/********************************************************************************
** Form generated from reading UI file 'HdrPreview.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HDRPREVIEW_H
#define UI_HDRPREVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_HdrPreview
{
public:
    QGridLayout *gridLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *acceptHDRButton;
    QFrame *hdrPreviewFrame;
    QGridLayout *gridLayout;
    QHBoxLayout *layout;
    QPushButton *pushButton;

    void setupUi(QDialog *HdrPreview)
    {
        if (HdrPreview->objectName().isEmpty())
            HdrPreview->setObjectName(QString::fromUtf8("HdrPreview"));
        HdrPreview->resize(660, 509);
        HdrPreview->setSizeGripEnabled(true);
        HdrPreview->setModal(true);
        gridLayout_2 = new QGridLayout(HdrPreview);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        horizontalSpacer = new QSpacerItem(555, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer, 1, 0, 1, 1);

        acceptHDRButton = new QPushButton(HdrPreview);
        acceptHDRButton->setObjectName(QString::fromUtf8("acceptHDRButton"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(acceptHDRButton->sizePolicy().hasHeightForWidth());
        acceptHDRButton->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(acceptHDRButton, 1, 2, 1, 1);

        hdrPreviewFrame = new QFrame(HdrPreview);
        hdrPreviewFrame->setObjectName(QString::fromUtf8("hdrPreviewFrame"));
        hdrPreviewFrame->setMinimumSize(QSize(500, 250));
        hdrPreviewFrame->setFrameShape(QFrame::StyledPanel);
        hdrPreviewFrame->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(hdrPreviewFrame);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        layout = new QHBoxLayout();
        layout->setObjectName(QString::fromUtf8("layout"));

        gridLayout->addLayout(layout, 0, 0, 1, 1);


        gridLayout_2->addWidget(hdrPreviewFrame, 0, 0, 1, 4);

        pushButton = new QPushButton(HdrPreview);
        pushButton->setObjectName(QString::fromUtf8("pushButton"));

        gridLayout_2->addWidget(pushButton, 1, 1, 1, 1);


        retranslateUi(HdrPreview);
        QObject::connect(pushButton, SIGNAL(clicked()), HdrPreview, SLOT(reject()));
        QObject::connect(acceptHDRButton, SIGNAL(clicked()), HdrPreview, SLOT(accept()));

        QMetaObject::connectSlotsByName(HdrPreview);
    } // setupUi

    void retranslateUi(QDialog *HdrPreview)
    {
        HdrPreview->setWindowTitle(QCoreApplication::translate("HdrPreview", "HDR Preview", nullptr));
        acceptHDRButton->setText(QCoreApplication::translate("HdrPreview", "Accept This HDR", nullptr));
        pushButton->setText(QCoreApplication::translate("HdrPreview", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HdrPreview: public Ui_HdrPreview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HDRPREVIEW_H
