/****************************************************************************
** Meta object code from reading C++ file 'Align.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/Alignment/Align.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Align.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Align_t {
    QByteArrayData data[15];
    char stringdata0[188];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Align_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Align_t qt_meta_stringdata_Align = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Align"
QT_MOC_LITERAL(1, 6, 16), // "finishedAligning"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 14), // "failedAligning"
QT_MOC_LITERAL(4, 39, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(5, 62, 9), // "dataReady"
QT_MOC_LITERAL(6, 72, 11), // "QByteArray&"
QT_MOC_LITERAL(7, 84, 12), // "ais_finished"
QT_MOC_LITERAL(8, 97, 8), // "exitcode"
QT_MOC_LITERAL(9, 106, 20), // "QProcess::ExitStatus"
QT_MOC_LITERAL(10, 127, 10), // "exitstatus"
QT_MOC_LITERAL(11, 138, 18), // "alignedFilesLoaded"
QT_MOC_LITERAL(12, 157, 15), // "ais_failed_slot"
QT_MOC_LITERAL(13, 173, 5), // "error"
QT_MOC_LITERAL(14, 179, 8) // "readData"

    },
    "Align\0finishedAligning\0\0failedAligning\0"
    "QProcess::ProcessError\0dataReady\0"
    "QByteArray&\0ais_finished\0exitcode\0"
    "QProcess::ExitStatus\0exitstatus\0"
    "alignedFilesLoaded\0ais_failed_slot\0"
    "error\0readData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Align[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       3,    1,   52,    2, 0x06 /* Public */,
       5,    1,   55,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   58,    2, 0x09 /* Protected */,
      11,    0,   63,    2, 0x09 /* Protected */,
      12,    1,   64,    2, 0x09 /* Protected */,
      14,    0,   67,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 6,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 9,    8,   10,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,   13,
    QMetaType::Void,

       0        // eod
};

void Align::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Align *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finishedAligning((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->failedAligning((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 2: _t->dataReady((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 3: _t->ais_finished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 4: _t->alignedFilesLoaded(); break;
        case 5: _t->ais_failed_slot((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 6: _t->readData(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Align::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Align::finishedAligning)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Align::*)(QProcess::ProcessError );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Align::failedAligning)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Align::*)(QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Align::dataReady)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Align::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Align.data,
    qt_meta_data_Align,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Align::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Align::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Align.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Align::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Align::finishedAligning(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Align::failedAligning(QProcess::ProcessError _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Align::dataReady(QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
