/********************************************************************************
** Form generated from reading UI file 'ImageQualityDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEQUALITYDIALOG_H
#define UI_IMAGEQUALITYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ImgQualityDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label_quality;
    QSlider *horizontalSlider;
    QSpinBox *spinBox;
    QFrame *fileSizePanel;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QLabel *label_filesize;
    QSpacerItem *horizontalSpacer;
    QPushButton *getSizeButton;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *cancelButton;
    QPushButton *saveButton;

    void setupUi(QDialog *ImgQualityDialog)
    {
        if (ImgQualityDialog->objectName().isEmpty())
            ImgQualityDialog->setObjectName(QString::fromUtf8("ImgQualityDialog"));
        ImgQualityDialog->resize(424, 161);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ImgQualityDialog->sizePolicy().hasHeightForWidth());
        ImgQualityDialog->setSizePolicy(sizePolicy);
        ImgQualityDialog->setMaximumSize(QSize(16777215, 165));
        ImgQualityDialog->setModal(true);
        verticalLayout = new QVBoxLayout(ImgQualityDialog);
        verticalLayout->setSpacing(4);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        groupBox = new QGroupBox(ImgQualityDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_quality = new QLabel(groupBox);
        label_quality->setObjectName(QString::fromUtf8("label_quality"));
        label_quality->setMinimumSize(QSize(60, 0));
        label_quality->setMaximumSize(QSize(60, 16777215));
        label_quality->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_quality, 0, 0, 2, 1);

        horizontalSlider = new QSlider(groupBox);
        horizontalSlider->setObjectName(QString::fromUtf8("horizontalSlider"));
        horizontalSlider->setMinimumSize(QSize(200, 0));
        horizontalSlider->setMaximumSize(QSize(200, 16777215));
        horizontalSlider->setMaximum(100);
        horizontalSlider->setPageStep(25);
        horizontalSlider->setValue(100);
        horizontalSlider->setOrientation(Qt::Horizontal);
        horizontalSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout->addWidget(horizontalSlider, 0, 2, 2, 1);

        spinBox = new QSpinBox(groupBox);
        spinBox->setObjectName(QString::fromUtf8("spinBox"));
        spinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        spinBox->setMaximum(100);
        spinBox->setValue(100);

        gridLayout->addWidget(spinBox, 0, 3, 2, 1);

        fileSizePanel = new QFrame(groupBox);
        fileSizePanel->setObjectName(QString::fromUtf8("fileSizePanel"));
        fileSizePanel->setFrameShape(QFrame::StyledPanel);
        fileSizePanel->setFrameShadow(QFrame::Raised);
        horizontalLayout_2 = new QHBoxLayout(fileSizePanel);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(fileSizePanel);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(label_2);

        label_filesize = new QLabel(fileSizePanel);
        label_filesize->setObjectName(QString::fromUtf8("label_filesize"));

        horizontalLayout_2->addWidget(label_filesize);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        getSizeButton = new QPushButton(fileSizePanel);
        getSizeButton->setObjectName(QString::fromUtf8("getSizeButton"));

        horizontalLayout_2->addWidget(getSizeButton);


        gridLayout->addWidget(fileSizePanel, 2, 2, 2, 2);


        verticalLayout->addWidget(groupBox);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalSpacer_3 = new QSpacerItem(48, 26, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        cancelButton = new QPushButton(ImgQualityDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        horizontalLayout_3->addWidget(cancelButton);

        saveButton = new QPushButton(ImgQualityDialog);
        saveButton->setObjectName(QString::fromUtf8("saveButton"));

        horizontalLayout_3->addWidget(saveButton);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(ImgQualityDialog);
        QObject::connect(horizontalSlider, SIGNAL(valueChanged(int)), spinBox, SLOT(setValue(int)));
        QObject::connect(spinBox, SIGNAL(valueChanged(int)), horizontalSlider, SLOT(setValue(int)));
        QObject::connect(saveButton, SIGNAL(clicked()), ImgQualityDialog, SLOT(accept()));
        QObject::connect(cancelButton, SIGNAL(clicked()), ImgQualityDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ImgQualityDialog);
    } // setupUi

    void retranslateUi(QDialog *ImgQualityDialog)
    {
        ImgQualityDialog->setWindowTitle(QCoreApplication::translate("ImgQualityDialog", "Save as", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ImgQualityDialog", "Saved File Quality", nullptr));
        label_quality->setText(QCoreApplication::translate("ImgQualityDialog", "Quality:", nullptr));
        label_2->setText(QCoreApplication::translate("ImgQualityDialog", "File size:", nullptr));
        label_filesize->setText(QCoreApplication::translate("ImgQualityDialog", "Unknown", nullptr));
        getSizeButton->setText(QCoreApplication::translate("ImgQualityDialog", "Ca&lculate", nullptr));
        cancelButton->setText(QCoreApplication::translate("ImgQualityDialog", "&Cancel", nullptr));
        saveButton->setText(QCoreApplication::translate("ImgQualityDialog", "&Save", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ImgQualityDialog: public Ui_ImgQualityDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEQUALITYDIALOG_H
