/********************************************************************************
** Form generated from reading UI file 'SupportedCamerasDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SUPPORTEDCAMERASDIALOG_H
#define UI_SUPPORTEDCAMERASDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_SupportedCamerasDialog
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label;
    QLabel *labelVersion;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout;
    QLabel *labelLibRawVersion;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelCount;
    QLineEdit *lineEdit;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton;
    QListWidget *listWidget;

    void setupUi(QDialog *SupportedCamerasDialog)
    {
        if (SupportedCamerasDialog->objectName().isEmpty())
            SupportedCamerasDialog->setObjectName(QString::fromUtf8("SupportedCamerasDialog"));
        SupportedCamerasDialog->resize(400, 581);
        gridLayout_2 = new QGridLayout(SupportedCamerasDialog);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer, 2, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout->addItem(verticalSpacer_2, 0, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        label = new QLabel(SupportedCamerasDialog);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);
        label->setPixmap(QPixmap(QString::fromUtf8(":/program-icons/luminance-hdr")));

        gridLayout->addWidget(label, 1, 1, 1, 1);

        labelVersion = new QLabel(SupportedCamerasDialog);
        labelVersion->setObjectName(QString::fromUtf8("labelVersion"));
        QFont font;
        font.setPointSize(12);
        font.setBold(true);
        font.setWeight(75);
        labelVersion->setFont(font);

        gridLayout->addWidget(labelVersion, 1, 3, 1, 1);

        label_2 = new QLabel(SupportedCamerasDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font1;
        font1.setPointSize(14);
        font1.setBold(true);
        font1.setWeight(75);
        label_2->setFont(font1);
        label_2->setTextFormat(Qt::RichText);

        gridLayout->addWidget(label_2, 0, 3, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 1, 0, 1, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        labelLibRawVersion = new QLabel(SupportedCamerasDialog);
        labelLibRawVersion->setObjectName(QString::fromUtf8("labelLibRawVersion"));

        horizontalLayout->addWidget(labelLibRawVersion);


        gridLayout_2->addLayout(horizontalLayout, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        labelCount = new QLabel(SupportedCamerasDialog);
        labelCount->setObjectName(QString::fromUtf8("labelCount"));

        horizontalLayout_2->addWidget(labelCount);


        gridLayout_2->addLayout(horizontalLayout_2, 2, 0, 1, 1);

        lineEdit = new QLineEdit(SupportedCamerasDialog);
        lineEdit->setObjectName(QString::fromUtf8("lineEdit"));
        lineEdit->setClearButtonEnabled(true);

        gridLayout_2->addWidget(lineEdit, 4, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);

        pushButton = new QPushButton(SupportedCamerasDialog);
        pushButton->setObjectName(QString::fromUtf8("pushButton"));

        horizontalLayout_3->addWidget(pushButton);


        gridLayout_2->addLayout(horizontalLayout_3, 5, 0, 1, 1);

        listWidget = new QListWidget(SupportedCamerasDialog);
        listWidget->setObjectName(QString::fromUtf8("listWidget"));

        gridLayout_2->addWidget(listWidget, 3, 0, 1, 1);

        QWidget::setTabOrder(lineEdit, pushButton);
        QWidget::setTabOrder(pushButton, listWidget);

        retranslateUi(SupportedCamerasDialog);
        QObject::connect(pushButton, SIGNAL(clicked()), SupportedCamerasDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(SupportedCamerasDialog);
    } // setupUi

    void retranslateUi(QDialog *SupportedCamerasDialog)
    {
        SupportedCamerasDialog->setWindowTitle(QCoreApplication::translate("SupportedCamerasDialog", "List of supported cameras", nullptr));
        label->setText(QString());
        labelVersion->setText(QCoreApplication::translate("SupportedCamerasDialog", "TextLabel", nullptr));
        label_2->setText(QCoreApplication::translate("SupportedCamerasDialog", "<html><head/><body><p><span style=\" font-weight:600; color:#0078dc;\">Luminance HDR</span></p></body></html>", nullptr));
        labelLibRawVersion->setText(QCoreApplication::translate("SupportedCamerasDialog", "With LibRaw version", nullptr));
        labelCount->setText(QCoreApplication::translate("SupportedCamerasDialog", "models listed", nullptr));
        lineEdit->setInputMask(QString());
        lineEdit->setText(QString());
        lineEdit->setPlaceholderText(QCoreApplication::translate("SupportedCamerasDialog", "search...", nullptr));
        pushButton->setText(QCoreApplication::translate("SupportedCamerasDialog", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SupportedCamerasDialog: public Ui_SupportedCamerasDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SUPPORTEDCAMERASDIALOG_H
