/********************************************************************************
** Form generated from reading UI file 'ExportToHtmlDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTTOHTMLDIALOG_H
#define UI_EXPORTTOHTMLDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExportToHtmlDialog
{
public:
    QGridLayout *gridLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QLineEdit *lineEditPageName;
    QLineEdit *lineEditOutputFolder;
    QLabel *label_4;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *checkBoxOpenOnBrowser;
    QLabel *label_6;
    QSpinBox *spinBoxSize;
    QLabel *label_2;
    QSpinBox *spinBoxQuality;
    QLineEdit *lineEditImagesFolder;
    QCheckBox *checkBoxImagesFolder;
    QLabel *label_5;
    QPushButton *OutputFolderButton;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *CancelButton;
    QPushButton *ExportButton;

    void setupUi(QDialog *ExportToHtmlDialog)
    {
        if (ExportToHtmlDialog->objectName().isEmpty())
            ExportToHtmlDialog->setObjectName(QString::fromUtf8("ExportToHtmlDialog"));
        ExportToHtmlDialog->resize(600, 360);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ExportToHtmlDialog->sizePolicy().hasHeightForWidth());
        ExportToHtmlDialog->setSizePolicy(sizePolicy);
        ExportToHtmlDialog->setMinimumSize(QSize(0, 0));
        ExportToHtmlDialog->setMaximumSize(QSize(16777215, 16777215));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        ExportToHtmlDialog->setWindowIcon(icon);
        gridLayout_2 = new QGridLayout(ExportToHtmlDialog);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(ExportToHtmlDialog);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(16);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_3 = new QLabel(ExportToHtmlDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        lineEditPageName = new QLineEdit(ExportToHtmlDialog);
        lineEditPageName->setObjectName(QString::fromUtf8("lineEditPageName"));

        gridLayout->addWidget(lineEditPageName, 0, 1, 1, 1);

        lineEditOutputFolder = new QLineEdit(ExportToHtmlDialog);
        lineEditOutputFolder->setObjectName(QString::fromUtf8("lineEditOutputFolder"));

        gridLayout->addWidget(lineEditOutputFolder, 1, 1, 1, 1);

        label_4 = new QLabel(ExportToHtmlDialog);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        sizePolicy1.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy1);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_4, 1, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        checkBoxOpenOnBrowser = new QCheckBox(ExportToHtmlDialog);
        checkBoxOpenOnBrowser->setObjectName(QString::fromUtf8("checkBoxOpenOnBrowser"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(checkBoxOpenOnBrowser->sizePolicy().hasHeightForWidth());
        checkBoxOpenOnBrowser->setSizePolicy(sizePolicy2);
        checkBoxOpenOnBrowser->setLayoutDirection(Qt::RightToLeft);

        horizontalLayout_2->addWidget(checkBoxOpenOnBrowser);

        label_6 = new QLabel(ExportToHtmlDialog);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);
        label_6->setLayoutDirection(Qt::LeftToRight);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label_6);

        spinBoxSize = new QSpinBox(ExportToHtmlDialog);
        spinBoxSize->setObjectName(QString::fromUtf8("spinBoxSize"));
        spinBoxSize->setMinimum(1);
        spinBoxSize->setMaximum(100);
        spinBoxSize->setValue(100);

        horizontalLayout_2->addWidget(spinBoxSize);

        label_2 = new QLabel(ExportToHtmlDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label_2);

        spinBoxQuality = new QSpinBox(ExportToHtmlDialog);
        spinBoxQuality->setObjectName(QString::fromUtf8("spinBoxQuality"));
        spinBoxQuality->setMinimum(1);
        spinBoxQuality->setMaximum(4);
        spinBoxQuality->setValue(2);

        horizontalLayout_2->addWidget(spinBoxQuality);


        gridLayout->addLayout(horizontalLayout_2, 3, 1, 1, 1);

        lineEditImagesFolder = new QLineEdit(ExportToHtmlDialog);
        lineEditImagesFolder->setObjectName(QString::fromUtf8("lineEditImagesFolder"));
        lineEditImagesFolder->setEnabled(false);

        gridLayout->addWidget(lineEditImagesFolder, 2, 1, 1, 1);

        checkBoxImagesFolder = new QCheckBox(ExportToHtmlDialog);
        checkBoxImagesFolder->setObjectName(QString::fromUtf8("checkBoxImagesFolder"));
        sizePolicy2.setHeightForWidth(checkBoxImagesFolder->sizePolicy().hasHeightForWidth());
        checkBoxImagesFolder->setSizePolicy(sizePolicy2);
        checkBoxImagesFolder->setLayoutDirection(Qt::LeftToRight);
        checkBoxImagesFolder->setText(QString::fromUtf8(""));

        gridLayout->addWidget(checkBoxImagesFolder, 2, 2, 1, 1);

        label_5 = new QLabel(ExportToHtmlDialog);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setEnabled(false);
        sizePolicy1.setHeightForWidth(label_5->sizePolicy().hasHeightForWidth());
        label_5->setSizePolicy(sizePolicy1);
        label_5->setMinimumSize(QSize(0, 0));
        label_5->setMaximumSize(QSize(16777215, 16777215));
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_5, 2, 0, 1, 1);

        OutputFolderButton = new QPushButton(ExportToHtmlDialog);
        OutputFolderButton->setObjectName(QString::fromUtf8("OutputFolderButton"));
        sizePolicy2.setHeightForWidth(OutputFolderButton->sizePolicy().hasHeightForWidth());
        OutputFolderButton->setSizePolicy(sizePolicy2);
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        OutputFolderButton->setIcon(icon1);
        OutputFolderButton->setAutoDefault(false);

        gridLayout->addWidget(OutputFolderButton, 1, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(203, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        CancelButton = new QPushButton(ExportToHtmlDialog);
        CancelButton->setObjectName(QString::fromUtf8("CancelButton"));
        CancelButton->setAutoDefault(false);

        horizontalLayout_3->addWidget(CancelButton);

        ExportButton = new QPushButton(ExportToHtmlDialog);
        ExportButton->setObjectName(QString::fromUtf8("ExportButton"));
        ExportButton->setEnabled(false);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("document-export");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        ExportButton->setIcon(icon2);
        ExportButton->setAutoDefault(false);

        horizontalLayout_3->addWidget(ExportButton);


        horizontalLayout->addLayout(horizontalLayout_3);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout_2->addLayout(verticalLayout, 0, 0, 1, 1);

        QWidget::setTabOrder(lineEditPageName, lineEditOutputFolder);
        QWidget::setTabOrder(lineEditOutputFolder, OutputFolderButton);
        QWidget::setTabOrder(OutputFolderButton, lineEditImagesFolder);
        QWidget::setTabOrder(lineEditImagesFolder, checkBoxImagesFolder);
        QWidget::setTabOrder(checkBoxImagesFolder, checkBoxOpenOnBrowser);
        QWidget::setTabOrder(checkBoxOpenOnBrowser, spinBoxSize);
        QWidget::setTabOrder(spinBoxSize, spinBoxQuality);
        QWidget::setTabOrder(spinBoxQuality, CancelButton);
        QWidget::setTabOrder(CancelButton, ExportButton);

        retranslateUi(ExportToHtmlDialog);
        QObject::connect(checkBoxImagesFolder, SIGNAL(clicked(bool)), lineEditImagesFolder, SLOT(setEnabled(bool)));
        QObject::connect(checkBoxImagesFolder, SIGNAL(clicked(bool)), label_5, SLOT(setEnabled(bool)));
        QObject::connect(CancelButton, SIGNAL(clicked()), ExportToHtmlDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ExportToHtmlDialog);
    } // setupUi

    void retranslateUi(QDialog *ExportToHtmlDialog)
    {
        ExportToHtmlDialog->setWindowTitle(QCoreApplication::translate("ExportToHtmlDialog", "Export HDR to HTML", nullptr));
        label->setText(QCoreApplication::translate("ExportToHtmlDialog", "Export to HTML", nullptr));
        label_3->setText(QCoreApplication::translate("ExportToHtmlDialog", "Web page:", nullptr));
        label_4->setText(QCoreApplication::translate("ExportToHtmlDialog", "Output folder:", nullptr));
        checkBoxOpenOnBrowser->setText(QCoreApplication::translate("ExportToHtmlDialog", "Open web browser", nullptr));
        label_6->setText(QCoreApplication::translate("ExportToHtmlDialog", "Size:", nullptr));
        spinBoxSize->setSuffix(QCoreApplication::translate("ExportToHtmlDialog", "%", nullptr));
        label_2->setText(QCoreApplication::translate("ExportToHtmlDialog", "Quality:", nullptr));
        label_5->setText(QCoreApplication::translate("ExportToHtmlDialog", "Images folder:", nullptr));
        OutputFolderButton->setText(QCoreApplication::translate("ExportToHtmlDialog", "Select...", nullptr));
        CancelButton->setText(QCoreApplication::translate("ExportToHtmlDialog", "Cancel", nullptr));
        ExportButton->setText(QCoreApplication::translate("ExportToHtmlDialog", "Export", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportToHtmlDialog: public Ui_ExportToHtmlDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTTOHTMLDIALOG_H
