/********************************************************************************
** Form generated from reading UI file 'BatchTMDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BATCHTMDIALOG_H
#define UI_BATCHTMDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_BatchTMDialog
{
public:
    QVBoxLayout *_2;
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox_3;
    QHBoxLayout *_6;
    QListWidget *listWidget_HDRs;
    QGridLayout *gridLayout_2;
    QLabel *label_2;
    QSpacerItem *verticalSpacer;
    QToolButton *remove_HDRs_Button;
    QToolButton *remove_all_HDRs_Button;
    QToolButton *add_HDRs_Button;
    QToolButton *add_dir_HDRs_Button;
    QSpacerItem *verticalSpacer_3;
    QGroupBox *groupBox_5;
    QHBoxLayout *_5;
    QGridLayout *gridLayout_4;
    QToolButton *add_TMopts_Button;
    QToolButton *add_dir_TMopts_Button;
    QToolButton *from_Database_Button;
    QToolButton *remove_TMOpts_Button;
    QSpacerItem *verticalSpacer_2;
    QToolButton *remove_all_TMOpts_Button;
    QSpacerItem *verticalSpacer_4;
    QListWidget *listWidget_TMopts;
    QGroupBox *groupBoxOutput;
    QGridLayout *gridLayout;
    QLabel *label;
    QLabel *label_3;
    QPushButton *out_folder_Button;
    QLineEdit *out_folder_widgets;
    QSpinBox *spinBox_Width;
    QSlider *horizontalSlider_Width;
    QLabel *label_4;
    QComboBox *comboBoxFormat;
    QToolButton *formatSettingsButton;
    QGroupBox *groupBox;
    QVBoxLayout *_8;
    QListView *Log_Widget;
    QHBoxLayout *hboxLayout1;
    QLabel *filterLabel1;
    QComboBox *filterComboBox;
    QLabel *filterLabel2;
    QLineEdit *filterLineEdit;
    QToolButton *clearTextToolButton;
    QProgressBar *overallProgressBar;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem;
    QPushButton *cancelbutton;
    QPushButton *BatchGoButton;

    void setupUi(QDialog *BatchTMDialog)
    {
        if (BatchTMDialog->objectName().isEmpty())
            BatchTMDialog->setObjectName(QString::fromUtf8("BatchTMDialog"));
        BatchTMDialog->resize(682, 652);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        BatchTMDialog->setWindowIcon(icon);
        _2 = new QVBoxLayout(BatchTMDialog);
        _2->setObjectName(QString::fromUtf8("_2"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(9);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        groupBox_3 = new QGroupBox(BatchTMDialog);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        groupBox_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        _6 = new QHBoxLayout(groupBox_3);
        _6->setObjectName(QString::fromUtf8("_6"));
        _6->setContentsMargins(2, 2, 2, 2);
        listWidget_HDRs = new QListWidget(groupBox_3);
        listWidget_HDRs->setObjectName(QString::fromUtf8("listWidget_HDRs"));
        listWidget_HDRs->setAlternatingRowColors(true);
        listWidget_HDRs->setSelectionMode(QAbstractItemView::MultiSelection);
        listWidget_HDRs->setSelectionBehavior(QAbstractItemView::SelectRows);

        _6->addWidget(listWidget_HDRs);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(-1, -1, 6, -1);
        label_2 = new QLabel(groupBox_3);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setMinimumSize(QSize(34, 40));

        gridLayout_2->addWidget(label_2, 2, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 6, 0, 1, 1);

        remove_HDRs_Button = new QToolButton(groupBox_3);
        remove_HDRs_Button->setObjectName(QString::fromUtf8("remove_HDRs_Button"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        remove_HDRs_Button->setIcon(icon1);
        remove_HDRs_Button->setIconSize(QSize(24, 24));

        gridLayout_2->addWidget(remove_HDRs_Button, 3, 0, 1, 1);

        remove_all_HDRs_Button = new QToolButton(groupBox_3);
        remove_all_HDRs_Button->setObjectName(QString::fromUtf8("remove_all_HDRs_Button"));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("edit-clear");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        remove_all_HDRs_Button->setIcon(icon2);
        remove_all_HDRs_Button->setIconSize(QSize(24, 24));

        gridLayout_2->addWidget(remove_all_HDRs_Button, 4, 0, 1, 1);

        add_HDRs_Button = new QToolButton(groupBox_3);
        add_HDRs_Button->setObjectName(QString::fromUtf8("add_HDRs_Button"));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        add_HDRs_Button->setIcon(icon3);
        add_HDRs_Button->setIconSize(QSize(24, 24));

        gridLayout_2->addWidget(add_HDRs_Button, 1, 0, 1, 1);

        add_dir_HDRs_Button = new QToolButton(groupBox_3);
        add_dir_HDRs_Button->setObjectName(QString::fromUtf8("add_dir_HDRs_Button"));
        QIcon icon4;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        add_dir_HDRs_Button->setIcon(icon4);
        add_dir_HDRs_Button->setIconSize(QSize(24, 24));

        gridLayout_2->addWidget(add_dir_HDRs_Button, 0, 0, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_2->addItem(verticalSpacer_3, 5, 0, 1, 1);


        _6->addLayout(gridLayout_2);


        hboxLayout->addWidget(groupBox_3);

        groupBox_5 = new QGroupBox(BatchTMDialog);
        groupBox_5->setObjectName(QString::fromUtf8("groupBox_5"));
        groupBox_5->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        _5 = new QHBoxLayout(groupBox_5);
        _5->setObjectName(QString::fromUtf8("_5"));
        _5->setContentsMargins(2, 2, 2, 2);
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setContentsMargins(6, -1, -1, -1);
        add_TMopts_Button = new QToolButton(groupBox_5);
        add_TMopts_Button->setObjectName(QString::fromUtf8("add_TMopts_Button"));
        add_TMopts_Button->setIcon(icon3);
        add_TMopts_Button->setIconSize(QSize(24, 24));

        gridLayout_4->addWidget(add_TMopts_Button, 1, 0, 1, 1);

        add_dir_TMopts_Button = new QToolButton(groupBox_5);
        add_dir_TMopts_Button->setObjectName(QString::fromUtf8("add_dir_TMopts_Button"));
        add_dir_TMopts_Button->setIcon(icon4);
        add_dir_TMopts_Button->setIconSize(QSize(24, 24));

        gridLayout_4->addWidget(add_dir_TMopts_Button, 0, 0, 1, 1);

        from_Database_Button = new QToolButton(groupBox_5);
        from_Database_Button->setObjectName(QString::fromUtf8("from_Database_Button"));
        QIcon icon5;
        iconThemeName = QString::fromUtf8("vcs-added");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        from_Database_Button->setIcon(icon5);
        from_Database_Button->setIconSize(QSize(24, 24));

        gridLayout_4->addWidget(from_Database_Button, 2, 0, 1, 1);

        remove_TMOpts_Button = new QToolButton(groupBox_5);
        remove_TMOpts_Button->setObjectName(QString::fromUtf8("remove_TMOpts_Button"));
        remove_TMOpts_Button->setIcon(icon1);
        remove_TMOpts_Button->setIconSize(QSize(24, 24));

        gridLayout_4->addWidget(remove_TMOpts_Button, 3, 0, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer_2, 6, 0, 1, 1);

        remove_all_TMOpts_Button = new QToolButton(groupBox_5);
        remove_all_TMOpts_Button->setObjectName(QString::fromUtf8("remove_all_TMOpts_Button"));
        remove_all_TMOpts_Button->setIcon(icon2);
        remove_all_TMOpts_Button->setIconSize(QSize(24, 24));

        gridLayout_4->addWidget(remove_all_TMOpts_Button, 4, 0, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer_4, 5, 0, 1, 1);


        _5->addLayout(gridLayout_4);

        listWidget_TMopts = new QListWidget(groupBox_5);
        listWidget_TMopts->setObjectName(QString::fromUtf8("listWidget_TMopts"));
        listWidget_TMopts->setAlternatingRowColors(true);
        listWidget_TMopts->setSelectionMode(QAbstractItemView::MultiSelection);
        listWidget_TMopts->setSelectionBehavior(QAbstractItemView::SelectRows);

        _5->addWidget(listWidget_TMopts);


        hboxLayout->addWidget(groupBox_5);


        _2->addLayout(hboxLayout);

        groupBoxOutput = new QGroupBox(BatchTMDialog);
        groupBoxOutput->setObjectName(QString::fromUtf8("groupBoxOutput"));
        gridLayout = new QGridLayout(groupBoxOutput);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(groupBoxOutput);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 1, 0, 1, 1);

        label_3 = new QLabel(groupBoxOutput);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        out_folder_Button = new QPushButton(groupBoxOutput);
        out_folder_Button->setObjectName(QString::fromUtf8("out_folder_Button"));
        out_folder_Button->setIcon(icon4);
        out_folder_Button->setAutoDefault(false);

        gridLayout->addWidget(out_folder_Button, 3, 5, 1, 1);

        out_folder_widgets = new QLineEdit(groupBoxOutput);
        out_folder_widgets->setObjectName(QString::fromUtf8("out_folder_widgets"));
        out_folder_widgets->setReadOnly(true);

        gridLayout->addWidget(out_folder_widgets, 3, 1, 1, 4);

        spinBox_Width = new QSpinBox(groupBoxOutput);
        spinBox_Width->setObjectName(QString::fromUtf8("spinBox_Width"));
        spinBox_Width->setMinimum(1);
        spinBox_Width->setMaximum(100);
        spinBox_Width->setValue(100);

        gridLayout->addWidget(spinBox_Width, 1, 4, 1, 1);

        horizontalSlider_Width = new QSlider(groupBoxOutput);
        horizontalSlider_Width->setObjectName(QString::fromUtf8("horizontalSlider_Width"));
        horizontalSlider_Width->setMinimum(1);
        horizontalSlider_Width->setMaximum(100);
        horizontalSlider_Width->setValue(100);
        horizontalSlider_Width->setOrientation(Qt::Horizontal);
        horizontalSlider_Width->setTickPosition(QSlider::TicksBelow);
        horizontalSlider_Width->setTickInterval(10);

        gridLayout->addWidget(horizontalSlider_Width, 1, 1, 1, 3);

        label_4 = new QLabel(groupBoxOutput);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 0, 0, 1, 1);

        comboBoxFormat = new QComboBox(groupBoxOutput);
        comboBoxFormat->setObjectName(QString::fromUtf8("comboBoxFormat"));

        gridLayout->addWidget(comboBoxFormat, 0, 1, 1, 1);

        formatSettingsButton = new QToolButton(groupBoxOutput);
        formatSettingsButton->setObjectName(QString::fromUtf8("formatSettingsButton"));
        formatSettingsButton->setText(QString::fromUtf8("..."));

        gridLayout->addWidget(formatSettingsButton, 0, 2, 1, 1);


        _2->addWidget(groupBoxOutput);

        groupBox = new QGroupBox(BatchTMDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        _8 = new QVBoxLayout(groupBox);
        _8->setObjectName(QString::fromUtf8("_8"));
        _8->setContentsMargins(2, 2, 2, 2);
        Log_Widget = new QListView(groupBox);
        Log_Widget->setObjectName(QString::fromUtf8("Log_Widget"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Log_Widget->sizePolicy().hasHeightForWidth());
        Log_Widget->setSizePolicy(sizePolicy1);
        Log_Widget->setMaximumSize(QSize(16777215, 120));
        Log_Widget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        Log_Widget->setSelectionMode(QAbstractItemView::ContiguousSelection);
        Log_Widget->setResizeMode(QListView::Adjust);

        _8->addWidget(Log_Widget);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        filterLabel1 = new QLabel(groupBox);
        filterLabel1->setObjectName(QString::fromUtf8("filterLabel1"));

        hboxLayout1->addWidget(filterLabel1);

        filterComboBox = new QComboBox(groupBox);
        filterComboBox->addItem(QString());
        filterComboBox->addItem(QString());
        filterComboBox->addItem(QString());
        filterComboBox->setObjectName(QString::fromUtf8("filterComboBox"));
        filterComboBox->setEditable(false);

        hboxLayout1->addWidget(filterComboBox);

        filterLabel2 = new QLabel(groupBox);
        filterLabel2->setObjectName(QString::fromUtf8("filterLabel2"));

        hboxLayout1->addWidget(filterLabel2);

        filterLineEdit = new QLineEdit(groupBox);
        filterLineEdit->setObjectName(QString::fromUtf8("filterLineEdit"));

        hboxLayout1->addWidget(filterLineEdit);

        clearTextToolButton = new QToolButton(groupBox);
        clearTextToolButton->setObjectName(QString::fromUtf8("clearTextToolButton"));
        clearTextToolButton->setEnabled(false);
        clearTextToolButton->setMinimumSize(QSize(28, 28));
        clearTextToolButton->setMaximumSize(QSize(28, 28));
        clearTextToolButton->setIcon(icon2);

        hboxLayout1->addWidget(clearTextToolButton);


        _8->addLayout(hboxLayout1);


        _2->addWidget(groupBox);

        overallProgressBar = new QProgressBar(BatchTMDialog);
        overallProgressBar->setObjectName(QString::fromUtf8("overallProgressBar"));
        overallProgressBar->setAlignment(Qt::AlignCenter);

        _2->addWidget(overallProgressBar);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem);

        cancelbutton = new QPushButton(BatchTMDialog);
        cancelbutton->setObjectName(QString::fromUtf8("cancelbutton"));
        cancelbutton->setAutoDefault(false);

        hboxLayout2->addWidget(cancelbutton);

        BatchGoButton = new QPushButton(BatchTMDialog);
        BatchGoButton->setObjectName(QString::fromUtf8("BatchGoButton"));
        BatchGoButton->setEnabled(false);
        BatchGoButton->setAutoDefault(false);

        hboxLayout2->addWidget(BatchGoButton);


        _2->addLayout(hboxLayout2);

#if QT_CONFIG(shortcut)
        filterLabel1->setBuddy(filterComboBox);
        filterLabel2->setBuddy(filterLineEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(listWidget_HDRs, add_dir_HDRs_Button);
        QWidget::setTabOrder(add_dir_HDRs_Button, add_HDRs_Button);
        QWidget::setTabOrder(add_HDRs_Button, remove_HDRs_Button);
        QWidget::setTabOrder(remove_HDRs_Button, remove_all_HDRs_Button);
        QWidget::setTabOrder(remove_all_HDRs_Button, listWidget_TMopts);
        QWidget::setTabOrder(listWidget_TMopts, add_dir_TMopts_Button);
        QWidget::setTabOrder(add_dir_TMopts_Button, add_TMopts_Button);
        QWidget::setTabOrder(add_TMopts_Button, from_Database_Button);
        QWidget::setTabOrder(from_Database_Button, remove_TMOpts_Button);
        QWidget::setTabOrder(remove_TMOpts_Button, remove_all_TMOpts_Button);
        QWidget::setTabOrder(remove_all_TMOpts_Button, comboBoxFormat);
        QWidget::setTabOrder(comboBoxFormat, formatSettingsButton);
        QWidget::setTabOrder(formatSettingsButton, horizontalSlider_Width);
        QWidget::setTabOrder(horizontalSlider_Width, spinBox_Width);
        QWidget::setTabOrder(spinBox_Width, out_folder_widgets);
        QWidget::setTabOrder(out_folder_widgets, out_folder_Button);
        QWidget::setTabOrder(out_folder_Button, Log_Widget);
        QWidget::setTabOrder(Log_Widget, filterComboBox);
        QWidget::setTabOrder(filterComboBox, filterLineEdit);
        QWidget::setTabOrder(filterLineEdit, clearTextToolButton);
        QWidget::setTabOrder(clearTextToolButton, cancelbutton);
        QWidget::setTabOrder(cancelbutton, BatchGoButton);

        retranslateUi(BatchTMDialog);
        QObject::connect(horizontalSlider_Width, SIGNAL(valueChanged(int)), spinBox_Width, SLOT(setValue(int)));
        QObject::connect(spinBox_Width, SIGNAL(valueChanged(int)), horizontalSlider_Width, SLOT(setValue(int)));

        BatchGoButton->setDefault(false);


        QMetaObject::connectSlotsByName(BatchTMDialog);
    } // setupUi

    void retranslateUi(QDialog *BatchTMDialog)
    {
        BatchTMDialog->setWindowTitle(QCoreApplication::translate("BatchTMDialog", "Batch Tone Mapping", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("BatchTMDialog", "HDR Images to Convert", nullptr));
#if QT_CONFIG(tooltip)
        listWidget_HDRs->setToolTip(QCoreApplication::translate("BatchTMDialog", "List of HDRs that will be tone mapped", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QString());
#if QT_CONFIG(tooltip)
        remove_HDRs_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Remove Files (Alt+R)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        remove_HDRs_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+R", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        remove_all_HDRs_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Remove All Files (Alt+X)", nullptr));
#endif // QT_CONFIG(tooltip)
        remove_all_HDRs_Button->setText(QCoreApplication::translate("BatchTMDialog", "...", nullptr));
#if QT_CONFIG(shortcut)
        remove_all_HDRs_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+X", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        add_HDRs_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Add Files (Alt+F)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        add_HDRs_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+F", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        add_dir_HDRs_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Add Directory (Alt+D)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        add_dir_HDRs_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        groupBox_5->setTitle(QCoreApplication::translate("BatchTMDialog", "Tone Mapping Settings", nullptr));
#if QT_CONFIG(tooltip)
        add_TMopts_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Add Files (Alt+L)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        add_TMopts_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        add_dir_TMopts_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Add Directory (Alt+I)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        add_dir_TMopts_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+I", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        from_Database_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Add from Database (Alt+B)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        from_Database_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+B", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        remove_TMOpts_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Remove Settings (Alt+M)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        remove_TMOpts_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+M", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        remove_all_TMOpts_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Remove All Settings (Alt+W)", nullptr));
#endif // QT_CONFIG(tooltip)
        remove_all_TMOpts_Button->setText(QCoreApplication::translate("BatchTMDialog", "...", nullptr));
#if QT_CONFIG(shortcut)
        remove_all_TMOpts_Button->setShortcut(QCoreApplication::translate("BatchTMDialog", "Alt+W", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        listWidget_TMopts->setToolTip(QCoreApplication::translate("BatchTMDialog", "List of Tone Mapping Setting files that will be used to tone map each HDR", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        groupBoxOutput->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        groupBoxOutput->setTitle(QCoreApplication::translate("BatchTMDialog", "Output", nullptr));
        label->setText(QCoreApplication::translate("BatchTMDialog", "Output Image Width:", nullptr));
        label_3->setText(QCoreApplication::translate("BatchTMDialog", "Output Folder:", nullptr));
#if QT_CONFIG(tooltip)
        out_folder_Button->setToolTip(QCoreApplication::translate("BatchTMDialog", "Specify output folder for the tone mapped files", nullptr));
#endif // QT_CONFIG(tooltip)
        out_folder_Button->setText(QCoreApplication::translate("BatchTMDialog", "Selec&t...", nullptr));
#if QT_CONFIG(tooltip)
        out_folder_widgets->setToolTip(QCoreApplication::translate("BatchTMDialog", "Specify output folder for the tone mapped files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox_Width->setToolTip(QCoreApplication::translate("BatchTMDialog", "Width of tonemapped image in percent of original image", nullptr));
#endif // QT_CONFIG(tooltip)
        spinBox_Width->setSuffix(QCoreApplication::translate("BatchTMDialog", " %", nullptr));
#if QT_CONFIG(tooltip)
        horizontalSlider_Width->setToolTip(QCoreApplication::translate("BatchTMDialog", "Width of tonemapped image in percent of original image", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("BatchTMDialog", "Output Format:", nullptr));
#if QT_CONFIG(tooltip)
        comboBoxFormat->setToolTip(QCoreApplication::translate("BatchTMDialog", "Image format of tonemapped images", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        formatSettingsButton->setToolTip(QCoreApplication::translate("BatchTMDialog", "Image format options", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("BatchTMDialog", "Conversion Log", nullptr));
#if QT_CONFIG(tooltip)
        Log_Widget->setToolTip(QCoreApplication::translate("BatchTMDialog", "Batch operations report", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel1->setText(QCoreApplication::translate("BatchTMDialog", "Show onl&y:", nullptr));
        filterComboBox->setItemText(0, QCoreApplication::translate("BatchTMDialog", "All messages", nullptr));
        filterComboBox->setItemText(1, QCoreApplication::translate("BatchTMDialog", "Errors only", nullptr));
        filterComboBox->setItemText(2, QCoreApplication::translate("BatchTMDialog", "Success messages", nullptr));

#if QT_CONFIG(tooltip)
        filterComboBox->setToolTip(QCoreApplication::translate("BatchTMDialog", "Filter messages based on severity", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel2->setText(QCoreApplication::translate("BatchTMDialog", "Filter &log messages:", nullptr));
#if QT_CONFIG(tooltip)
        clearTextToolButton->setToolTip(QCoreApplication::translate("BatchTMDialog", "Clear filter text", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        overallProgressBar->setToolTip(QCoreApplication::translate("BatchTMDialog", "Overall completion progress", nullptr));
#endif // QT_CONFIG(tooltip)
        cancelbutton->setText(QCoreApplication::translate("BatchTMDialog", "&Cancel", nullptr));
#if QT_CONFIG(tooltip)
        BatchGoButton->setToolTip(QCoreApplication::translate("BatchTMDialog", "Start batch tone mapping", nullptr));
#endif // QT_CONFIG(tooltip)
        BatchGoButton->setText(QCoreApplication::translate("BatchTMDialog", "&Start", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BatchTMDialog: public Ui_BatchTMDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BATCHTMDIALOG_H
