/****************************************************************************
** Meta object code from reading C++ file 'commandline.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/MainCli/commandline.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'commandline.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CommandLineInterfaceManager_t {
    QByteArrayData data[13];
    char stringdata0[215];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CommandLineInterfaceManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CommandLineInterfaceManager_t qt_meta_stringdata_CommandLineInterfaceManager = {
    {
QT_MOC_LITERAL(0, 0, 27), // "CommandLineInterfaceManager"
QT_MOC_LITERAL(1, 28, 15), // "finishedParsing"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 25), // "finishedLoadingInputFiles"
QT_MOC_LITERAL(4, 71, 10), // "ais_failed"
QT_MOC_LITERAL(5, 82, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(6, 105, 17), // "errorWhileLoading"
QT_MOC_LITERAL(7, 123, 9), // "createHDR"
QT_MOC_LITERAL(8, 133, 25), // "execCommandLineParamsSlot"
QT_MOC_LITERAL(9, 159, 14), // "setProgressBar"
QT_MOC_LITERAL(10, 174, 17), // "updateProgressBar"
QT_MOC_LITERAL(11, 192, 8), // "readData"
QT_MOC_LITERAL(12, 201, 13) // "tonemapFailed"

    },
    "CommandLineInterfaceManager\0finishedParsing\0"
    "\0finishedLoadingInputFiles\0ais_failed\0"
    "QProcess::ProcessError\0errorWhileLoading\0"
    "createHDR\0execCommandLineParamsSlot\0"
    "setProgressBar\0updateProgressBar\0"
    "readData\0tonemapFailed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CommandLineInterfaceManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   65,    2, 0x08 /* Private */,
       4,    1,   66,    2, 0x08 /* Private */,
       6,    1,   69,    2, 0x08 /* Private */,
       7,    1,   72,    2, 0x08 /* Private */,
       8,    0,   75,    2, 0x08 /* Private */,
       9,    1,   76,    2, 0x08 /* Private */,
      10,    1,   79,    2, 0x08 /* Private */,
      11,    1,   82,    2, 0x08 /* Private */,
      12,    1,   85,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void CommandLineInterfaceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CommandLineInterfaceManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finishedParsing(); break;
        case 1: _t->finishedLoadingInputFiles(); break;
        case 2: _t->ais_failed((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 3: _t->errorWhileLoading((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->createHDR((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->execCommandLineParamsSlot(); break;
        case 6: _t->setProgressBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->updateProgressBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->readData((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 9: _t->tonemapFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CommandLineInterfaceManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CommandLineInterfaceManager::finishedParsing)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CommandLineInterfaceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CommandLineInterfaceManager.data,
    qt_meta_data_CommandLineInterfaceManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CommandLineInterfaceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CommandLineInterfaceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CommandLineInterfaceManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CommandLineInterfaceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void CommandLineInterfaceManager::finishedParsing()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
