/****************************************************************************
** Meta object code from reading C++ file 'IOWorker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/Core/IOWorker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'IOWorker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_IOWorker_t {
    QByteArrayData data[26];
    char stringdata0[325];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_IOWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_IOWorker_t qt_meta_stringdata_IOWorker = {
    {
QT_MOC_LITERAL(0, 0, 8), // "IOWorker"
QT_MOC_LITERAL(1, 9, 15), // "read_hdr_failed"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 16), // "read_hdr_success"
QT_MOC_LITERAL(4, 43, 11), // "pfs::Frame*"
QT_MOC_LITERAL(5, 55, 16), // "write_hdr_failed"
QT_MOC_LITERAL(6, 72, 17), // "write_hdr_success"
QT_MOC_LITERAL(7, 90, 14), // "GenericViewer*"
QT_MOC_LITERAL(8, 105, 16), // "write_ldr_failed"
QT_MOC_LITERAL(9, 122, 17), // "write_ldr_success"
QT_MOC_LITERAL(10, 140, 10), // "setMaximum"
QT_MOC_LITERAL(11, 151, 8), // "setValue"
QT_MOC_LITERAL(12, 160, 7), // "IO_init"
QT_MOC_LITERAL(13, 168, 9), // "IO_finish"
QT_MOC_LITERAL(14, 178, 14), // "read_hdr_frame"
QT_MOC_LITERAL(15, 193, 8), // "filename"
QT_MOC_LITERAL(16, 202, 15), // "write_hdr_frame"
QT_MOC_LITERAL(17, 218, 5), // "frame"
QT_MOC_LITERAL(18, 224, 11), // "pfs::Params"
QT_MOC_LITERAL(19, 236, 6), // "params"
QT_MOC_LITERAL(20, 243, 15), // "write_ldr_frame"
QT_MOC_LITERAL(21, 259, 13), // "inputFileName"
QT_MOC_LITERAL(22, 273, 14), // "QVector<float>"
QT_MOC_LITERAL(23, 288, 9), // "expoTimes"
QT_MOC_LITERAL(24, 298, 19), // "TonemappingOptions*"
QT_MOC_LITERAL(25, 318, 6) // "tmopts"

    },
    "IOWorker\0read_hdr_failed\0\0read_hdr_success\0"
    "pfs::Frame*\0write_hdr_failed\0"
    "write_hdr_success\0GenericViewer*\0"
    "write_ldr_failed\0write_ldr_success\0"
    "setMaximum\0setValue\0IO_init\0IO_finish\0"
    "read_hdr_frame\0filename\0write_hdr_frame\0"
    "frame\0pfs::Params\0params\0write_ldr_frame\0"
    "inputFileName\0QVector<float>\0expoTimes\0"
    "TonemappingOptions*\0tmopts"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_IOWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  129,    2, 0x06 /* Public */,
       3,    2,  132,    2, 0x06 /* Public */,
       5,    1,  137,    2, 0x06 /* Public */,
       6,    2,  140,    2, 0x06 /* Public */,
       6,    2,  145,    2, 0x06 /* Public */,
       8,    1,  150,    2, 0x06 /* Public */,
       9,    2,  153,    2, 0x06 /* Public */,
       9,    2,  158,    2, 0x06 /* Public */,
      10,    1,  163,    2, 0x06 /* Public */,
      11,    1,  166,    2, 0x06 /* Public */,
      12,    0,  169,    2, 0x06 /* Public */,
      13,    0,  170,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    1,  171,    2, 0x0a /* Public */,
      16,    3,  174,    2, 0x0a /* Public */,
      16,    2,  181,    2, 0x2a /* Public | MethodCloned */,
      16,    3,  186,    2, 0x0a /* Public */,
      16,    2,  193,    2, 0x2a /* Public | MethodCloned */,
      20,    6,  198,    2, 0x0a /* Public */,
      20,    5,  211,    2, 0x2a /* Public | MethodCloned */,
      20,    4,  222,    2, 0x2a /* Public | MethodCloned */,
      20,    6,  231,    2, 0x0a /* Public */,
      20,    5,  244,    2, 0x2a /* Public | MethodCloned */,
      20,    4,  255,    2, 0x2a /* Public | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 4, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 4, QMetaType::QString,    2,    2,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 4, QMetaType::QString,    2,    2,
    QMetaType::Void, 0x80000000 | 7, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    0x80000000 | 4, QMetaType::QString,   15,
    QMetaType::Bool, 0x80000000 | 4, QMetaType::QString, 0x80000000 | 18,   17,   15,   19,
    QMetaType::Bool, 0x80000000 | 4, QMetaType::QString,   17,   15,
    QMetaType::Bool, 0x80000000 | 7, QMetaType::QString, 0x80000000 | 18,   17,   15,   19,
    QMetaType::Bool, 0x80000000 | 7, QMetaType::QString,   17,   15,
    QMetaType::Bool, 0x80000000 | 4, QMetaType::QString, QMetaType::QString, 0x80000000 | 22, 0x80000000 | 24, 0x80000000 | 18,   17,   15,   21,   23,   25,   19,
    QMetaType::Bool, 0x80000000 | 4, QMetaType::QString, QMetaType::QString, 0x80000000 | 22, 0x80000000 | 24,   17,   15,   21,   23,   25,
    QMetaType::Bool, 0x80000000 | 4, QMetaType::QString, QMetaType::QString, 0x80000000 | 22,   17,   15,   21,   23,
    QMetaType::Bool, 0x80000000 | 7, QMetaType::QString, QMetaType::QString, 0x80000000 | 22, 0x80000000 | 24, 0x80000000 | 18,   17,   15,   21,   23,   25,   19,
    QMetaType::Bool, 0x80000000 | 7, QMetaType::QString, QMetaType::QString, 0x80000000 | 22, 0x80000000 | 24,   17,   15,   21,   23,   25,
    QMetaType::Bool, 0x80000000 | 7, QMetaType::QString, QMetaType::QString, 0x80000000 | 22,   17,   15,   21,   23,

       0        // eod
};

void IOWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IOWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->read_hdr_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->read_hdr_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->write_hdr_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->write_hdr_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->write_hdr_success((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 5: _t->write_ldr_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->write_ldr_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->write_ldr_success((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->setMaximum((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->setValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->IO_init(); break;
        case 11: _t->IO_finish(); break;
        case 12: { pfs::Frame* _r = _t->read_hdr_frame((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< pfs::Frame**>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->write_hdr_frame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const pfs::Params(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->write_hdr_frame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->write_hdr_frame((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const pfs::Params(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->write_hdr_frame((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->write_ldr_frame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[5])),(*reinterpret_cast< const pfs::Params(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->write_ldr_frame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->write_ldr_frame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->write_ldr_frame((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[5])),(*reinterpret_cast< const pfs::Params(*)>(_a[6])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->write_ldr_frame((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[5])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->write_ldr_frame((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QVector<float>(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (IOWorker::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::read_hdr_failed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(pfs::Frame * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::read_hdr_success)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_hdr_failed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(pfs::Frame * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_hdr_success)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(GenericViewer * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_hdr_success)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_ldr_failed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(pfs::Frame * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_ldr_success)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(GenericViewer * , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::write_ldr_success)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::setMaximum)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::setValue)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::IO_init)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (IOWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IOWorker::IO_finish)) {
                *result = 11;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject IOWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_IOWorker.data,
    qt_meta_data_IOWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *IOWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *IOWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_IOWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int IOWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void IOWorker::read_hdr_failed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void IOWorker::read_hdr_success(pfs::Frame * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void IOWorker::write_hdr_failed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void IOWorker::write_hdr_success(pfs::Frame * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void IOWorker::write_hdr_success(GenericViewer * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void IOWorker::write_ldr_failed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void IOWorker::write_ldr_success(pfs::Frame * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void IOWorker::write_ldr_success(GenericViewer * _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void IOWorker::setMaximum(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void IOWorker::setValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void IOWorker::IO_init()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void IOWorker::IO_finish()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
