/****************************************************************************
** Meta object code from reading C++ file 'HdrCreationManager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/HdrWizard/HdrCreationManager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'HdrCreationManager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HdrCreationManager_t {
    QByteArrayData data[28];
    char stringdata0[403];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HdrCreationManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HdrCreationManager_t qt_meta_stringdata_HdrCreationManager = {
    {
QT_MOC_LITERAL(0, 0, 18), // "HdrCreationManager"
QT_MOC_LITERAL(1, 19, 15), // "progressStarted"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 16), // "progressFinished"
QT_MOC_LITERAL(4, 53, 14), // "progressCancel"
QT_MOC_LITERAL(5, 68, 20), // "progressRangeChanged"
QT_MOC_LITERAL(6, 89, 20), // "progressValueChanged"
QT_MOC_LITERAL(7, 110, 20), // "finishedLoadingFiles"
QT_MOC_LITERAL(8, 131, 25), // "finishedLoadingInputFiles"
QT_MOC_LITERAL(9, 157, 16), // "filesLackingExif"
QT_MOC_LITERAL(10, 174, 17), // "errorWhileLoading"
QT_MOC_LITERAL(11, 192, 7), // "message"
QT_MOC_LITERAL(12, 200, 10), // "fileLoaded"
QT_MOC_LITERAL(13, 211, 5), // "index"
QT_MOC_LITERAL(14, 217, 5), // "fname"
QT_MOC_LITERAL(15, 223, 8), // "expotime"
QT_MOC_LITERAL(16, 232, 16), // "finishedAligning"
QT_MOC_LITERAL(17, 249, 20), // "expotimeValueChanged"
QT_MOC_LITERAL(18, 270, 10), // "ais_failed"
QT_MOC_LITERAL(19, 281, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(20, 304, 12), // "aisDataReady"
QT_MOC_LITERAL(21, 317, 11), // "QByteArray&"
QT_MOC_LITERAL(22, 329, 4), // "data"
QT_MOC_LITERAL(23, 334, 9), // "processed"
QT_MOC_LITERAL(24, 344, 11), // "imagesSaved"
QT_MOC_LITERAL(25, 356, 16), // "loadFilesAborted"
QT_MOC_LITERAL(26, 373, 15), // "ais_failed_slot"
QT_MOC_LITERAL(27, 389, 13) // "loadFilesDone"

    },
    "HdrCreationManager\0progressStarted\0\0"
    "progressFinished\0progressCancel\0"
    "progressRangeChanged\0progressValueChanged\0"
    "finishedLoadingFiles\0finishedLoadingInputFiles\0"
    "filesLackingExif\0errorWhileLoading\0"
    "message\0fileLoaded\0index\0fname\0expotime\0"
    "finishedAligning\0expotimeValueChanged\0"
    "ais_failed\0QProcess::ProcessError\0"
    "aisDataReady\0QByteArray&\0data\0processed\0"
    "imagesSaved\0loadFilesAborted\0"
    "ais_failed_slot\0loadFilesDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HdrCreationManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  104,    2, 0x06 /* Public */,
       3,    0,  105,    2, 0x06 /* Public */,
       4,    0,  106,    2, 0x06 /* Public */,
       5,    2,  107,    2, 0x06 /* Public */,
       6,    1,  112,    2, 0x06 /* Public */,
       7,    0,  115,    2, 0x06 /* Public */,
       8,    1,  116,    2, 0x06 /* Public */,
      10,    1,  119,    2, 0x06 /* Public */,
      12,    3,  122,    2, 0x06 /* Public */,
      16,    1,  129,    2, 0x06 /* Public */,
      17,    2,  132,    2, 0x06 /* Public */,
      18,    1,  137,    2, 0x06 /* Public */,
      20,    1,  140,    2, 0x06 /* Public */,
      23,    0,  143,    2, 0x06 /* Public */,
      24,    0,  144,    2, 0x06 /* Public */,
      25,    0,  145,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      26,    1,  146,    2, 0x08 /* Private */,
      27,    0,  149,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,    9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::Float,   13,   14,   15,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Int,    2,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,

       0        // eod
};

void HdrCreationManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HdrCreationManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->progressStarted(); break;
        case 1: _t->progressFinished(); break;
        case 2: _t->progressCancel(); break;
        case 3: _t->progressRangeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->progressValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->finishedLoadingFiles(); break;
        case 6: _t->finishedLoadingInputFiles((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 7: _t->errorWhileLoading((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->fileLoaded((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3]))); break;
        case 9: _t->finishedAligning((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->expotimeValueChanged((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: _t->ais_failed((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 12: _t->aisDataReady((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 13: _t->processed(); break;
        case 14: _t->imagesSaved(); break;
        case 15: _t->loadFilesAborted(); break;
        case 16: _t->ais_failed_slot((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 17: _t->loadFilesDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::progressStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::progressFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::progressCancel)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::progressRangeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::progressValueChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::finishedLoadingFiles)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::finishedLoadingInputFiles)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::errorWhileLoading)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(int , const QString & , float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::fileLoaded)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::finishedAligning)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(float , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::expotimeValueChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(QProcess::ProcessError );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::ais_failed)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)(QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::aisDataReady)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::processed)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::imagesSaved)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (HdrCreationManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrCreationManager::loadFilesAborted)) {
                *result = 15;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HdrCreationManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_HdrCreationManager.data,
    qt_meta_data_HdrCreationManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HdrCreationManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HdrCreationManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HdrCreationManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int HdrCreationManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void HdrCreationManager::progressStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void HdrCreationManager::progressFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void HdrCreationManager::progressCancel()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void HdrCreationManager::progressRangeChanged(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HdrCreationManager::progressValueChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HdrCreationManager::finishedLoadingFiles()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void HdrCreationManager::finishedLoadingInputFiles(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void HdrCreationManager::errorWhileLoading(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void HdrCreationManager::fileLoaded(int _t1, const QString & _t2, float _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void HdrCreationManager::finishedAligning(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void HdrCreationManager::expotimeValueChanged(float _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void HdrCreationManager::ais_failed(QProcess::ProcessError _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void HdrCreationManager::aisDataReady(QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void HdrCreationManager::processed()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void HdrCreationManager::imagesSaved()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void HdrCreationManager::loadFilesAborted()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
