/****************************************************************************
** Meta object code from reading C++ file 'GammaAndLevels.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/UI/GammaAndLevels.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'GammaAndLevels.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GrayBar_t {
    QByteArrayData data[11];
    char stringdata0[145];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GrayBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GrayBar_t qt_meta_stringdata_GrayBar = {
    {
QT_MOC_LITERAL(0, 0, 7), // "GrayBar"
QT_MOC_LITERAL(1, 8, 13), // "black_changed"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 13), // "gamma_changed"
QT_MOC_LITERAL(4, 37, 13), // "white_changed"
QT_MOC_LITERAL(5, 51, 25), // "default_gamma_black_white"
QT_MOC_LITERAL(6, 77, 19), // "default_black_white"
QT_MOC_LITERAL(7, 97, 11), // "resetvalues"
QT_MOC_LITERAL(8, 109, 11), // "changeBlack"
QT_MOC_LITERAL(9, 121, 11), // "changeGamma"
QT_MOC_LITERAL(10, 133, 11) // "changeWhite"

    },
    "GrayBar\0black_changed\0\0gamma_changed\0"
    "white_changed\0default_gamma_black_white\0"
    "default_black_white\0resetvalues\0"
    "changeBlack\0changeGamma\0changeWhite"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GrayBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       3,    1,   62,    2, 0x06 /* Public */,
       4,    1,   65,    2, 0x06 /* Public */,
       5,    0,   68,    2, 0x06 /* Public */,
       6,    0,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   70,    2, 0x0a /* Public */,
       8,    1,   71,    2, 0x0a /* Public */,
       9,    1,   74,    2, 0x0a /* Public */,
      10,    1,   77,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void GrayBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GrayBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->black_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->gamma_changed((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->white_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->default_gamma_black_white(); break;
        case 4: _t->default_black_white(); break;
        case 5: _t->resetvalues(); break;
        case 6: _t->changeBlack((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->changeGamma((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 8: _t->changeWhite((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GrayBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GrayBar::black_changed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GrayBar::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GrayBar::gamma_changed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GrayBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GrayBar::white_changed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (GrayBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GrayBar::default_gamma_black_white)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (GrayBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GrayBar::default_black_white)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject GrayBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_GrayBar.data,
    qt_meta_data_GrayBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GrayBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GrayBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GrayBar.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int GrayBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void GrayBar::black_changed(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void GrayBar::gamma_changed(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void GrayBar::white_changed(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void GrayBar::default_gamma_black_white()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GrayBar::default_black_white()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_HistogramLDR_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HistogramLDR_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HistogramLDR_t qt_meta_stringdata_HistogramLDR = {
    {
QT_MOC_LITERAL(0, 0, 12) // "HistogramLDR"

    },
    "HistogramLDR"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HistogramLDR[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void HistogramLDR::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject HistogramLDR::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_HistogramLDR.data,
    qt_meta_data_HistogramLDR,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HistogramLDR::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HistogramLDR::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HistogramLDR.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int HistogramLDR::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_GammaAndLevels_t {
    QByteArrayData data[12];
    char stringdata0[163];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GammaAndLevels_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GammaAndLevels_t qt_meta_stringdata_GammaAndLevels = {
    {
QT_MOC_LITERAL(0, 0, 14), // "GammaAndLevels"
QT_MOC_LITERAL(1, 15, 12), // "updateQImage"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 5), // "image"
QT_MOC_LITERAL(4, 35, 11), // "resetValues"
QT_MOC_LITERAL(5, 47, 13), // "updateBlackIn"
QT_MOC_LITERAL(6, 61, 11), // "updateGamma"
QT_MOC_LITERAL(7, 73, 13), // "updateWhiteIn"
QT_MOC_LITERAL(8, 87, 14), // "updateBlackOut"
QT_MOC_LITERAL(9, 102, 14), // "updateWhiteOut"
QT_MOC_LITERAL(10, 117, 24), // "defaultGammaBlackWhiteIn"
QT_MOC_LITERAL(11, 142, 20) // "defaultBlackWhiteOut"

    },
    "GammaAndLevels\0updateQImage\0\0image\0"
    "resetValues\0updateBlackIn\0updateGamma\0"
    "updateWhiteIn\0updateBlackOut\0"
    "updateWhiteOut\0defaultGammaBlackWhiteIn\0"
    "defaultBlackWhiteOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GammaAndLevels[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   62,    2, 0x08 /* Private */,
       5,    1,   63,    2, 0x08 /* Private */,
       6,    1,   66,    2, 0x08 /* Private */,
       7,    1,   69,    2, 0x08 /* Private */,
       8,    1,   72,    2, 0x08 /* Private */,
       9,    1,   75,    2, 0x08 /* Private */,
      10,    0,   78,    2, 0x08 /* Private */,
      11,    0,   79,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QImage,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void GammaAndLevels::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GammaAndLevels *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateQImage((*reinterpret_cast< QImage(*)>(_a[1]))); break;
        case 1: _t->resetValues(); break;
        case 2: _t->updateBlackIn((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->updateGamma((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->updateWhiteIn((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->updateBlackOut((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->updateWhiteOut((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->defaultGammaBlackWhiteIn(); break;
        case 8: _t->defaultBlackWhiteOut(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GammaAndLevels::*)(QImage );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GammaAndLevels::updateQImage)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject GammaAndLevels::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_GammaAndLevels.data,
    qt_meta_data_GammaAndLevels,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GammaAndLevels::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GammaAndLevels::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GammaAndLevels.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int GammaAndLevels::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void GammaAndLevels::updateQImage(QImage _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
