/********************************************************************************
** Form generated from reading UI file 'DnDOption.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DNDOPTION_H
#define UI_DNDOPTION_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCommandLinkButton>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DnDOption
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QCommandLinkButton *btnOpenHDR;
    QCommandLinkButton *btnCreateNewHDR;
    QCommandLinkButton *btnCancel;

    void setupUi(QDialog *DnDOption)
    {
        if (DnDOption->objectName().isEmpty())
            DnDOption->setObjectName(QString::fromUtf8("DnDOption"));
        DnDOption->setWindowModality(Qt::WindowModal);
        DnDOption->resize(484, 307);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        DnDOption->setWindowIcon(icon);
        DnDOption->setModal(true);
        verticalLayout = new QVBoxLayout(DnDOption);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(DnDOption);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setBold(false);
        font.setWeight(50);
        font.setKerning(false);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(DnDOption);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        btnOpenHDR = new QCommandLinkButton(DnDOption);
        btnOpenHDR->setObjectName(QString::fromUtf8("btnOpenHDR"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("folder-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        btnOpenHDR->setIcon(icon1);
        btnOpenHDR->setIconSize(QSize(36, 36));

        verticalLayout->addWidget(btnOpenHDR);

        btnCreateNewHDR = new QCommandLinkButton(DnDOption);
        btnCreateNewHDR->setObjectName(QString::fromUtf8("btnCreateNewHDR"));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("insert-image");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        btnCreateNewHDR->setIcon(icon2);
        btnCreateNewHDR->setIconSize(QSize(36, 36));

        verticalLayout->addWidget(btnCreateNewHDR);

        btnCancel = new QCommandLinkButton(DnDOption);
        btnCancel->setObjectName(QString::fromUtf8("btnCancel"));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("window-close");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        btnCancel->setIcon(icon3);
        btnCancel->setIconSize(QSize(36, 36));

        verticalLayout->addWidget(btnCancel);


        retranslateUi(DnDOption);

        QMetaObject::connectSlotsByName(DnDOption);
    } // setupUi

    void retranslateUi(QDialog *DnDOption)
    {
        DnDOption->setWindowTitle(QCoreApplication::translate("DnDOption", "Choose action", nullptr));
        label->setText(QCoreApplication::translate("DnDOption", "<b>What would you like to do with the file(s)?</b>", nullptr));
        btnOpenHDR->setText(QCoreApplication::translate("DnDOption", "&Open", nullptr));
        btnOpenHDR->setDescription(QCoreApplication::translate("DnDOption", "Open an existing HDR image", nullptr));
        btnCreateNewHDR->setText(QCoreApplication::translate("DnDOption", "&New", nullptr));
        btnCreateNewHDR->setDescription(QCoreApplication::translate("DnDOption", "Create a new HDR image from one or more images", nullptr));
        btnCancel->setText(QCoreApplication::translate("DnDOption", "&Cancel", nullptr));
        btnCancel->setDescription(QCoreApplication::translate("DnDOption", "Cancel the drag and drop process and return to the main window", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DnDOption: public Ui_DnDOption {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DNDOPTION_H
