/****************************************************************************
** Meta object code from reading C++ file 'BatchTMDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/BatchTM/BatchTMDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'BatchTMDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BatchTMDialog_t {
    QByteArrayData data[24];
    char stringdata0[337];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BatchTMDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BatchTMDialog_t qt_meta_stringdata_BatchTMDialog = {
    {
QT_MOC_LITERAL(0, 0, 13), // "BatchTMDialog"
QT_MOC_LITERAL(1, 14, 12), // "add_dir_HDRs"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 8), // "add_HDRs"
QT_MOC_LITERAL(4, 37, 14), // "add_dir_TMopts"
QT_MOC_LITERAL(5, 52, 10), // "add_TMopts"
QT_MOC_LITERAL(6, 63, 18), // "out_folder_clicked"
QT_MOC_LITERAL(7, 82, 18), // "check_enable_start"
QT_MOC_LITERAL(8, 101, 11), // "remove_HDRs"
QT_MOC_LITERAL(9, 113, 13), // "remove_TMOpts"
QT_MOC_LITERAL(10, 127, 15), // "remove_all_HDRs"
QT_MOC_LITERAL(11, 143, 17), // "remove_all_TMOpts"
QT_MOC_LITERAL(12, 161, 13), // "filterChanged"
QT_MOC_LITERAL(13, 175, 23), // "filterComboBoxActivated"
QT_MOC_LITERAL(14, 199, 5), // "abort"
QT_MOC_LITERAL(15, 205, 15), // "add_log_message"
QT_MOC_LITERAL(16, 221, 10), // "batch_core"
QT_MOC_LITERAL(17, 232, 14), // "release_thread"
QT_MOC_LITERAL(18, 247, 4), // "t_id"
QT_MOC_LITERAL(19, 252, 18), // "start_batch_thread"
QT_MOC_LITERAL(20, 271, 16), // "stop_batch_tm_ui"
QT_MOC_LITERAL(21, 288, 22), // "increment_progress_bar"
QT_MOC_LITERAL(22, 311, 13), // "from_database"
QT_MOC_LITERAL(23, 325, 11) // "updateWidth"

    },
    "BatchTMDialog\0add_dir_HDRs\0\0add_HDRs\0"
    "add_dir_TMopts\0add_TMopts\0out_folder_clicked\0"
    "check_enable_start\0remove_HDRs\0"
    "remove_TMOpts\0remove_all_HDRs\0"
    "remove_all_TMOpts\0filterChanged\0"
    "filterComboBoxActivated\0abort\0"
    "add_log_message\0batch_core\0release_thread\0"
    "t_id\0start_batch_thread\0stop_batch_tm_ui\0"
    "increment_progress_bar\0from_database\0"
    "updateWidth"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BatchTMDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x08 /* Private */,
       3,    0,  120,    2, 0x08 /* Private */,
       4,    0,  121,    2, 0x08 /* Private */,
       5,    0,  122,    2, 0x08 /* Private */,
       6,    0,  123,    2, 0x08 /* Private */,
       7,    0,  124,    2, 0x08 /* Private */,
       8,    0,  125,    2, 0x08 /* Private */,
       9,    0,  126,    2, 0x08 /* Private */,
      10,    0,  127,    2, 0x08 /* Private */,
      11,    0,  128,    2, 0x08 /* Private */,
      12,    1,  129,    2, 0x08 /* Private */,
      13,    1,  132,    2, 0x08 /* Private */,
      14,    0,  135,    2, 0x08 /* Private */,
      15,    1,  136,    2, 0x08 /* Private */,
      16,    0,  139,    2, 0x08 /* Private */,
      17,    1,  140,    2, 0x08 /* Private */,
      19,    0,  143,    2, 0x08 /* Private */,
      20,    0,  144,    2, 0x08 /* Private */,
      21,    1,  145,    2, 0x08 /* Private */,
      22,    0,  148,    2, 0x08 /* Private */,
      23,    1,  149,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void BatchTMDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BatchTMDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->add_dir_HDRs(); break;
        case 1: _t->add_HDRs(); break;
        case 2: _t->add_dir_TMopts(); break;
        case 3: _t->add_TMopts(); break;
        case 4: _t->out_folder_clicked(); break;
        case 5: _t->check_enable_start(); break;
        case 6: _t->remove_HDRs(); break;
        case 7: _t->remove_TMOpts(); break;
        case 8: _t->remove_all_HDRs(); break;
        case 9: _t->remove_all_TMOpts(); break;
        case 10: _t->filterChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->filterComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->abort(); break;
        case 13: _t->add_log_message((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->batch_core(); break;
        case 15: _t->release_thread((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->start_batch_thread(); break;
        case 17: _t->stop_batch_tm_ui(); break;
        case 18: _t->increment_progress_bar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->from_database(); break;
        case 20: _t->updateWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BatchTMDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_BatchTMDialog.data,
    qt_meta_data_BatchTMDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BatchTMDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BatchTMDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BatchTMDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int BatchTMDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
