/********************************************************************************
** Form generated from reading UI file 'BatchHDRDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BATCHHDRDIALOG_H
#define UI_BATCHHDRDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_BatchHDRDialog
{
public:
    QGridLayout *gridLayout_9;
    QHBoxLayout *horizontalLayout_4;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QSlider *horizontalSlider;
    QSpinBox *spinBox;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *groupBoxOutput;
    QGridLayout *gridLayout_7;
    QGridLayout *gridLayout_5;
    QComboBox *formatComboBox;
    QLabel *label_4;
    QComboBox *profileComboBox;
    QLabel *label_3;
    QLineEdit *prefixLineEdit;
    QCheckBox *proposedFilenameCheckBox;
    QLabel *label_10;
    QLabel *label_11;
    QLabel *label_2;
    QToolButton *formatSettingsButton;
    QLabel *label_12;
    QLabel *labelModel;
    QLabel *labelWeights;
    QLabel *labelResponse;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBoxAlignment;
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout_6;
    QRadioButton *aisRadioButton;
    QRadioButton *MTBRadioButton;
    QCheckBox *autoAlignCheckBox;
    QCheckBox *autoCropCheckBox;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *cancelButton;
    QPushButton *startButton;
    QPushButton *closeButton;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_6;
    QTextEdit *textEdit;
    QProgressBar *progressBar;
    QGroupBox *groupBoxIO;
    QGridLayout *gridLayout_8;
    QGridLayout *gridLayout_4;
    QLineEdit *outputLineEdit;
    QLineEdit *inputLineEdit;
    QPushButton *selectOutputFolder;
    QPushButton *selectInputFolder;
    QLabel *label_5;
    QLabel *label_6;
    QGroupBox *groupBoxAg;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_8;
    QLabel *label_9;
    QSlider *threshold_horizontalSlider;
    QDoubleSpinBox *threshold_doubleSpinBox;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_7;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *autoAG_checkBox;

    void setupUi(QDialog *BatchHDRDialog)
    {
        if (BatchHDRDialog->objectName().isEmpty())
            BatchHDRDialog->setObjectName(QString::fromUtf8("BatchHDRDialog"));
        BatchHDRDialog->resize(622, 729);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        BatchHDRDialog->setWindowIcon(icon);
        gridLayout_9 = new QGridLayout(BatchHDRDialog);
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));

        gridLayout_9->addLayout(horizontalLayout_4, 1, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(BatchHDRDialog);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        horizontalSlider = new QSlider(BatchHDRDialog);
        horizontalSlider->setObjectName(QString::fromUtf8("horizontalSlider"));
        horizontalSlider->setMinimum(1);
        horizontalSlider->setMaximum(20);
        horizontalSlider->setValue(3);
        horizontalSlider->setOrientation(Qt::Horizontal);
        horizontalSlider->setTickPosition(QSlider::TicksBelow);
        horizontalSlider->setTickInterval(1);

        horizontalLayout->addWidget(horizontalSlider);

        spinBox = new QSpinBox(BatchHDRDialog);
        spinBox->setObjectName(QString::fromUtf8("spinBox"));
        spinBox->setMinimum(1);
        spinBox->setMaximum(20);
        spinBox->setValue(3);

        horizontalLayout->addWidget(spinBox);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 1);
        horizontalLayout->setStretch(2, 2);

        gridLayout_9->addLayout(horizontalLayout, 0, 0, 1, 2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        groupBoxOutput = new QGroupBox(BatchHDRDialog);
        groupBoxOutput->setObjectName(QString::fromUtf8("groupBoxOutput"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBoxOutput->sizePolicy().hasHeightForWidth());
        groupBoxOutput->setSizePolicy(sizePolicy);
        gridLayout_7 = new QGridLayout(groupBoxOutput);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        formatComboBox = new QComboBox(groupBoxOutput);
        formatComboBox->setObjectName(QString::fromUtf8("formatComboBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(formatComboBox->sizePolicy().hasHeightForWidth());
        formatComboBox->setSizePolicy(sizePolicy1);

        gridLayout_5->addWidget(formatComboBox, 0, 1, 1, 1);

        label_4 = new QLabel(groupBoxOutput);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setEnabled(false);

        gridLayout_5->addWidget(label_4, 6, 0, 1, 1);

        profileComboBox = new QComboBox(groupBoxOutput);
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->setObjectName(QString::fromUtf8("profileComboBox"));

        gridLayout_5->addWidget(profileComboBox, 1, 1, 1, 2);

        label_3 = new QLabel(groupBoxOutput);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout_5->addWidget(label_3, 0, 0, 1, 1);

        prefixLineEdit = new QLineEdit(groupBoxOutput);
        prefixLineEdit->setObjectName(QString::fromUtf8("prefixLineEdit"));
        prefixLineEdit->setEnabled(false);

        gridLayout_5->addWidget(prefixLineEdit, 6, 1, 1, 1);

        proposedFilenameCheckBox = new QCheckBox(groupBoxOutput);
        proposedFilenameCheckBox->setObjectName(QString::fromUtf8("proposedFilenameCheckBox"));
        proposedFilenameCheckBox->setChecked(true);

        gridLayout_5->addWidget(proposedFilenameCheckBox, 5, 1, 1, 1);

        label_10 = new QLabel(groupBoxOutput);
        label_10->setObjectName(QString::fromUtf8("label_10"));
        label_10->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_10, 4, 0, 1, 1);

        label_11 = new QLabel(groupBoxOutput);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_11, 3, 0, 1, 1);

        label_2 = new QLabel(groupBoxOutput);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_5->addWidget(label_2, 1, 0, 1, 1);

        formatSettingsButton = new QToolButton(groupBoxOutput);
        formatSettingsButton->setObjectName(QString::fromUtf8("formatSettingsButton"));
        formatSettingsButton->setText(QString::fromUtf8("..."));

        gridLayout_5->addWidget(formatSettingsButton, 0, 2, 1, 1);

        label_12 = new QLabel(groupBoxOutput);
        label_12->setObjectName(QString::fromUtf8("label_12"));
        label_12->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_12, 2, 0, 1, 1);

        labelModel = new QLabel(groupBoxOutput);
        labelModel->setObjectName(QString::fromUtf8("labelModel"));

        gridLayout_5->addWidget(labelModel, 2, 1, 1, 1);

        labelWeights = new QLabel(groupBoxOutput);
        labelWeights->setObjectName(QString::fromUtf8("labelWeights"));

        gridLayout_5->addWidget(labelWeights, 3, 1, 1, 1);

        labelResponse = new QLabel(groupBoxOutput);
        labelResponse->setObjectName(QString::fromUtf8("labelResponse"));

        gridLayout_5->addWidget(labelResponse, 4, 1, 1, 1);


        gridLayout_7->addLayout(gridLayout_5, 0, 0, 1, 1);


        horizontalLayout_3->addWidget(groupBoxOutput);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        groupBoxAlignment = new QGroupBox(BatchHDRDialog);
        groupBoxAlignment->setObjectName(QString::fromUtf8("groupBoxAlignment"));
        gridLayout_2 = new QGridLayout(groupBoxAlignment);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_6 = new QGridLayout();
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        aisRadioButton = new QRadioButton(groupBoxAlignment);
        aisRadioButton->setObjectName(QString::fromUtf8("aisRadioButton"));
        aisRadioButton->setEnabled(false);
        aisRadioButton->setChecked(true);

        gridLayout_6->addWidget(aisRadioButton, 2, 0, 1, 2);

        MTBRadioButton = new QRadioButton(groupBoxAlignment);
        MTBRadioButton->setObjectName(QString::fromUtf8("MTBRadioButton"));
        MTBRadioButton->setEnabled(false);

        gridLayout_6->addWidget(MTBRadioButton, 3, 0, 1, 2);

        autoAlignCheckBox = new QCheckBox(groupBoxAlignment);
        autoAlignCheckBox->setObjectName(QString::fromUtf8("autoAlignCheckBox"));
        autoAlignCheckBox->setEnabled(true);

        gridLayout_6->addWidget(autoAlignCheckBox, 0, 0, 1, 1);

        autoCropCheckBox = new QCheckBox(groupBoxAlignment);
        autoCropCheckBox->setObjectName(QString::fromUtf8("autoCropCheckBox"));
        autoCropCheckBox->setEnabled(false);

        gridLayout_6->addWidget(autoCropCheckBox, 4, 0, 1, 1);


        gridLayout_2->addLayout(gridLayout_6, 0, 0, 1, 1);


        verticalLayout_4->addWidget(groupBoxAlignment);


        horizontalLayout_3->addLayout(verticalLayout_4);

        horizontalLayout_3->setStretch(0, 1);
        horizontalLayout_3->setStretch(1, 1);

        gridLayout_9->addLayout(horizontalLayout_3, 2, 0, 1, 2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);

        cancelButton = new QPushButton(BatchHDRDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        horizontalLayout_6->addWidget(cancelButton);

        startButton = new QPushButton(BatchHDRDialog);
        startButton->setObjectName(QString::fromUtf8("startButton"));
        startButton->setEnabled(false);
        startButton->setAutoDefault(false);

        horizontalLayout_6->addWidget(startButton);

        closeButton = new QPushButton(BatchHDRDialog);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));
        closeButton->setAutoDefault(false);

        horizontalLayout_6->addWidget(closeButton);


        gridLayout_9->addLayout(horizontalLayout_6, 6, 0, 1, 2);

        groupBox_2 = new QGroupBox(BatchHDRDialog);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        verticalLayout_6 = new QVBoxLayout(groupBox_2);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        textEdit = new QTextEdit(groupBox_2);
        textEdit->setObjectName(QString::fromUtf8("textEdit"));
        textEdit->setReadOnly(true);

        verticalLayout_6->addWidget(textEdit);

        progressBar = new QProgressBar(groupBox_2);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setValue(0);

        verticalLayout_6->addWidget(progressBar);


        gridLayout_9->addWidget(groupBox_2, 5, 0, 1, 2);

        groupBoxIO = new QGroupBox(BatchHDRDialog);
        groupBoxIO->setObjectName(QString::fromUtf8("groupBoxIO"));
        gridLayout_8 = new QGridLayout(groupBoxIO);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        outputLineEdit = new QLineEdit(groupBoxIO);
        outputLineEdit->setObjectName(QString::fromUtf8("outputLineEdit"));
        outputLineEdit->setReadOnly(true);

        gridLayout_4->addWidget(outputLineEdit, 1, 2, 1, 1);

        inputLineEdit = new QLineEdit(groupBoxIO);
        inputLineEdit->setObjectName(QString::fromUtf8("inputLineEdit"));
        inputLineEdit->setReadOnly(true);

        gridLayout_4->addWidget(inputLineEdit, 0, 2, 1, 1);

        selectOutputFolder = new QPushButton(groupBoxIO);
        selectOutputFolder->setObjectName(QString::fromUtf8("selectOutputFolder"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        selectOutputFolder->setIcon(icon1);
        selectOutputFolder->setAutoDefault(false);

        gridLayout_4->addWidget(selectOutputFolder, 1, 3, 1, 1);

        selectInputFolder = new QPushButton(groupBoxIO);
        selectInputFolder->setObjectName(QString::fromUtf8("selectInputFolder"));
        selectInputFolder->setIcon(icon1);
        selectInputFolder->setAutoDefault(false);

        gridLayout_4->addWidget(selectInputFolder, 0, 3, 1, 1);

        label_5 = new QLabel(groupBoxIO);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setScaledContents(false);

        gridLayout_4->addWidget(label_5, 0, 1, 1, 1);

        label_6 = new QLabel(groupBoxIO);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        gridLayout_4->addWidget(label_6, 1, 1, 1, 1);


        gridLayout_8->addLayout(gridLayout_4, 0, 0, 1, 1);


        gridLayout_9->addWidget(groupBoxIO, 4, 0, 1, 2);

        groupBoxAg = new QGroupBox(BatchHDRDialog);
        groupBoxAg->setObjectName(QString::fromUtf8("groupBoxAg"));
        gridLayout = new QGridLayout(groupBoxAg);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        label_8 = new QLabel(groupBoxAg);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        horizontalLayout_9->addWidget(label_8);


        gridLayout->addLayout(horizontalLayout_9, 1, 0, 1, 2);

        label_9 = new QLabel(groupBoxAg);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        label_9->setEnabled(false);

        gridLayout->addWidget(label_9, 2, 0, 1, 1);

        threshold_horizontalSlider = new QSlider(groupBoxAg);
        threshold_horizontalSlider->setObjectName(QString::fromUtf8("threshold_horizontalSlider"));
        threshold_horizontalSlider->setEnabled(false);
        threshold_horizontalSlider->setMaximum(10000);
        threshold_horizontalSlider->setSingleStep(1);
        threshold_horizontalSlider->setPageStep(10);
        threshold_horizontalSlider->setValue(5000);
        threshold_horizontalSlider->setOrientation(Qt::Horizontal);
        threshold_horizontalSlider->setTickPosition(QSlider::NoTicks);

        gridLayout->addWidget(threshold_horizontalSlider, 2, 1, 1, 1);

        threshold_doubleSpinBox = new QDoubleSpinBox(groupBoxAg);
        threshold_doubleSpinBox->setObjectName(QString::fromUtf8("threshold_doubleSpinBox"));
        threshold_doubleSpinBox->setEnabled(false);
        threshold_doubleSpinBox->setDecimals(4);
        threshold_doubleSpinBox->setMaximum(1.000000000000000);
        threshold_doubleSpinBox->setSingleStep(0.001000000000000);
        threshold_doubleSpinBox->setValue(0.100000000000000);

        gridLayout->addWidget(threshold_doubleSpinBox, 2, 2, 1, 1);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        label_7 = new QLabel(groupBoxAg);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        horizontalLayout_8->addWidget(label_7);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_3);

        autoAG_checkBox = new QCheckBox(groupBoxAg);
        autoAG_checkBox->setObjectName(QString::fromUtf8("autoAG_checkBox"));

        horizontalLayout_8->addWidget(autoAG_checkBox);


        gridLayout->addLayout(horizontalLayout_8, 0, 0, 1, 3);


        gridLayout_9->addWidget(groupBoxAg, 3, 0, 1, 2);

        QWidget::setTabOrder(horizontalSlider, spinBox);
        QWidget::setTabOrder(spinBox, formatComboBox);
        QWidget::setTabOrder(formatComboBox, formatSettingsButton);
        QWidget::setTabOrder(formatSettingsButton, profileComboBox);
        QWidget::setTabOrder(profileComboBox, proposedFilenameCheckBox);
        QWidget::setTabOrder(proposedFilenameCheckBox, prefixLineEdit);
        QWidget::setTabOrder(prefixLineEdit, autoAlignCheckBox);
        QWidget::setTabOrder(autoAlignCheckBox, aisRadioButton);
        QWidget::setTabOrder(aisRadioButton, MTBRadioButton);
        QWidget::setTabOrder(MTBRadioButton, autoAG_checkBox);
        QWidget::setTabOrder(autoAG_checkBox, threshold_horizontalSlider);
        QWidget::setTabOrder(threshold_horizontalSlider, threshold_doubleSpinBox);
        QWidget::setTabOrder(threshold_doubleSpinBox, inputLineEdit);
        QWidget::setTabOrder(inputLineEdit, selectInputFolder);
        QWidget::setTabOrder(selectInputFolder, outputLineEdit);
        QWidget::setTabOrder(outputLineEdit, selectOutputFolder);
        QWidget::setTabOrder(selectOutputFolder, textEdit);
        QWidget::setTabOrder(textEdit, cancelButton);
        QWidget::setTabOrder(cancelButton, startButton);
        QWidget::setTabOrder(startButton, closeButton);

        retranslateUi(BatchHDRDialog);
        QObject::connect(spinBox, SIGNAL(valueChanged(int)), horizontalSlider, SLOT(setValue(int)));
        QObject::connect(horizontalSlider, SIGNAL(valueChanged(int)), spinBox, SLOT(setValue(int)));
        QObject::connect(closeButton, SIGNAL(clicked()), BatchHDRDialog, SLOT(accept()));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), threshold_horizontalSlider, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), threshold_doubleSpinBox, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), label_9, SLOT(setEnabled(bool)));
        QObject::connect(autoAlignCheckBox, SIGNAL(toggled(bool)), aisRadioButton, SLOT(setEnabled(bool)));
        QObject::connect(autoAlignCheckBox, SIGNAL(toggled(bool)), MTBRadioButton, SLOT(setEnabled(bool)));
        QObject::connect(autoAlignCheckBox, SIGNAL(toggled(bool)), autoCropCheckBox, SLOT(setEnabled(bool)));
        QObject::connect(proposedFilenameCheckBox, SIGNAL(toggled(bool)), label_4, SLOT(setDisabled(bool)));
        QObject::connect(proposedFilenameCheckBox, SIGNAL(toggled(bool)), prefixLineEdit, SLOT(setDisabled(bool)));

        formatComboBox->setCurrentIndex(-1);
        profileComboBox->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(BatchHDRDialog);
    } // setupUi

    void retranslateUi(QDialog *BatchHDRDialog)
    {
        BatchHDRDialog->setWindowTitle(QCoreApplication::translate("BatchHDRDialog", "Batch HDR", nullptr));
        label->setText(QCoreApplication::translate("BatchHDRDialog", "Number of bracketed images:", nullptr));
#if QT_CONFIG(tooltip)
        horizontalSlider->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Select the number of bracketed pictures used to create the HDRs", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        spinBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Select the number of bracketed pictures used to create the HDRs", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBoxOutput->setTitle(QCoreApplication::translate("BatchHDRDialog", "Output", nullptr));
#if QT_CONFIG(tooltip)
        formatComboBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Choose an HDR output file format", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("BatchHDRDialog", "Filename prefix:", nullptr));
        profileComboBox->setItemText(0, QCoreApplication::translate("BatchHDRDialog", "Profile 1", nullptr));
        profileComboBox->setItemText(1, QCoreApplication::translate("BatchHDRDialog", "Profile 2", nullptr));
        profileComboBox->setItemText(2, QCoreApplication::translate("BatchHDRDialog", "Profile 3", nullptr));
        profileComboBox->setItemText(3, QCoreApplication::translate("BatchHDRDialog", "Profile 4", nullptr));
        profileComboBox->setItemText(4, QCoreApplication::translate("BatchHDRDialog", "Profile 5", nullptr));
        profileComboBox->setItemText(5, QCoreApplication::translate("BatchHDRDialog", "Profile 6", nullptr));

#if QT_CONFIG(tooltip)
        profileComboBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Choose one of the creation predefined profiles. \n"
"See documentation for more information.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("BatchHDRDialog", "Output format:", nullptr));
        prefixLineEdit->setText(QCoreApplication::translate("BatchHDRDialog", "hdr_", nullptr));
#if QT_CONFIG(tooltip)
        proposedFilenameCheckBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "<html><head/><body><p>With leaving this checked, filenames for created HDRs will be of the form <span style=\" font-style:italic;\">first-last_HdrCreationModel.ext </span>where first is the file name of 1st braketed image in the set, last is the file name of the last bracketed image in the set and extension is the selected HDR file format extension.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        proposedFilenameCheckBox->setText(QCoreApplication::translate("BatchHDRDialog", "Use Proposed Filename Scheme", nullptr));
        label_10->setText(QCoreApplication::translate("BatchHDRDialog", "Response:", nullptr));
        label_11->setText(QCoreApplication::translate("BatchHDRDialog", "Weights:", nullptr));
        label_2->setText(QCoreApplication::translate("BatchHDRDialog", "Predefined profile:", nullptr));
        label_12->setText(QCoreApplication::translate("BatchHDRDialog", "Model:", nullptr));
        labelModel->setText(QCoreApplication::translate("BatchHDRDialog", "Debevec", nullptr));
        labelWeights->setText(QCoreApplication::translate("BatchHDRDialog", "Triangular", nullptr));
        labelResponse->setText(QCoreApplication::translate("BatchHDRDialog", "Linear", nullptr));
        groupBoxAlignment->setTitle(QCoreApplication::translate("BatchHDRDialog", "Alignment", nullptr));
#if QT_CONFIG(tooltip)
        aisRadioButton->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Use Hugin's align_image_stack engine", nullptr));
#endif // QT_CONFIG(tooltip)
        aisRadioButton->setText(QCoreApplication::translate("BatchHDRDialog", "Hugin's align_&image_stack", nullptr));
#if QT_CONFIG(tooltip)
        MTBRadioButton->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Use MTB (Median Threshold Bitmap) engine", nullptr));
#endif // QT_CONFIG(tooltip)
        MTBRadioButton->setText(QCoreApplication::translate("BatchHDRDialog", "&MTB", nullptr));
#if QT_CONFIG(tooltip)
        autoAlignCheckBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Auto align the bracketed pictures", nullptr));
#endif // QT_CONFIG(tooltip)
        autoAlignCheckBox->setText(QCoreApplication::translate("BatchHDRDialog", "Auto-align images", nullptr));
        autoCropCheckBox->setText(QCoreApplication::translate("BatchHDRDialog", "Auto-crop", nullptr));
        cancelButton->setText(QCoreApplication::translate("BatchHDRDialog", "&Cancel", nullptr));
        startButton->setText(QCoreApplication::translate("BatchHDRDialog", "&Start", nullptr));
        closeButton->setText(QCoreApplication::translate("BatchHDRDialog", "&Close", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("BatchHDRDialog", "Progress", nullptr));
        groupBoxIO->setTitle(QCoreApplication::translate("BatchHDRDialog", "Input/Output", nullptr));
#if QT_CONFIG(tooltip)
        selectOutputFolder->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Folder where created HDRs are saved", nullptr));
#endif // QT_CONFIG(tooltip)
        selectOutputFolder->setText(QCoreApplication::translate("BatchHDRDialog", "Select...", nullptr));
#if QT_CONFIG(tooltip)
        selectInputFolder->setToolTip(QCoreApplication::translate("BatchHDRDialog", "Folder where bracketed pictures are located (in alphabetical order)", nullptr));
#endif // QT_CONFIG(tooltip)
        selectInputFolder->setText(QCoreApplication::translate("BatchHDRDialog", "Select...", nullptr));
        label_5->setText(QCoreApplication::translate("BatchHDRDialog", "Input folder:", nullptr));
        label_6->setText(QCoreApplication::translate("BatchHDRDialog", "Output folder:", nullptr));
        groupBoxAg->setTitle(QCoreApplication::translate("BatchHDRDialog", "Anti-ghosting", nullptr));
        label_8->setText(QCoreApplication::translate("BatchHDRDialog", "Artifact-free High Dynamic Range Imaging", nullptr));
        label_9->setText(QCoreApplication::translate("BatchHDRDialog", "Threshold", nullptr));
#if QT_CONFIG(tooltip)
        threshold_horizontalSlider->setToolTip(QCoreApplication::translate("BatchHDRDialog", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range. Use the &quot;Recompute&quot; button to see the actual patches coverage and adjust the threshold accordingly</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        threshold_doubleSpinBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range. Use the &quot;Recompute&quot; button to see the actual patches coverage and adjust the threshold accordingly</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        label_7->setText(QCoreApplication::translate("BatchHDRDialog", "O. Gallo and others", nullptr));
#if QT_CONFIG(tooltip)
        autoAG_checkBox->setToolTip(QCoreApplication::translate("BatchHDRDialog", "<html><head/><body><p>Enable auto anti-ghosting algorithm</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        autoAG_checkBox->setText(QCoreApplication::translate("BatchHDRDialog", "Auto anti-ghosting", nullptr));
    } // retranslateUi

};

namespace Ui {
    class BatchHDRDialog: public Ui_BatchHDRDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BATCHHDRDIALOG_H
