/********************************************************************************
** Form generated from reading UI file 'TonemappingSettings.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TONEMAPPINGSETTINGS_H
#define UI_TONEMAPPINGSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_TonemappingSettings
{
public:
    QGridLayout *gridLayout_5;
    QSplitter *splitter;
    QWidget *gridLayoutWidget;
    QGridLayout *gridLayout_3;
    QListWidget *listWidget;
    QWidget *layoutWidget;
    QGridLayout *gridLayout_4;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QLabel *label;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer_2;
    QComboBox *comboBox;
    QFrame *frame_3;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *btnTonemap;
    QPushButton *applyButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *TonemappingSettings)
    {
        if (TonemappingSettings->objectName().isEmpty())
            TonemappingSettings->setObjectName(QString::fromUtf8("TonemappingSettings"));
        TonemappingSettings->resize(823, 534);
        gridLayout_5 = new QGridLayout(TonemappingSettings);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        splitter = new QSplitter(TonemappingSettings);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        splitter->setChildrenCollapsible(false);
        gridLayoutWidget = new QWidget(splitter);
        gridLayoutWidget->setObjectName(QString::fromUtf8("gridLayoutWidget"));
        gridLayout_3 = new QGridLayout(gridLayoutWidget);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        listWidget = new QListWidget(gridLayoutWidget);
        listWidget->setObjectName(QString::fromUtf8("listWidget"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(listWidget->sizePolicy().hasHeightForWidth());
        listWidget->setSizePolicy(sizePolicy);
        listWidget->setMinimumSize(QSize(50, 0));

        gridLayout_3->addWidget(listWidget, 0, 0, 1, 1);

        splitter->addWidget(gridLayoutWidget);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName(QString::fromUtf8("layoutWidget"));
        gridLayout_4 = new QGridLayout(layoutWidget);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(layoutWidget);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(4);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(scrollArea->sizePolicy().hasHeightForWidth());
        scrollArea->setSizePolicy(sizePolicy1);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 367, 453));
        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout_4->addWidget(scrollArea, 0, 0, 1, 2);

        splitter->addWidget(layoutWidget);

        gridLayout_5->addWidget(splitter, 1, 0, 1, 5);

        label = new QLabel(TonemappingSettings);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout_5->addWidget(label, 0, 2, 1, 1);

        label_2 = new QLabel(TonemappingSettings);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_5->addWidget(label_2, 0, 0, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_2, 0, 1, 1, 1);

        comboBox = new QComboBox(TonemappingSettings);
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->addItem(QString());
        comboBox->setObjectName(QString::fromUtf8("comboBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(comboBox->sizePolicy().hasHeightForWidth());
        comboBox->setSizePolicy(sizePolicy2);

        gridLayout_5->addWidget(comboBox, 0, 3, 1, 2);

        frame_3 = new QFrame(TonemappingSettings);
        frame_3->setObjectName(QString::fromUtf8("frame_3"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(frame_3->sizePolicy().hasHeightForWidth());
        frame_3->setSizePolicy(sizePolicy3);
        frame_3->setFrameShape(QFrame::StyledPanel);
        frame_3->setFrameShadow(QFrame::Raised);
        gridLayout_2 = new QGridLayout(frame_3);
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        btnTonemap = new QPushButton(frame_3);
        btnTonemap->setObjectName(QString::fromUtf8("btnTonemap"));

        horizontalLayout_2->addWidget(btnTonemap);

        applyButton = new QPushButton(frame_3);
        applyButton->setObjectName(QString::fromUtf8("applyButton"));

        horizontalLayout_2->addWidget(applyButton);

        cancelButton = new QPushButton(frame_3);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        horizontalLayout_2->addWidget(cancelButton);


        gridLayout_2->addLayout(horizontalLayout_2, 0, 0, 1, 1);


        gridLayout_5->addWidget(frame_3, 4, 0, 1, 5);


        retranslateUi(TonemappingSettings);
        QObject::connect(cancelButton, SIGNAL(clicked()), TonemappingSettings, SLOT(reject()));
        QObject::connect(applyButton, SIGNAL(clicked()), TonemappingSettings, SLOT(accept()));

        QMetaObject::connectSlotsByName(TonemappingSettings);
    } // setupUi

    void retranslateUi(QDialog *TonemappingSettings)
    {
        TonemappingSettings->setWindowTitle(QCoreApplication::translate("TonemappingSettings", "Load Custom Settings", nullptr));
        label->setText(QCoreApplication::translate("TonemappingSettings", "Sort by", nullptr));
        label_2->setText(QCoreApplication::translate("TonemappingSettings", "Comments:", nullptr));
        comboBox->setItemText(0, QCoreApplication::translate("TonemappingSettings", "Comment", nullptr));
        comboBox->setItemText(1, QCoreApplication::translate("TonemappingSettings", "Operator", nullptr));
        comboBox->setItemText(2, QCoreApplication::translate("TonemappingSettings", "Most Useful Operators", nullptr));

        btnTonemap->setText(QCoreApplication::translate("TonemappingSettings", "Tonemap", nullptr));
        applyButton->setText(QCoreApplication::translate("TonemappingSettings", "Load", nullptr));
        cancelButton->setText(QCoreApplication::translate("TonemappingSettings", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TonemappingSettings: public Ui_TonemappingSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TONEMAPPINGSETTINGS_H
