/****************************************************************************
** Meta object code from reading C++ file 'PreviewWidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/HdrWizard/PreviewWidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PreviewWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PreviewWidget_t {
    QByteArrayData data[42];
    char stringdata0[559];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreviewWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreviewWidget_t qt_meta_stringdata_PreviewWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PreviewWidget"
QT_MOC_LITERAL(1, 14, 5), // "moved"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 4), // "diff"
QT_MOC_LITERAL(4, 26, 14), // "selectionReady"
QT_MOC_LITERAL(5, 41, 7), // "isReady"
QT_MOC_LITERAL(6, 49, 7), // "changed"
QT_MOC_LITERAL(7, 57, 14), // "PreviewWidget*"
QT_MOC_LITERAL(8, 72, 1), // "v"
QT_MOC_LITERAL(9, 74, 13), // "patchesEdited"
QT_MOC_LITERAL(10, 88, 18), // "requestedBlendMode"
QT_MOC_LITERAL(11, 107, 10), // "updateView"
QT_MOC_LITERAL(12, 118, 18), // "updatePreviewImage"
QT_MOC_LITERAL(13, 137, 6), // "zoomIn"
QT_MOC_LITERAL(14, 144, 7), // "zoomOut"
QT_MOC_LITERAL(15, 152, 11), // "fitToWindow"
QT_MOC_LITERAL(16, 164, 12), // "fillToWindow"
QT_MOC_LITERAL(17, 177, 10), // "normalSize"
QT_MOC_LITERAL(18, 188, 13), // "getViewerMode"
QT_MOC_LITERAL(19, 202, 10), // "ViewerMode"
QT_MOC_LITERAL(20, 213, 13), // "setViewerMode"
QT_MOC_LITERAL(21, 227, 11), // "viewer_mode"
QT_MOC_LITERAL(22, 239, 12), // "hasSelection"
QT_MOC_LITERAL(23, 252, 16), // "setSelectionTool"
QT_MOC_LITERAL(24, 269, 16), // "getSelectionRect"
QT_MOC_LITERAL(25, 286, 15), // "removeSelection"
QT_MOC_LITERAL(26, 302, 22), // "switchAntighostingMode"
QT_MOC_LITERAL(27, 325, 21), // "switchViewPatchesMode"
QT_MOC_LITERAL(28, 347, 10), // "bool[][40]"
QT_MOC_LITERAL(29, 358, 10), // "getPatches"
QT_MOC_LITERAL(30, 369, 12), // "setBrushSize"
QT_MOC_LITERAL(31, 382, 16), // "setBrushStrength"
QT_MOC_LITERAL(32, 399, 13), // "setBrushColor"
QT_MOC_LITERAL(33, 413, 13), // "setLassoColor"
QT_MOC_LITERAL(34, 427, 12), // "setBrushMode"
QT_MOC_LITERAL(35, 440, 10), // "saveAgMask"
QT_MOC_LITERAL(36, 451, 14), // "getSavedAgMask"
QT_MOC_LITERAL(37, 466, 7), // "QImage*"
QT_MOC_LITERAL(38, 474, 25), // "slotPanIconSelectionMoved"
QT_MOC_LITERAL(39, 500, 17), // "slotPanIconHidden"
QT_MOC_LITERAL(40, 518, 23), // "slotCornerButtonPressed"
QT_MOC_LITERAL(41, 542, 16) // "scrollBarChanged"

    },
    "PreviewWidget\0moved\0\0diff\0selectionReady\0"
    "isReady\0changed\0PreviewWidget*\0v\0"
    "patchesEdited\0requestedBlendMode\0"
    "updateView\0updatePreviewImage\0zoomIn\0"
    "zoomOut\0fitToWindow\0fillToWindow\0"
    "normalSize\0getViewerMode\0ViewerMode\0"
    "setViewerMode\0viewer_mode\0hasSelection\0"
    "setSelectionTool\0getSelectionRect\0"
    "removeSelection\0switchAntighostingMode\0"
    "switchViewPatchesMode\0bool[][40]\0"
    "getPatches\0setBrushSize\0setBrushStrength\0"
    "setBrushColor\0setLassoColor\0setBrushMode\0"
    "saveAgMask\0getSavedAgMask\0QImage*\0"
    "slotPanIconSelectionMoved\0slotPanIconHidden\0"
    "slotCornerButtonPressed\0scrollBarChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreviewWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      32,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  174,    2, 0x06 /* Public */,
       4,    1,  177,    2, 0x06 /* Public */,
       6,    1,  180,    2, 0x06 /* Public */,
       9,    0,  183,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,  184,    2, 0x0a /* Public */,
      11,    0,  187,    2, 0x0a /* Public */,
      12,    0,  188,    2, 0x0a /* Public */,
      13,    0,  189,    2, 0x0a /* Public */,
      14,    0,  190,    2, 0x0a /* Public */,
      15,    0,  191,    2, 0x0a /* Public */,
      16,    0,  192,    2, 0x0a /* Public */,
      17,    0,  193,    2, 0x0a /* Public */,
      18,    0,  194,    2, 0x0a /* Public */,
      20,    1,  195,    2, 0x0a /* Public */,
      22,    0,  198,    2, 0x0a /* Public */,
      23,    1,  199,    2, 0x0a /* Public */,
      24,    0,  202,    2, 0x0a /* Public */,
      25,    0,  203,    2, 0x0a /* Public */,
      26,    1,  204,    2, 0x0a /* Public */,
      27,    4,  207,    2, 0x0a /* Public */,
      29,    1,  216,    2, 0x0a /* Public */,
      30,    1,  219,    2, 0x0a /* Public */,
      31,    1,  222,    2, 0x0a /* Public */,
      32,    1,  225,    2, 0x0a /* Public */,
      33,    1,  228,    2, 0x0a /* Public */,
      34,    1,  231,    2, 0x0a /* Public */,
      35,    0,  234,    2, 0x0a /* Public */,
      36,    0,  235,    2, 0x0a /* Public */,
      38,    1,  236,    2, 0x09 /* Protected */,
      39,    0,  239,    2, 0x09 /* Protected */,
      40,    0,  240,    2, 0x09 /* Protected */,
      41,    1,  241,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QPoint,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 19,
    QMetaType::Void, 0x80000000 | 19,   21,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::QRect,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 28, QMetaType::Int, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, 0x80000000 | 28,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QColor,    2,
    QMetaType::Void, QMetaType::QColor,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    0x80000000 | 37,
    QMetaType::Void, QMetaType::QRect,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void PreviewWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PreviewWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->moved((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 1: _t->selectionReady((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->changed((*reinterpret_cast< PreviewWidget*(*)>(_a[1]))); break;
        case 3: _t->patchesEdited(); break;
        case 4: _t->requestedBlendMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->updateView(); break;
        case 6: _t->updatePreviewImage(); break;
        case 7: _t->zoomIn(); break;
        case 8: _t->zoomOut(); break;
        case 9: _t->fitToWindow(); break;
        case 10: _t->fillToWindow(); break;
        case 11: _t->normalSize(); break;
        case 12: { ViewerMode _r = _t->getViewerMode();
            if (_a[0]) *reinterpret_cast< ViewerMode*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setViewerMode((*reinterpret_cast< ViewerMode(*)>(_a[1]))); break;
        case 14: { bool _r = _t->hasSelection();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setSelectionTool((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: { QRect _r = _t->getSelectionRect();
            if (_a[0]) *reinterpret_cast< QRect*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->removeSelection(); break;
        case 18: _t->switchAntighostingMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->switchViewPatchesMode((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)[][40]>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3])),(*reinterpret_cast< const int(*)>(_a[4]))); break;
        case 20: _t->getPatches((*reinterpret_cast< bool(*)[][40]>(_a[1]))); break;
        case 21: _t->setBrushSize((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 22: _t->setBrushStrength((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 23: _t->setBrushColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 24: _t->setLassoColor((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 25: _t->setBrushMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->saveAgMask(); break;
        case 27: { QImage* _r = _t->getSavedAgMask();
            if (_a[0]) *reinterpret_cast< QImage**>(_a[0]) = std::move(_r); }  break;
        case 28: _t->slotPanIconSelectionMoved((*reinterpret_cast< QRect(*)>(_a[1]))); break;
        case 29: _t->slotPanIconHidden(); break;
        case 30: _t->slotCornerButtonPressed(); break;
        case 31: _t->scrollBarChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PreviewWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PreviewWidget::*)(QPoint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewWidget::moved)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PreviewWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewWidget::selectionReady)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PreviewWidget::*)(PreviewWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewWidget::changed)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PreviewWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewWidget::patchesEdited)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PreviewWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PreviewWidget.data,
    qt_meta_data_PreviewWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreviewWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreviewWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreviewWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int PreviewWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    }
    return _id;
}

// SIGNAL 0
void PreviewWidget::moved(QPoint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PreviewWidget::selectionReady(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PreviewWidget::changed(PreviewWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PreviewWidget::patchesEdited()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
