/****************************************************************************
** Meta object code from reading C++ file 'TonemappingPanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/TonemappingPanel/TonemappingPanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TonemappingPanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TonemappingPanel_t {
    QByteArrayData data[55];
    char stringdata0[999];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TonemappingPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TonemappingPanel_t qt_meta_stringdata_TonemappingPanel = {
    {
QT_MOC_LITERAL(0, 0, 16), // "TonemappingPanel"
QT_MOC_LITERAL(1, 17, 16), // "startTonemapping"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 19), // "TonemappingOptions*"
QT_MOC_LITERAL(4, 55, 11), // "startExport"
QT_MOC_LITERAL(5, 67, 10), // "autoLevels"
QT_MOC_LITERAL(6, 78, 11), // "changeEvent"
QT_MOC_LITERAL(7, 90, 7), // "QEvent*"
QT_MOC_LITERAL(8, 98, 5), // "event"
QT_MOC_LITERAL(9, 104, 26), // "on_pregammadefault_clicked"
QT_MOC_LITERAL(10, 131, 27), // "on_postgammadefault_clicked"
QT_MOC_LITERAL(11, 159, 32), // "on_postsaturationdefault_clicked"
QT_MOC_LITERAL(12, 192, 24), // "on_defaultButton_clicked"
QT_MOC_LITERAL(13, 217, 22), // "on_applyButton_clicked"
QT_MOC_LITERAL(14, 240, 21), // "on_undoButton_clicked"
QT_MOC_LITERAL(15, 262, 21), // "on_redoButton_clicked"
QT_MOC_LITERAL(16, 284, 29), // "on_savesettingsbutton_clicked"
QT_MOC_LITERAL(17, 314, 29), // "on_loadsettingsbutton_clicked"
QT_MOC_LITERAL(18, 344, 22), // "on_queueButton_clicked"
QT_MOC_LITERAL(19, 367, 29), // "on_lblOpenQueue_linkActivated"
QT_MOC_LITERAL(20, 397, 4), // "link"
QT_MOC_LITERAL(21, 402, 11), // "fromTxt2Gui"
QT_MOC_LITERAL(22, 414, 30), // "on_addCustomSizeButton_clicked"
QT_MOC_LITERAL(23, 445, 22), // "fillCustomSizeComboBox"
QT_MOC_LITERAL(24, 468, 24), // "updateCurrentTmoOperator"
QT_MOC_LITERAL(25, 493, 15), // "updateUndoState"
QT_MOC_LITERAL(26, 509, 14), // "loadParameters"
QT_MOC_LITERAL(27, 524, 14), // "saveParameters"
QT_MOC_LITERAL(28, 539, 18), // "execMantiuk06Query"
QT_MOC_LITERAL(29, 558, 18), // "execMantiuk08Query"
QT_MOC_LITERAL(30, 577, 18), // "execAshikhminQuery"
QT_MOC_LITERAL(31, 596, 14), // "execDragoQuery"
QT_MOC_LITERAL(32, 611, 15), // "execDurandQuery"
QT_MOC_LITERAL(33, 627, 15), // "execFattalQuery"
QT_MOC_LITERAL(34, 643, 18), // "execFerradansQuery"
QT_MOC_LITERAL(35, 662, 17), // "execFerwerdaQuery"
QT_MOC_LITERAL(36, 680, 17), // "execKimKautzQuery"
QT_MOC_LITERAL(37, 698, 18), // "execPattanaikQuery"
QT_MOC_LITERAL(38, 717, 19), // "execReinhard02Query"
QT_MOC_LITERAL(39, 737, 19), // "execReinhard05Query"
QT_MOC_LITERAL(40, 757, 19), // "execVanHaterenQuery"
QT_MOC_LITERAL(41, 777, 19), // "execLischinskiQuery"
QT_MOC_LITERAL(42, 797, 14), // "updatePreviews"
QT_MOC_LITERAL(43, 812, 16), // "updatePreviewsCB"
QT_MOC_LITERAL(44, 829, 16), // "updatePreviewsRB"
QT_MOC_LITERAL(45, 846, 21), // "on_pattalocal_toggled"
QT_MOC_LITERAL(46, 868, 30), // "on_toolButtonThreshold_clicked"
QT_MOC_LITERAL(47, 899, 14), // "thresholdReady"
QT_MOC_LITERAL(48, 914, 10), // "setEnabled"
QT_MOC_LITERAL(49, 925, 10), // "updatedHDR"
QT_MOC_LITERAL(50, 936, 11), // "pfs::Frame*"
QT_MOC_LITERAL(51, 948, 23), // "updateTonemappingParams"
QT_MOC_LITERAL(52, 972, 4), // "opts"
QT_MOC_LITERAL(53, 977, 19), // "setRealtimePreviews"
QT_MOC_LITERAL(54, 997, 1) // "b"

    },
    "TonemappingPanel\0startTonemapping\0\0"
    "TonemappingOptions*\0startExport\0"
    "autoLevels\0changeEvent\0QEvent*\0event\0"
    "on_pregammadefault_clicked\0"
    "on_postgammadefault_clicked\0"
    "on_postsaturationdefault_clicked\0"
    "on_defaultButton_clicked\0"
    "on_applyButton_clicked\0on_undoButton_clicked\0"
    "on_redoButton_clicked\0"
    "on_savesettingsbutton_clicked\0"
    "on_loadsettingsbutton_clicked\0"
    "on_queueButton_clicked\0"
    "on_lblOpenQueue_linkActivated\0link\0"
    "fromTxt2Gui\0on_addCustomSizeButton_clicked\0"
    "fillCustomSizeComboBox\0updateCurrentTmoOperator\0"
    "updateUndoState\0loadParameters\0"
    "saveParameters\0execMantiuk06Query\0"
    "execMantiuk08Query\0execAshikhminQuery\0"
    "execDragoQuery\0execDurandQuery\0"
    "execFattalQuery\0execFerradansQuery\0"
    "execFerwerdaQuery\0execKimKautzQuery\0"
    "execPattanaikQuery\0execReinhard02Query\0"
    "execReinhard05Query\0execVanHaterenQuery\0"
    "execLischinskiQuery\0updatePreviews\0"
    "updatePreviewsCB\0updatePreviewsRB\0"
    "on_pattalocal_toggled\0"
    "on_toolButtonThreshold_clicked\0"
    "thresholdReady\0setEnabled\0updatedHDR\0"
    "pfs::Frame*\0updateTonemappingParams\0"
    "opts\0setRealtimePreviews\0b"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TonemappingPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      47,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  249,    2, 0x06 /* Public */,
       4,    1,  252,    2, 0x06 /* Public */,
       5,    2,  255,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,  260,    2, 0x09 /* Protected */,
       9,    0,  263,    2, 0x09 /* Protected */,
      10,    0,  264,    2, 0x09 /* Protected */,
      11,    0,  265,    2, 0x09 /* Protected */,
      12,    0,  266,    2, 0x09 /* Protected */,
      13,    0,  267,    2, 0x09 /* Protected */,
      14,    0,  268,    2, 0x09 /* Protected */,
      15,    0,  269,    2, 0x09 /* Protected */,
      16,    0,  270,    2, 0x09 /* Protected */,
      17,    0,  271,    2, 0x09 /* Protected */,
      18,    0,  272,    2, 0x09 /* Protected */,
      19,    1,  273,    2, 0x09 /* Protected */,
      21,    0,  276,    2, 0x09 /* Protected */,
      22,    0,  277,    2, 0x09 /* Protected */,
      23,    0,  278,    2, 0x09 /* Protected */,
      24,    1,  279,    2, 0x09 /* Protected */,
      25,    0,  282,    2, 0x09 /* Protected */,
      26,    0,  283,    2, 0x09 /* Protected */,
      27,    0,  284,    2, 0x09 /* Protected */,
      28,    5,  285,    2, 0x09 /* Protected */,
      29,    5,  296,    2, 0x09 /* Protected */,
      30,    4,  307,    2, 0x09 /* Protected */,
      31,    2,  316,    2, 0x09 /* Protected */,
      32,    4,  321,    2, 0x09 /* Protected */,
      33,    6,  330,    2, 0x09 /* Protected */,
      34,    3,  343,    2, 0x09 /* Protected */,
      35,    3,  350,    2, 0x09 /* Protected */,
      36,    3,  357,    2, 0x09 /* Protected */,
      37,    6,  364,    2, 0x09 /* Protected */,
      38,    7,  377,    2, 0x09 /* Protected */,
      39,    4,  392,    2, 0x09 /* Protected */,
      40,    2,  401,    2, 0x09 /* Protected */,
      41,    2,  406,    2, 0x09 /* Protected */,
      42,    1,  411,    2, 0x09 /* Protected */,
      43,    1,  414,    2, 0x09 /* Protected */,
      44,    1,  417,    2, 0x09 /* Protected */,
      45,    1,  420,    2, 0x09 /* Protected */,
      46,    0,  423,    2, 0x09 /* Protected */,
      47,    0,  424,    2, 0x09 /* Protected */,
      48,    1,  425,    2, 0x0a /* Public */,
      49,    1,  428,    2, 0x0a /* Public */,
      51,    1,  431,    2, 0x0a /* Public */,
      53,    1,  434,    2, 0x0a /* Public */,
       5,    1,  437,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Float,    2,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::Bool, QMetaType::QString,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool, QMetaType::Float, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::Bool, QMetaType::QString,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Float, QMetaType::Float, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::QString,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::Float, QMetaType::Float, QMetaType::QString,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::Float, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 50,    2,
    QMetaType::Void, 0x80000000 | 3,   52,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,   54,

       0        // eod
};

void TonemappingPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TonemappingPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startTonemapping((*reinterpret_cast< TonemappingOptions*(*)>(_a[1]))); break;
        case 1: _t->startExport((*reinterpret_cast< TonemappingOptions*(*)>(_a[1]))); break;
        case 2: _t->autoLevels((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 3: _t->changeEvent((*reinterpret_cast< QEvent*(*)>(_a[1]))); break;
        case 4: _t->on_pregammadefault_clicked(); break;
        case 5: _t->on_postgammadefault_clicked(); break;
        case 6: _t->on_postsaturationdefault_clicked(); break;
        case 7: _t->on_defaultButton_clicked(); break;
        case 8: _t->on_applyButton_clicked(); break;
        case 9: _t->on_undoButton_clicked(); break;
        case 10: _t->on_redoButton_clicked(); break;
        case 11: _t->on_savesettingsbutton_clicked(); break;
        case 12: _t->on_loadsettingsbutton_clicked(); break;
        case 13: _t->on_queueButton_clicked(); break;
        case 14: _t->on_lblOpenQueue_linkActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->fromTxt2Gui(); break;
        case 16: _t->on_addCustomSizeButton_clicked(); break;
        case 17: _t->fillCustomSizeComboBox(); break;
        case 18: _t->updateCurrentTmoOperator((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->updateUndoState(); break;
        case 20: _t->loadParameters(); break;
        case 21: _t->saveParameters(); break;
        case 22: _t->execMantiuk06Query((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< float(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        case 23: _t->execMantiuk08Query((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5]))); break;
        case 24: _t->execAshikhminQuery((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4]))); break;
        case 25: _t->execDragoQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 26: _t->execDurandQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4]))); break;
        case 27: _t->execFattalQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< float(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6]))); break;
        case 28: _t->execFerradansQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 29: _t->execFerwerdaQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 30: _t->execKimKautzQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 31: _t->execPattanaikQuery((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< float(*)>(_a[4])),(*reinterpret_cast< float(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6]))); break;
        case 32: _t->execReinhard02Query((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< int(*)>(_a[6])),(*reinterpret_cast< QString(*)>(_a[7]))); break;
        case 33: _t->execReinhard05Query((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2])),(*reinterpret_cast< float(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4]))); break;
        case 34: _t->execVanHaterenQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 35: _t->execLischinskiQuery((*reinterpret_cast< float(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 36: _t->updatePreviews((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 37: _t->updatePreviewsCB((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->updatePreviewsRB((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->on_pattalocal_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 40: _t->on_toolButtonThreshold_clicked(); break;
        case 41: _t->thresholdReady(); break;
        case 42: _t->setEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->updatedHDR((*reinterpret_cast< pfs::Frame*(*)>(_a[1]))); break;
        case 44: _t->updateTonemappingParams((*reinterpret_cast< TonemappingOptions*(*)>(_a[1]))); break;
        case 45: _t->setRealtimePreviews((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 46: _t->autoLevels((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TonemappingPanel::*)(TonemappingOptions * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TonemappingPanel::startTonemapping)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TonemappingPanel::*)(TonemappingOptions * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TonemappingPanel::startExport)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TonemappingPanel::*)(bool , float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TonemappingPanel::autoLevels)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TonemappingPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TonemappingPanel.data,
    qt_meta_data_TonemappingPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TonemappingPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TonemappingPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TonemappingPanel.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TonemappingPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 47)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 47;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 47)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 47;
    }
    return _id;
}

// SIGNAL 0
void TonemappingPanel::startTonemapping(TonemappingOptions * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TonemappingPanel::startExport(TonemappingOptions * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TonemappingPanel::autoLevels(bool _t1, float _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
