/****************************************************************************
** Meta object code from reading C++ file 'TMWorker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/Core/TMWorker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TMWorker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TMWorker_t {
    QByteArrayData data[23];
    char stringdata0[318];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TMWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TMWorker_t qt_meta_stringdata_TMWorker = {
    {
QT_MOC_LITERAL(0, 0, 8), // "TMWorker"
QT_MOC_LITERAL(1, 9, 14), // "tonemapSuccess"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 11), // "pfs::Frame*"
QT_MOC_LITERAL(4, 37, 19), // "TonemappingOptions*"
QT_MOC_LITERAL(5, 57, 13), // "tonemapFailed"
QT_MOC_LITERAL(6, 71, 12), // "tonemapBegin"
QT_MOC_LITERAL(7, 84, 10), // "tonemapEnd"
QT_MOC_LITERAL(8, 95, 17), // "tonemapSetMaximum"
QT_MOC_LITERAL(9, 113, 17), // "tonemapSetMinimum"
QT_MOC_LITERAL(10, 131, 15), // "tonemapSetValue"
QT_MOC_LITERAL(11, 147, 25), // "tonemapRequestTermination"
QT_MOC_LITERAL(12, 173, 14), // "computeTonemap"
QT_MOC_LITERAL(13, 188, 19), // "InterpolationMethod"
QT_MOC_LITERAL(14, 208, 1), // "m"
QT_MOC_LITERAL(15, 210, 23), // "computeTonemapAndExport"
QT_MOC_LITERAL(16, 234, 11), // "pfs::Params"
QT_MOC_LITERAL(17, 246, 9), // "exportDir"
QT_MOC_LITERAL(18, 256, 7), // "hdrName"
QT_MOC_LITERAL(19, 264, 10), // "inputfname"
QT_MOC_LITERAL(20, 275, 14), // "QVector<float>"
QT_MOC_LITERAL(21, 290, 14), // "inputExpoTimes"
QT_MOC_LITERAL(22, 305, 12) // "tonemapFrame"

    },
    "TMWorker\0tonemapSuccess\0\0pfs::Frame*\0"
    "TonemappingOptions*\0tonemapFailed\0"
    "tonemapBegin\0tonemapEnd\0tonemapSetMaximum\0"
    "tonemapSetMinimum\0tonemapSetValue\0"
    "tonemapRequestTermination\0computeTonemap\0"
    "InterpolationMethod\0m\0computeTonemapAndExport\0"
    "pfs::Params\0exportDir\0hdrName\0inputfname\0"
    "QVector<float>\0inputExpoTimes\0"
    "tonemapFrame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TMWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   69,    2, 0x06 /* Public */,
       5,    1,   74,    2, 0x06 /* Public */,
       6,    0,   77,    2, 0x06 /* Public */,
       7,    0,   78,    2, 0x06 /* Public */,
       8,    1,   79,    2, 0x06 /* Public */,
       9,    1,   82,    2, 0x06 /* Public */,
      10,    1,   85,    2, 0x06 /* Public */,
      11,    1,   88,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    3,   91,    2, 0x0a /* Public */,
      15,    8,   98,    2, 0x0a /* Public */,
      22,    2,  115,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 4,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 4, 0x80000000 | 13,    2,    2,   14,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 4, 0x80000000 | 16, QMetaType::QString, QMetaType::QString, QMetaType::QString, 0x80000000 | 20, 0x80000000 | 13,    2,    2,    2,   17,   18,   19,   21,   14,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 4,    2,    2,

       0        // eod
};

void TMWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TMWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->tonemapSuccess((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[2]))); break;
        case 1: _t->tonemapFailed((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->tonemapBegin(); break;
        case 3: _t->tonemapEnd(); break;
        case 4: _t->tonemapSetMaximum((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->tonemapSetMinimum((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->tonemapSetValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->tonemapRequestTermination((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: { pfs::Frame* _r = _t->computeTonemap((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[2])),(*reinterpret_cast< InterpolationMethod(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< pfs::Frame**>(_a[0]) = std::move(_r); }  break;
        case 9: _t->computeTonemapAndExport((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[2])),(*reinterpret_cast< pfs::Params(*)>(_a[3])),(*reinterpret_cast< QString(*)>(_a[4])),(*reinterpret_cast< QString(*)>(_a[5])),(*reinterpret_cast< QString(*)>(_a[6])),(*reinterpret_cast< QVector<float>(*)>(_a[7])),(*reinterpret_cast< InterpolationMethod(*)>(_a[8]))); break;
        case 10: _t->tonemapFrame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<float> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TMWorker::*)(pfs::Frame * , TonemappingOptions * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapSuccess)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapFailed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapBegin)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapEnd)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapSetMaximum)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapSetMinimum)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapSetValue)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TMWorker::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TMWorker::tonemapRequestTermination)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TMWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TMWorker.data,
    qt_meta_data_TMWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TMWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TMWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TMWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TMWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TMWorker::tonemapSuccess(pfs::Frame * _t1, TonemappingOptions * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TMWorker::tonemapFailed(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TMWorker::tonemapBegin()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TMWorker::tonemapEnd()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TMWorker::tonemapSetMaximum(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TMWorker::tonemapSetMinimum(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TMWorker::tonemapSetValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void TMWorker::tonemapRequestTermination(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
