/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/MainWindow/MainWindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[113];
    char stringdata0[2287];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 10), // "updatedHDR"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 11), // "pfs::Frame*"
QT_MOC_LITERAL(4, 35, 16), // "save_hdr_success"
QT_MOC_LITERAL(5, 52, 14), // "GenericViewer*"
QT_MOC_LITERAL(6, 67, 9), // "saved_hdr"
QT_MOC_LITERAL(7, 77, 5), // "fname"
QT_MOC_LITERAL(8, 83, 15), // "save_hdr_failed"
QT_MOC_LITERAL(9, 99, 16), // "save_ldr_success"
QT_MOC_LITERAL(10, 116, 9), // "saved_ldr"
QT_MOC_LITERAL(11, 126, 15), // "save_ldr_failed"
QT_MOC_LITERAL(12, 142, 11), // "load_failed"
QT_MOC_LITERAL(13, 154, 12), // "load_success"
QT_MOC_LITERAL(14, 167, 13), // "new_hdr_frame"
QT_MOC_LITERAL(15, 181, 9), // "new_fname"
QT_MOC_LITERAL(16, 191, 14), // "inputFileNames"
QT_MOC_LITERAL(17, 206, 10), // "needSaving"
QT_MOC_LITERAL(18, 217, 7), // "ioBegin"
QT_MOC_LITERAL(19, 225, 5), // "ioEnd"
QT_MOC_LITERAL(20, 231, 21), // "setMainWindowModified"
QT_MOC_LITERAL(21, 253, 1), // "b"
QT_MOC_LITERAL(22, 255, 8), // "openFile"
QT_MOC_LITERAL(23, 264, 4), // "file"
QT_MOC_LITERAL(24, 269, 9), // "openFiles"
QT_MOC_LITERAL(25, 279, 5), // "files"
QT_MOC_LITERAL(26, 285, 26), // "on_fileNewAction_triggered"
QT_MOC_LITERAL(27, 312, 12), // "createNewHdr"
QT_MOC_LITERAL(28, 325, 27), // "on_fileOpenAction_triggered"
QT_MOC_LITERAL(29, 353, 29), // "on_fileSaveAsAction_triggered"
QT_MOC_LITERAL(30, 383, 30), // "on_fileSaveAllAction_triggered"
QT_MOC_LITERAL(31, 414, 35), // "on_actionSave_Hdr_Preview_tri..."
QT_MOC_LITERAL(32, 450, 22), // "on_rotateccw_triggered"
QT_MOC_LITERAL(33, 473, 21), // "on_rotatecw_triggered"
QT_MOC_LITERAL(34, 495, 28), // "on_actionResizeHDR_triggered"
QT_MOC_LITERAL(35, 524, 45), // "on_action_Projective_Transfor..."
QT_MOC_LITERAL(36, 570, 28), // "on_actionBatch_HDR_triggered"
QT_MOC_LITERAL(37, 599, 37), // "on_actionBatch_Tone_Mapping_t..."
QT_MOC_LITERAL(38, 637, 31), // "on_actionExportToHTML_triggered"
QT_MOC_LITERAL(39, 669, 36), // "on_actionSupported_Cameras_tr..."
QT_MOC_LITERAL(40, 706, 30), // "on_Increase_exposure_triggered"
QT_MOC_LITERAL(41, 737, 30), // "on_Decrease_exposure_triggered"
QT_MOC_LITERAL(42, 768, 33), // "on_Extend_dynamic_range_trigg..."
QT_MOC_LITERAL(43, 802, 33), // "on_Shrink_dynamic_range_trigg..."
QT_MOC_LITERAL(44, 836, 33), // "on_Fit_to_dynamic_range_trigg..."
QT_MOC_LITERAL(45, 870, 30), // "on_Low_dynamic_range_triggered"
QT_MOC_LITERAL(46, 901, 22), // "on_zoomInAct_triggered"
QT_MOC_LITERAL(47, 924, 23), // "on_zoomOutAct_triggered"
QT_MOC_LITERAL(48, 948, 27), // "on_fitToWindowAct_triggered"
QT_MOC_LITERAL(49, 976, 33), // "on_actionFill_to_Window_trigg..."
QT_MOC_LITERAL(50, 1010, 26), // "on_normalSizeAct_triggered"
QT_MOC_LITERAL(51, 1037, 26), // "updateMagnificationButtons"
QT_MOC_LITERAL(52, 1064, 14), // "enterWhatsThis"
QT_MOC_LITERAL(53, 1079, 26), // "on_OptionsAction_triggered"
QT_MOC_LITERAL(54, 1106, 40), // "on_Transplant_Exif_Data_actio..."
QT_MOC_LITERAL(55, 1147, 30), // "on_actionFix_Histogram_toggled"
QT_MOC_LITERAL(56, 1178, 7), // "checked"
QT_MOC_LITERAL(57, 1186, 32), // "on_actionWhite_Balance_triggered"
QT_MOC_LITERAL(58, 1219, 16), // "whiteBalanceDone"
QT_MOC_LITERAL(59, 1236, 16), // "Text_Under_Icons"
QT_MOC_LITERAL(60, 1253, 10), // "Icons_Only"
QT_MOC_LITERAL(61, 1264, 20), // "Text_Alongside_Icons"
QT_MOC_LITERAL(62, 1285, 9), // "Text_Only"
QT_MOC_LITERAL(63, 1295, 22), // "showPreviewsOnTheRight"
QT_MOC_LITERAL(64, 1318, 23), // "showPreviewsOnTheBottom"
QT_MOC_LITERAL(65, 1342, 16), // "updateWindowMenu"
QT_MOC_LITERAL(66, 1359, 37), // "on_actionBring_All_to_Front_t..."
QT_MOC_LITERAL(67, 1397, 14), // "openRecentFile"
QT_MOC_LITERAL(68, 1412, 14), // "setCurrentFile"
QT_MOC_LITERAL(69, 1427, 8), // "fileName"
QT_MOC_LITERAL(70, 1436, 34), // "on_actionAbout_Luminance_trig..."
QT_MOC_LITERAL(71, 1471, 13), // "updateActions"
QT_MOC_LITERAL(72, 1485, 1), // "w"
QT_MOC_LITERAL(73, 1487, 19), // "setActiveMainWindow"
QT_MOC_LITERAL(74, 1507, 8), // "QWidget*"
QT_MOC_LITERAL(75, 1516, 15), // "cropToSelection"
QT_MOC_LITERAL(76, 1532, 10), // "enableCrop"
QT_MOC_LITERAL(77, 1543, 11), // "disableCrop"
QT_MOC_LITERAL(78, 1555, 25), // "on_actionDonate_triggered"
QT_MOC_LITERAL(79, 1581, 17), // "onUpdateAvailable"
QT_MOC_LITERAL(80, 1599, 34), // "on_actionUpdateAvailable_trig..."
QT_MOC_LITERAL(81, 1634, 12), // "tonemapBegin"
QT_MOC_LITERAL(82, 1647, 10), // "tonemapEnd"
QT_MOC_LITERAL(83, 1658, 12), // "tonemapImage"
QT_MOC_LITERAL(84, 1671, 19), // "TonemappingOptions*"
QT_MOC_LITERAL(85, 1691, 4), // "opts"
QT_MOC_LITERAL(86, 1696, 11), // "exportImage"
QT_MOC_LITERAL(87, 1708, 11), // "addLdrFrame"
QT_MOC_LITERAL(88, 1720, 13), // "tonemapFailed"
QT_MOC_LITERAL(89, 1734, 11), // "exportBegin"
QT_MOC_LITERAL(90, 1746, 9), // "exportEnd"
QT_MOC_LITERAL(91, 1756, 21), // "on_actionLock_toggled"
QT_MOC_LITERAL(92, 1778, 11), // "syncViewers"
QT_MOC_LITERAL(93, 1790, 16), // "showPreviewPanel"
QT_MOC_LITERAL(94, 1807, 9), // "removeTab"
QT_MOC_LITERAL(95, 1817, 29), // "on_actionRemove_Tab_triggered"
QT_MOC_LITERAL(96, 1847, 30), // "on_actionSoft_Proofing_toggled"
QT_MOC_LITERAL(97, 1878, 28), // "on_actionGamut_Check_toggled"
QT_MOC_LITERAL(98, 1907, 18), // "updateSoftProofing"
QT_MOC_LITERAL(99, 1926, 32), // "on_actionFits_Importer_triggered"
QT_MOC_LITERAL(100, 1959, 33), // "on_actionShow_Full_Screen_tog..."
QT_MOC_LITERAL(101, 1993, 41), // "on_actionShow_Image_Full_Scre..."
QT_MOC_LITERAL(102, 2035, 44), // "on_actionSelect_Interpolation..."
QT_MOC_LITERAL(103, 2080, 14), // "updatePreviews"
QT_MOC_LITERAL(104, 2095, 14), // "reparentViewer"
QT_MOC_LITERAL(105, 2110, 4), // "g_vi"
QT_MOC_LITERAL(106, 2115, 14), // "showNextViewer"
QT_MOC_LITERAL(107, 2130, 3), // "g_v"
QT_MOC_LITERAL(108, 2134, 18), // "showPreviousViewer"
QT_MOC_LITERAL(109, 2153, 14), // "setSyncViewers"
QT_MOC_LITERAL(110, 2168, 39), // "on_actionOnline_Documentation..."
QT_MOC_LITERAL(111, 2208, 37), // "on_actionDevelopers_Website_t..."
QT_MOC_LITERAL(112, 2246, 40) // "on_actionLuminance_HDR_Websit..."

    },
    "MainWindow\0updatedHDR\0\0pfs::Frame*\0"
    "save_hdr_success\0GenericViewer*\0"
    "saved_hdr\0fname\0save_hdr_failed\0"
    "save_ldr_success\0saved_ldr\0save_ldr_failed\0"
    "load_failed\0load_success\0new_hdr_frame\0"
    "new_fname\0inputFileNames\0needSaving\0"
    "ioBegin\0ioEnd\0setMainWindowModified\0"
    "b\0openFile\0file\0openFiles\0files\0"
    "on_fileNewAction_triggered\0createNewHdr\0"
    "on_fileOpenAction_triggered\0"
    "on_fileSaveAsAction_triggered\0"
    "on_fileSaveAllAction_triggered\0"
    "on_actionSave_Hdr_Preview_triggered\0"
    "on_rotateccw_triggered\0on_rotatecw_triggered\0"
    "on_actionResizeHDR_triggered\0"
    "on_action_Projective_Transformation_triggered\0"
    "on_actionBatch_HDR_triggered\0"
    "on_actionBatch_Tone_Mapping_triggered\0"
    "on_actionExportToHTML_triggered\0"
    "on_actionSupported_Cameras_triggered\0"
    "on_Increase_exposure_triggered\0"
    "on_Decrease_exposure_triggered\0"
    "on_Extend_dynamic_range_triggered\0"
    "on_Shrink_dynamic_range_triggered\0"
    "on_Fit_to_dynamic_range_triggered\0"
    "on_Low_dynamic_range_triggered\0"
    "on_zoomInAct_triggered\0on_zoomOutAct_triggered\0"
    "on_fitToWindowAct_triggered\0"
    "on_actionFill_to_Window_triggered\0"
    "on_normalSizeAct_triggered\0"
    "updateMagnificationButtons\0enterWhatsThis\0"
    "on_OptionsAction_triggered\0"
    "on_Transplant_Exif_Data_action_triggered\0"
    "on_actionFix_Histogram_toggled\0checked\0"
    "on_actionWhite_Balance_triggered\0"
    "whiteBalanceDone\0Text_Under_Icons\0"
    "Icons_Only\0Text_Alongside_Icons\0"
    "Text_Only\0showPreviewsOnTheRight\0"
    "showPreviewsOnTheBottom\0updateWindowMenu\0"
    "on_actionBring_All_to_Front_triggered\0"
    "openRecentFile\0setCurrentFile\0fileName\0"
    "on_actionAbout_Luminance_triggered\0"
    "updateActions\0w\0setActiveMainWindow\0"
    "QWidget*\0cropToSelection\0enableCrop\0"
    "disableCrop\0on_actionDonate_triggered\0"
    "onUpdateAvailable\0on_actionUpdateAvailable_triggered\0"
    "tonemapBegin\0tonemapEnd\0tonemapImage\0"
    "TonemappingOptions*\0opts\0exportImage\0"
    "addLdrFrame\0tonemapFailed\0exportBegin\0"
    "exportEnd\0on_actionLock_toggled\0"
    "syncViewers\0showPreviewPanel\0removeTab\0"
    "on_actionRemove_Tab_triggered\0"
    "on_actionSoft_Proofing_toggled\0"
    "on_actionGamut_Check_toggled\0"
    "updateSoftProofing\0on_actionFits_Importer_triggered\0"
    "on_actionShow_Full_Screen_toggled\0"
    "on_actionShow_Image_Full_Screen_triggered\0"
    "on_actionSelect_Interpolation_Method_toggled\0"
    "updatePreviews\0reparentViewer\0g_vi\0"
    "showNextViewer\0g_v\0showPreviousViewer\0"
    "setSyncViewers\0on_actionOnline_Documentation_triggered\0"
    "on_actionDevelopers_Website_triggered\0"
    "on_actionLuminance_HDR_Website_triggered"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      94,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  484,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    2,  487,    2, 0x0a /* Public */,
       8,    1,  492,    2, 0x0a /* Public */,
       9,    2,  495,    2, 0x0a /* Public */,
      11,    1,  500,    2, 0x0a /* Public */,
      12,    1,  503,    2, 0x0a /* Public */,
      13,    4,  506,    2, 0x0a /* Public */,
      13,    3,  515,    2, 0x2a /* Public | MethodCloned */,
      13,    2,  522,    2, 0x2a /* Public | MethodCloned */,
      18,    0,  527,    2, 0x0a /* Public */,
      19,    0,  528,    2, 0x0a /* Public */,
      20,    1,  529,    2, 0x0a /* Public */,
      22,    1,  532,    2, 0x0a /* Public */,
      24,    1,  535,    2, 0x0a /* Public */,
      26,    0,  538,    2, 0x09 /* Protected */,
      27,    1,  539,    2, 0x09 /* Protected */,
      28,    0,  542,    2, 0x09 /* Protected */,
      29,    0,  543,    2, 0x09 /* Protected */,
      30,    0,  544,    2, 0x09 /* Protected */,
      31,    0,  545,    2, 0x09 /* Protected */,
      32,    0,  546,    2, 0x09 /* Protected */,
      33,    0,  547,    2, 0x09 /* Protected */,
      34,    0,  548,    2, 0x09 /* Protected */,
      35,    0,  549,    2, 0x09 /* Protected */,
      36,    0,  550,    2, 0x09 /* Protected */,
      37,    0,  551,    2, 0x09 /* Protected */,
      38,    0,  552,    2, 0x09 /* Protected */,
      39,    0,  553,    2, 0x09 /* Protected */,
      40,    0,  554,    2, 0x09 /* Protected */,
      41,    0,  555,    2, 0x09 /* Protected */,
      42,    0,  556,    2, 0x09 /* Protected */,
      43,    0,  557,    2, 0x09 /* Protected */,
      44,    0,  558,    2, 0x09 /* Protected */,
      45,    0,  559,    2, 0x09 /* Protected */,
      46,    0,  560,    2, 0x09 /* Protected */,
      47,    0,  561,    2, 0x09 /* Protected */,
      48,    0,  562,    2, 0x09 /* Protected */,
      49,    0,  563,    2, 0x09 /* Protected */,
      50,    0,  564,    2, 0x09 /* Protected */,
      51,    1,  565,    2, 0x09 /* Protected */,
      52,    0,  568,    2, 0x09 /* Protected */,
      53,    0,  569,    2, 0x09 /* Protected */,
      54,    0,  570,    2, 0x09 /* Protected */,
      55,    1,  571,    2, 0x09 /* Protected */,
      57,    0,  574,    2, 0x09 /* Protected */,
      58,    0,  575,    2, 0x09 /* Protected */,
      59,    0,  576,    2, 0x09 /* Protected */,
      60,    0,  577,    2, 0x09 /* Protected */,
      61,    0,  578,    2, 0x09 /* Protected */,
      62,    0,  579,    2, 0x09 /* Protected */,
      63,    0,  580,    2, 0x09 /* Protected */,
      64,    0,  581,    2, 0x09 /* Protected */,
      65,    0,  582,    2, 0x09 /* Protected */,
      66,    0,  583,    2, 0x09 /* Protected */,
      67,    0,  584,    2, 0x09 /* Protected */,
      68,    1,  585,    2, 0x09 /* Protected */,
      70,    0,  588,    2, 0x09 /* Protected */,
      71,    1,  589,    2, 0x09 /* Protected */,
      73,    1,  592,    2, 0x09 /* Protected */,
      75,    0,  595,    2, 0x09 /* Protected */,
      76,    1,  596,    2, 0x09 /* Protected */,
      77,    0,  599,    2, 0x09 /* Protected */,
      78,    0,  600,    2, 0x09 /* Protected */,
      79,    0,  601,    2, 0x09 /* Protected */,
      80,    0,  602,    2, 0x09 /* Protected */,
      81,    0,  603,    2, 0x09 /* Protected */,
      82,    0,  604,    2, 0x09 /* Protected */,
      83,    1,  605,    2, 0x09 /* Protected */,
      86,    1,  608,    2, 0x09 /* Protected */,
      87,    2,  611,    2, 0x09 /* Protected */,
      88,    1,  616,    2, 0x09 /* Protected */,
      89,    0,  619,    2, 0x09 /* Protected */,
      90,    0,  620,    2, 0x09 /* Protected */,
      91,    1,  621,    2, 0x09 /* Protected */,
      92,    1,  624,    2, 0x09 /* Protected */,
      93,    1,  627,    2, 0x09 /* Protected */,
      94,    1,  630,    2, 0x09 /* Protected */,
      95,    0,  633,    2, 0x09 /* Protected */,
      96,    1,  634,    2, 0x09 /* Protected */,
      97,    1,  637,    2, 0x09 /* Protected */,
      98,    1,  640,    2, 0x09 /* Protected */,
      99,    0,  643,    2, 0x09 /* Protected */,
     100,    1,  644,    2, 0x09 /* Protected */,
     101,    0,  647,    2, 0x09 /* Protected */,
     102,    1,  648,    2, 0x09 /* Protected */,
     103,    2,  651,    2, 0x09 /* Protected */,
     104,    2,  656,    2, 0x09 /* Protected */,
     104,    1,  661,    2, 0x29 /* Protected | MethodCloned */,
     106,    1,  664,    2, 0x09 /* Protected */,
     108,    1,  667,    2, 0x09 /* Protected */,
     109,    1,  670,    2, 0x09 /* Protected */,
     110,    0,  673,    2, 0x09 /* Protected */,
     111,    0,  674,    2, 0x09 /* Protected */,
     112,    0,  675,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5, QMetaType::QString,    6,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, 0x80000000 | 5, QMetaType::QString,   10,    7,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::QStringList, QMetaType::Bool,   14,   15,   16,   17,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString, QMetaType::QStringList,   14,   15,   16,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,   14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void, QMetaType::QStringList,   25,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   69,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   72,
    QMetaType::Void, 0x80000000 | 74,   72,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 84,   85,
    QMetaType::Void, 0x80000000 | 84,   85,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 84,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Float,    2,    2,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,  105,    2,
    QMetaType::Void, 0x80000000 | 5,  105,
    QMetaType::Void, 0x80000000 | 5,  107,
    QMetaType::Void, 0x80000000 | 5,  107,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updatedHDR((*reinterpret_cast< pfs::Frame*(*)>(_a[1]))); break;
        case 1: _t->save_hdr_success((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->save_hdr_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->save_ldr_success((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->save_ldr_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->load_failed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->load_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 7: _t->load_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3]))); break;
        case 8: _t->load_success((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->ioBegin(); break;
        case 10: _t->ioEnd(); break;
        case 11: _t->setMainWindowModified((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->openFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->openFiles((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 14: _t->on_fileNewAction_triggered(); break;
        case 15: _t->createNewHdr((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 16: _t->on_fileOpenAction_triggered(); break;
        case 17: _t->on_fileSaveAsAction_triggered(); break;
        case 18: _t->on_fileSaveAllAction_triggered(); break;
        case 19: _t->on_actionSave_Hdr_Preview_triggered(); break;
        case 20: _t->on_rotateccw_triggered(); break;
        case 21: _t->on_rotatecw_triggered(); break;
        case 22: _t->on_actionResizeHDR_triggered(); break;
        case 23: _t->on_action_Projective_Transformation_triggered(); break;
        case 24: _t->on_actionBatch_HDR_triggered(); break;
        case 25: _t->on_actionBatch_Tone_Mapping_triggered(); break;
        case 26: _t->on_actionExportToHTML_triggered(); break;
        case 27: _t->on_actionSupported_Cameras_triggered(); break;
        case 28: _t->on_Increase_exposure_triggered(); break;
        case 29: _t->on_Decrease_exposure_triggered(); break;
        case 30: _t->on_Extend_dynamic_range_triggered(); break;
        case 31: _t->on_Shrink_dynamic_range_triggered(); break;
        case 32: _t->on_Fit_to_dynamic_range_triggered(); break;
        case 33: _t->on_Low_dynamic_range_triggered(); break;
        case 34: _t->on_zoomInAct_triggered(); break;
        case 35: _t->on_zoomOutAct_triggered(); break;
        case 36: _t->on_fitToWindowAct_triggered(); break;
        case 37: _t->on_actionFill_to_Window_triggered(); break;
        case 38: _t->on_normalSizeAct_triggered(); break;
        case 39: _t->updateMagnificationButtons((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 40: _t->enterWhatsThis(); break;
        case 41: _t->on_OptionsAction_triggered(); break;
        case 42: _t->on_Transplant_Exif_Data_action_triggered(); break;
        case 43: _t->on_actionFix_Histogram_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 44: _t->on_actionWhite_Balance_triggered(); break;
        case 45: _t->whiteBalanceDone(); break;
        case 46: _t->Text_Under_Icons(); break;
        case 47: _t->Icons_Only(); break;
        case 48: _t->Text_Alongside_Icons(); break;
        case 49: _t->Text_Only(); break;
        case 50: _t->showPreviewsOnTheRight(); break;
        case 51: _t->showPreviewsOnTheBottom(); break;
        case 52: _t->updateWindowMenu(); break;
        case 53: _t->on_actionBring_All_to_Front_triggered(); break;
        case 54: _t->openRecentFile(); break;
        case 55: _t->setCurrentFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 56: _t->on_actionAbout_Luminance_triggered(); break;
        case 57: _t->updateActions((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 58: _t->setActiveMainWindow((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 59: _t->cropToSelection(); break;
        case 60: _t->enableCrop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 61: _t->disableCrop(); break;
        case 62: _t->on_actionDonate_triggered(); break;
        case 63: _t->onUpdateAvailable(); break;
        case 64: _t->on_actionUpdateAvailable_triggered(); break;
        case 65: _t->tonemapBegin(); break;
        case 66: _t->tonemapEnd(); break;
        case 67: _t->tonemapImage((*reinterpret_cast< TonemappingOptions*(*)>(_a[1]))); break;
        case 68: _t->exportImage((*reinterpret_cast< TonemappingOptions*(*)>(_a[1]))); break;
        case 69: _t->addLdrFrame((*reinterpret_cast< pfs::Frame*(*)>(_a[1])),(*reinterpret_cast< TonemappingOptions*(*)>(_a[2]))); break;
        case 70: _t->tonemapFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 71: _t->exportBegin(); break;
        case 72: _t->exportEnd(); break;
        case 73: _t->on_actionLock_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 74: _t->syncViewers((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 75: _t->showPreviewPanel((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 76: _t->removeTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 77: _t->on_actionRemove_Tab_triggered(); break;
        case 78: _t->on_actionSoft_Proofing_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 79: _t->on_actionGamut_Check_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 80: _t->updateSoftProofing((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 81: _t->on_actionFits_Importer_triggered(); break;
        case 82: _t->on_actionShow_Full_Screen_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 83: _t->on_actionShow_Image_Full_Screen_triggered(); break;
        case 84: _t->on_actionSelect_Interpolation_Method_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 85: _t->updatePreviews((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< float(*)>(_a[2]))); break;
        case 86: _t->reparentViewer((*reinterpret_cast< GenericViewer*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 87: _t->reparentViewer((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 88: _t->showNextViewer((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 89: _t->showPreviousViewer((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 90: _t->setSyncViewers((*reinterpret_cast< GenericViewer*(*)>(_a[1]))); break;
        case 91: _t->on_actionOnline_Documentation_triggered(); break;
        case 92: _t->on_actionDevelopers_Website_triggered(); break;
        case 93: _t->on_actionLuminance_HDR_Website_triggered(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(pfs::Frame * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::updatedHDR)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 94)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 94;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 94)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 94;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::updatedHDR(pfs::Frame * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
