/****************************************************************************
** Meta object code from reading C++ file 'Gang.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/UI/Gang.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Gang.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Gang_t {
    QByteArrayData data[13];
    char stringdata0[164];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Gang_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Gang_t qt_meta_stringdata_Gang = {
    {
QT_MOC_LITERAL(0, 0, 4), // "Gang"
QT_MOC_LITERAL(1, 5, 8), // "finished"
QT_MOC_LITERAL(2, 14, 0), // ""
QT_MOC_LITERAL(3, 15, 10), // "enableUndo"
QT_MOC_LITERAL(4, 26, 10), // "enableRedo"
QT_MOC_LITERAL(5, 37, 11), // "sliderMoved"
QT_MOC_LITERAL(6, 49, 1), // "p"
QT_MOC_LITERAL(7, 51, 18), // "sliderValueChanged"
QT_MOC_LITERAL(8, 70, 19), // "spinboxValueChanged"
QT_MOC_LITERAL(9, 90, 16), // "checkBox1Checked"
QT_MOC_LITERAL(10, 107, 16), // "checkBox2Checked"
QT_MOC_LITERAL(11, 124, 19), // "radioButton1Checked"
QT_MOC_LITERAL(12, 144, 19) // "radioButton2Checked"

    },
    "Gang\0finished\0\0enableUndo\0enableRedo\0"
    "sliderMoved\0p\0sliderValueChanged\0"
    "spinboxValueChanged\0checkBox1Checked\0"
    "checkBox2Checked\0radioButton1Checked\0"
    "radioButton2Checked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Gang[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,
       3,    1,   65,    2, 0x06 /* Public */,
       4,    1,   68,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   71,    2, 0x09 /* Protected */,
       7,    1,   74,    2, 0x09 /* Protected */,
       8,    1,   77,    2, 0x09 /* Protected */,
       9,    1,   80,    2, 0x09 /* Protected */,
      10,    1,   83,    2, 0x09 /* Protected */,
      11,    1,   86,    2, 0x09 /* Protected */,
      12,    1,   89,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void Gang::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Gang *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->enableUndo((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->enableRedo((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->sliderMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->sliderValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->spinboxValueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 6: _t->checkBox1Checked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->checkBox2Checked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->radioButton1Checked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->radioButton2Checked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Gang::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Gang::finished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Gang::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Gang::enableUndo)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Gang::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Gang::enableRedo)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Gang::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Gang.data,
    qt_meta_data_Gang,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Gang::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Gang::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Gang.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Gang::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Gang::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Gang::enableUndo(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Gang::enableRedo(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
