/****************************************************************************
** Meta object code from reading C++ file 'BatchHDRDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/BatchHDR/BatchHDRDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'BatchHDRDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BatchHDRDialog_t {
    QByteArrayData data[27];
    char stringdata0[462];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BatchHDRDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BatchHDRDialog_t qt_meta_stringdata_BatchHDRDialog = {
    {
QT_MOC_LITERAL(0, 0, 14), // "BatchHDRDialog"
QT_MOC_LITERAL(1, 15, 8), // "setValue"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 21), // "num_bracketed_changed"
QT_MOC_LITERAL(4, 47, 28), // "on_selectInputFolder_clicked"
QT_MOC_LITERAL(5, 76, 29), // "on_selectOutputFolder_clicked"
QT_MOC_LITERAL(6, 106, 20), // "add_output_directory"
QT_MOC_LITERAL(7, 127, 3), // "dir"
QT_MOC_LITERAL(8, 131, 22), // "on_startButton_clicked"
QT_MOC_LITERAL(9, 154, 9), // "batch_hdr"
QT_MOC_LITERAL(10, 164, 5), // "align"
QT_MOC_LITERAL(11, 170, 10), // "create_hdr"
QT_MOC_LITERAL(12, 181, 19), // "error_while_loading"
QT_MOC_LITERAL(13, 201, 12), // "writeAisData"
QT_MOC_LITERAL(14, 214, 11), // "QByteArray&"
QT_MOC_LITERAL(15, 226, 18), // "check_start_button"
QT_MOC_LITERAL(16, 245, 23), // "on_cancelButton_clicked"
QT_MOC_LITERAL(17, 269, 23), // "align_selection_clicked"
QT_MOC_LITERAL(18, 293, 9), // "processed"
QT_MOC_LITERAL(19, 303, 15), // "try_to_continue"
QT_MOC_LITERAL(20, 319, 21), // "updateThresholdSlider"
QT_MOC_LITERAL(21, 341, 22), // "updateThresholdSpinBox"
QT_MOC_LITERAL(22, 364, 10), // "ais_failed"
QT_MOC_LITERAL(23, 375, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(24, 398, 17), // "createHdrFinished"
QT_MOC_LITERAL(25, 416, 16), // "loadFilesAborted"
QT_MOC_LITERAL(26, 433, 28) // "on_profileComboBox_activated"

    },
    "BatchHDRDialog\0setValue\0\0num_bracketed_changed\0"
    "on_selectInputFolder_clicked\0"
    "on_selectOutputFolder_clicked\0"
    "add_output_directory\0dir\0"
    "on_startButton_clicked\0batch_hdr\0align\0"
    "create_hdr\0error_while_loading\0"
    "writeAisData\0QByteArray&\0check_start_button\0"
    "on_cancelButton_clicked\0align_selection_clicked\0"
    "processed\0try_to_continue\0"
    "updateThresholdSlider\0updateThresholdSpinBox\0"
    "ais_failed\0QProcess::ProcessError\0"
    "createHdrFinished\0loadFilesAborted\0"
    "on_profileComboBox_activated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BatchHDRDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  129,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  132,    2, 0x09 /* Protected */,
       4,    0,  135,    2, 0x09 /* Protected */,
       5,    0,  136,    2, 0x09 /* Protected */,
       6,    1,  137,    2, 0x09 /* Protected */,
       6,    0,  140,    2, 0x29 /* Protected | MethodCloned */,
       8,    0,  141,    2, 0x09 /* Protected */,
       9,    0,  142,    2, 0x09 /* Protected */,
      10,    0,  143,    2, 0x09 /* Protected */,
      11,    1,  144,    2, 0x09 /* Protected */,
      12,    1,  147,    2, 0x09 /* Protected */,
      13,    1,  150,    2, 0x09 /* Protected */,
      15,    0,  153,    2, 0x09 /* Protected */,
      16,    0,  154,    2, 0x09 /* Protected */,
      17,    0,  155,    2, 0x09 /* Protected */,
      18,    0,  156,    2, 0x09 /* Protected */,
      19,    0,  157,    2, 0x09 /* Protected */,
      20,    1,  158,    2, 0x09 /* Protected */,
      21,    1,  161,    2, 0x09 /* Protected */,
      22,    1,  164,    2, 0x09 /* Protected */,
      24,    0,  167,    2, 0x09 /* Protected */,
      25,    0,  168,    2, 0x09 /* Protected */,
      26,    1,  169,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 14,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, 0x80000000 | 23,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void BatchHDRDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BatchHDRDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->num_bracketed_changed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->on_selectInputFolder_clicked(); break;
        case 3: _t->on_selectOutputFolder_clicked(); break;
        case 4: _t->add_output_directory((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->add_output_directory(); break;
        case 6: _t->on_startButton_clicked(); break;
        case 7: _t->batch_hdr(); break;
        case 8: _t->align(); break;
        case 9: _t->create_hdr((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->error_while_loading((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->writeAisData((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 12: _t->check_start_button(); break;
        case 13: _t->on_cancelButton_clicked(); break;
        case 14: _t->align_selection_clicked(); break;
        case 15: _t->processed(); break;
        case 16: _t->try_to_continue(); break;
        case 17: _t->updateThresholdSlider((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->updateThresholdSpinBox((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 19: _t->ais_failed((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 20: _t->createHdrFinished(); break;
        case 21: _t->loadFilesAborted(); break;
        case 22: _t->on_profileComboBox_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BatchHDRDialog::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BatchHDRDialog::setValue)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BatchHDRDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_BatchHDRDialog.data,
    qt_meta_data_BatchHDRDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BatchHDRDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BatchHDRDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BatchHDRDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int BatchHDRDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void BatchHDRDialog::setValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
