/****************************************************************************
** Meta object code from reading C++ file 'HdrWizard.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/HdrWizard/HdrWizard.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'HdrWizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HdrWizard_t {
    QByteArrayData data[47];
    char stringdata0[832];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HdrWizard_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HdrWizard_t qt_meta_stringdata_HdrWizard = {
    {
QT_MOC_LITERAL(0, 0, 9), // "HdrWizard"
QT_MOC_LITERAL(1, 10, 8), // "setValue"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 5), // "value"
QT_MOC_LITERAL(4, 26, 8), // "setRange"
QT_MOC_LITERAL(5, 35, 3), // "min"
QT_MOC_LITERAL(6, 39, 3), // "max"
QT_MOC_LITERAL(7, 43, 14), // "loadInputFiles"
QT_MOC_LITERAL(8, 58, 5), // "files"
QT_MOC_LITERAL(9, 64, 18), // "loadInputFilesDone"
QT_MOC_LITERAL(10, 83, 23), // "loadImagesButtonClicked"
QT_MOC_LITERAL(11, 107, 24), // "removeImageButtonClicked"
QT_MOC_LITERAL(12, 132, 22), // "clearListButtonClicked"
QT_MOC_LITERAL(13, 155, 20), // "inputHdrFileSelected"
QT_MOC_LITERAL(14, 176, 10), // "currentRow"
QT_MOC_LITERAL(15, 187, 14), // "updateEVSlider"
QT_MOC_LITERAL(16, 202, 8), // "newValue"
QT_MOC_LITERAL(17, 211, 15), // "updateEVSpinBox"
QT_MOC_LITERAL(18, 227, 17), // "errorWhileLoading"
QT_MOC_LITERAL(19, 245, 12), // "errormessage"
QT_MOC_LITERAL(20, 258, 16), // "finishedAligning"
QT_MOC_LITERAL(21, 275, 21), // "alignSelectionClicked"
QT_MOC_LITERAL(22, 297, 30), // "predefConfigsComboBoxActivated"
QT_MOC_LITERAL(23, 328, 27), // "customConfigCheckBoxToggled"
QT_MOC_LITERAL(24, 356, 34), // "weightingFunctionComboBoxActi..."
QT_MOC_LITERAL(25, 391, 30), // "responseCurveComboBoxActivated"
QT_MOC_LITERAL(26, 422, 22), // "modelComboBoxActivated"
QT_MOC_LITERAL(27, 445, 13), // "loadRespCurve"
QT_MOC_LITERAL(28, 459, 30), // "saveRespCurveFileButtonClicked"
QT_MOC_LITERAL(29, 490, 34), // "on_loadRespCurveFileButton_cl..."
QT_MOC_LITERAL(30, 525, 23), // "NextFinishButtonClicked"
QT_MOC_LITERAL(31, 549, 22), // "currentPageChangedInto"
QT_MOC_LITERAL(32, 572, 17), // "editingEVfinished"
QT_MOC_LITERAL(33, 590, 6), // "reject"
QT_MOC_LITERAL(34, 597, 10), // "ais_failed"
QT_MOC_LITERAL(35, 608, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(36, 631, 12), // "writeAisData"
QT_MOC_LITERAL(37, 644, 4), // "data"
QT_MOC_LITERAL(38, 649, 16), // "setupConnections"
QT_MOC_LITERAL(39, 666, 33), // "on_pushButtonSaveSettings_cli..."
QT_MOC_LITERAL(40, 700, 17), // "updateProgressBar"
QT_MOC_LITERAL(41, 718, 21), // "updateThresholdSlider"
QT_MOC_LITERAL(42, 740, 22), // "updateThresholdSpinBox"
QT_MOC_LITERAL(43, 763, 16), // "startComputation"
QT_MOC_LITERAL(44, 780, 9), // "createHdr"
QT_MOC_LITERAL(45, 790, 17), // "createHdrFinished"
QT_MOC_LITERAL(46, 808, 23) // "updateHideLogButtonText"

    },
    "HdrWizard\0setValue\0\0value\0setRange\0"
    "min\0max\0loadInputFiles\0files\0"
    "loadInputFilesDone\0loadImagesButtonClicked\0"
    "removeImageButtonClicked\0"
    "clearListButtonClicked\0inputHdrFileSelected\0"
    "currentRow\0updateEVSlider\0newValue\0"
    "updateEVSpinBox\0errorWhileLoading\0"
    "errormessage\0finishedAligning\0"
    "alignSelectionClicked\0"
    "predefConfigsComboBoxActivated\0"
    "customConfigCheckBoxToggled\0"
    "weightingFunctionComboBoxActivated\0"
    "responseCurveComboBoxActivated\0"
    "modelComboBoxActivated\0loadRespCurve\0"
    "saveRespCurveFileButtonClicked\0"
    "on_loadRespCurveFileButton_clicked\0"
    "NextFinishButtonClicked\0currentPageChangedInto\0"
    "editingEVfinished\0reject\0ais_failed\0"
    "QProcess::ProcessError\0writeAisData\0"
    "data\0setupConnections\0"
    "on_pushButtonSaveSettings_clicked\0"
    "updateProgressBar\0updateThresholdSlider\0"
    "updateThresholdSpinBox\0startComputation\0"
    "createHdr\0createHdrFinished\0"
    "updateHideLogButtonText"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HdrWizard[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  194,    2, 0x06 /* Public */,
       4,    2,  197,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  202,    2, 0x08 /* Private */,
       9,    0,  205,    2, 0x08 /* Private */,
      10,    0,  206,    2, 0x08 /* Private */,
      11,    0,  207,    2, 0x08 /* Private */,
      12,    0,  208,    2, 0x08 /* Private */,
      13,    1,  209,    2, 0x08 /* Private */,
      15,    1,  212,    2, 0x08 /* Private */,
      17,    1,  215,    2, 0x08 /* Private */,
      18,    1,  218,    2, 0x08 /* Private */,
      20,    1,  221,    2, 0x08 /* Private */,
      21,    0,  224,    2, 0x08 /* Private */,
      22,    1,  225,    2, 0x08 /* Private */,
      23,    1,  228,    2, 0x08 /* Private */,
      24,    1,  231,    2, 0x08 /* Private */,
      25,    1,  234,    2, 0x08 /* Private */,
      26,    1,  237,    2, 0x08 /* Private */,
      27,    1,  240,    2, 0x08 /* Private */,
      28,    0,  243,    2, 0x08 /* Private */,
      29,    0,  244,    2, 0x08 /* Private */,
      30,    0,  245,    2, 0x08 /* Private */,
      31,    1,  246,    2, 0x08 /* Private */,
      32,    0,  249,    2, 0x08 /* Private */,
      33,    0,  250,    2, 0x08 /* Private */,
      34,    1,  251,    2, 0x08 /* Private */,
      36,    1,  254,    2, 0x08 /* Private */,
      38,    0,  257,    2, 0x08 /* Private */,
      39,    0,  258,    2, 0x08 /* Private */,
      40,    1,  259,    2, 0x08 /* Private */,
      41,    1,  262,    2, 0x08 /* Private */,
      42,    1,  265,    2, 0x08 /* Private */,
      43,    0,  268,    2, 0x08 /* Private */,
      44,    0,  269,    2, 0x08 /* Private */,
      45,    0,  270,    2, 0x08 /* Private */,
      46,    1,  271,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    5,    6,

 // slots: parameters
    QMetaType::Void, QMetaType::QStringList,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Double,   16,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Bool, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 35,    2,
    QMetaType::Void, QMetaType::QByteArray,   37,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void HdrWizard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HdrWizard *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setRange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->loadInputFiles((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 3: _t->loadInputFilesDone(); break;
        case 4: _t->loadImagesButtonClicked(); break;
        case 5: _t->removeImageButtonClicked(); break;
        case 6: _t->clearListButtonClicked(); break;
        case 7: _t->inputHdrFileSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->updateEVSlider((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->updateEVSpinBox((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 10: _t->errorWhileLoading((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->finishedAligning((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->alignSelectionClicked(); break;
        case 13: _t->predefConfigsComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->customConfigCheckBoxToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->weightingFunctionComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->responseCurveComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->modelComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: { bool _r = _t->loadRespCurve((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->saveRespCurveFileButtonClicked(); break;
        case 20: _t->on_loadRespCurveFileButton_clicked(); break;
        case 21: _t->NextFinishButtonClicked(); break;
        case 22: _t->currentPageChangedInto((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->editingEVfinished(); break;
        case 24: _t->reject(); break;
        case 25: _t->ais_failed((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 26: _t->writeAisData((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 27: _t->setupConnections(); break;
        case 28: _t->on_pushButtonSaveSettings_clicked(); break;
        case 29: _t->updateProgressBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 30: _t->updateThresholdSlider((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->updateThresholdSpinBox((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 32: _t->startComputation(); break;
        case 33: _t->createHdr(); break;
        case 34: _t->createHdrFinished(); break;
        case 35: _t->updateHideLogButtonText((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HdrWizard::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrWizard::setValue)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HdrWizard::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HdrWizard::setRange)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HdrWizard::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_HdrWizard.data,
    qt_meta_data_HdrWizard,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HdrWizard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HdrWizard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HdrWizard.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int HdrWizard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 36;
    }
    return _id;
}

// SIGNAL 0
void HdrWizard::setValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HdrWizard::setRange(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
