/********************************************************************************
** Form generated from reading UI file 'EditingTools.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITINGTOOLS_H
#define UI_EDITINGTOOLS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_EditingToolsDialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_10;
    QVBoxLayout *verticalLayout;
    QGroupBox *toolsGroupBox;
    QGridLayout *gridLayout_8;
    QGroupBox *listsGroupBox;
    QHBoxLayout *_4;
    QVBoxLayout *_5;
    QLabel *label_editable_list;
    QListWidget *movableListWidget;
    QVBoxLayout *_6;
    QSpacerItem *verticalSpacer;
    QToolButton *prevBothButton;
    QToolButton *nextBothButton;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *_7;
    QLabel *label_reference_list;
    QListWidget *referenceListWidget;
    QHBoxLayout *_8;
    QToolButton *cropButton;
    QToolButton *antighostToolButton;
    QToolButton *saveImagesButton;
    QGroupBox *visualizationGroupBox;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *_2;
    QToolButton *fitButton;
    QToolButton *fillButton;
    QToolButton *origSizeButton;
    QHBoxLayout *_3;
    QSpacerItem *horizontalSpacer_7;
    QLabel *label_7;
    QComboBox *blendModeCB;
    QSpacerItem *horizontalSpacer_10;
    QToolButton *maskColorButton;
    QPushButton *lassoColorButton;
    QSpacerItem *horizontalSpacer_8;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_12;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QGridLayout *gridLayout_7;
    QFrame *frame_3;
    QGridLayout *gridLayout_10;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_6;
    QCheckBox *autoAG_checkBox;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_8;
    QSpacerItem *horizontalSpacer;
    QLabel *label_2;
    QLineEdit *totalPatches_lineEdit;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_5;
    QSlider *threshold_horizontalSlider;
    QDoubleSpinBox *threshold_doubleSpinBox;
    QPushButton *recomputePatches_pushButton;
    QWidget *page_2;
    QGridLayout *gridLayout_9;
    QVBoxLayout *verticalLayout_7;
    QFrame *frame_2;
    QGridLayout *gridLayout_4;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_6;
    QFrame *toolOptionsFrame;
    QGridLayout *gridLayout_6;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *_9;
    QSpacerItem *horizontalSpacer_5;
    QRadioButton *addMaskRadioButton;
    QRadioButton *removeMaskRadioButton;
    QSpacerItem *horizontalSpacer_6;
    QGridLayout *_10;
    QSlider *agBrushSizeQSlider;
    QSpinBox *agBrushStrengthQSpinbox;
    QLabel *labelBrushSize;
    QSlider *agBrushStrengthQSlider;
    QSpinBox *agBrushSizeQSpinbox;
    QLabel *labelBrushStrength;
    QFrame *frame;
    QGridLayout *gridLayout_3;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_4;
    QToolButton *toolButtonPath;
    QToolButton *toolButtonPaint;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout;
    QToolButton *toolButtonSaveMask;
    QToolButton *toolButtonApplyMask;
    QFrame *line;
    QVBoxLayout *verticalLayout_2;
    QFrame *previewImageFrame;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *correctionsGroupBox;
    QHBoxLayout *_11;
    QVBoxLayout *_12;
    QPushButton *resetButton;
    QPushButton *resetAllButton;
    QSpacerItem *horizontalSpacer_9;
    QGridLayout *_13;
    QLabel *label_3;
    QLabel *label_4;
    QSpinBox *horizShiftSB;
    QSpinBox *vertShiftSB;
    QGridLayout *_14;
    QToolButton *upToolButton;
    QToolButton *downToolButton;
    QToolButton *rightToolButton;
    QToolButton *leftToolButton;
    QGroupBox *groupBoxHistogram;
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout_11;
    QHBoxLayout *horizontalLayout_3;
    QToolButton *whatsThisButton;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *Next_Finishbutton;
    QPushButton *cancelbutton;

    void setupUi(QDialog *EditingToolsDialog)
    {
        if (EditingToolsDialog->objectName().isEmpty())
            EditingToolsDialog->setObjectName(QString::fromUtf8("EditingToolsDialog"));
        EditingToolsDialog->resize(1004, 737);
        EditingToolsDialog->setMinimumSize(QSize(0, 0));
        EditingToolsDialog->setMaximumSize(QSize(16777215, 737));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        EditingToolsDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(EditingToolsDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        toolsGroupBox = new QGroupBox(EditingToolsDialog);
        toolsGroupBox->setObjectName(QString::fromUtf8("toolsGroupBox"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(toolsGroupBox->sizePolicy().hasHeightForWidth());
        toolsGroupBox->setSizePolicy(sizePolicy);
        toolsGroupBox->setMaximumSize(QSize(400, 16777215));
        gridLayout_8 = new QGridLayout(toolsGroupBox);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        listsGroupBox = new QGroupBox(toolsGroupBox);
        listsGroupBox->setObjectName(QString::fromUtf8("listsGroupBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(listsGroupBox->sizePolicy().hasHeightForWidth());
        listsGroupBox->setSizePolicy(sizePolicy1);
        listsGroupBox->setMaximumSize(QSize(400, 16777215));
        listsGroupBox->setAlignment(Qt::AlignCenter);
        _4 = new QHBoxLayout(listsGroupBox);
        _4->setSpacing(1);
        _4->setObjectName(QString::fromUtf8("_4"));
        _4->setContentsMargins(3, 3, 3, 3);
        _5 = new QVBoxLayout();
        _5->setSpacing(6);
        _5->setObjectName(QString::fromUtf8("_5"));
        _5->setContentsMargins(0, 0, 0, 0);
        label_editable_list = new QLabel(listsGroupBox);
        label_editable_list->setObjectName(QString::fromUtf8("label_editable_list"));
        label_editable_list->setAlignment(Qt::AlignCenter);

        _5->addWidget(label_editable_list);

        movableListWidget = new QListWidget(listsGroupBox);
        movableListWidget->setObjectName(QString::fromUtf8("movableListWidget"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(movableListWidget->sizePolicy().hasHeightForWidth());
        movableListWidget->setSizePolicy(sizePolicy2);
        movableListWidget->setMaximumSize(QSize(130, 16777215));

        _5->addWidget(movableListWidget);


        _4->addLayout(_5);

        _6 = new QVBoxLayout();
        _6->setSpacing(6);
        _6->setObjectName(QString::fromUtf8("_6"));
        _6->setContentsMargins(0, 0, 0, 0);
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        _6->addItem(verticalSpacer);

        prevBothButton = new QToolButton(listsGroupBox);
        prevBothButton->setObjectName(QString::fromUtf8("prevBothButton"));
        prevBothButton->setMinimumSize(QSize(44, 0));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("arrow-up");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        prevBothButton->setIcon(icon1);

        _6->addWidget(prevBothButton);

        nextBothButton = new QToolButton(listsGroupBox);
        nextBothButton->setObjectName(QString::fromUtf8("nextBothButton"));
        nextBothButton->setMinimumSize(QSize(44, 0));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("arrow-down");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        nextBothButton->setIcon(icon2);

        _6->addWidget(nextBothButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        _6->addItem(verticalSpacer_2);


        _4->addLayout(_6);

        _7 = new QVBoxLayout();
        _7->setSpacing(6);
        _7->setObjectName(QString::fromUtf8("_7"));
        _7->setContentsMargins(0, 0, 0, 0);
        label_reference_list = new QLabel(listsGroupBox);
        label_reference_list->setObjectName(QString::fromUtf8("label_reference_list"));
        label_reference_list->setAlignment(Qt::AlignCenter);

        _7->addWidget(label_reference_list);

        referenceListWidget = new QListWidget(listsGroupBox);
        referenceListWidget->setObjectName(QString::fromUtf8("referenceListWidget"));
        sizePolicy2.setHeightForWidth(referenceListWidget->sizePolicy().hasHeightForWidth());
        referenceListWidget->setSizePolicy(sizePolicy2);
        referenceListWidget->setMaximumSize(QSize(130, 16777215));

        _7->addWidget(referenceListWidget);


        _4->addLayout(_7);


        gridLayout_8->addWidget(listsGroupBox, 1, 0, 1, 1);

        _8 = new QHBoxLayout();
        _8->setSpacing(6);
        _8->setObjectName(QString::fromUtf8("_8"));
        _8->setContentsMargins(0, 0, 0, 0);
        cropButton = new QToolButton(toolsGroupBox);
        cropButton->setObjectName(QString::fromUtf8("cropButton"));
        cropButton->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("transform-crop-and-resize");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        cropButton->setIcon(icon3);
        cropButton->setIconSize(QSize(32, 32));

        _8->addWidget(cropButton);

        antighostToolButton = new QToolButton(toolsGroupBox);
        antighostToolButton->setObjectName(QString::fromUtf8("antighostToolButton"));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/program-icons/ghost"), QSize(), QIcon::Normal, QIcon::Off);
        antighostToolButton->setIcon(icon4);
        antighostToolButton->setIconSize(QSize(32, 32));
        antighostToolButton->setCheckable(true);

        _8->addWidget(antighostToolButton);

        saveImagesButton = new QToolButton(toolsGroupBox);
        saveImagesButton->setObjectName(QString::fromUtf8("saveImagesButton"));
        QIcon icon5;
        iconThemeName = QString::fromUtf8("document-save");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        saveImagesButton->setIcon(icon5);
        saveImagesButton->setIconSize(QSize(32, 32));

        _8->addWidget(saveImagesButton);


        gridLayout_8->addLayout(_8, 3, 0, 1, 1);

        visualizationGroupBox = new QGroupBox(toolsGroupBox);
        visualizationGroupBox->setObjectName(QString::fromUtf8("visualizationGroupBox"));
        sizePolicy.setHeightForWidth(visualizationGroupBox->sizePolicy().hasHeightForWidth());
        visualizationGroupBox->setSizePolicy(sizePolicy);
        visualizationGroupBox->setMaximumSize(QSize(400, 120));
        visualizationGroupBox->setAlignment(Qt::AlignCenter);
        vboxLayout = new QVBoxLayout(visualizationGroupBox);
        vboxLayout->setSpacing(3);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(6, 6, 6, 6);
        _2 = new QHBoxLayout();
        _2->setSpacing(6);
        _2->setObjectName(QString::fromUtf8("_2"));
        _2->setContentsMargins(0, 0, 0, 0);
        fitButton = new QToolButton(visualizationGroupBox);
        fitButton->setObjectName(QString::fromUtf8("fitButton"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(fitButton->sizePolicy().hasHeightForWidth());
        fitButton->setSizePolicy(sizePolicy3);
        fitButton->setMinimumSize(QSize(70, 0));
        fitButton->setMaximumSize(QSize(16777215, 16777215));
        QFont font;
        font.setPointSize(8);
        fitButton->setFont(font);
        QIcon icon6;
        iconThemeName = QString::fromUtf8("zoom-fit-best");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fitButton->setIcon(icon6);
        fitButton->setIconSize(QSize(22, 22));
        fitButton->setCheckable(true);
        fitButton->setChecked(true);
        fitButton->setAutoExclusive(true);

        _2->addWidget(fitButton);

        fillButton = new QToolButton(visualizationGroupBox);
        fillButton->setObjectName(QString::fromUtf8("fillButton"));
        sizePolicy3.setHeightForWidth(fillButton->sizePolicy().hasHeightForWidth());
        fillButton->setSizePolicy(sizePolicy3);
        fillButton->setMinimumSize(QSize(70, 0));
        fillButton->setMaximumSize(QSize(16777215, 16777215));
        fillButton->setFont(font);
        QIcon icon7;
        iconThemeName = QString::fromUtf8("zoom");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fillButton->setIcon(icon7);
        fillButton->setIconSize(QSize(22, 22));
        fillButton->setCheckable(true);
        fillButton->setAutoExclusive(true);

        _2->addWidget(fillButton);

        origSizeButton = new QToolButton(visualizationGroupBox);
        origSizeButton->setObjectName(QString::fromUtf8("origSizeButton"));
        sizePolicy3.setHeightForWidth(origSizeButton->sizePolicy().hasHeightForWidth());
        origSizeButton->setSizePolicy(sizePolicy3);
        origSizeButton->setMinimumSize(QSize(70, 0));
        origSizeButton->setMaximumSize(QSize(16777215, 16777215));
        origSizeButton->setFont(font);
        QIcon icon8;
        iconThemeName = QString::fromUtf8("zoom-original");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon8 = QIcon::fromTheme(iconThemeName);
        } else {
            icon8.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        origSizeButton->setIcon(icon8);
        origSizeButton->setIconSize(QSize(22, 22));
        origSizeButton->setCheckable(true);
        origSizeButton->setAutoExclusive(true);

        _2->addWidget(origSizeButton);


        vboxLayout->addLayout(_2);

        _3 = new QHBoxLayout();
        _3->setSpacing(6);
        _3->setObjectName(QString::fromUtf8("_3"));
        _3->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _3->addItem(horizontalSpacer_7);

        label_7 = new QLabel(visualizationGroupBox);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        _3->addWidget(label_7);

        blendModeCB = new QComboBox(visualizationGroupBox);
        blendModeCB->addItem(QString());
        blendModeCB->addItem(QString());
        blendModeCB->addItem(QString());
        blendModeCB->addItem(QString());
        blendModeCB->addItem(QString());
        blendModeCB->setObjectName(QString::fromUtf8("blendModeCB"));

        _3->addWidget(blendModeCB);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _3->addItem(horizontalSpacer_10);

        maskColorButton = new QToolButton(visualizationGroupBox);
        maskColorButton->setObjectName(QString::fromUtf8("maskColorButton"));
        maskColorButton->setAutoFillBackground(true);
        QIcon icon9;
        iconThemeName = QString::fromUtf8("draw-brush");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon9 = QIcon::fromTheme(iconThemeName);
        } else {
            icon9.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        maskColorButton->setIcon(icon9);
        maskColorButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        _3->addWidget(maskColorButton);

        lassoColorButton = new QPushButton(visualizationGroupBox);
        lassoColorButton->setObjectName(QString::fromUtf8("lassoColorButton"));
        QIcon icon10;
        iconThemeName = QString::fromUtf8("edit-select-lasso");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon10 = QIcon::fromTheme(iconThemeName);
        } else {
            icon10.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        lassoColorButton->setIcon(icon10);
        lassoColorButton->setAutoDefault(false);

        _3->addWidget(lassoColorButton);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _3->addItem(horizontalSpacer_8);


        vboxLayout->addLayout(_3);


        gridLayout_8->addWidget(visualizationGroupBox, 0, 0, 1, 1);

        groupBox = new QGroupBox(toolsGroupBox);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        groupBox->setMaximumSize(QSize(400, 16777215));
        groupBox->setAlignment(Qt::AlignCenter);
        gridLayout_12 = new QGridLayout(groupBox);
        gridLayout_12->setObjectName(QString::fromUtf8("gridLayout_12"));
        stackedWidget = new QStackedWidget(groupBox);
        stackedWidget->setObjectName(QString::fromUtf8("stackedWidget"));
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        gridLayout_7 = new QGridLayout(page);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        frame_3 = new QFrame(page);
        frame_3->setObjectName(QString::fromUtf8("frame_3"));
        frame_3->setFrameShape(QFrame::StyledPanel);
        frame_3->setFrameShadow(QFrame::Raised);
        gridLayout_10 = new QGridLayout(frame_3);
        gridLayout_10->setObjectName(QString::fromUtf8("gridLayout_10"));
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        label_6 = new QLabel(frame_3);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        horizontalLayout_8->addWidget(label_6);

        autoAG_checkBox = new QCheckBox(frame_3);
        autoAG_checkBox->setObjectName(QString::fromUtf8("autoAG_checkBox"));

        horizontalLayout_8->addWidget(autoAG_checkBox);


        verticalLayout_3->addLayout(horizontalLayout_8);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        label_8 = new QLabel(frame_3);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        horizontalLayout_9->addWidget(label_8);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer);

        label_2 = new QLabel(frame_3);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setEnabled(false);

        horizontalLayout_9->addWidget(label_2);

        totalPatches_lineEdit = new QLineEdit(frame_3);
        totalPatches_lineEdit->setObjectName(QString::fromUtf8("totalPatches_lineEdit"));
        totalPatches_lineEdit->setEnabled(false);
        QSizePolicy sizePolicy4(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(totalPatches_lineEdit->sizePolicy().hasHeightForWidth());
        totalPatches_lineEdit->setSizePolicy(sizePolicy4);
        totalPatches_lineEdit->setMinimumSize(QSize(60, 0));
        totalPatches_lineEdit->setMaximumSize(QSize(60, 16777215));
        totalPatches_lineEdit->setReadOnly(true);

        horizontalLayout_9->addWidget(totalPatches_lineEdit);


        verticalLayout_3->addLayout(horizontalLayout_9);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        label_5 = new QLabel(frame_3);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setEnabled(false);

        horizontalLayout_7->addWidget(label_5);

        threshold_horizontalSlider = new QSlider(frame_3);
        threshold_horizontalSlider->setObjectName(QString::fromUtf8("threshold_horizontalSlider"));
        threshold_horizontalSlider->setEnabled(false);
        threshold_horizontalSlider->setMaximum(10000);
        threshold_horizontalSlider->setSingleStep(1);
        threshold_horizontalSlider->setPageStep(10);
        threshold_horizontalSlider->setValue(1000);
        threshold_horizontalSlider->setOrientation(Qt::Horizontal);
        threshold_horizontalSlider->setTickPosition(QSlider::NoTicks);

        horizontalLayout_7->addWidget(threshold_horizontalSlider);

        threshold_doubleSpinBox = new QDoubleSpinBox(frame_3);
        threshold_doubleSpinBox->setObjectName(QString::fromUtf8("threshold_doubleSpinBox"));
        threshold_doubleSpinBox->setEnabled(false);
        threshold_doubleSpinBox->setDecimals(4);
        threshold_doubleSpinBox->setMaximum(1.000000000000000);
        threshold_doubleSpinBox->setSingleStep(0.001000000000000);
        threshold_doubleSpinBox->setValue(0.100000000000000);

        horizontalLayout_7->addWidget(threshold_doubleSpinBox);

        recomputePatches_pushButton = new QPushButton(frame_3);
        recomputePatches_pushButton->setObjectName(QString::fromUtf8("recomputePatches_pushButton"));
        recomputePatches_pushButton->setEnabled(false);
        recomputePatches_pushButton->setAutoDefault(false);

        horizontalLayout_7->addWidget(recomputePatches_pushButton);


        verticalLayout_3->addLayout(horizontalLayout_7);


        gridLayout_10->addLayout(verticalLayout_3, 0, 0, 1, 1);


        gridLayout_7->addWidget(frame_3, 0, 0, 1, 1);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        gridLayout_9 = new QGridLayout(page_2);
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        frame_2 = new QFrame(page_2);
        frame_2->setObjectName(QString::fromUtf8("frame_2"));
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        gridLayout_4 = new QGridLayout(frame_2);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        label = new QLabel(frame_2);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout_5->addWidget(label);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);


        gridLayout_4->addLayout(horizontalLayout_5, 0, 0, 1, 1);


        verticalLayout_7->addWidget(frame_2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        toolOptionsFrame = new QFrame(page_2);
        toolOptionsFrame->setObjectName(QString::fromUtf8("toolOptionsFrame"));
        toolOptionsFrame->setFrameShape(QFrame::StyledPanel);
        toolOptionsFrame->setFrameShadow(QFrame::Raised);
        gridLayout_6 = new QGridLayout(toolOptionsFrame);
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        _9 = new QHBoxLayout();
        _9->setSpacing(6);
        _9->setObjectName(QString::fromUtf8("_9"));
        _9->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _9->addItem(horizontalSpacer_5);

        addMaskRadioButton = new QRadioButton(toolOptionsFrame);
        addMaskRadioButton->setObjectName(QString::fromUtf8("addMaskRadioButton"));
        addMaskRadioButton->setChecked(true);

        _9->addWidget(addMaskRadioButton);

        removeMaskRadioButton = new QRadioButton(toolOptionsFrame);
        removeMaskRadioButton->setObjectName(QString::fromUtf8("removeMaskRadioButton"));

        _9->addWidget(removeMaskRadioButton);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _9->addItem(horizontalSpacer_6);


        verticalLayout_6->addLayout(_9);

        _10 = new QGridLayout();
        _10->setSpacing(6);
        _10->setObjectName(QString::fromUtf8("_10"));
        _10->setContentsMargins(0, 0, 0, 0);
        agBrushSizeQSlider = new QSlider(toolOptionsFrame);
        agBrushSizeQSlider->setObjectName(QString::fromUtf8("agBrushSizeQSlider"));
        agBrushSizeQSlider->setMinimum(1);
        agBrushSizeQSlider->setMaximum(64);
        agBrushSizeQSlider->setValue(32);
        agBrushSizeQSlider->setOrientation(Qt::Horizontal);
        agBrushSizeQSlider->setTickPosition(QSlider::NoTicks);

        _10->addWidget(agBrushSizeQSlider, 0, 1, 1, 1);

        agBrushStrengthQSpinbox = new QSpinBox(toolOptionsFrame);
        agBrushStrengthQSpinbox->setObjectName(QString::fromUtf8("agBrushStrengthQSpinbox"));
        agBrushStrengthQSpinbox->setMinimum(1);
        agBrushStrengthQSpinbox->setMaximum(255);
        agBrushStrengthQSpinbox->setValue(255);

        _10->addWidget(agBrushStrengthQSpinbox, 1, 2, 1, 1);

        labelBrushSize = new QLabel(toolOptionsFrame);
        labelBrushSize->setObjectName(QString::fromUtf8("labelBrushSize"));

        _10->addWidget(labelBrushSize, 0, 0, 1, 1);

        agBrushStrengthQSlider = new QSlider(toolOptionsFrame);
        agBrushStrengthQSlider->setObjectName(QString::fromUtf8("agBrushStrengthQSlider"));
        agBrushStrengthQSlider->setMinimum(1);
        agBrushStrengthQSlider->setMaximum(255);
        agBrushStrengthQSlider->setValue(255);
        agBrushStrengthQSlider->setOrientation(Qt::Horizontal);
        agBrushStrengthQSlider->setTickPosition(QSlider::NoTicks);

        _10->addWidget(agBrushStrengthQSlider, 1, 1, 1, 1);

        agBrushSizeQSpinbox = new QSpinBox(toolOptionsFrame);
        agBrushSizeQSpinbox->setObjectName(QString::fromUtf8("agBrushSizeQSpinbox"));
        agBrushSizeQSpinbox->setMinimum(1);
        agBrushSizeQSpinbox->setMaximum(64);
        agBrushSizeQSpinbox->setValue(32);

        _10->addWidget(agBrushSizeQSpinbox, 0, 2, 1, 1);

        labelBrushStrength = new QLabel(toolOptionsFrame);
        labelBrushStrength->setObjectName(QString::fromUtf8("labelBrushStrength"));

        _10->addWidget(labelBrushStrength, 1, 0, 1, 1);


        verticalLayout_6->addLayout(_10);


        gridLayout_6->addLayout(verticalLayout_6, 0, 0, 1, 1);


        horizontalLayout_6->addWidget(toolOptionsFrame);

        frame = new QFrame(page_2);
        frame->setObjectName(QString::fromUtf8("frame"));
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        gridLayout_3 = new QGridLayout(frame);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        toolButtonPath = new QToolButton(frame);
        toolButtonPath->setObjectName(QString::fromUtf8("toolButtonPath"));
        toolButtonPath->setMinimumSize(QSize(38, 38));
        toolButtonPath->setMaximumSize(QSize(38, 38));
        toolButtonPath->setIcon(icon10);
        toolButtonPath->setIconSize(QSize(32, 32));
        toolButtonPath->setCheckable(true);
        toolButtonPath->setAutoExclusive(true);

        horizontalLayout_4->addWidget(toolButtonPath);

        toolButtonPaint = new QToolButton(frame);
        toolButtonPaint->setObjectName(QString::fromUtf8("toolButtonPaint"));
        toolButtonPaint->setMinimumSize(QSize(38, 38));
        toolButtonPaint->setMaximumSize(QSize(38, 38));
        toolButtonPaint->setIcon(icon9);
        toolButtonPaint->setIconSize(QSize(32, 32));
        toolButtonPaint->setCheckable(true);
        toolButtonPaint->setChecked(true);
        toolButtonPaint->setAutoExclusive(true);

        horizontalLayout_4->addWidget(toolButtonPaint);


        verticalLayout_5->addLayout(horizontalLayout_4);

        line_2 = new QFrame(frame);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        toolButtonSaveMask = new QToolButton(frame);
        toolButtonSaveMask->setObjectName(QString::fromUtf8("toolButtonSaveMask"));
        toolButtonSaveMask->setMinimumSize(QSize(38, 38));
        toolButtonSaveMask->setMaximumSize(QSize(38, 38));
        toolButtonSaveMask->setIcon(icon5);
        toolButtonSaveMask->setIconSize(QSize(32, 32));

        horizontalLayout->addWidget(toolButtonSaveMask);

        toolButtonApplyMask = new QToolButton(frame);
        toolButtonApplyMask->setObjectName(QString::fromUtf8("toolButtonApplyMask"));
        toolButtonApplyMask->setEnabled(false);
        toolButtonApplyMask->setMinimumSize(QSize(38, 38));
        toolButtonApplyMask->setMaximumSize(QSize(38, 38));
        QIcon icon11;
        iconThemeName = QString::fromUtf8("dialog-ok-apply");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon11 = QIcon::fromTheme(iconThemeName);
        } else {
            icon11.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        toolButtonApplyMask->setIcon(icon11);
        toolButtonApplyMask->setIconSize(QSize(32, 32));

        horizontalLayout->addWidget(toolButtonApplyMask);


        verticalLayout_5->addLayout(horizontalLayout);


        gridLayout_3->addLayout(verticalLayout_5, 0, 0, 1, 1);


        horizontalLayout_6->addWidget(frame);


        verticalLayout_7->addLayout(horizontalLayout_6);


        gridLayout_9->addLayout(verticalLayout_7, 0, 0, 1, 1);

        stackedWidget->addWidget(page_2);

        gridLayout_12->addWidget(stackedWidget, 0, 0, 1, 1);


        gridLayout_8->addWidget(groupBox, 2, 0, 1, 1);


        verticalLayout->addWidget(toolsGroupBox);


        horizontalLayout_10->addLayout(verticalLayout);

        line = new QFrame(EditingToolsDialog);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        horizontalLayout_10->addWidget(line);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        previewImageFrame = new QFrame(EditingToolsDialog);
        previewImageFrame->setObjectName(QString::fromUtf8("previewImageFrame"));
        QSizePolicy sizePolicy5(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(previewImageFrame->sizePolicy().hasHeightForWidth());
        previewImageFrame->setSizePolicy(sizePolicy5);
        previewImageFrame->setMinimumSize(QSize(555, 300));
        previewImageFrame->setFocusPolicy(Qt::StrongFocus);
        previewImageFrame->setFrameShape(QFrame::NoFrame);
        previewImageFrame->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(previewImageFrame);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        correctionsGroupBox = new QGroupBox(EditingToolsDialog);
        correctionsGroupBox->setObjectName(QString::fromUtf8("correctionsGroupBox"));
        QSizePolicy sizePolicy6(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(correctionsGroupBox->sizePolicy().hasHeightForWidth());
        correctionsGroupBox->setSizePolicy(sizePolicy6);
        correctionsGroupBox->setMaximumSize(QSize(450, 150));
        correctionsGroupBox->setAlignment(Qt::AlignCenter);
        _11 = new QHBoxLayout(correctionsGroupBox);
        _11->setSpacing(9);
        _11->setObjectName(QString::fromUtf8("_11"));
        _11->setContentsMargins(5, 5, 5, 5);
        _12 = new QVBoxLayout();
        _12->setSpacing(6);
        _12->setObjectName(QString::fromUtf8("_12"));
        _12->setContentsMargins(0, 0, 0, 0);
        resetButton = new QPushButton(correctionsGroupBox);
        resetButton->setObjectName(QString::fromUtf8("resetButton"));
        resetButton->setAutoDefault(false);

        _12->addWidget(resetButton);

        resetAllButton = new QPushButton(correctionsGroupBox);
        resetAllButton->setObjectName(QString::fromUtf8("resetAllButton"));
        resetAllButton->setAutoDefault(false);

        _12->addWidget(resetAllButton);


        _11->addLayout(_12);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _11->addItem(horizontalSpacer_9);

        _13 = new QGridLayout();
        _13->setSpacing(6);
        _13->setObjectName(QString::fromUtf8("_13"));
        _13->setContentsMargins(0, 0, 0, 0);
        label_3 = new QLabel(correctionsGroupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        _13->addWidget(label_3, 0, 0, 1, 1);

        label_4 = new QLabel(correctionsGroupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setLayoutDirection(Qt::LeftToRight);
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        _13->addWidget(label_4, 1, 0, 1, 1);

        horizShiftSB = new QSpinBox(correctionsGroupBox);
        horizShiftSB->setObjectName(QString::fromUtf8("horizShiftSB"));
        horizShiftSB->setMinimum(-9000);
        horizShiftSB->setMaximum(9000);

        _13->addWidget(horizShiftSB, 0, 1, 1, 1);

        vertShiftSB = new QSpinBox(correctionsGroupBox);
        vertShiftSB->setObjectName(QString::fromUtf8("vertShiftSB"));
        vertShiftSB->setMinimum(-9000);
        vertShiftSB->setMaximum(9000);

        _13->addWidget(vertShiftSB, 1, 1, 1, 1);


        _11->addLayout(_13);

        _14 = new QGridLayout();
        _14->setSpacing(6);
        _14->setObjectName(QString::fromUtf8("_14"));
        _14->setContentsMargins(0, 0, 0, 0);
        upToolButton = new QToolButton(correctionsGroupBox);
        upToolButton->setObjectName(QString::fromUtf8("upToolButton"));
        upToolButton->setAutoRepeat(true);
        upToolButton->setArrowType(Qt::UpArrow);

        _14->addWidget(upToolButton, 0, 1, 1, 1);

        downToolButton = new QToolButton(correctionsGroupBox);
        downToolButton->setObjectName(QString::fromUtf8("downToolButton"));
        downToolButton->setAutoRepeat(true);
        downToolButton->setArrowType(Qt::DownArrow);

        _14->addWidget(downToolButton, 2, 1, 1, 1);

        rightToolButton = new QToolButton(correctionsGroupBox);
        rightToolButton->setObjectName(QString::fromUtf8("rightToolButton"));
        rightToolButton->setAutoRepeat(true);
        rightToolButton->setArrowType(Qt::RightArrow);

        _14->addWidget(rightToolButton, 1, 2, 1, 1);

        leftToolButton = new QToolButton(correctionsGroupBox);
        leftToolButton->setObjectName(QString::fromUtf8("leftToolButton"));
        leftToolButton->setAutoRepeat(true);
        leftToolButton->setArrowType(Qt::LeftArrow);

        _14->addWidget(leftToolButton, 1, 0, 1, 1);


        _11->addLayout(_14);


        horizontalLayout_2->addWidget(correctionsGroupBox);

        groupBoxHistogram = new QGroupBox(EditingToolsDialog);
        groupBoxHistogram->setObjectName(QString::fromUtf8("groupBoxHistogram"));
        groupBoxHistogram->setMaximumSize(QSize(16777215, 150));
        groupBoxHistogram->setAlignment(Qt::AlignCenter);
        gridLayout_2 = new QGridLayout(groupBoxHistogram);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_11 = new QGridLayout();
        gridLayout_11->setObjectName(QString::fromUtf8("gridLayout_11"));

        gridLayout_2->addLayout(gridLayout_11, 0, 0, 1, 1);


        horizontalLayout_2->addWidget(groupBoxHistogram);


        verticalLayout_2->addLayout(horizontalLayout_2);


        horizontalLayout_10->addLayout(verticalLayout_2);


        gridLayout->addLayout(horizontalLayout_10, 0, 0, 1, 1);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        whatsThisButton = new QToolButton(EditingToolsDialog);
        whatsThisButton->setObjectName(QString::fromUtf8("whatsThisButton"));
        QIcon icon12;
        iconThemeName = QString::fromUtf8("help-whatsthis");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon12 = QIcon::fromTheme(iconThemeName);
        } else {
            icon12.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        whatsThisButton->setIcon(icon12);
        whatsThisButton->setIconSize(QSize(22, 22));

        horizontalLayout_3->addWidget(whatsThisButton);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_4);

        Next_Finishbutton = new QPushButton(EditingToolsDialog);
        Next_Finishbutton->setObjectName(QString::fromUtf8("Next_Finishbutton"));
        Next_Finishbutton->setAutoDefault(false);

        horizontalLayout_3->addWidget(Next_Finishbutton);

        cancelbutton = new QPushButton(EditingToolsDialog);
        cancelbutton->setObjectName(QString::fromUtf8("cancelbutton"));
        cancelbutton->setAutoDefault(false);

        horizontalLayout_3->addWidget(cancelbutton);


        gridLayout->addLayout(horizontalLayout_3, 3, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_editable_list->setBuddy(movableListWidget);
        label_reference_list->setBuddy(referenceListWidget);
        label_7->setBuddy(blendModeCB);
        labelBrushStrength->setBuddy(agBrushStrengthQSlider);
        label_3->setBuddy(horizShiftSB);
        label_4->setBuddy(vertShiftSB);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(movableListWidget, referenceListWidget);
        QWidget::setTabOrder(referenceListWidget, prevBothButton);
        QWidget::setTabOrder(prevBothButton, nextBothButton);
        QWidget::setTabOrder(nextBothButton, fitButton);
        QWidget::setTabOrder(fitButton, fillButton);
        QWidget::setTabOrder(fillButton, origSizeButton);
        QWidget::setTabOrder(origSizeButton, blendModeCB);
        QWidget::setTabOrder(blendModeCB, maskColorButton);
        QWidget::setTabOrder(maskColorButton, lassoColorButton);
        QWidget::setTabOrder(lassoColorButton, autoAG_checkBox);
        QWidget::setTabOrder(autoAG_checkBox, totalPatches_lineEdit);
        QWidget::setTabOrder(totalPatches_lineEdit, threshold_horizontalSlider);
        QWidget::setTabOrder(threshold_horizontalSlider, threshold_doubleSpinBox);
        QWidget::setTabOrder(threshold_doubleSpinBox, recomputePatches_pushButton);
        QWidget::setTabOrder(recomputePatches_pushButton, cropButton);
        QWidget::setTabOrder(cropButton, antighostToolButton);
        QWidget::setTabOrder(antighostToolButton, saveImagesButton);
        QWidget::setTabOrder(saveImagesButton, resetButton);
        QWidget::setTabOrder(resetButton, resetAllButton);
        QWidget::setTabOrder(resetAllButton, horizShiftSB);
        QWidget::setTabOrder(horizShiftSB, vertShiftSB);
        QWidget::setTabOrder(vertShiftSB, upToolButton);
        QWidget::setTabOrder(upToolButton, leftToolButton);
        QWidget::setTabOrder(leftToolButton, rightToolButton);
        QWidget::setTabOrder(rightToolButton, downToolButton);
        QWidget::setTabOrder(downToolButton, Next_Finishbutton);
        QWidget::setTabOrder(Next_Finishbutton, cancelbutton);
        QWidget::setTabOrder(cancelbutton, whatsThisButton);
        QWidget::setTabOrder(whatsThisButton, addMaskRadioButton);
        QWidget::setTabOrder(addMaskRadioButton, removeMaskRadioButton);
        QWidget::setTabOrder(removeMaskRadioButton, agBrushSizeQSlider);
        QWidget::setTabOrder(agBrushSizeQSlider, agBrushSizeQSpinbox);
        QWidget::setTabOrder(agBrushSizeQSpinbox, agBrushStrengthQSlider);
        QWidget::setTabOrder(agBrushStrengthQSlider, agBrushStrengthQSpinbox);
        QWidget::setTabOrder(agBrushStrengthQSpinbox, toolButtonPath);
        QWidget::setTabOrder(toolButtonPath, toolButtonPaint);
        QWidget::setTabOrder(toolButtonPaint, toolButtonSaveMask);
        QWidget::setTabOrder(toolButtonSaveMask, toolButtonApplyMask);
        QWidget::setTabOrder(toolButtonApplyMask, previewImageFrame);

        retranslateUi(EditingToolsDialog);
        QObject::connect(agBrushSizeQSlider, SIGNAL(valueChanged(int)), agBrushSizeQSpinbox, SLOT(setValue(int)));
        QObject::connect(agBrushStrengthQSlider, SIGNAL(valueChanged(int)), agBrushStrengthQSpinbox, SLOT(setValue(int)));
        QObject::connect(agBrushSizeQSpinbox, SIGNAL(valueChanged(int)), agBrushSizeQSlider, SLOT(setValue(int)));
        QObject::connect(agBrushStrengthQSpinbox, SIGNAL(valueChanged(int)), agBrushStrengthQSlider, SLOT(setValue(int)));
        QObject::connect(cancelbutton, SIGNAL(clicked()), EditingToolsDialog, SLOT(reject()));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), label_2, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), totalPatches_lineEdit, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), label_5, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), threshold_horizontalSlider, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), threshold_doubleSpinBox, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(clicked(bool)), recomputePatches_pushButton, SLOT(setEnabled(bool)));

        stackedWidget->setCurrentIndex(0);
        Next_Finishbutton->setDefault(false);


        QMetaObject::connectSlotsByName(EditingToolsDialog);
    } // setupUi

    void retranslateUi(QDialog *EditingToolsDialog)
    {
        EditingToolsDialog->setWindowTitle(QCoreApplication::translate("EditingToolsDialog", "Editing Tools", nullptr));
        listsGroupBox->setTitle(QCoreApplication::translate("EditingToolsDialog", "Images List", nullptr));
        label_editable_list->setText(QCoreApplication::translate("EditingToolsDialog", "Ed&itable", nullptr));
#if QT_CONFIG(tooltip)
        movableListWidget->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The currently selected image is the one that you can shift and paint an anti-ghosting mask on.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        movableListWidget->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "The image currently selected in this list can be translated up,down,left and right using the controls below.\n"
"This image's histogram is displayed in the widget above.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        prevBothButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Select the previous image in both lists", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        prevBothButton->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "Use this button to move the selection to the previous image in both lists.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        nextBothButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Select the next image in both lists", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        nextBothButton->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "Use this button to move the selection to the next image in both lists.", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_reference_list->setText(QCoreApplication::translate("EditingToolsDialog", "Referen&ce", nullptr));
#if QT_CONFIG(tooltip)
        referenceListWidget->setToolTip(QCoreApplication::translate("EditingToolsDialog", "The reference image is the one currently selected", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        referenceListWidget->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The image currently selected in this list serves as a reference position for the editable image</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        cropButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Crop the stack of images after a selection with the left mouse button has been made", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        cropButton->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "Clicking this button you will be able to crop all the images to the size that you have previously selected with the left mouse button (press and hold the left mouse button and move the pointer to select a crop area, then press this button).", nullptr));
#endif // QT_CONFIG(whatsthis)
        cropButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Crop All Images", nullptr));
#if QT_CONFIG(shortcut)
        cropButton->setShortcut(QCoreApplication::translate("EditingToolsDialog", "Ctrl+X", nullptr));
#endif // QT_CONFIG(shortcut)
        antighostToolButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Anti-Ghosting", nullptr));
        saveImagesButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Save Images", nullptr));
#if QT_CONFIG(shortcut)
        saveImagesButton->setShortcut(QCoreApplication::translate("EditingToolsDialog", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        visualizationGroupBox->setTitle(QCoreApplication::translate("EditingToolsDialog", "Visualization and Navigation", nullptr));
#if QT_CONFIG(tooltip)
        fitButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Fit to window", nullptr));
#endif // QT_CONFIG(tooltip)
        fitButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Fit to Window", nullptr));
#if QT_CONFIG(tooltip)
        fillButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Fill the window", nullptr));
#endif // QT_CONFIG(tooltip)
        fillButton->setText(QCoreApplication::translate("EditingToolsDialog", "Fill &window", nullptr));
#if QT_CONFIG(tooltip)
        origSizeButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Original size", nullptr));
#endif // QT_CONFIG(tooltip)
        origSizeButton->setText(QCoreApplication::translate("EditingToolsDialog", "Origina&l Size", nullptr));
        label_7->setText(QCoreApplication::translate("EditingToolsDialog", "&Mode:", nullptr));
        blendModeCB->setItemText(0, QCoreApplication::translate("EditingToolsDialog", "Difference (E-P)", nullptr));
        blendModeCB->setItemText(1, QCoreApplication::translate("EditingToolsDialog", "Overlay (E+P)", nullptr));
        blendModeCB->setItemText(2, QCoreApplication::translate("EditingToolsDialog", "Editable Only (E)", nullptr));
        blendModeCB->setItemText(3, QCoreApplication::translate("EditingToolsDialog", "Pivot Only (P)", nullptr));
        blendModeCB->setItemText(4, QCoreApplication::translate("EditingToolsDialog", "Antighosting Mask", nullptr));

        maskColorButton->setText(QCoreApplication::translate("EditingToolsDialog", "Brush", nullptr));
        lassoColorButton->setText(QCoreApplication::translate("EditingToolsDialog", "Lasso", nullptr));
        groupBox->setTitle(QCoreApplication::translate("EditingToolsDialog", "Anti Ghosting", nullptr));
        label_6->setText(QCoreApplication::translate("EditingToolsDialog", "O. Gallo and others", nullptr));
#if QT_CONFIG(tooltip)
        autoAG_checkBox->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Enable auto anti-ghosting algorithm</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        autoAG_checkBox->setText(QCoreApplication::translate("EditingToolsDialog", "Auto anti-ghosting", nullptr));
        label_8->setText(QCoreApplication::translate("EditingToolsDialog", "Artifact-free HDR Imaging", nullptr));
        label_2->setText(QCoreApplication::translate("EditingToolsDialog", "Total Patches:", nullptr));
#if QT_CONFIG(tooltip)
        totalPatches_lineEdit->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Shows the amount of patches in percentage computed by the auto anti-ghosting algorithm</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        label_5->setText(QCoreApplication::translate("EditingToolsDialog", "Threshold", nullptr));
#if QT_CONFIG(tooltip)
        threshold_horizontalSlider->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range. Use the &quot;Recompute&quot; button to see the actual patches coverage and adjust the threshold accordingly.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        threshold_doubleSpinBox->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range. Use the &quot;Recompute&quot; button to see the actual patches coverage and adjust the threshold accordingly</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        recomputePatches_pushButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Click here to recompute patches based on selected threshold</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        recomputePatches_pushButton->setText(QCoreApplication::translate("EditingToolsDialog", "Recompute", nullptr));
        label->setText(QCoreApplication::translate("EditingToolsDialog", "Mask", nullptr));
#if QT_CONFIG(tooltip)
        addMaskRadioButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Add a mask using lasso or brush</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        addMaskRadioButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Add", nullptr));
#if QT_CONFIG(tooltip)
        removeMaskRadioButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Remove mask using lasso or brush</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        removeMaskRadioButton->setText(QCoreApplication::translate("EditingToolsDialog", "R&emove", nullptr));
#if QT_CONFIG(tooltip)
        agBrushSizeQSlider->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Size of the brush</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        agBrushStrengthQSpinbox->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Strength of the mask (semi transparent)</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        labelBrushSize->setText(QCoreApplication::translate("EditingToolsDialog", "Size:", nullptr));
#if QT_CONFIG(tooltip)
        agBrushStrengthQSlider->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Strength of the mask (semi transparent)</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        agBrushSizeQSpinbox->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head/><body><p>Size of the brush</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        agBrushSizeQSpinbox->setSuffix(QCoreApplication::translate("EditingToolsDialog", " px", nullptr));
        labelBrushStrength->setText(QCoreApplication::translate("EditingToolsDialog", "S&trength:", nullptr));
#if QT_CONFIG(tooltip)
        toolButtonPath->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Lasso - Draw a free hand selection area", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        toolButtonPaint->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Brush - Paint the mask using the brush", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        toolButtonSaveMask->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Save mask", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        toolButtonApplyMask->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Apply saved mask", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        previewImageFrame->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This area shows the preview (using the preview mode on the left) between the editable and reference image.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">If the same image is selected in both the editable and reference list, this area will show the image itself.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Clicking and holding the left mouse button you will be able to select a crop area.</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        correctionsGroupBox->setToolTip(QCoreApplication::translate("EditingToolsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Shift values for the currently selected editable image</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        correctionsGroupBox->setWhatsThis(QCoreApplication::translate("EditingToolsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Here the user can view and set the shift values for the currently selected editable image</p></body></html>", nullptr));
#endif // QT_CONFIG(whatsthis)
        correctionsGroupBox->setTitle(QCoreApplication::translate("EditingToolsDialog", "Shift values for editable image", nullptr));
#if QT_CONFIG(tooltip)
        resetButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Reset the shift values for the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        resetButton->setText(QCoreApplication::translate("EditingToolsDialog", "&Reset", nullptr));
#if QT_CONFIG(tooltip)
        resetAllButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Reset the shift values for all the images", nullptr));
#endif // QT_CONFIG(tooltip)
        resetAllButton->setText(QCoreApplication::translate("EditingToolsDialog", "Reset &All", nullptr));
        label_3->setText(QCoreApplication::translate("EditingToolsDialog", "Hori&zontal:", nullptr));
        label_4->setText(QCoreApplication::translate("EditingToolsDialog", "Vertica&l:", nullptr));
#if QT_CONFIG(tooltip)
        horizShiftSB->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Insert an horizontal shift value for the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        horizShiftSB->setSuffix(QCoreApplication::translate("EditingToolsDialog", " px", nullptr));
#if QT_CONFIG(tooltip)
        vertShiftSB->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Insert a vertical shift value for the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        vertShiftSB->setSuffix(QCoreApplication::translate("EditingToolsDialog", " px", nullptr));
#if QT_CONFIG(tooltip)
        upToolButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Move up of 1 pixel the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        upToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        downToolButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Move down of 1 pixel the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        rightToolButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Move right of 1 pixel the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        rightToolButton->setText(QCoreApplication::translate("EditingToolsDialog", ">", nullptr));
#if QT_CONFIG(tooltip)
        leftToolButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Move left of 1 pixel the currently selected movable image", nullptr));
#endif // QT_CONFIG(tooltip)
        leftToolButton->setText(QCoreApplication::translate("EditingToolsDialog", "<", nullptr));
        groupBoxHistogram->setTitle(QCoreApplication::translate("EditingToolsDialog", "Histogram", nullptr));
#if QT_CONFIG(tooltip)
        whatsThisButton->setToolTip(QCoreApplication::translate("EditingToolsDialog", "Get information about the graphical elements", nullptr));
#endif // QT_CONFIG(tooltip)
        whatsThisButton->setText(QCoreApplication::translate("EditingToolsDialog", "&What's this?", nullptr));
#if QT_CONFIG(shortcut)
        whatsThisButton->setShortcut(QCoreApplication::translate("EditingToolsDialog", "Shift+F1", nullptr));
#endif // QT_CONFIG(shortcut)
        Next_Finishbutton->setText(QCoreApplication::translate("EditingToolsDialog", "&Next >", nullptr));
        cancelbutton->setText(QCoreApplication::translate("EditingToolsDialog", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class EditingToolsDialog: public Ui_EditingToolsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EDITINGTOOLS_H
