/****************************************************************************
** Meta object code from reading C++ file 'EditingTools.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/HdrWizard/EditingTools.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'EditingTools.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_EditingTools_t {
    QByteArrayData data[47];
    char stringdata0[712];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EditingTools_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EditingTools_t qt_meta_stringdata_EditingTools = {
    {
QT_MOC_LITERAL(0, 0, 12), // "EditingTools"
QT_MOC_LITERAL(1, 13, 23), // "saveImagesButtonClicked"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 11), // "updatePivot"
QT_MOC_LITERAL(4, 50, 13), // "updateMovable"
QT_MOC_LITERAL(5, 64, 9), // "upClicked"
QT_MOC_LITERAL(6, 74, 12), // "rightClicked"
QT_MOC_LITERAL(7, 87, 11), // "downClicked"
QT_MOC_LITERAL(8, 99, 11), // "leftClicked"
QT_MOC_LITERAL(9, 111, 17), // "horizShiftChanged"
QT_MOC_LITERAL(10, 129, 16), // "vertShiftChanged"
QT_MOC_LITERAL(11, 146, 12), // "resetCurrent"
QT_MOC_LITERAL(12, 159, 8), // "resetAll"
QT_MOC_LITERAL(13, 168, 8), // "prevLeft"
QT_MOC_LITERAL(14, 177, 8), // "nextLeft"
QT_MOC_LITERAL(15, 186, 8), // "prevBoth"
QT_MOC_LITERAL(16, 195, 8), // "nextBoth"
QT_MOC_LITERAL(17, 204, 9), // "prevRight"
QT_MOC_LITERAL(18, 214, 9), // "nextRight"
QT_MOC_LITERAL(19, 224, 14), // "enterWhatsThis"
QT_MOC_LITERAL(20, 239, 6), // "zoomIn"
QT_MOC_LITERAL(21, 246, 7), // "zoomOut"
QT_MOC_LITERAL(22, 254, 10), // "fitPreview"
QT_MOC_LITERAL(23, 265, 11), // "fillPreview"
QT_MOC_LITERAL(24, 277, 8), // "origSize"
QT_MOC_LITERAL(25, 286, 9), // "cropStack"
QT_MOC_LITERAL(26, 296, 11), // "nextClicked"
QT_MOC_LITERAL(27, 308, 22), // "maskColorButtonClicked"
QT_MOC_LITERAL(28, 331, 23), // "lassoColorButtonClicked"
QT_MOC_LITERAL(29, 355, 26), // "antighostToolButtonToggled"
QT_MOC_LITERAL(30, 382, 23), // "blendModeCBIndexChanged"
QT_MOC_LITERAL(31, 406, 16), // "setupConnections"
QT_MOC_LITERAL(32, 423, 28), // "restoreSaveImagesButtonState"
QT_MOC_LITERAL(33, 452, 12), // "addGoodImage"
QT_MOC_LITERAL(34, 465, 15), // "removeGoodImage"
QT_MOC_LITERAL(35, 481, 12), // "updateAgMask"
QT_MOC_LITERAL(36, 494, 31), // "antighostToolButtonPaintToggled"
QT_MOC_LITERAL(37, 526, 10), // "saveAgMask"
QT_MOC_LITERAL(38, 537, 16), // "applySavedAgMask"
QT_MOC_LITERAL(39, 554, 38), // "on_recomputePatches_pushButto..."
QT_MOC_LITERAL(40, 593, 26), // "on_autoAG_checkBox_toggled"
QT_MOC_LITERAL(41, 620, 7), // "toggled"
QT_MOC_LITERAL(42, 628, 21), // "updateThresholdSlider"
QT_MOC_LITERAL(43, 650, 8), // "newValue"
QT_MOC_LITERAL(44, 659, 22), // "updateThresholdSpinBox"
QT_MOC_LITERAL(45, 682, 12), // "newThreshold"
QT_MOC_LITERAL(46, 695, 16) // "setPatchesEdited"

    },
    "EditingTools\0saveImagesButtonClicked\0"
    "\0updatePivot\0updateMovable\0upClicked\0"
    "rightClicked\0downClicked\0leftClicked\0"
    "horizShiftChanged\0vertShiftChanged\0"
    "resetCurrent\0resetAll\0prevLeft\0nextLeft\0"
    "prevBoth\0nextBoth\0prevRight\0nextRight\0"
    "enterWhatsThis\0zoomIn\0zoomOut\0fitPreview\0"
    "fillPreview\0origSize\0cropStack\0"
    "nextClicked\0maskColorButtonClicked\0"
    "lassoColorButtonClicked\0"
    "antighostToolButtonToggled\0"
    "blendModeCBIndexChanged\0setupConnections\0"
    "restoreSaveImagesButtonState\0addGoodImage\0"
    "removeGoodImage\0updateAgMask\0"
    "antighostToolButtonPaintToggled\0"
    "saveAgMask\0applySavedAgMask\0"
    "on_recomputePatches_pushButton_clicked\0"
    "on_autoAG_checkBox_toggled\0toggled\0"
    "updateThresholdSlider\0newValue\0"
    "updateThresholdSpinBox\0newThreshold\0"
    "setPatchesEdited"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EditingTools[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  224,    2, 0x08 /* Private */,
       3,    1,  225,    2, 0x08 /* Private */,
       4,    1,  228,    2, 0x08 /* Private */,
       5,    0,  231,    2, 0x08 /* Private */,
       6,    0,  232,    2, 0x08 /* Private */,
       7,    0,  233,    2, 0x08 /* Private */,
       8,    0,  234,    2, 0x08 /* Private */,
       9,    1,  235,    2, 0x08 /* Private */,
      10,    1,  238,    2, 0x08 /* Private */,
      11,    0,  241,    2, 0x08 /* Private */,
      12,    0,  242,    2, 0x08 /* Private */,
      13,    0,  243,    2, 0x08 /* Private */,
      14,    0,  244,    2, 0x08 /* Private */,
      15,    0,  245,    2, 0x08 /* Private */,
      16,    0,  246,    2, 0x08 /* Private */,
      17,    0,  247,    2, 0x08 /* Private */,
      18,    0,  248,    2, 0x08 /* Private */,
      19,    0,  249,    2, 0x08 /* Private */,
      20,    0,  250,    2, 0x08 /* Private */,
      21,    0,  251,    2, 0x08 /* Private */,
      22,    0,  252,    2, 0x08 /* Private */,
      23,    0,  253,    2, 0x08 /* Private */,
      24,    0,  254,    2, 0x08 /* Private */,
      25,    0,  255,    2, 0x08 /* Private */,
      26,    0,  256,    2, 0x08 /* Private */,
      27,    0,  257,    2, 0x08 /* Private */,
      28,    0,  258,    2, 0x08 /* Private */,
      29,    1,  259,    2, 0x08 /* Private */,
      30,    1,  262,    2, 0x08 /* Private */,
      31,    0,  265,    2, 0x08 /* Private */,
      32,    0,  266,    2, 0x08 /* Private */,
      33,    0,  267,    2, 0x08 /* Private */,
      34,    0,  268,    2, 0x08 /* Private */,
      35,    1,  269,    2, 0x08 /* Private */,
      36,    1,  272,    2, 0x08 /* Private */,
      37,    0,  275,    2, 0x08 /* Private */,
      38,    0,  276,    2, 0x08 /* Private */,
      39,    0,  277,    2, 0x08 /* Private */,
      40,    1,  278,    2, 0x08 /* Private */,
      42,    1,  281,    2, 0x08 /* Private */,
      44,    1,  284,    2, 0x08 /* Private */,
      46,    0,  287,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   41,
    QMetaType::Void, QMetaType::Int,   43,
    QMetaType::Void, QMetaType::Double,   45,
    QMetaType::Void,

       0        // eod
};

void EditingTools::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EditingTools *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->saveImagesButtonClicked(); break;
        case 1: _t->updatePivot((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->updateMovable((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->upClicked(); break;
        case 4: _t->rightClicked(); break;
        case 5: _t->downClicked(); break;
        case 6: _t->leftClicked(); break;
        case 7: _t->horizShiftChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->vertShiftChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->resetCurrent(); break;
        case 10: _t->resetAll(); break;
        case 11: _t->prevLeft(); break;
        case 12: _t->nextLeft(); break;
        case 13: _t->prevBoth(); break;
        case 14: _t->nextBoth(); break;
        case 15: _t->prevRight(); break;
        case 16: _t->nextRight(); break;
        case 17: _t->enterWhatsThis(); break;
        case 18: _t->zoomIn(); break;
        case 19: _t->zoomOut(); break;
        case 20: _t->fitPreview(); break;
        case 21: _t->fillPreview(); break;
        case 22: _t->origSize(); break;
        case 23: _t->cropStack(); break;
        case 24: _t->nextClicked(); break;
        case 25: _t->maskColorButtonClicked(); break;
        case 26: _t->lassoColorButtonClicked(); break;
        case 27: _t->antighostToolButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 28: _t->blendModeCBIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->setupConnections(); break;
        case 30: _t->restoreSaveImagesButtonState(); break;
        case 31: _t->addGoodImage(); break;
        case 32: _t->removeGoodImage(); break;
        case 33: _t->updateAgMask((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->antighostToolButtonPaintToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 35: _t->saveAgMask(); break;
        case 36: _t->applySavedAgMask(); break;
        case 37: _t->on_recomputePatches_pushButton_clicked(); break;
        case 38: _t->on_autoAG_checkBox_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->updateThresholdSlider((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: _t->updateThresholdSpinBox((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 41: _t->setPatchesEdited(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject EditingTools::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_EditingTools.data,
    qt_meta_data_EditingTools,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EditingTools::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EditingTools::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EditingTools.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int EditingTools::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 42;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
