/********************************************************************************
** Form generated from reading UI file 'SavedParametersDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SAVEDPARAMETERSDIALOG_H
#define UI_SAVEDPARAMETERSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SavedParametersDialog
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QTableView *tableView;
    QHBoxLayout *horizontalLayout;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SavedParametersDialog)
    {
        if (SavedParametersDialog->objectName().isEmpty())
            SavedParametersDialog->setObjectName(QString::fromUtf8("SavedParametersDialog"));
        SavedParametersDialog->setWindowModality(Qt::ApplicationModal);
        SavedParametersDialog->resize(835, 300);
        gridLayout = new QGridLayout(SavedParametersDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        tableView = new QTableView(SavedParametersDialog);
        tableView->setObjectName(QString::fromUtf8("tableView"));
        tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
        tableView->horizontalHeader()->setStretchLastSection(true);

        verticalLayout->addWidget(tableView);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        buttonBox = new QDialogButtonBox(SavedParametersDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(SavedParametersDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), SavedParametersDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), SavedParametersDialog, SLOT(reject()));
        QObject::connect(tableView, SIGNAL(doubleClicked(QModelIndex)), SavedParametersDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(SavedParametersDialog);
    } // setupUi

    void retranslateUi(QDialog *SavedParametersDialog)
    {
        SavedParametersDialog->setWindowTitle(QCoreApplication::translate("SavedParametersDialog", "Saved Parameters", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SavedParametersDialog: public Ui_SavedParametersDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SAVEDPARAMETERSDIALOG_H
