/********************************************************************************
** Form generated from reading UI file 'ResizeDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESIZEDIALOG_H
#define UI_RESIZEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_ResizeDialog
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QToolButton *restoredefault;
    QComboBox *px_or_percentage;
    QDoubleSpinBox *widthSpinBox;
    QLabel *sizepreview;
    QDoubleSpinBox *heightSpinBox;
    QLabel *label_2;
    QLabel *label;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *cancelButton;
    QPushButton *scaleButton;

    void setupUi(QDialog *ResizeDialog)
    {
        if (ResizeDialog->objectName().isEmpty())
            ResizeDialog->setObjectName(QString::fromUtf8("ResizeDialog"));
        ResizeDialog->resize(400, 170);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ResizeDialog->sizePolicy().hasHeightForWidth());
        ResizeDialog->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        ResizeDialog->setWindowIcon(icon);
        ResizeDialog->setModal(true);
        gridLayout_3 = new QGridLayout(ResizeDialog);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        groupBox = new QGroupBox(ResizeDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        restoredefault = new QToolButton(groupBox);
        restoredefault->setObjectName(QString::fromUtf8("restoredefault"));
        restoredefault->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(restoredefault->sizePolicy().hasHeightForWidth());
        restoredefault->setSizePolicy(sizePolicy1);
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("edit-undo");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        restoredefault->setIcon(icon1);
        restoredefault->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(restoredefault, 1, 3, 1, 1);

        px_or_percentage = new QComboBox(groupBox);
        px_or_percentage->addItem(QString());
        px_or_percentage->addItem(QString());
        px_or_percentage->setObjectName(QString::fromUtf8("px_or_percentage"));
        sizePolicy1.setHeightForWidth(px_or_percentage->sizePolicy().hasHeightForWidth());
        px_or_percentage->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(px_or_percentage, 0, 3, 1, 1);

        widthSpinBox = new QDoubleSpinBox(groupBox);
        widthSpinBox->setObjectName(QString::fromUtf8("widthSpinBox"));
        widthSpinBox->setMinimumSize(QSize(85, 0));
        widthSpinBox->setMaximumSize(QSize(85, 16777215));

        gridLayout->addWidget(widthSpinBox, 0, 1, 1, 1);

        sizepreview = new QLabel(groupBox);
        sizepreview->setObjectName(QString::fromUtf8("sizepreview"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(sizepreview->sizePolicy().hasHeightForWidth());
        sizepreview->setSizePolicy(sizePolicy2);
        sizepreview->setMinimumSize(QSize(100, 0));
        sizepreview->setMaximumSize(QSize(150, 16777215));
        sizepreview->setFrameShape(QFrame::StyledPanel);
        sizepreview->setFrameShadow(QFrame::Sunken);
        sizepreview->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(sizepreview, 1, 2, 1, 1);

        heightSpinBox = new QDoubleSpinBox(groupBox);
        heightSpinBox->setObjectName(QString::fromUtf8("heightSpinBox"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(heightSpinBox->sizePolicy().hasHeightForWidth());
        heightSpinBox->setSizePolicy(sizePolicy3);
        heightSpinBox->setMinimumSize(QSize(85, 0));
        heightSpinBox->setMaximumSize(QSize(85, 16777215));

        gridLayout->addWidget(heightSpinBox, 1, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        sizePolicy2.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy2);
        label_2->setMinimumSize(QSize(0, 0));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName(QString::fromUtf8("label"));
        sizePolicy2.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy2);
        label->setMinimumSize(QSize(0, 0));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label, 0, 0, 1, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);


        gridLayout_3->addWidget(groupBox, 0, 0, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(117, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        cancelButton = new QPushButton(ResizeDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        hboxLayout->addWidget(cancelButton);

        scaleButton = new QPushButton(ResizeDialog);
        scaleButton->setObjectName(QString::fromUtf8("scaleButton"));
        scaleButton->setAutoDefault(false);

        hboxLayout->addWidget(scaleButton);


        gridLayout_3->addLayout(hboxLayout, 1, 0, 1, 1);

        QWidget::setTabOrder(widthSpinBox, px_or_percentage);
        QWidget::setTabOrder(px_or_percentage, heightSpinBox);
        QWidget::setTabOrder(heightSpinBox, restoredefault);
        QWidget::setTabOrder(restoredefault, cancelButton);
        QWidget::setTabOrder(cancelButton, scaleButton);

        retranslateUi(ResizeDialog);
        QObject::connect(cancelButton, SIGNAL(clicked()), ResizeDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ResizeDialog);
    } // setupUi

    void retranslateUi(QDialog *ResizeDialog)
    {
        ResizeDialog->setWindowTitle(QCoreApplication::translate("ResizeDialog", "Scale Image", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ResizeDialog", "Hdr Image Size", nullptr));
#if QT_CONFIG(tooltip)
        restoredefault->setToolTip(QCoreApplication::translate("ResizeDialog", "Restore original size", nullptr));
#endif // QT_CONFIG(tooltip)
        restoredefault->setText(QCoreApplication::translate("ResizeDialog", "&Reset", nullptr));
        px_or_percentage->setItemText(0, QCoreApplication::translate("ResizeDialog", "Pixels", nullptr));
        px_or_percentage->setItemText(1, QCoreApplication::translate("ResizeDialog", "Percent", nullptr));

#if QT_CONFIG(tooltip)
        px_or_percentage->setToolTip(QCoreApplication::translate("ResizeDialog", "Switch between pixels or percentage", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        sizepreview->setToolTip(QCoreApplication::translate("ResizeDialog", "Result size", nullptr));
#endif // QT_CONFIG(tooltip)
        sizepreview->setText(QString());
        label_2->setText(QCoreApplication::translate("ResizeDialog", "Height:", nullptr));
        label->setText(QCoreApplication::translate("ResizeDialog", "Width:", nullptr));
        cancelButton->setText(QCoreApplication::translate("ResizeDialog", "&Cancel", nullptr));
        scaleButton->setText(QCoreApplication::translate("ResizeDialog", "&Scale", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ResizeDialog: public Ui_ResizeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RESIZEDIALOG_H
