/****************************************************************************
** Meta object code from reading C++ file 'FitsImporter.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/UI/FitsImporter.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'FitsImporter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FitsImporter_t {
    QByteArrayData data[28];
    char stringdata0[578];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FitsImporter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FitsImporter_t qt_meta_stringdata_FitsImporter = {
    {
QT_MOC_LITERAL(0, 0, 12), // "FitsImporter"
QT_MOC_LITERAL(1, 13, 8), // "setValue"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 8), // "setRange"
QT_MOC_LITERAL(4, 32, 31), // "on_pushButtonLuminosity_clicked"
QT_MOC_LITERAL(5, 64, 24), // "on_pushButtonRed_clicked"
QT_MOC_LITERAL(6, 89, 26), // "on_pushButtonGreen_clicked"
QT_MOC_LITERAL(7, 116, 25), // "on_pushButtonBlue_clicked"
QT_MOC_LITERAL(8, 142, 22), // "on_pushButtonH_clicked"
QT_MOC_LITERAL(9, 165, 25), // "on_pushButtonLoad_clicked"
QT_MOC_LITERAL(10, 191, 30), // "on_pushButtonClockwise_clicked"
QT_MOC_LITERAL(11, 222, 28), // "on_pushButtonPreview_clicked"
QT_MOC_LITERAL(12, 251, 13), // "loadFilesDone"
QT_MOC_LITERAL(13, 265, 12), // "ais_finished"
QT_MOC_LITERAL(14, 278, 15), // "ais_failed_slot"
QT_MOC_LITERAL(15, 294, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(16, 317, 8), // "readData"
QT_MOC_LITERAL(17, 326, 20), // "previewLabelSelected"
QT_MOC_LITERAL(18, 347, 5), // "index"
QT_MOC_LITERAL(19, 353, 24), // "on_hsRedRed_valueChanged"
QT_MOC_LITERAL(20, 378, 8), // "newValue"
QT_MOC_LITERAL(21, 387, 25), // "on_dsbRedRed_valueChanged"
QT_MOC_LITERAL(22, 413, 28), // "on_hsGreenGreen_valueChanged"
QT_MOC_LITERAL(23, 442, 29), // "on_dsbGreenGreen_valueChanged"
QT_MOC_LITERAL(24, 472, 26), // "on_hsBlueBlue_valueChanged"
QT_MOC_LITERAL(25, 499, 27), // "on_dsbBlueBlue_valueChanged"
QT_MOC_LITERAL(26, 527, 23), // "on_vsGamma_valueChanged"
QT_MOC_LITERAL(27, 551, 26) // "on_pushButtonReset_clicked"

    },
    "FitsImporter\0setValue\0\0setRange\0"
    "on_pushButtonLuminosity_clicked\0"
    "on_pushButtonRed_clicked\0"
    "on_pushButtonGreen_clicked\0"
    "on_pushButtonBlue_clicked\0"
    "on_pushButtonH_clicked\0on_pushButtonLoad_clicked\0"
    "on_pushButtonClockwise_clicked\0"
    "on_pushButtonPreview_clicked\0loadFilesDone\0"
    "ais_finished\0ais_failed_slot\0"
    "QProcess::ProcessError\0readData\0"
    "previewLabelSelected\0index\0"
    "on_hsRedRed_valueChanged\0newValue\0"
    "on_dsbRedRed_valueChanged\0"
    "on_hsGreenGreen_valueChanged\0"
    "on_dsbGreenGreen_valueChanged\0"
    "on_hsBlueBlue_valueChanged\0"
    "on_dsbBlueBlue_valueChanged\0"
    "on_vsGamma_valueChanged\0"
    "on_pushButtonReset_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FitsImporter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  129,    2, 0x06 /* Public */,
       3,    2,  132,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  137,    2, 0x09 /* Protected */,
       5,    0,  138,    2, 0x09 /* Protected */,
       6,    0,  139,    2, 0x09 /* Protected */,
       7,    0,  140,    2, 0x09 /* Protected */,
       8,    0,  141,    2, 0x09 /* Protected */,
       9,    0,  142,    2, 0x09 /* Protected */,
      10,    0,  143,    2, 0x09 /* Protected */,
      11,    0,  144,    2, 0x09 /* Protected */,
      12,    1,  145,    2, 0x09 /* Protected */,
      13,    1,  148,    2, 0x09 /* Protected */,
      14,    1,  151,    2, 0x09 /* Protected */,
      16,    1,  154,    2, 0x09 /* Protected */,
      17,    1,  157,    2, 0x09 /* Protected */,
      19,    1,  160,    2, 0x09 /* Protected */,
      21,    1,  163,    2, 0x09 /* Protected */,
      22,    1,  166,    2, 0x09 /* Protected */,
      23,    1,  169,    2, 0x09 /* Protected */,
      24,    1,  172,    2, 0x09 /* Protected */,
      25,    1,  175,    2, 0x09 /* Protected */,
      26,    1,  178,    2, 0x09 /* Protected */,
      27,    0,  181,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, QMetaType::QByteArray,    2,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Double,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Double,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Double,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,

       0        // eod
};

void FitsImporter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FitsImporter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setRange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->on_pushButtonLuminosity_clicked(); break;
        case 3: _t->on_pushButtonRed_clicked(); break;
        case 4: _t->on_pushButtonGreen_clicked(); break;
        case 5: _t->on_pushButtonBlue_clicked(); break;
        case 6: _t->on_pushButtonH_clicked(); break;
        case 7: _t->on_pushButtonLoad_clicked(); break;
        case 8: _t->on_pushButtonClockwise_clicked(); break;
        case 9: _t->on_pushButtonPreview_clicked(); break;
        case 10: _t->loadFilesDone((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 11: _t->ais_finished((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->ais_failed_slot((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 13: _t->readData((*reinterpret_cast< QByteArray(*)>(_a[1]))); break;
        case 14: _t->previewLabelSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->on_hsRedRed_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_dsbRedRed_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 17: _t->on_hsGreenGreen_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->on_dsbGreenGreen_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 19: _t->on_hsBlueBlue_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->on_dsbBlueBlue_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 21: _t->on_vsGamma_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->on_pushButtonReset_clicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FitsImporter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FitsImporter::setValue)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FitsImporter::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FitsImporter::setRange)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FitsImporter::staticMetaObject = { {
    QMetaObject::SuperData::link<QWizard::staticMetaObject>(),
    qt_meta_stringdata_FitsImporter.data,
    qt_meta_data_FitsImporter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FitsImporter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FitsImporter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FitsImporter.stringdata0))
        return static_cast<void*>(this);
    return QWizard::qt_metacast(_clname);
}

int FitsImporter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWizard::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void FitsImporter::setValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FitsImporter::setRange(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
