/********************************************************************************
** Form generated from reading UI file 'HdrWizard.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HDRWIZARD_H
#define UI_HDRWIZARD_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_HdrWizard
{
public:
    QGridLayout *gridLayout_5;
    QVBoxLayout *verticalLayout_9;
    QStackedWidget *pagestack;
    QWidget *choose_files;
    QGridLayout *gridLayout_12;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QTableWidget *tableWidget;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QToolButton *loadImagesButton;
    QToolButton *removeImageButton;
    QToolButton *clearListButton;
    QSpacerItem *verticalSpacer_4;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_3;
    QFrame *previewFrame;
    QGridLayout *gridLayout;
    QLabel *previewLabel;
    QGroupBox *EVgroupBox;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_9;
    QSlider *EVSlider;
    QDoubleSpinBox *ImageEVdsb;
    QHBoxLayout *horizontalLayout_4;
    QGroupBox *alignGroupBox;
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout_1;
    QRadioButton *mtb_radioButton;
    QCheckBox *alignCheckBox;
    QRadioButton *ais_radioButton;
    QCheckBox *autoCropCheckBox;
    QSpacerItem *horizontalSpacer_8;
    QGroupBox *agGroupBox;
    QGridLayout *gridLayout_8;
    QGridLayout *gridLayout_13;
    QLabel *label_7;
    QSlider *threshold_horizontalSlider;
    QDoubleSpinBox *threshold_doubleSpinBox;
    QCheckBox *autoAG_checkBox;
    QSpacerItem *horizontalSpacer_9;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_16;
    QSpacerItem *horizontalSpacer_15;
    QCheckBox *checkBoxEditingTools;
    QLabel *confirmloadlabel;
    QWidget *chooseHDRconfig;
    QGridLayout *gridLayout_4;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout_14;
    QLabel *responseCurveInputFileLabel;
    QLabel *responseCurveOutputFileLabel;
    QComboBox *weightFunctionComboBox;
    QLabel *weightFunctionLabel;
    QPushButton *pushButtonSaveSettings;
    QLabel *modelLabel;
    QLabel *profileLabel;
    QLabel *responseCurveLabel;
    QCheckBox *customConfigCheckBox;
    QComboBox *profileComboBox;
    QComboBox *responseCurveComboBox;
    QHBoxLayout *horizontalLayout_6;
    QLineEdit *RespCurveFileLoadedLineEdit;
    QPushButton *loadRespCurveFileButton;
    QHBoxLayout *horizontalLayout_11;
    QLineEdit *CurveFileNameSaveLineEdit;
    QPushButton *saveRespCurveFileButton;
    QComboBox *modelComboBox;
    QTextEdit *textEdit;
    QHBoxLayout *horizontalLayout_5;
    QProgressBar *progressBar;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *HideLogButton;
    QPushButton *NextFinishButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *HdrWizard)
    {
        if (HdrWizard->objectName().isEmpty())
            HdrWizard->setObjectName(QString::fromUtf8("HdrWizard"));
        HdrWizard->resize(782, 683);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        HdrWizard->setWindowIcon(icon);
        gridLayout_5 = new QGridLayout(HdrWizard);
        gridLayout_5->setSpacing(6);
        gridLayout_5->setContentsMargins(6, 6, 6, 6);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setSpacing(6);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        verticalLayout_9->setContentsMargins(6, 6, 6, 6);
        pagestack = new QStackedWidget(HdrWizard);
        pagestack->setObjectName(QString::fromUtf8("pagestack"));
        pagestack->setAutoFillBackground(true);
        choose_files = new QWidget();
        choose_files->setObjectName(QString::fromUtf8("choose_files"));
        gridLayout_12 = new QGridLayout(choose_files);
        gridLayout_12->setSpacing(6);
        gridLayout_12->setContentsMargins(6, 6, 6, 6);
        gridLayout_12->setObjectName(QString::fromUtf8("gridLayout_12"));
        groupBox = new QGroupBox(choose_files);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(6);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setMaximumSize(QSize(222, 16777215));
        label_2->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(label_2);

        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        tableWidget = new QTableWidget(groupBox);
        if (tableWidget->columnCount() < 2)
            tableWidget->setColumnCount(2);
        tableWidget->setObjectName(QString::fromUtf8("tableWidget"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableWidget->sizePolicy().hasHeightForWidth());
        tableWidget->setSizePolicy(sizePolicy);
        tableWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableWidget->setAlternatingRowColors(true);
        tableWidget->setSelectionMode(QAbstractItemView::SingleSelection);
        tableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
        tableWidget->setColumnCount(2);
        tableWidget->horizontalHeader()->setStretchLastSection(true);

        verticalLayout->addWidget(tableWidget);


        horizontalLayout_3->addLayout(verticalLayout);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        loadImagesButton = new QToolButton(groupBox);
        loadImagesButton->setObjectName(QString::fromUtf8("loadImagesButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(loadImagesButton->sizePolicy().hasHeightForWidth());
        loadImagesButton->setSizePolicy(sizePolicy1);
        loadImagesButton->setMinimumSize(QSize(42, 42));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        loadImagesButton->setIcon(icon1);

        verticalLayout_4->addWidget(loadImagesButton);

        removeImageButton = new QToolButton(groupBox);
        removeImageButton->setObjectName(QString::fromUtf8("removeImageButton"));
        removeImageButton->setEnabled(false);
        removeImageButton->setMinimumSize(QSize(42, 42));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        removeImageButton->setIcon(icon2);

        verticalLayout_4->addWidget(removeImageButton);

        clearListButton = new QToolButton(groupBox);
        clearListButton->setObjectName(QString::fromUtf8("clearListButton"));
        clearListButton->setEnabled(false);
        clearListButton->setMinimumSize(QSize(42, 42));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("edit-clear-list");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        clearListButton->setIcon(icon3);

        verticalLayout_4->addWidget(clearListButton);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_4);


        horizontalLayout_3->addLayout(verticalLayout_4);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label_3 = new QLabel(groupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        verticalLayout_2->addWidget(label_3);

        previewFrame = new QFrame(groupBox);
        previewFrame->setObjectName(QString::fromUtf8("previewFrame"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(previewFrame->sizePolicy().hasHeightForWidth());
        previewFrame->setSizePolicy(sizePolicy2);
        previewFrame->setFrameShape(QFrame::StyledPanel);
        previewFrame->setFrameShadow(QFrame::Sunken);
        gridLayout = new QGridLayout(previewFrame);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(6, 6, 6, 6);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        previewLabel = new QLabel(previewFrame);
        previewLabel->setObjectName(QString::fromUtf8("previewLabel"));
        QSizePolicy sizePolicy3(QSizePolicy::Ignored, QSizePolicy::Ignored);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(previewLabel->sizePolicy().hasHeightForWidth());
        previewLabel->setSizePolicy(sizePolicy3);
        previewLabel->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(previewLabel, 0, 0, 1, 1);


        verticalLayout_2->addWidget(previewFrame);

        EVgroupBox = new QGroupBox(groupBox);
        EVgroupBox->setObjectName(QString::fromUtf8("EVgroupBox"));
        EVgroupBox->setEnabled(false);
        horizontalLayout_2 = new QHBoxLayout(EVgroupBox);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setContentsMargins(6, 6, 6, 6);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_9 = new QLabel(EVgroupBox);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        QSizePolicy sizePolicy4(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(label_9->sizePolicy().hasHeightForWidth());
        label_9->setSizePolicy(sizePolicy4);

        horizontalLayout_2->addWidget(label_9);

        EVSlider = new QSlider(EVgroupBox);
        EVSlider->setObjectName(QString::fromUtf8("EVSlider"));
        QSizePolicy sizePolicy5(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(EVSlider->sizePolicy().hasHeightForWidth());
        EVSlider->setSizePolicy(sizePolicy5);
        EVSlider->setMinimum(-3000);
        EVSlider->setMaximum(3000);
        EVSlider->setPageStep(100);
        EVSlider->setValue(0);
        EVSlider->setSliderPosition(0);
        EVSlider->setOrientation(Qt::Horizontal);
        EVSlider->setTickPosition(QSlider::TicksAbove);
        EVSlider->setTickInterval(100);

        horizontalLayout_2->addWidget(EVSlider);

        ImageEVdsb = new QDoubleSpinBox(EVgroupBox);
        ImageEVdsb->setObjectName(QString::fromUtf8("ImageEVdsb"));
        QSizePolicy sizePolicy6(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(ImageEVdsb->sizePolicy().hasHeightForWidth());
        ImageEVdsb->setSizePolicy(sizePolicy6);
        ImageEVdsb->setDecimals(2);
        ImageEVdsb->setMinimum(-30.000000000000000);
        ImageEVdsb->setMaximum(30.000000000000000);
        ImageEVdsb->setSingleStep(0.100000000000000);

        horizontalLayout_2->addWidget(ImageEVdsb);


        verticalLayout_2->addWidget(EVgroupBox);


        horizontalLayout_3->addLayout(verticalLayout_2);


        verticalLayout_7->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(6);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        alignGroupBox = new QGroupBox(groupBox);
        alignGroupBox->setObjectName(QString::fromUtf8("alignGroupBox"));
        alignGroupBox->setEnabled(false);
        gridLayout_3 = new QGridLayout(alignGroupBox);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(6, 6, 6, 6);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(6, 6, -1, -1);
        gridLayout_1 = new QGridLayout();
        gridLayout_1->setSpacing(6);
        gridLayout_1->setObjectName(QString::fromUtf8("gridLayout_1"));
        mtb_radioButton = new QRadioButton(alignGroupBox);
        mtb_radioButton->setObjectName(QString::fromUtf8("mtb_radioButton"));
        mtb_radioButton->setEnabled(false);

        gridLayout_1->addWidget(mtb_radioButton, 0, 3, 1, 1);

        alignCheckBox = new QCheckBox(alignGroupBox);
        alignCheckBox->setObjectName(QString::fromUtf8("alignCheckBox"));
        alignCheckBox->setEnabled(false);
        sizePolicy6.setHeightForWidth(alignCheckBox->sizePolicy().hasHeightForWidth());
        alignCheckBox->setSizePolicy(sizePolicy6);

        gridLayout_1->addWidget(alignCheckBox, 0, 0, 1, 2);

        ais_radioButton = new QRadioButton(alignGroupBox);
        ais_radioButton->setObjectName(QString::fromUtf8("ais_radioButton"));
        ais_radioButton->setEnabled(false);
        ais_radioButton->setChecked(true);

        gridLayout_1->addWidget(ais_radioButton, 0, 2, 1, 1);

        autoCropCheckBox = new QCheckBox(alignGroupBox);
        autoCropCheckBox->setObjectName(QString::fromUtf8("autoCropCheckBox"));
        autoCropCheckBox->setEnabled(false);
        sizePolicy6.setHeightForWidth(autoCropCheckBox->sizePolicy().hasHeightForWidth());
        autoCropCheckBox->setSizePolicy(sizePolicy6);

        gridLayout_1->addWidget(autoCropCheckBox, 2, 1, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_1->addItem(horizontalSpacer_8, 2, 0, 1, 1);


        gridLayout_3->addLayout(gridLayout_1, 0, 0, 1, 1);


        horizontalLayout_4->addWidget(alignGroupBox);

        agGroupBox = new QGroupBox(groupBox);
        agGroupBox->setObjectName(QString::fromUtf8("agGroupBox"));
        agGroupBox->setEnabled(false);
        gridLayout_8 = new QGridLayout(agGroupBox);
        gridLayout_8->setSpacing(6);
        gridLayout_8->setContentsMargins(6, 6, 6, 6);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        gridLayout_13 = new QGridLayout();
        gridLayout_13->setSpacing(6);
        gridLayout_13->setObjectName(QString::fromUtf8("gridLayout_13"));
        label_7 = new QLabel(agGroupBox);
        label_7->setObjectName(QString::fromUtf8("label_7"));
        label_7->setEnabled(false);

        gridLayout_13->addWidget(label_7, 1, 1, 1, 1);

        threshold_horizontalSlider = new QSlider(agGroupBox);
        threshold_horizontalSlider->setObjectName(QString::fromUtf8("threshold_horizontalSlider"));
        threshold_horizontalSlider->setEnabled(false);
        threshold_horizontalSlider->setMaximum(10000);
        threshold_horizontalSlider->setSingleStep(1);
        threshold_horizontalSlider->setPageStep(10);
        threshold_horizontalSlider->setValue(1000);
        threshold_horizontalSlider->setOrientation(Qt::Horizontal);
        threshold_horizontalSlider->setTickPosition(QSlider::NoTicks);

        gridLayout_13->addWidget(threshold_horizontalSlider, 1, 2, 1, 1);

        threshold_doubleSpinBox = new QDoubleSpinBox(agGroupBox);
        threshold_doubleSpinBox->setObjectName(QString::fromUtf8("threshold_doubleSpinBox"));
        threshold_doubleSpinBox->setEnabled(false);
        threshold_doubleSpinBox->setDecimals(4);
        threshold_doubleSpinBox->setMaximum(1.000000000000000);
        threshold_doubleSpinBox->setSingleStep(0.001000000000000);
        threshold_doubleSpinBox->setValue(0.100000000000000);

        gridLayout_13->addWidget(threshold_doubleSpinBox, 1, 3, 1, 1);

        autoAG_checkBox = new QCheckBox(agGroupBox);
        autoAG_checkBox->setObjectName(QString::fromUtf8("autoAG_checkBox"));

        gridLayout_13->addWidget(autoAG_checkBox, 0, 0, 1, 3);

        horizontalSpacer_9 = new QSpacerItem(20, 1, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_13->addItem(horizontalSpacer_9, 1, 0, 1, 1);


        gridLayout_8->addLayout(gridLayout_13, 0, 0, 1, 1);


        horizontalLayout_4->addWidget(agGroupBox);


        verticalLayout_7->addLayout(horizontalLayout_4);


        gridLayout_2->addLayout(verticalLayout_7, 0, 0, 2, 2);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setSpacing(6);
        horizontalLayout_16->setObjectName(QString::fromUtf8("horizontalLayout_16"));
        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_15);

        checkBoxEditingTools = new QCheckBox(groupBox);
        checkBoxEditingTools->setObjectName(QString::fromUtf8("checkBoxEditingTools"));

        horizontalLayout_16->addWidget(checkBoxEditingTools);


        verticalLayout_3->addLayout(horizontalLayout_16);

        confirmloadlabel = new QLabel(groupBox);
        confirmloadlabel->setObjectName(QString::fromUtf8("confirmloadlabel"));
        sizePolicy5.setHeightForWidth(confirmloadlabel->sizePolicy().hasHeightForWidth());
        confirmloadlabel->setSizePolicy(sizePolicy5);
        confirmloadlabel->setMinimumSize(QSize(0, 42));
        confirmloadlabel->setMaximumSize(QSize(16777215, 42));

        verticalLayout_3->addWidget(confirmloadlabel);


        gridLayout_2->addLayout(verticalLayout_3, 2, 0, 1, 2);


        gridLayout_12->addWidget(groupBox, 0, 0, 1, 1);

        pagestack->addWidget(choose_files);
        chooseHDRconfig = new QWidget();
        chooseHDRconfig->setObjectName(QString::fromUtf8("chooseHDRconfig"));
        gridLayout_4 = new QGridLayout(chooseHDRconfig);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(6, 6, 6, 6);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        groupBox_2 = new QGroupBox(chooseHDRconfig);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        gridLayout_14 = new QGridLayout(groupBox_2);
        gridLayout_14->setSpacing(6);
        gridLayout_14->setContentsMargins(6, 6, 6, 6);
        gridLayout_14->setObjectName(QString::fromUtf8("gridLayout_14"));
        responseCurveInputFileLabel = new QLabel(groupBox_2);
        responseCurveInputFileLabel->setObjectName(QString::fromUtf8("responseCurveInputFileLabel"));
        responseCurveInputFileLabel->setEnabled(false);
        responseCurveInputFileLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_14->addWidget(responseCurveInputFileLabel, 5, 0, 1, 1);

        responseCurveOutputFileLabel = new QLabel(groupBox_2);
        responseCurveOutputFileLabel->setObjectName(QString::fromUtf8("responseCurveOutputFileLabel"));
        responseCurveOutputFileLabel->setEnabled(false);
        responseCurveOutputFileLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_14->addWidget(responseCurveOutputFileLabel, 6, 0, 1, 1);

        weightFunctionComboBox = new QComboBox(groupBox_2);
        weightFunctionComboBox->addItem(QString());
        weightFunctionComboBox->addItem(QString());
        weightFunctionComboBox->addItem(QString());
        weightFunctionComboBox->addItem(QString());
        weightFunctionComboBox->setObjectName(QString::fromUtf8("weightFunctionComboBox"));
        weightFunctionComboBox->setEnabled(false);
        weightFunctionComboBox->setMinimumSize(QSize(250, 0));

        gridLayout_14->addWidget(weightFunctionComboBox, 3, 1, 1, 2);

        weightFunctionLabel = new QLabel(groupBox_2);
        weightFunctionLabel->setObjectName(QString::fromUtf8("weightFunctionLabel"));
        weightFunctionLabel->setEnabled(false);
        weightFunctionLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_14->addWidget(weightFunctionLabel, 3, 0, 1, 1);

        pushButtonSaveSettings = new QPushButton(groupBox_2);
        pushButtonSaveSettings->setObjectName(QString::fromUtf8("pushButtonSaveSettings"));
        pushButtonSaveSettings->setEnabled(false);
        pushButtonSaveSettings->setAutoDefault(false);

        gridLayout_14->addWidget(pushButtonSaveSettings, 7, 1, 1, 2);

        modelLabel = new QLabel(groupBox_2);
        modelLabel->setObjectName(QString::fromUtf8("modelLabel"));
        modelLabel->setEnabled(false);
        modelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_14->addWidget(modelLabel, 2, 0, 1, 1);

        profileLabel = new QLabel(groupBox_2);
        profileLabel->setObjectName(QString::fromUtf8("profileLabel"));
        profileLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        profileLabel->setWordWrap(true);

        gridLayout_14->addWidget(profileLabel, 0, 0, 1, 1);

        responseCurveLabel = new QLabel(groupBox_2);
        responseCurveLabel->setObjectName(QString::fromUtf8("responseCurveLabel"));
        responseCurveLabel->setEnabled(false);
        responseCurveLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_14->addWidget(responseCurveLabel, 4, 0, 1, 1);

        customConfigCheckBox = new QCheckBox(groupBox_2);
        customConfigCheckBox->setObjectName(QString::fromUtf8("customConfigCheckBox"));
        customConfigCheckBox->setChecked(false);

        gridLayout_14->addWidget(customConfigCheckBox, 1, 1, 1, 2);

        profileComboBox = new QComboBox(groupBox_2);
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->addItem(QString());
        profileComboBox->setObjectName(QString::fromUtf8("profileComboBox"));
        profileComboBox->setMinimumSize(QSize(250, 0));

        gridLayout_14->addWidget(profileComboBox, 0, 1, 1, 2);

        responseCurveComboBox = new QComboBox(groupBox_2);
        responseCurveComboBox->addItem(QString());
        responseCurveComboBox->addItem(QString());
        responseCurveComboBox->addItem(QString());
        responseCurveComboBox->addItem(QString());
        responseCurveComboBox->addItem(QString());
        responseCurveComboBox->setObjectName(QString::fromUtf8("responseCurveComboBox"));
        responseCurveComboBox->setEnabled(false);
        QSizePolicy sizePolicy7(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(responseCurveComboBox->sizePolicy().hasHeightForWidth());
        responseCurveComboBox->setSizePolicy(sizePolicy7);
        responseCurveComboBox->setMinimumSize(QSize(250, 0));

        gridLayout_14->addWidget(responseCurveComboBox, 4, 1, 1, 2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(6);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        RespCurveFileLoadedLineEdit = new QLineEdit(groupBox_2);
        RespCurveFileLoadedLineEdit->setObjectName(QString::fromUtf8("RespCurveFileLoadedLineEdit"));
        RespCurveFileLoadedLineEdit->setEnabled(false);
        RespCurveFileLoadedLineEdit->setMinimumSize(QSize(240, 0));
        RespCurveFileLoadedLineEdit->setReadOnly(true);

        horizontalLayout_6->addWidget(RespCurveFileLoadedLineEdit);

        loadRespCurveFileButton = new QPushButton(groupBox_2);
        loadRespCurveFileButton->setObjectName(QString::fromUtf8("loadRespCurveFileButton"));
        loadRespCurveFileButton->setEnabled(false);

        horizontalLayout_6->addWidget(loadRespCurveFileButton);


        gridLayout_14->addLayout(horizontalLayout_6, 5, 1, 1, 2);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setSpacing(6);
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        CurveFileNameSaveLineEdit = new QLineEdit(groupBox_2);
        CurveFileNameSaveLineEdit->setObjectName(QString::fromUtf8("CurveFileNameSaveLineEdit"));
        CurveFileNameSaveLineEdit->setEnabled(false);
        CurveFileNameSaveLineEdit->setMinimumSize(QSize(240, 0));
        CurveFileNameSaveLineEdit->setReadOnly(true);

        horizontalLayout_11->addWidget(CurveFileNameSaveLineEdit);

        saveRespCurveFileButton = new QPushButton(groupBox_2);
        saveRespCurveFileButton->setObjectName(QString::fromUtf8("saveRespCurveFileButton"));
        saveRespCurveFileButton->setEnabled(false);
        saveRespCurveFileButton->setAutoDefault(false);

        horizontalLayout_11->addWidget(saveRespCurveFileButton);


        gridLayout_14->addLayout(horizontalLayout_11, 6, 1, 1, 2);

        modelComboBox = new QComboBox(groupBox_2);
        modelComboBox->addItem(QString());
        modelComboBox->addItem(QString());
        modelComboBox->addItem(QString());
        modelComboBox->setObjectName(QString::fromUtf8("modelComboBox"));
        modelComboBox->setEnabled(false);
        modelComboBox->setMinimumSize(QSize(250, 0));

        gridLayout_14->addWidget(modelComboBox, 2, 1, 1, 2);


        gridLayout_4->addWidget(groupBox_2, 0, 0, 1, 1);

        pagestack->addWidget(chooseHDRconfig);

        verticalLayout_9->addWidget(pagestack);

        textEdit = new QTextEdit(HdrWizard);
        textEdit->setObjectName(QString::fromUtf8("textEdit"));
        textEdit->setReadOnly(true);

        verticalLayout_9->addWidget(textEdit);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(6);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalLayout_5->setContentsMargins(6, 6, 6, 6);
        progressBar = new QProgressBar(HdrWizard);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setValue(0);

        horizontalLayout_5->addWidget(progressBar);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);

        HideLogButton = new QPushButton(HdrWizard);
        HideLogButton->setObjectName(QString::fromUtf8("HideLogButton"));
        HideLogButton->setCheckable(true);

        horizontalLayout_5->addWidget(HideLogButton);

        NextFinishButton = new QPushButton(HdrWizard);
        NextFinishButton->setObjectName(QString::fromUtf8("NextFinishButton"));
        NextFinishButton->setEnabled(false);
        NextFinishButton->setAutoDefault(false);

        horizontalLayout_5->addWidget(NextFinishButton);

        cancelButton = new QPushButton(HdrWizard);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        horizontalLayout_5->addWidget(cancelButton);


        verticalLayout_9->addLayout(horizontalLayout_5);


        gridLayout_5->addLayout(verticalLayout_9, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_2->setBuddy(tableWidget);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tableWidget, loadImagesButton);
        QWidget::setTabOrder(loadImagesButton, removeImageButton);
        QWidget::setTabOrder(removeImageButton, clearListButton);
        QWidget::setTabOrder(clearListButton, EVSlider);
        QWidget::setTabOrder(EVSlider, ImageEVdsb);
        QWidget::setTabOrder(ImageEVdsb, alignCheckBox);
        QWidget::setTabOrder(alignCheckBox, ais_radioButton);
        QWidget::setTabOrder(ais_radioButton, mtb_radioButton);
        QWidget::setTabOrder(mtb_radioButton, autoCropCheckBox);
        QWidget::setTabOrder(autoCropCheckBox, autoAG_checkBox);
        QWidget::setTabOrder(autoAG_checkBox, threshold_horizontalSlider);
        QWidget::setTabOrder(threshold_horizontalSlider, threshold_doubleSpinBox);
        QWidget::setTabOrder(threshold_doubleSpinBox, checkBoxEditingTools);
        QWidget::setTabOrder(checkBoxEditingTools, NextFinishButton);
        QWidget::setTabOrder(NextFinishButton, cancelButton);
        QWidget::setTabOrder(cancelButton, profileComboBox);
        QWidget::setTabOrder(profileComboBox, customConfigCheckBox);
        QWidget::setTabOrder(customConfigCheckBox, modelComboBox);
        QWidget::setTabOrder(modelComboBox, weightFunctionComboBox);
        QWidget::setTabOrder(weightFunctionComboBox, responseCurveComboBox);
        QWidget::setTabOrder(responseCurveComboBox, RespCurveFileLoadedLineEdit);
        QWidget::setTabOrder(RespCurveFileLoadedLineEdit, CurveFileNameSaveLineEdit);
        QWidget::setTabOrder(CurveFileNameSaveLineEdit, saveRespCurveFileButton);
        QWidget::setTabOrder(saveRespCurveFileButton, pushButtonSaveSettings);
        QWidget::setTabOrder(pushButtonSaveSettings, textEdit);

        retranslateUi(HdrWizard);
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), weightFunctionLabel, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), responseCurveLabel, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), profileComboBox, SLOT(setDisabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), profileLabel, SLOT(setDisabled(bool)));
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), ais_radioButton, SLOT(setEnabled(bool)));
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), mtb_radioButton, SLOT(setEnabled(bool)));
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), autoCropCheckBox, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), label_7, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), threshold_horizontalSlider, SLOT(setEnabled(bool)));
        QObject::connect(autoAG_checkBox, SIGNAL(toggled(bool)), threshold_doubleSpinBox, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), modelLabel, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), weightFunctionComboBox, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), modelComboBox, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), responseCurveComboBox, SLOT(setEnabled(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(stateChanged(int)), customConfigCheckBox, SLOT(setFocus()));
        QObject::connect(HideLogButton, SIGNAL(clicked(bool)), textEdit, SLOT(setVisible(bool)));
        QObject::connect(customConfigCheckBox, SIGNAL(toggled(bool)), pushButtonSaveSettings, SLOT(setEnabled(bool)));

        pagestack->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(HdrWizard);
    } // setupUi

    void retranslateUi(QDialog *HdrWizard)
    {
        HdrWizard->setWindowTitle(QCoreApplication::translate("HdrWizard", "HDR Creation Wizard", nullptr));
        groupBox->setTitle(QCoreApplication::translate("HdrWizard", "[1/2] Load Input Images", nullptr));
        label_2->setText(QCoreApplication::translate("HdrWizard", "Currently &Loaded Files", nullptr));
#if QT_CONFIG(tooltip)
        loadImagesButton->setToolTip(QCoreApplication::translate("HdrWizard", "Add Images", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        removeImageButton->setToolTip(QCoreApplication::translate("HdrWizard", "Remove Selected Image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        clearListButton->setToolTip(QCoreApplication::translate("HdrWizard", "Clear List", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("HdrWizard", "Preview", nullptr));
        previewLabel->setText(QString());
        EVgroupBox->setTitle(QCoreApplication::translate("HdrWizard", "Selected Image &Exposure", nullptr));
        label_9->setText(QCoreApplication::translate("HdrWizard", "EV:", nullptr));
        ImageEVdsb->setPrefix(QString());
        alignGroupBox->setTitle(QCoreApplication::translate("HdrWizard", "Alignment", nullptr));
        mtb_radioButton->setText(QCoreApplication::translate("HdrWizard", "M&TB", nullptr));
        alignCheckBox->setText(QCoreApplication::translate("HdrWizard", "&Autoalign images", nullptr));
        ais_radioButton->setText(QCoreApplication::translate("HdrWizard", "Hu&gin's align_image_stack", nullptr));
        autoCropCheckBox->setText(QCoreApplication::translate("HdrWizard", "Auto-crop", nullptr));
        agGroupBox->setTitle(QCoreApplication::translate("HdrWizard", "Anti-ghosting", nullptr));
        label_7->setText(QCoreApplication::translate("HdrWizard", "Threshold", nullptr));
#if QT_CONFIG(tooltip)
        threshold_horizontalSlider->setToolTip(QCoreApplication::translate("HdrWizard", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        threshold_doubleSpinBox->setToolTip(QCoreApplication::translate("HdrWizard", "<html><head/><body><p>Threshold used by auto anti-ghosting algorithm. A lower threshold means a more aggressive processing but that can lower the final dynamic range. Use the &quot;Recompute&quot; button to see the actual patches coverage and adjust the threshold accordingly</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        autoAG_checkBox->setToolTip(QCoreApplication::translate("HdrWizard", "Enable the auto anti-ghosting algorithm (by O. Gallo and others)", nullptr));
#endif // QT_CONFIG(tooltip)
        autoAG_checkBox->setText(QCoreApplication::translate("HdrWizard", "Auto anti-ghosting", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxEditingTools->setToolTip(QCoreApplication::translate("HdrWizard", "Enable this option for manual alignment or manual Anti-Ghosting. It's intended for users which a strong knowledge of the different HDR creation steps!", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxEditingTools->setText(QCoreApplication::translate("HdrWizard", "Advanced Editing Tools", nullptr));
        confirmloadlabel->setText(QCoreApplication::translate("HdrWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:12pt; font-weight:600;\">Start loading a set of images with different exposure</span></p></body></html>", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("HdrWizard", "[2/2] Choose Settings for HDR Creation", nullptr));
        responseCurveInputFileLabel->setText(QCoreApplication::translate("HdrWizard", "Response Curve Input File", nullptr));
        responseCurveOutputFileLabel->setText(QCoreApplication::translate("HdrWizard", "Response Curve Output File", nullptr));
        weightFunctionComboBox->setItemText(0, QCoreApplication::translate("HdrWizard", "Triangular", nullptr));
        weightFunctionComboBox->setItemText(1, QCoreApplication::translate("HdrWizard", "Gaussian", nullptr));
        weightFunctionComboBox->setItemText(2, QCoreApplication::translate("HdrWizard", "Plateau", nullptr));
        weightFunctionComboBox->setItemText(3, QCoreApplication::translate("HdrWizard", "Flat", nullptr));

        weightFunctionLabel->setText(QCoreApplication::translate("HdrWizard", "Weighting Function", nullptr));
        pushButtonSaveSettings->setText(QCoreApplication::translate("HdrWizard", "Save HDR creation Settings into database to use in Batch HDR", nullptr));
        modelLabel->setText(QCoreApplication::translate("HdrWizard", "HDR Creation Model", nullptr));
        profileLabel->setText(QCoreApplication::translate("HdrWizard", "Profile", nullptr));
        responseCurveLabel->setText(QCoreApplication::translate("HdrWizard", "Response Curve", nullptr));
#if QT_CONFIG(tooltip)
        customConfigCheckBox->setToolTip(QCoreApplication::translate("HdrWizard", "Use this only if the default profiles above do not yield good results", nullptr));
#endif // QT_CONFIG(tooltip)
        customConfigCheckBox->setText(QCoreApplication::translate("HdrWizard", "Use Custom Configuration", nullptr));
        profileComboBox->setItemText(0, QCoreApplication::translate("HdrWizard", "Profile 1 - Debevec, Triangular, Linear", nullptr));
        profileComboBox->setItemText(1, QCoreApplication::translate("HdrWizard", "Profile 2 - Debevec, Triangular, Gamma", nullptr));
        profileComboBox->setItemText(2, QCoreApplication::translate("HdrWizard", "Profile 3 - Debevec, Plateau, Linear", nullptr));
        profileComboBox->setItemText(3, QCoreApplication::translate("HdrWizard", "Profile 4 - Debevec, Plateau, Gamma", nullptr));
        profileComboBox->setItemText(4, QCoreApplication::translate("HdrWizard", "Profile 5 - Debevec, Gaussian, Linear", nullptr));
        profileComboBox->setItemText(5, QCoreApplication::translate("HdrWizard", "Profile 6 - Debevec, Gaussian, Gamma", nullptr));

#if QT_CONFIG(tooltip)
        profileComboBox->setToolTip(QCoreApplication::translate("HdrWizard", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif';\">The first predefined profile in the list usually leads to best results. Change it only if the resulting HDR image is not good enough.</span></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        responseCurveComboBox->setItemText(0, QCoreApplication::translate("HdrWizard", "Linear", nullptr));
        responseCurveComboBox->setItemText(1, QCoreApplication::translate("HdrWizard", "Gamma", nullptr));
        responseCurveComboBox->setItemText(2, QCoreApplication::translate("HdrWizard", "Log", nullptr));
        responseCurveComboBox->setItemText(3, QCoreApplication::translate("HdrWizard", "sRGB", nullptr));
        responseCurveComboBox->setItemText(4, QCoreApplication::translate("HdrWizard", "Custom (From File)", nullptr));

        loadRespCurveFileButton->setText(QCoreApplication::translate("HdrWizard", "Load Response", nullptr));
        saveRespCurveFileButton->setText(QCoreApplication::translate("HdrWizard", "Save As...", nullptr));
        modelComboBox->setItemText(0, QCoreApplication::translate("HdrWizard", "Debevec", nullptr));
        modelComboBox->setItemText(1, QCoreApplication::translate("HdrWizard", "Robertson (Simple)", nullptr));
        modelComboBox->setItemText(2, QCoreApplication::translate("HdrWizard", "Robertson (Response Recovery)", nullptr));

        HideLogButton->setText(QCoreApplication::translate("HdrWizard", "Show Log", nullptr));
        NextFinishButton->setText(QCoreApplication::translate("HdrWizard", "&Next >", nullptr));
        cancelButton->setText(QCoreApplication::translate("HdrWizard", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HdrWizard: public Ui_HdrWizard {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HDRWIZARD_H
