/********************************************************************************
** Form generated from reading UI file 'SavingParametersDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SAVINGPARAMETERSDIALOG_H
#define UI_SAVINGPARAMETERSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SavingParameters
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *comment;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SavingParameters)
    {
        if (SavingParameters->objectName().isEmpty())
            SavingParameters->setObjectName(QString::fromUtf8("SavingParameters"));
        SavingParameters->setWindowModality(Qt::ApplicationModal);
        gridLayout = new QGridLayout(SavingParameters);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(SavingParameters);
        label->setObjectName(QString::fromUtf8("label"));

        verticalLayout->addWidget(label);

        comment = new QLineEdit(SavingParameters);
        comment->setObjectName(QString::fromUtf8("comment"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(3);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comment->sizePolicy().hasHeightForWidth());
        comment->setSizePolicy(sizePolicy);

        verticalLayout->addWidget(comment);

        buttonBox = new QDialogButtonBox(SavingParameters);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(SavingParameters);
        QObject::connect(buttonBox, SIGNAL(accepted()), SavingParameters, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), SavingParameters, SLOT(reject()));

        QMetaObject::connectSlotsByName(SavingParameters);
    } // setupUi

    void retranslateUi(QDialog *SavingParameters)
    {
        SavingParameters->setWindowTitle(QCoreApplication::translate("SavingParameters", "Saving Parameters", nullptr));
        label->setText(QCoreApplication::translate("SavingParameters", "Enter a short comment for the saved parameters:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SavingParameters: public Ui_SavingParameters {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SAVINGPARAMETERSDIALOG_H
