/********************************************************************************
** Form generated from reading UI file 'FitsImporter.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FITSIMPORTER_H
#define UI_FITSIMPORTER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>
#include <QtWidgets/QWizard>
#include <QtWidgets/QWizardPage>
#include "UI/ExtWizardPage.h"

QT_BEGIN_NAMESPACE

class Ui_FitsImporter
{
public:
    ExtWizardPage *wizardPageLoadFiles;
    QGridLayout *gridLayout_4;
    QLineEdit *lineEditRed;
    QLabel *label_8;
    QLineEdit *lineEditH;
    QPushButton *pushButtonGreen;
    QLineEdit *lineEditBlue;
    QLabel *label;
    QPushButton *pushButtonBlue;
    QPushButton *pushButtonRed;
    QLabel *label_5;
    QLabel *label_2;
    QPushButton *pushButtonLuminosity;
    QLineEdit *lineEditGreen;
    QLabel *label_3;
    QLineEdit *lineEditLuminosity;
    QWidget *widget;
    QGridLayout *gridLayout;
    QPushButton *pushButtonClockwise;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButtonReset;
    QPushButton *pushButtonLoad;
    QPushButton *pushButtonPreview;
    QGroupBox *grpPreviews;
    QHBoxLayout *horizontalLayout_4;
    QHBoxLayout *verticalLayoutPreviews;
    QPushButton *pushButtonH;
    QGroupBox *alignGroupBox;
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout1;
    QCheckBox *autoCropCheckBox;
    QRadioButton *mtb_radioButton;
    QRadioButton *ais_radioButton;
    QSpacerItem *horizontalSpacer_6;
    QCheckBox *alignCheckBox;
    QSpacerItem *horizontalSpacer_7;
    ExtWizardPage *wizardPageAlignment;
    QGridLayout *gridLayout_2;
    QProgressBar *progressBar;
    QLabel *label_9;
    QGroupBox *groupBoxMessages_2;
    QHBoxLayout *horizontalLayout_6;
    QTextEdit *textEdit;
    QWizardPage *wizardPagePreview;
    QGridLayout *gridLayout_5;
    QLabel *label_23;
    QDoubleSpinBox *dsbRedRed;
    QSlider *hsRedRed;
    QLabel *label_20;
    QDoubleSpinBox *dsbBlueBlue;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_10;
    QLabel *previewLabel;
    QSlider *vsGamma;
    QLabel *label_17;
    QDoubleSpinBox *dsbGreenGreen;
    QSlider *hsBlueBlue;
    QSlider *hsGreenGreen;

    void setupUi(QWizard *FitsImporter)
    {
        if (FitsImporter->objectName().isEmpty())
            FitsImporter->setObjectName(QString::fromUtf8("FitsImporter"));
        FitsImporter->resize(760, 630);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/images/applications-astronomy.png"), QSize(), QIcon::Normal, QIcon::Off);
        FitsImporter->setWindowIcon(icon);
        wizardPageLoadFiles = new ExtWizardPage();
        wizardPageLoadFiles->setObjectName(QString::fromUtf8("wizardPageLoadFiles"));
        gridLayout_4 = new QGridLayout(wizardPageLoadFiles);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        lineEditRed = new QLineEdit(wizardPageLoadFiles);
        lineEditRed->setObjectName(QString::fromUtf8("lineEditRed"));
        lineEditRed->setReadOnly(true);

        gridLayout_4->addWidget(lineEditRed, 0, 1, 1, 1);

        label_8 = new QLabel(wizardPageLoadFiles);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        gridLayout_4->addWidget(label_8, 4, 0, 1, 1);

        lineEditH = new QLineEdit(wizardPageLoadFiles);
        lineEditH->setObjectName(QString::fromUtf8("lineEditH"));
        lineEditH->setReadOnly(true);

        gridLayout_4->addWidget(lineEditH, 4, 1, 1, 1);

        pushButtonGreen = new QPushButton(wizardPageLoadFiles);
        pushButtonGreen->setObjectName(QString::fromUtf8("pushButtonGreen"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        pushButtonGreen->setIcon(icon1);

        gridLayout_4->addWidget(pushButtonGreen, 1, 2, 1, 1);

        lineEditBlue = new QLineEdit(wizardPageLoadFiles);
        lineEditBlue->setObjectName(QString::fromUtf8("lineEditBlue"));
        lineEditBlue->setReadOnly(true);

        gridLayout_4->addWidget(lineEditBlue, 2, 1, 1, 1);

        label = new QLabel(wizardPageLoadFiles);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout_4->addWidget(label, 0, 0, 1, 1);

        pushButtonBlue = new QPushButton(wizardPageLoadFiles);
        pushButtonBlue->setObjectName(QString::fromUtf8("pushButtonBlue"));
        pushButtonBlue->setIcon(icon1);

        gridLayout_4->addWidget(pushButtonBlue, 2, 2, 1, 1);

        pushButtonRed = new QPushButton(wizardPageLoadFiles);
        pushButtonRed->setObjectName(QString::fromUtf8("pushButtonRed"));
        pushButtonRed->setIcon(icon1);

        gridLayout_4->addWidget(pushButtonRed, 0, 2, 1, 1);

        label_5 = new QLabel(wizardPageLoadFiles);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout_4->addWidget(label_5, 3, 0, 1, 1);

        label_2 = new QLabel(wizardPageLoadFiles);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_4->addWidget(label_2, 1, 0, 1, 1);

        pushButtonLuminosity = new QPushButton(wizardPageLoadFiles);
        pushButtonLuminosity->setObjectName(QString::fromUtf8("pushButtonLuminosity"));
        pushButtonLuminosity->setIcon(icon1);

        gridLayout_4->addWidget(pushButtonLuminosity, 3, 2, 1, 1);

        lineEditGreen = new QLineEdit(wizardPageLoadFiles);
        lineEditGreen->setObjectName(QString::fromUtf8("lineEditGreen"));
        lineEditGreen->setReadOnly(true);

        gridLayout_4->addWidget(lineEditGreen, 1, 1, 1, 1);

        label_3 = new QLabel(wizardPageLoadFiles);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout_4->addWidget(label_3, 2, 0, 1, 1);

        lineEditLuminosity = new QLineEdit(wizardPageLoadFiles);
        lineEditLuminosity->setObjectName(QString::fromUtf8("lineEditLuminosity"));
        lineEditLuminosity->setReadOnly(true);

        gridLayout_4->addWidget(lineEditLuminosity, 3, 1, 1, 1);

        widget = new QWidget(wizardPageLoadFiles);
        widget->setObjectName(QString::fromUtf8("widget"));
        gridLayout = new QGridLayout(widget);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        pushButtonClockwise = new QPushButton(widget);
        pushButtonClockwise->setObjectName(QString::fromUtf8("pushButtonClockwise"));
        pushButtonClockwise->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pushButtonClockwise->sizePolicy().hasHeightForWidth());
        pushButtonClockwise->setSizePolicy(sizePolicy);
        QIcon icon2;
        iconThemeName = QString::fromUtf8("object-rotate-right");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        pushButtonClockwise->setIcon(icon2);

        gridLayout->addWidget(pushButtonClockwise, 0, 5, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 0, 0, 1, 1);

        pushButtonReset = new QPushButton(widget);
        pushButtonReset->setObjectName(QString::fromUtf8("pushButtonReset"));
        pushButtonReset->setEnabled(false);
        sizePolicy.setHeightForWidth(pushButtonReset->sizePolicy().hasHeightForWidth());
        pushButtonReset->setSizePolicy(sizePolicy);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("chronometer-reset");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        pushButtonReset->setIcon(icon3);

        gridLayout->addWidget(pushButtonReset, 0, 2, 1, 1);

        pushButtonLoad = new QPushButton(widget);
        pushButtonLoad->setObjectName(QString::fromUtf8("pushButtonLoad"));
        pushButtonLoad->setEnabled(false);
        sizePolicy.setHeightForWidth(pushButtonLoad->sizePolicy().hasHeightForWidth());
        pushButtonLoad->setSizePolicy(sizePolicy);
        QIcon icon4;
        iconThemeName = QString::fromUtf8("upload-media");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        pushButtonLoad->setIcon(icon4);
        pushButtonLoad->setIconSize(QSize(20, 20));
        pushButtonLoad->setFlat(false);

        gridLayout->addWidget(pushButtonLoad, 0, 1, 1, 1);

        pushButtonPreview = new QPushButton(widget);
        pushButtonPreview->setObjectName(QString::fromUtf8("pushButtonPreview"));
        pushButtonPreview->setEnabled(false);
        sizePolicy.setHeightForWidth(pushButtonPreview->sizePolicy().hasHeightForWidth());
        pushButtonPreview->setSizePolicy(sizePolicy);
        QIcon icon5;
        iconThemeName = QString::fromUtf8("document-preview");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        pushButtonPreview->setIcon(icon5);
        pushButtonPreview->setCheckable(true);
        pushButtonPreview->setAutoDefault(false);

        gridLayout->addWidget(pushButtonPreview, 0, 4, 1, 1);


        gridLayout_4->addWidget(widget, 9, 0, 1, 3);

        grpPreviews = new QGroupBox(wizardPageLoadFiles);
        grpPreviews->setObjectName(QString::fromUtf8("grpPreviews"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(grpPreviews->sizePolicy().hasHeightForWidth());
        grpPreviews->setSizePolicy(sizePolicy1);
        grpPreviews->setMinimumSize(QSize(700, 130));
        horizontalLayout_4 = new QHBoxLayout(grpPreviews);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        verticalLayoutPreviews = new QHBoxLayout();
        verticalLayoutPreviews->setObjectName(QString::fromUtf8("verticalLayoutPreviews"));

        horizontalLayout_4->addLayout(verticalLayoutPreviews);


        gridLayout_4->addWidget(grpPreviews, 6, 0, 3, 3);

        pushButtonH = new QPushButton(wizardPageLoadFiles);
        pushButtonH->setObjectName(QString::fromUtf8("pushButtonH"));
        pushButtonH->setIcon(icon1);

        gridLayout_4->addWidget(pushButtonH, 4, 2, 1, 1);

        alignGroupBox = new QGroupBox(wizardPageLoadFiles);
        alignGroupBox->setObjectName(QString::fromUtf8("alignGroupBox"));
        alignGroupBox->setEnabled(true);
        gridLayout_3 = new QGridLayout(alignGroupBox);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(-1, 0, 0, 0);
        autoCropCheckBox = new QCheckBox(alignGroupBox);
        autoCropCheckBox->setObjectName(QString::fromUtf8("autoCropCheckBox"));
        autoCropCheckBox->setEnabled(false);
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(autoCropCheckBox->sizePolicy().hasHeightForWidth());
        autoCropCheckBox->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(autoCropCheckBox, 5, 1, 1, 1);

        mtb_radioButton = new QRadioButton(alignGroupBox);
        mtb_radioButton->setObjectName(QString::fromUtf8("mtb_radioButton"));
        mtb_radioButton->setEnabled(false);

        gridLayout1->addWidget(mtb_radioButton, 1, 3, 1, 1);

        ais_radioButton = new QRadioButton(alignGroupBox);
        ais_radioButton->setObjectName(QString::fromUtf8("ais_radioButton"));
        ais_radioButton->setEnabled(false);
        ais_radioButton->setChecked(true);

        gridLayout1->addWidget(ais_radioButton, 1, 2, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(1, 1, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout1->addItem(horizontalSpacer_6, 5, 0, 1, 1);

        alignCheckBox = new QCheckBox(alignGroupBox);
        alignCheckBox->setObjectName(QString::fromUtf8("alignCheckBox"));
        alignCheckBox->setEnabled(true);
        sizePolicy2.setHeightForWidth(alignCheckBox->sizePolicy().hasHeightForWidth());
        alignCheckBox->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(alignCheckBox, 1, 0, 1, 2);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout1->addItem(horizontalSpacer_7, 1, 4, 1, 1);


        gridLayout_3->addLayout(gridLayout1, 0, 0, 1, 1);


        gridLayout_4->addWidget(alignGroupBox, 10, 0, 1, 3);

        gridLayout_4->setColumnStretch(1, 1);
        FitsImporter->setPage(21, wizardPageLoadFiles);
        wizardPageAlignment = new ExtWizardPage();
        wizardPageAlignment->setObjectName(QString::fromUtf8("wizardPageAlignment"));
        gridLayout_2 = new QGridLayout(wizardPageAlignment);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        progressBar = new QProgressBar(wizardPageAlignment);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setValue(0);

        gridLayout_2->addWidget(progressBar, 0, 1, 1, 1);

        label_9 = new QLabel(wizardPageAlignment);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        gridLayout_2->addWidget(label_9, 0, 0, 1, 1);

        groupBoxMessages_2 = new QGroupBox(wizardPageAlignment);
        groupBoxMessages_2->setObjectName(QString::fromUtf8("groupBoxMessages_2"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(groupBoxMessages_2->sizePolicy().hasHeightForWidth());
        groupBoxMessages_2->setSizePolicy(sizePolicy3);
        horizontalLayout_6 = new QHBoxLayout(groupBoxMessages_2);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        textEdit = new QTextEdit(groupBoxMessages_2);
        textEdit->setObjectName(QString::fromUtf8("textEdit"));
        textEdit->setReadOnly(true);

        horizontalLayout_6->addWidget(textEdit);


        gridLayout_2->addWidget(groupBoxMessages_2, 1, 0, 1, 2);

        FitsImporter->setPage(22, wizardPageAlignment);
        wizardPagePreview = new QWizardPage();
        wizardPagePreview->setObjectName(QString::fromUtf8("wizardPagePreview"));
        gridLayout_5 = new QGridLayout(wizardPagePreview);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        label_23 = new QLabel(wizardPagePreview);
        label_23->setObjectName(QString::fromUtf8("label_23"));
        label_23->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_23, 2, 0, 1, 1);

        dsbRedRed = new QDoubleSpinBox(wizardPagePreview);
        dsbRedRed->setObjectName(QString::fromUtf8("dsbRedRed"));
        dsbRedRed->setMaximum(1.000000000000000);
        dsbRedRed->setSingleStep(0.010000000000000);
        dsbRedRed->setValue(1.000000000000000);

        gridLayout_5->addWidget(dsbRedRed, 0, 2, 1, 1);

        hsRedRed = new QSlider(wizardPagePreview);
        hsRedRed->setObjectName(QString::fromUtf8("hsRedRed"));
        hsRedRed->setMaximum(10000);
        hsRedRed->setValue(10000);
        hsRedRed->setOrientation(Qt::Horizontal);

        gridLayout_5->addWidget(hsRedRed, 0, 1, 1, 1);

        label_20 = new QLabel(wizardPagePreview);
        label_20->setObjectName(QString::fromUtf8("label_20"));
        label_20->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_20, 1, 0, 1, 1);

        dsbBlueBlue = new QDoubleSpinBox(wizardPagePreview);
        dsbBlueBlue->setObjectName(QString::fromUtf8("dsbBlueBlue"));
        dsbBlueBlue->setMaximum(1.000000000000000);
        dsbBlueBlue->setSingleStep(0.010000000000000);
        dsbBlueBlue->setValue(1.000000000000000);

        gridLayout_5->addWidget(dsbBlueBlue, 2, 2, 1, 1);

        groupBox = new QGroupBox(wizardPagePreview);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_10 = new QGridLayout(groupBox);
        gridLayout_10->setObjectName(QString::fromUtf8("gridLayout_10"));
        previewLabel = new QLabel(groupBox);
        previewLabel->setObjectName(QString::fromUtf8("previewLabel"));
        previewLabel->setMinimumSize(QSize(300, 200));
        previewLabel->setFrameShape(QFrame::Box);
        previewLabel->setLineWidth(3);
        previewLabel->setAlignment(Qt::AlignCenter);

        gridLayout_10->addWidget(previewLabel, 0, 0, 1, 1);

        vsGamma = new QSlider(groupBox);
        vsGamma->setObjectName(QString::fromUtf8("vsGamma"));
        vsGamma->setMaximum(10000);
        vsGamma->setSingleStep(100);
        vsGamma->setPageStep(1000);
        vsGamma->setValue(2200);
        vsGamma->setOrientation(Qt::Vertical);
        vsGamma->setTickPosition(QSlider::NoTicks);

        gridLayout_10->addWidget(vsGamma, 0, 1, 1, 1);


        gridLayout_5->addWidget(groupBox, 3, 0, 1, 4);

        label_17 = new QLabel(wizardPagePreview);
        label_17->setObjectName(QString::fromUtf8("label_17"));
        label_17->setMinimumSize(QSize(50, 0));
        label_17->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_17, 0, 0, 1, 1);

        dsbGreenGreen = new QDoubleSpinBox(wizardPagePreview);
        dsbGreenGreen->setObjectName(QString::fromUtf8("dsbGreenGreen"));
        dsbGreenGreen->setMaximum(1.000000000000000);
        dsbGreenGreen->setSingleStep(0.010000000000000);
        dsbGreenGreen->setValue(1.000000000000000);

        gridLayout_5->addWidget(dsbGreenGreen, 1, 2, 1, 1);

        hsBlueBlue = new QSlider(wizardPagePreview);
        hsBlueBlue->setObjectName(QString::fromUtf8("hsBlueBlue"));
        hsBlueBlue->setMaximum(10000);
        hsBlueBlue->setValue(10000);
        hsBlueBlue->setOrientation(Qt::Horizontal);

        gridLayout_5->addWidget(hsBlueBlue, 2, 1, 1, 1);

        hsGreenGreen = new QSlider(wizardPagePreview);
        hsGreenGreen->setObjectName(QString::fromUtf8("hsGreenGreen"));
        hsGreenGreen->setMaximum(10000);
        hsGreenGreen->setValue(10000);
        hsGreenGreen->setOrientation(Qt::Horizontal);

        gridLayout_5->addWidget(hsGreenGreen, 1, 1, 1, 1);

        FitsImporter->setPage(99, wizardPagePreview);
        QWidget::setTabOrder(lineEditRed, pushButtonRed);
        QWidget::setTabOrder(pushButtonRed, lineEditGreen);
        QWidget::setTabOrder(lineEditGreen, pushButtonGreen);
        QWidget::setTabOrder(pushButtonGreen, lineEditBlue);
        QWidget::setTabOrder(lineEditBlue, pushButtonBlue);
        QWidget::setTabOrder(pushButtonBlue, lineEditLuminosity);
        QWidget::setTabOrder(lineEditLuminosity, pushButtonLuminosity);
        QWidget::setTabOrder(pushButtonLuminosity, lineEditH);
        QWidget::setTabOrder(lineEditH, pushButtonH);
        QWidget::setTabOrder(pushButtonH, pushButtonLoad);
        QWidget::setTabOrder(pushButtonLoad, pushButtonReset);
        QWidget::setTabOrder(pushButtonReset, pushButtonPreview);
        QWidget::setTabOrder(pushButtonPreview, pushButtonClockwise);
        QWidget::setTabOrder(pushButtonClockwise, alignCheckBox);
        QWidget::setTabOrder(alignCheckBox, ais_radioButton);
        QWidget::setTabOrder(ais_radioButton, mtb_radioButton);
        QWidget::setTabOrder(mtb_radioButton, autoCropCheckBox);
        QWidget::setTabOrder(autoCropCheckBox, textEdit);
        QWidget::setTabOrder(textEdit, hsRedRed);
        QWidget::setTabOrder(hsRedRed, dsbRedRed);
        QWidget::setTabOrder(dsbRedRed, hsGreenGreen);
        QWidget::setTabOrder(hsGreenGreen, dsbGreenGreen);
        QWidget::setTabOrder(dsbGreenGreen, hsBlueBlue);
        QWidget::setTabOrder(hsBlueBlue, dsbBlueBlue);
        QWidget::setTabOrder(dsbBlueBlue, vsGamma);

        retranslateUi(FitsImporter);
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), ais_radioButton, SLOT(setEnabled(bool)));
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), mtb_radioButton, SLOT(setEnabled(bool)));
        QObject::connect(alignCheckBox, SIGNAL(toggled(bool)), autoCropCheckBox, SLOT(setEnabled(bool)));

        pushButtonPreview->setDefault(false);


        QMetaObject::connectSlotsByName(FitsImporter);
    } // setupUi

    void retranslateUi(QWizard *FitsImporter)
    {
        FitsImporter->setWindowTitle(QCoreApplication::translate("FitsImporter", "Import FITS files", nullptr));
        wizardPageLoadFiles->setTitle(QCoreApplication::translate("FitsImporter", "Setup files", nullptr));
        wizardPageLoadFiles->setSubTitle(QCoreApplication::translate("FitsImporter", "Select the different files for the channels. Optionally, they can be aligned.", nullptr));
        label_8->setText(QCoreApplication::translate("FitsImporter", "H-alpha (optional)", nullptr));
        pushButtonGreen->setText(QCoreApplication::translate("FitsImporter", "Select...", nullptr));
        label->setText(QCoreApplication::translate("FitsImporter", "Channel 1 (Red)", nullptr));
        pushButtonBlue->setText(QCoreApplication::translate("FitsImporter", "Select...", nullptr));
        pushButtonRed->setText(QCoreApplication::translate("FitsImporter", "Select...", nullptr));
        label_5->setText(QCoreApplication::translate("FitsImporter", "Luminosity (optional)", nullptr));
        label_2->setText(QCoreApplication::translate("FitsImporter", "Channel 2 (Green)", nullptr));
        pushButtonLuminosity->setText(QCoreApplication::translate("FitsImporter", "Select...", nullptr));
        label_3->setText(QCoreApplication::translate("FitsImporter", "Channel 3 (Blue)", nullptr));
#if QT_CONFIG(tooltip)
        pushButtonClockwise->setToolTip(QCoreApplication::translate("FitsImporter", "<html><head/><body><p>Rotate selected image 180\303\202\302\260</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonClockwise->setText(QCoreApplication::translate("FitsImporter", "Rotate", nullptr));
        pushButtonReset->setText(QCoreApplication::translate("FitsImporter", "Reset", nullptr));
        pushButtonLoad->setText(QCoreApplication::translate("FitsImporter", "Load Previews", nullptr));
#if QT_CONFIG(shortcut)
        pushButtonLoad->setShortcut(QCoreApplication::translate("FitsImporter", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pushButtonPreview->setToolTip(QCoreApplication::translate("FitsImporter", "Shows a larger preview of selected image", nullptr));
#endif // QT_CONFIG(tooltip)
        pushButtonPreview->setText(QCoreApplication::translate("FitsImporter", "Preview", nullptr));
        grpPreviews->setTitle(QCoreApplication::translate("FitsImporter", "Previews", nullptr));
        pushButtonH->setText(QCoreApplication::translate("FitsImporter", "Select...", nullptr));
        alignGroupBox->setTitle(QCoreApplication::translate("FitsImporter", "Alignment", nullptr));
        autoCropCheckBox->setText(QCoreApplication::translate("FitsImporter", "Auto-crop", nullptr));
        mtb_radioButton->setText(QCoreApplication::translate("FitsImporter", "&MTB", nullptr));
        ais_radioButton->setText(QCoreApplication::translate("FitsImporter", "Hugin's align_&image_stack", nullptr));
        alignCheckBox->setText(QCoreApplication::translate("FitsImporter", "&Autoalign images", nullptr));
        wizardPageAlignment->setTitle(QCoreApplication::translate("FitsImporter", "Alignment", nullptr));
        wizardPageAlignment->setSubTitle(QCoreApplication::translate("FitsImporter", "The alignment of the input files is in progress. Please be patient!", nullptr));
        label_9->setText(QCoreApplication::translate("FitsImporter", "Progress:", nullptr));
        groupBoxMessages_2->setTitle(QCoreApplication::translate("FitsImporter", "Messages", nullptr));
        wizardPagePreview->setTitle(QCoreApplication::translate("FitsImporter", "Preview", nullptr));
        wizardPagePreview->setSubTitle(QCoreApplication::translate("FitsImporter", "Please fine-tune the color part of the channels.", nullptr));
        label_23->setText(QCoreApplication::translate("FitsImporter", "Blue", nullptr));
        label_20->setText(QCoreApplication::translate("FitsImporter", "Green", nullptr));
        groupBox->setTitle(QCoreApplication::translate("FitsImporter", "Preview", nullptr));
        previewLabel->setText(QString());
        label_17->setText(QCoreApplication::translate("FitsImporter", "Red", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FitsImporter: public Ui_FitsImporter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FITSIMPORTER_H
