/********************************************************************************
** Form generated from reading UI file 'TiffModeDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TIFFMODEDIALOG_H
#define UI_TIFFMODEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TiffModeDialog
{
public:
    QVBoxLayout *verticalLayout;
    QComboBox *comboBox;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TiffModeDialog)
    {
        if (TiffModeDialog->objectName().isEmpty())
            TiffModeDialog->setObjectName(QString::fromUtf8("TiffModeDialog"));
        TiffModeDialog->resize(400, 84);
        verticalLayout = new QVBoxLayout(TiffModeDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        comboBox = new QComboBox(TiffModeDialog);
        comboBox->setObjectName(QString::fromUtf8("comboBox"));

        verticalLayout->addWidget(comboBox);

        buttonBox = new QDialogButtonBox(TiffModeDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(TiffModeDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), TiffModeDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), TiffModeDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(TiffModeDialog);
    } // setupUi

    void retranslateUi(QDialog *TiffModeDialog)
    {
        TiffModeDialog->setWindowTitle(QCoreApplication::translate("TiffModeDialog", "Save as ...TIFF", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TiffModeDialog: public Ui_TiffModeDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TIFFMODEDIALOG_H
