/****************************************************************************
** Meta object code from reading C++ file 'LuminanceOptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/Common/LuminanceOptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LuminanceOptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LuminanceOptions_t {
    QByteArrayData data[136];
    char stringdata0[2499];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LuminanceOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LuminanceOptions_t qt_meta_stringdata_LuminanceOptions = {
    {
QT_MOC_LITERAL(0, 0, 16), // "LuminanceOptions"
QT_MOC_LITERAL(1, 17, 17), // "isRawFourColorRGB"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 18), // "setRawFourColorRGB"
QT_MOC_LITERAL(4, 55, 23), // "isRawDoNotUseFujiRotate"
QT_MOC_LITERAL(5, 79, 24), // "setRawDoNotUseFujiRotate"
QT_MOC_LITERAL(6, 104, 18), // "isRawUseChromaAber"
QT_MOC_LITERAL(7, 123, 19), // "setRawUseChromaAber"
QT_MOC_LITERAL(8, 143, 11), // "getRawAber0"
QT_MOC_LITERAL(9, 155, 11), // "setRawAber0"
QT_MOC_LITERAL(10, 167, 11), // "getRawAber1"
QT_MOC_LITERAL(11, 179, 11), // "setRawAber1"
QT_MOC_LITERAL(12, 191, 11), // "getRawAber2"
QT_MOC_LITERAL(13, 203, 11), // "setRawAber2"
QT_MOC_LITERAL(14, 215, 11), // "getRawAber3"
QT_MOC_LITERAL(15, 227, 11), // "setRawAber3"
QT_MOC_LITERAL(16, 239, 11), // "getRawGamm0"
QT_MOC_LITERAL(17, 251, 11), // "setRawGamm0"
QT_MOC_LITERAL(18, 263, 11), // "getRawGamm1"
QT_MOC_LITERAL(19, 275, 11), // "setRawGamm1"
QT_MOC_LITERAL(20, 287, 24), // "getRawWhiteBalanceMethod"
QT_MOC_LITERAL(21, 312, 24), // "setRawWhiteBalanceMethod"
QT_MOC_LITERAL(22, 337, 23), // "getRawTemperatureKelvin"
QT_MOC_LITERAL(23, 361, 23), // "setRawTemperatureKelvin"
QT_MOC_LITERAL(24, 385, 11), // "getRawGreen"
QT_MOC_LITERAL(25, 397, 11), // "setRawGreen"
QT_MOC_LITERAL(26, 409, 14), // "getRawUserMul0"
QT_MOC_LITERAL(27, 424, 14), // "setRawUserMul0"
QT_MOC_LITERAL(28, 439, 14), // "getRawUserMul1"
QT_MOC_LITERAL(29, 454, 14), // "setRawUserMul1"
QT_MOC_LITERAL(30, 469, 14), // "getRawUserMul2"
QT_MOC_LITERAL(31, 484, 14), // "setRawUserMul2"
QT_MOC_LITERAL(32, 499, 14), // "getRawUserMul3"
QT_MOC_LITERAL(33, 514, 14), // "setRawUserMul3"
QT_MOC_LITERAL(34, 529, 19), // "isRawAutoBrightness"
QT_MOC_LITERAL(35, 549, 20), // "setRawAutoBrightness"
QT_MOC_LITERAL(36, 570, 29), // "getRawAutoBrightnessThreshold"
QT_MOC_LITERAL(37, 600, 29), // "setRawAutoBrightnessThreshold"
QT_MOC_LITERAL(38, 630, 16), // "getRawBrightness"
QT_MOC_LITERAL(39, 647, 16), // "setRawBrightness"
QT_MOC_LITERAL(40, 664, 1), // "f"
QT_MOC_LITERAL(41, 666, 14), // "getRawHalfSize"
QT_MOC_LITERAL(42, 681, 14), // "setRawHalfSize"
QT_MOC_LITERAL(43, 696, 17), // "getRawOutputColor"
QT_MOC_LITERAL(44, 714, 19), // "getRawOutputProfile"
QT_MOC_LITERAL(45, 734, 19), // "setRawOutputProfile"
QT_MOC_LITERAL(46, 754, 19), // "getRawCameraProfile"
QT_MOC_LITERAL(47, 774, 19), // "setRawCameraProfile"
QT_MOC_LITERAL(48, 794, 14), // "getRawUserFlip"
QT_MOC_LITERAL(49, 809, 17), // "getRawUserQuality"
QT_MOC_LITERAL(50, 827, 17), // "setRawUserQuality"
QT_MOC_LITERAL(51, 845, 15), // "getRawMedPasses"
QT_MOC_LITERAL(52, 861, 15), // "setRawMedPasses"
QT_MOC_LITERAL(53, 877, 20), // "getRawHighlightsMode"
QT_MOC_LITERAL(54, 898, 20), // "setRawHighlightsMode"
QT_MOC_LITERAL(55, 919, 11), // "getRawLevel"
QT_MOC_LITERAL(56, 931, 11), // "setRawLevel"
QT_MOC_LITERAL(57, 943, 22), // "getRawMaximumThreshold"
QT_MOC_LITERAL(58, 966, 22), // "setRawMaximumThreshold"
QT_MOC_LITERAL(59, 989, 13), // "isRawUseBlack"
QT_MOC_LITERAL(60, 1003, 14), // "setRawUseBlack"
QT_MOC_LITERAL(61, 1018, 15), // "getRawUserBlack"
QT_MOC_LITERAL(62, 1034, 15), // "setRawUserBlack"
QT_MOC_LITERAL(63, 1050, 18), // "isRawUseSaturation"
QT_MOC_LITERAL(64, 1069, 19), // "setRawUseSaturation"
QT_MOC_LITERAL(65, 1089, 20), // "getRawUserSaturation"
QT_MOC_LITERAL(66, 1110, 20), // "setRawUserSaturation"
QT_MOC_LITERAL(67, 1131, 22), // "isRawUseNoiseReduction"
QT_MOC_LITERAL(68, 1154, 23), // "setRawUseNoiseReduction"
QT_MOC_LITERAL(69, 1178, 29), // "getRawNoiseReductionThreshold"
QT_MOC_LITERAL(70, 1208, 29), // "setRawNoiseReductionThreshold"
QT_MOC_LITERAL(71, 1238, 11), // "getGuiTheme"
QT_MOC_LITERAL(72, 1250, 11), // "setGuiTheme"
QT_MOC_LITERAL(73, 1262, 13), // "isGuiDarkMode"
QT_MOC_LITERAL(74, 1276, 14), // "setGuiDarkMode"
QT_MOC_LITERAL(75, 1291, 10), // "applyTheme"
QT_MOC_LITERAL(76, 1302, 4), // "init"
QT_MOC_LITERAL(77, 1307, 10), // "getGuiLang"
QT_MOC_LITERAL(78, 1318, 10), // "setGuiLang"
QT_MOC_LITERAL(79, 1329, 20), // "getBatchHdrPathInput"
QT_MOC_LITERAL(80, 1350, 11), // "defaultPath"
QT_MOC_LITERAL(81, 1362, 21), // "getBatchHdrPathOutput"
QT_MOC_LITERAL(82, 1384, 20), // "setBatchHdrPathInput"
QT_MOC_LITERAL(83, 1405, 21), // "setBatchHdrPathOutput"
QT_MOC_LITERAL(84, 1427, 22), // "getBatchTmPathHdrInput"
QT_MOC_LITERAL(85, 1450, 25), // "getBatchTmPathTmoSettings"
QT_MOC_LITERAL(86, 1476, 23), // "getBatchTmPathLdrOutput"
QT_MOC_LITERAL(87, 1500, 20), // "getBatchTmNumThreads"
QT_MOC_LITERAL(88, 1521, 22), // "setBatchTmPathHdrInput"
QT_MOC_LITERAL(89, 1544, 25), // "setBatchTmPathTmoSettings"
QT_MOC_LITERAL(90, 1570, 23), // "setBatchTmPathLdrOutput"
QT_MOC_LITERAL(91, 1594, 20), // "setBatchTmNumThreads"
QT_MOC_LITERAL(92, 1615, 13), // "getNumThreads"
QT_MOC_LITERAL(93, 1629, 13), // "setNumThreads"
QT_MOC_LITERAL(94, 1643, 1), // "i"
QT_MOC_LITERAL(95, 1645, 10), // "getTempDir"
QT_MOC_LITERAL(96, 1656, 19), // "getDefaultPathHdrIn"
QT_MOC_LITERAL(97, 1676, 20), // "getDefaultPathHdrOut"
QT_MOC_LITERAL(98, 1697, 19), // "getDefaultPathLdrIn"
QT_MOC_LITERAL(99, 1717, 20), // "getDefaultPathLdrOut"
QT_MOC_LITERAL(100, 1738, 25), // "getDefaultPathTmoSettings"
QT_MOC_LITERAL(101, 1764, 31), // "getDefaultResponseCurveFilename"
QT_MOC_LITERAL(102, 1796, 10), // "setTempDir"
QT_MOC_LITERAL(103, 1807, 19), // "setDefaultPathHdrIn"
QT_MOC_LITERAL(104, 1827, 20), // "setDefaultPathHdrOut"
QT_MOC_LITERAL(105, 1848, 19), // "setDefaultPathLdrIn"
QT_MOC_LITERAL(106, 1868, 20), // "setDefaultPathLdrOut"
QT_MOC_LITERAL(107, 1889, 25), // "setDefaultPathTmoSettings"
QT_MOC_LITERAL(108, 1915, 31), // "setDefaultResponseCurveFilename"
QT_MOC_LITERAL(109, 1947, 25), // "getAlignImageStackOptions"
QT_MOC_LITERAL(110, 1973, 25), // "setAlignImageStackOptions"
QT_MOC_LITERAL(111, 1999, 7), // "verbose"
QT_MOC_LITERAL(112, 2007, 19), // "isShowFattalWarning"
QT_MOC_LITERAL(113, 2027, 20), // "setShowFattalWarning"
QT_MOC_LITERAL(114, 2048, 1), // "b"
QT_MOC_LITERAL(115, 2050, 23), // "isShowMissingEVsWarning"
QT_MOC_LITERAL(116, 2074, 24), // "setShowMissingEVsWarning"
QT_MOC_LITERAL(117, 2099, 24), // "getMainWindowToolBarMode"
QT_MOC_LITERAL(118, 2124, 24), // "setMainWindowToolBarMode"
QT_MOC_LITERAL(119, 2149, 20), // "isPreviewPanelActive"
QT_MOC_LITERAL(120, 2170, 21), // "setPreviewPanelActive"
QT_MOC_LITERAL(121, 2192, 15), // "getPreviewWidth"
QT_MOC_LITERAL(122, 2208, 15), // "setPreviewWidth"
QT_MOC_LITERAL(123, 2224, 24), // "isRealtimePreviewsActive"
QT_MOC_LITERAL(124, 2249, 25), // "setRealtimePreviewsActive"
QT_MOC_LITERAL(125, 2275, 24), // "getCameraProfileFileName"
QT_MOC_LITERAL(126, 2300, 24), // "setCameraProfileFileName"
QT_MOC_LITERAL(127, 2325, 25), // "getMonitorProfileFileName"
QT_MOC_LITERAL(128, 2351, 25), // "setMonitorProfileFileName"
QT_MOC_LITERAL(129, 2377, 25), // "getPrinterProfileFileName"
QT_MOC_LITERAL(130, 2403, 25), // "setPrinterProfileFileName"
QT_MOC_LITERAL(131, 2429, 19), // "getPreviewPanelMode"
QT_MOC_LITERAL(132, 2449, 19), // "setPreviewPanelMode"
QT_MOC_LITERAL(133, 2469, 12), // "getExportDir"
QT_MOC_LITERAL(134, 2482, 12), // "setExportDir"
QT_MOC_LITERAL(135, 2495, 3) // "dir"

    },
    "LuminanceOptions\0isRawFourColorRGB\0\0"
    "setRawFourColorRGB\0isRawDoNotUseFujiRotate\0"
    "setRawDoNotUseFujiRotate\0isRawUseChromaAber\0"
    "setRawUseChromaAber\0getRawAber0\0"
    "setRawAber0\0getRawAber1\0setRawAber1\0"
    "getRawAber2\0setRawAber2\0getRawAber3\0"
    "setRawAber3\0getRawGamm0\0setRawGamm0\0"
    "getRawGamm1\0setRawGamm1\0"
    "getRawWhiteBalanceMethod\0"
    "setRawWhiteBalanceMethod\0"
    "getRawTemperatureKelvin\0setRawTemperatureKelvin\0"
    "getRawGreen\0setRawGreen\0getRawUserMul0\0"
    "setRawUserMul0\0getRawUserMul1\0"
    "setRawUserMul1\0getRawUserMul2\0"
    "setRawUserMul2\0getRawUserMul3\0"
    "setRawUserMul3\0isRawAutoBrightness\0"
    "setRawAutoBrightness\0getRawAutoBrightnessThreshold\0"
    "setRawAutoBrightnessThreshold\0"
    "getRawBrightness\0setRawBrightness\0f\0"
    "getRawHalfSize\0setRawHalfSize\0"
    "getRawOutputColor\0getRawOutputProfile\0"
    "setRawOutputProfile\0getRawCameraProfile\0"
    "setRawCameraProfile\0getRawUserFlip\0"
    "getRawUserQuality\0setRawUserQuality\0"
    "getRawMedPasses\0setRawMedPasses\0"
    "getRawHighlightsMode\0setRawHighlightsMode\0"
    "getRawLevel\0setRawLevel\0getRawMaximumThreshold\0"
    "setRawMaximumThreshold\0isRawUseBlack\0"
    "setRawUseBlack\0getRawUserBlack\0"
    "setRawUserBlack\0isRawUseSaturation\0"
    "setRawUseSaturation\0getRawUserSaturation\0"
    "setRawUserSaturation\0isRawUseNoiseReduction\0"
    "setRawUseNoiseReduction\0"
    "getRawNoiseReductionThreshold\0"
    "setRawNoiseReductionThreshold\0getGuiTheme\0"
    "setGuiTheme\0isGuiDarkMode\0setGuiDarkMode\0"
    "applyTheme\0init\0getGuiLang\0setGuiLang\0"
    "getBatchHdrPathInput\0defaultPath\0"
    "getBatchHdrPathOutput\0setBatchHdrPathInput\0"
    "setBatchHdrPathOutput\0getBatchTmPathHdrInput\0"
    "getBatchTmPathTmoSettings\0"
    "getBatchTmPathLdrOutput\0getBatchTmNumThreads\0"
    "setBatchTmPathHdrInput\0setBatchTmPathTmoSettings\0"
    "setBatchTmPathLdrOutput\0setBatchTmNumThreads\0"
    "getNumThreads\0setNumThreads\0i\0getTempDir\0"
    "getDefaultPathHdrIn\0getDefaultPathHdrOut\0"
    "getDefaultPathLdrIn\0getDefaultPathLdrOut\0"
    "getDefaultPathTmoSettings\0"
    "getDefaultResponseCurveFilename\0"
    "setTempDir\0setDefaultPathHdrIn\0"
    "setDefaultPathHdrOut\0setDefaultPathLdrIn\0"
    "setDefaultPathLdrOut\0setDefaultPathTmoSettings\0"
    "setDefaultResponseCurveFilename\0"
    "getAlignImageStackOptions\0"
    "setAlignImageStackOptions\0verbose\0"
    "isShowFattalWarning\0setShowFattalWarning\0"
    "b\0isShowMissingEVsWarning\0"
    "setShowMissingEVsWarning\0"
    "getMainWindowToolBarMode\0"
    "setMainWindowToolBarMode\0isPreviewPanelActive\0"
    "setPreviewPanelActive\0getPreviewWidth\0"
    "setPreviewWidth\0isRealtimePreviewsActive\0"
    "setRealtimePreviewsActive\0"
    "getCameraProfileFileName\0"
    "setCameraProfileFileName\0"
    "getMonitorProfileFileName\0"
    "setMonitorProfileFileName\0"
    "getPrinterProfileFileName\0"
    "setPrinterProfileFileName\0getPreviewPanelMode\0"
    "setPreviewPanelMode\0getExportDir\0"
    "setExportDir\0dir"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LuminanceOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     130,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  664,    2, 0x0a /* Public */,
       3,    1,  665,    2, 0x0a /* Public */,
       4,    0,  668,    2, 0x0a /* Public */,
       5,    1,  669,    2, 0x0a /* Public */,
       6,    0,  672,    2, 0x0a /* Public */,
       7,    1,  673,    2, 0x0a /* Public */,
       8,    0,  676,    2, 0x0a /* Public */,
       9,    1,  677,    2, 0x0a /* Public */,
      10,    0,  680,    2, 0x0a /* Public */,
      11,    1,  681,    2, 0x0a /* Public */,
      12,    0,  684,    2, 0x0a /* Public */,
      13,    1,  685,    2, 0x0a /* Public */,
      14,    0,  688,    2, 0x0a /* Public */,
      15,    1,  689,    2, 0x0a /* Public */,
      16,    0,  692,    2, 0x0a /* Public */,
      17,    1,  693,    2, 0x0a /* Public */,
      18,    0,  696,    2, 0x0a /* Public */,
      19,    1,  697,    2, 0x0a /* Public */,
      20,    0,  700,    2, 0x0a /* Public */,
      21,    1,  701,    2, 0x0a /* Public */,
      22,    0,  704,    2, 0x0a /* Public */,
      23,    1,  705,    2, 0x0a /* Public */,
      24,    0,  708,    2, 0x0a /* Public */,
      25,    1,  709,    2, 0x0a /* Public */,
      26,    0,  712,    2, 0x0a /* Public */,
      27,    1,  713,    2, 0x0a /* Public */,
      28,    0,  716,    2, 0x0a /* Public */,
      29,    1,  717,    2, 0x0a /* Public */,
      30,    0,  720,    2, 0x0a /* Public */,
      31,    1,  721,    2, 0x0a /* Public */,
      32,    0,  724,    2, 0x0a /* Public */,
      33,    1,  725,    2, 0x0a /* Public */,
      34,    0,  728,    2, 0x0a /* Public */,
      35,    1,  729,    2, 0x0a /* Public */,
      36,    0,  732,    2, 0x0a /* Public */,
      37,    1,  733,    2, 0x0a /* Public */,
      38,    0,  736,    2, 0x0a /* Public */,
      39,    1,  737,    2, 0x0a /* Public */,
      41,    0,  740,    2, 0x0a /* Public */,
      42,    1,  741,    2, 0x0a /* Public */,
      43,    0,  744,    2, 0x0a /* Public */,
      44,    0,  745,    2, 0x0a /* Public */,
      45,    1,  746,    2, 0x0a /* Public */,
      46,    0,  749,    2, 0x0a /* Public */,
      47,    1,  750,    2, 0x0a /* Public */,
      48,    0,  753,    2, 0x0a /* Public */,
      49,    0,  754,    2, 0x0a /* Public */,
      50,    1,  755,    2, 0x0a /* Public */,
      51,    0,  758,    2, 0x0a /* Public */,
      52,    1,  759,    2, 0x0a /* Public */,
      53,    0,  762,    2, 0x0a /* Public */,
      54,    1,  763,    2, 0x0a /* Public */,
      55,    0,  766,    2, 0x0a /* Public */,
      56,    1,  767,    2, 0x0a /* Public */,
      57,    0,  770,    2, 0x0a /* Public */,
      58,    1,  771,    2, 0x0a /* Public */,
      59,    0,  774,    2, 0x0a /* Public */,
      60,    1,  775,    2, 0x0a /* Public */,
      61,    0,  778,    2, 0x0a /* Public */,
      62,    1,  779,    2, 0x0a /* Public */,
      63,    0,  782,    2, 0x0a /* Public */,
      64,    1,  783,    2, 0x0a /* Public */,
      65,    0,  786,    2, 0x0a /* Public */,
      66,    1,  787,    2, 0x0a /* Public */,
      67,    0,  790,    2, 0x0a /* Public */,
      68,    1,  791,    2, 0x0a /* Public */,
      69,    0,  794,    2, 0x0a /* Public */,
      70,    1,  795,    2, 0x0a /* Public */,
      71,    0,  798,    2, 0x0a /* Public */,
      72,    1,  799,    2, 0x0a /* Public */,
      73,    0,  802,    2, 0x0a /* Public */,
      74,    1,  803,    2, 0x0a /* Public */,
      75,    1,  806,    2, 0x0a /* Public */,
      77,    0,  809,    2, 0x0a /* Public */,
      78,    1,  810,    2, 0x0a /* Public */,
      79,    1,  813,    2, 0x0a /* Public */,
      79,    0,  816,    2, 0x2a /* Public | MethodCloned */,
      81,    1,  817,    2, 0x0a /* Public */,
      81,    0,  820,    2, 0x2a /* Public | MethodCloned */,
      82,    1,  821,    2, 0x0a /* Public */,
      83,    1,  824,    2, 0x0a /* Public */,
      84,    0,  827,    2, 0x0a /* Public */,
      85,    0,  828,    2, 0x0a /* Public */,
      86,    0,  829,    2, 0x0a /* Public */,
      87,    0,  830,    2, 0x0a /* Public */,
      88,    1,  831,    2, 0x0a /* Public */,
      89,    1,  834,    2, 0x0a /* Public */,
      90,    1,  837,    2, 0x0a /* Public */,
      91,    1,  840,    2, 0x0a /* Public */,
      92,    0,  843,    2, 0x0a /* Public */,
      93,    1,  844,    2, 0x0a /* Public */,
      95,    0,  847,    2, 0x0a /* Public */,
      96,    0,  848,    2, 0x0a /* Public */,
      97,    0,  849,    2, 0x0a /* Public */,
      98,    0,  850,    2, 0x0a /* Public */,
      99,    0,  851,    2, 0x0a /* Public */,
     100,    0,  852,    2, 0x0a /* Public */,
     101,    0,  853,    2, 0x0a /* Public */,
     102,    1,  854,    2, 0x0a /* Public */,
     103,    1,  857,    2, 0x0a /* Public */,
     104,    1,  860,    2, 0x0a /* Public */,
     105,    1,  863,    2, 0x0a /* Public */,
     106,    1,  866,    2, 0x0a /* Public */,
     107,    1,  869,    2, 0x0a /* Public */,
     108,    1,  872,    2, 0x0a /* Public */,
     109,    0,  875,    2, 0x0a /* Public */,
     110,    2,  876,    2, 0x0a /* Public */,
     110,    1,  881,    2, 0x2a /* Public | MethodCloned */,
     112,    0,  884,    2, 0x0a /* Public */,
     113,    1,  885,    2, 0x0a /* Public */,
     115,    0,  888,    2, 0x0a /* Public */,
     116,    1,  889,    2, 0x0a /* Public */,
     117,    0,  892,    2, 0x0a /* Public */,
     118,    1,  893,    2, 0x0a /* Public */,
     119,    0,  896,    2, 0x0a /* Public */,
     120,    1,  897,    2, 0x0a /* Public */,
     121,    0,  900,    2, 0x0a /* Public */,
     122,    1,  901,    2, 0x0a /* Public */,
     123,    0,  904,    2, 0x0a /* Public */,
     124,    1,  905,    2, 0x0a /* Public */,
     125,    0,  908,    2, 0x0a /* Public */,
     126,    1,  909,    2, 0x0a /* Public */,
     127,    0,  912,    2, 0x0a /* Public */,
     128,    1,  913,    2, 0x0a /* Public */,
     129,    0,  916,    2, 0x0a /* Public */,
     130,    1,  917,    2, 0x0a /* Public */,
     131,    0,  920,    2, 0x0a /* Public */,
     132,    1,  921,    2, 0x0a /* Public */,
     133,    0,  924,    2, 0x0a /* Public */,
     134,    1,  925,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Double,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,   40,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Int,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Float,
    QMetaType::Void, QMetaType::Float,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,   76,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QString, QMetaType::QString,   80,
    QMetaType::QString,
    QMetaType::QString, QMetaType::QString,   80,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Int,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,   94,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QStringList,
    QMetaType::Void, QMetaType::QStringList, QMetaType::Bool,    2,  111,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,  114,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,  114,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Int,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::QString,
    QMetaType::Void, QMetaType::QString,  135,

       0        // eod
};

void LuminanceOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LuminanceOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->isRawFourColorRGB();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->setRawFourColorRGB((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: { bool _r = _t->isRawDoNotUseFujiRotate();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->setRawDoNotUseFujiRotate((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: { bool _r = _t->isRawUseChromaAber();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setRawUseChromaAber((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: { double _r = _t->getRawAber0();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setRawAber0((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 8: { double _r = _t->getRawAber1();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setRawAber1((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 10: { double _r = _t->getRawAber2();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setRawAber2((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 12: { double _r = _t->getRawAber3();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setRawAber3((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 14: { double _r = _t->getRawGamm0();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->setRawGamm0((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 16: { double _r = _t->getRawGamm1();
            if (_a[0]) *reinterpret_cast< double*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->setRawGamm1((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 18: { int _r = _t->getRawWhiteBalanceMethod();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->setRawWhiteBalanceMethod((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: { int _r = _t->getRawTemperatureKelvin();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->setRawTemperatureKelvin((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: { float _r = _t->getRawGreen();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->setRawGreen((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 24: { float _r = _t->getRawUserMul0();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->setRawUserMul0((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 26: { float _r = _t->getRawUserMul1();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setRawUserMul1((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 28: { float _r = _t->getRawUserMul2();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 29: _t->setRawUserMul2((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 30: { float _r = _t->getRawUserMul3();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->setRawUserMul3((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 32: { bool _r = _t->isRawAutoBrightness();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->setRawAutoBrightness((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 34: { float _r = _t->getRawAutoBrightnessThreshold();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 35: _t->setRawAutoBrightnessThreshold((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 36: { float _r = _t->getRawBrightness();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->setRawBrightness((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 38: { int _r = _t->getRawHalfSize();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 39: _t->setRawHalfSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: { int _r = _t->getRawOutputColor();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 41: { QString _r = _t->getRawOutputProfile();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 42: _t->setRawOutputProfile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 43: { QString _r = _t->getRawCameraProfile();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->setRawCameraProfile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 45: { int _r = _t->getRawUserFlip();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 46: { int _r = _t->getRawUserQuality();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 47: _t->setRawUserQuality((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 48: { int _r = _t->getRawMedPasses();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 49: _t->setRawMedPasses((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 50: { int _r = _t->getRawHighlightsMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 51: _t->setRawHighlightsMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: { int _r = _t->getRawLevel();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 53: _t->setRawLevel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 54: { float _r = _t->getRawMaximumThreshold();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 55: _t->setRawMaximumThreshold((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 56: { bool _r = _t->isRawUseBlack();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 57: _t->setRawUseBlack((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 58: { int _r = _t->getRawUserBlack();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 59: _t->setRawUserBlack((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 60: { bool _r = _t->isRawUseSaturation();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 61: _t->setRawUseSaturation((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: { int _r = _t->getRawUserSaturation();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 63: _t->setRawUserSaturation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 64: { bool _r = _t->isRawUseNoiseReduction();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 65: _t->setRawUseNoiseReduction((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 66: { float _r = _t->getRawNoiseReductionThreshold();
            if (_a[0]) *reinterpret_cast< float*>(_a[0]) = std::move(_r); }  break;
        case 67: _t->setRawNoiseReductionThreshold((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 68: { QString _r = _t->getGuiTheme();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 69: _t->setGuiTheme((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 70: { bool _r = _t->isGuiDarkMode();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 71: _t->setGuiDarkMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 72: _t->applyTheme((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 73: { QString _r = _t->getGuiLang();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 74: _t->setGuiLang((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 75: { QString _r = _t->getBatchHdrPathInput((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 76: { QString _r = _t->getBatchHdrPathInput();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 77: { QString _r = _t->getBatchHdrPathOutput((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 78: { QString _r = _t->getBatchHdrPathOutput();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 79: _t->setBatchHdrPathInput((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 80: _t->setBatchHdrPathOutput((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 81: { QString _r = _t->getBatchTmPathHdrInput();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 82: { QString _r = _t->getBatchTmPathTmoSettings();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 83: { QString _r = _t->getBatchTmPathLdrOutput();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 84: { int _r = _t->getBatchTmNumThreads();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 85: _t->setBatchTmPathHdrInput((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 86: _t->setBatchTmPathTmoSettings((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 87: _t->setBatchTmPathLdrOutput((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 88: _t->setBatchTmNumThreads((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 89: { int _r = _t->getNumThreads();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 90: _t->setNumThreads((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 91: { QString _r = _t->getTempDir();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 92: { QString _r = _t->getDefaultPathHdrIn();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 93: { QString _r = _t->getDefaultPathHdrOut();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 94: { QString _r = _t->getDefaultPathLdrIn();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 95: { QString _r = _t->getDefaultPathLdrOut();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 96: { QString _r = _t->getDefaultPathTmoSettings();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 97: { QString _r = _t->getDefaultResponseCurveFilename();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 98: _t->setTempDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 99: _t->setDefaultPathHdrIn((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 100: _t->setDefaultPathHdrOut((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 101: _t->setDefaultPathLdrIn((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 102: _t->setDefaultPathLdrOut((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 103: _t->setDefaultPathTmoSettings((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 104: _t->setDefaultResponseCurveFilename((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 105: { QStringList _r = _t->getAlignImageStackOptions();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = std::move(_r); }  break;
        case 106: _t->setAlignImageStackOptions((*reinterpret_cast< const QStringList(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 107: _t->setAlignImageStackOptions((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 108: { bool _r = _t->isShowFattalWarning();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 109: _t->setShowFattalWarning((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 110: { bool _r = _t->isShowMissingEVsWarning();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 111: _t->setShowMissingEVsWarning((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 112: { int _r = _t->getMainWindowToolBarMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 113: _t->setMainWindowToolBarMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 114: { bool _r = _t->isPreviewPanelActive();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 115: _t->setPreviewPanelActive((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 116: { int _r = _t->getPreviewWidth();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 117: _t->setPreviewWidth((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 118: { bool _r = _t->isRealtimePreviewsActive();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 119: _t->setRealtimePreviewsActive((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 120: { QString _r = _t->getCameraProfileFileName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 121: _t->setCameraProfileFileName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 122: { QString _r = _t->getMonitorProfileFileName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 123: _t->setMonitorProfileFileName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 124: { QString _r = _t->getPrinterProfileFileName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 125: _t->setPrinterProfileFileName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 126: { int _r = _t->getPreviewPanelMode();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 127: _t->setPreviewPanelMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 128: { QString _r = _t->getExportDir();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 129: _t->setExportDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LuminanceOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_LuminanceOptions.data,
    qt_meta_data_LuminanceOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LuminanceOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LuminanceOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LuminanceOptions.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int LuminanceOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 130)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 130;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 130)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 130;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
