/********************************************************************************
** Form generated from reading UI file 'ProjectionsDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROJECTIONSDIALOG_H
#define UI_PROJECTIONSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ProjectionsDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *projectionGroupBox;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QHBoxLayout *hboxLayout1;
    QLabel *labelSrcProj;
    QComboBox *sourceProjection;
    QHBoxLayout *hboxLayout2;
    QLabel *labelDestProj;
    QComboBox *destProjection;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem;
    QLabel *labelAngles;
    QSpinBox *anglesSpinBox;
    QFrame *line_2;
    QHBoxLayout *hboxLayout4;
    QCheckBox *bilinearCheckBox;
    QSpacerItem *spacerItem1;
    QLabel *oversampleLabel;
    QSpinBox *oversampleSpinBox;
    QGroupBox *rotationGroupBox;
    QHBoxLayout *hboxLayout5;
    QVBoxLayout *vboxLayout2;
    QLabel *XrotLabel;
    QHBoxLayout *hboxLayout6;
    QSpacerItem *spacerItem2;
    QSpinBox *XrotSpinBox;
    QSpacerItem *spacerItem3;
    QSlider *XSlider;
    QFrame *line_3;
    QVBoxLayout *vboxLayout3;
    QLabel *YrotLabel;
    QHBoxLayout *hboxLayout7;
    QSpacerItem *spacerItem4;
    QSpinBox *YrotSpinBox;
    QSpacerItem *spacerItem5;
    QSlider *YSlider;
    QFrame *line_4;
    QVBoxLayout *vboxLayout4;
    QLabel *ZrotLabel;
    QHBoxLayout *hboxLayout8;
    QSpacerItem *spacerItem6;
    QSpinBox *ZrotSpinBox;
    QSpacerItem *spacerItem7;
    QSlider *ZSlider;
    QFrame *line;
    QHBoxLayout *hboxLayout9;
    QProgressBar *progressBar;
    QSpacerItem *spacerItem8;
    QPushButton *cancelButton;
    QPushButton *okButton;

    void setupUi(QDialog *ProjectionsDialog)
    {
        if (ProjectionsDialog->objectName().isEmpty())
            ProjectionsDialog->setObjectName(QString::fromUtf8("ProjectionsDialog"));
        ProjectionsDialog->resize(521, 377);
        vboxLayout = new QVBoxLayout(ProjectionsDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        projectionGroupBox = new QGroupBox(ProjectionsDialog);
        projectionGroupBox->setObjectName(QString::fromUtf8("projectionGroupBox"));
        vboxLayout1 = new QVBoxLayout(projectionGroupBox);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        labelSrcProj = new QLabel(projectionGroupBox);
        labelSrcProj->setObjectName(QString::fromUtf8("labelSrcProj"));

        hboxLayout1->addWidget(labelSrcProj);

        sourceProjection = new QComboBox(projectionGroupBox);
        sourceProjection->addItem(QString());
        sourceProjection->addItem(QString());
        sourceProjection->addItem(QString());
        sourceProjection->addItem(QString());
        sourceProjection->setObjectName(QString::fromUtf8("sourceProjection"));

        hboxLayout1->addWidget(sourceProjection);


        hboxLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        labelDestProj = new QLabel(projectionGroupBox);
        labelDestProj->setObjectName(QString::fromUtf8("labelDestProj"));

        hboxLayout2->addWidget(labelDestProj);

        destProjection = new QComboBox(projectionGroupBox);
        destProjection->addItem(QString());
        destProjection->addItem(QString());
        destProjection->addItem(QString());
        destProjection->addItem(QString());
        destProjection->setObjectName(QString::fromUtf8("destProjection"));

        hboxLayout2->addWidget(destProjection);


        hboxLayout->addLayout(hboxLayout2);


        vboxLayout1->addLayout(hboxLayout);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(81, 26, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem);

        labelAngles = new QLabel(projectionGroupBox);
        labelAngles->setObjectName(QString::fromUtf8("labelAngles"));
        labelAngles->setEnabled(false);

        hboxLayout3->addWidget(labelAngles);

        anglesSpinBox = new QSpinBox(projectionGroupBox);
        anglesSpinBox->setObjectName(QString::fromUtf8("anglesSpinBox"));
        anglesSpinBox->setEnabled(false);
        anglesSpinBox->setMaximum(360);
        anglesSpinBox->setValue(360);

        hboxLayout3->addWidget(anglesSpinBox);


        vboxLayout1->addLayout(hboxLayout3);

        line_2 = new QFrame(projectionGroupBox);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        vboxLayout1->addWidget(line_2);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        bilinearCheckBox = new QCheckBox(projectionGroupBox);
        bilinearCheckBox->setObjectName(QString::fromUtf8("bilinearCheckBox"));
        bilinearCheckBox->setChecked(true);

        hboxLayout4->addWidget(bilinearCheckBox);

        spacerItem1 = new QSpacerItem(101, 26, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout4->addItem(spacerItem1);

        oversampleLabel = new QLabel(projectionGroupBox);
        oversampleLabel->setObjectName(QString::fromUtf8("oversampleLabel"));

        hboxLayout4->addWidget(oversampleLabel);

        oversampleSpinBox = new QSpinBox(projectionGroupBox);
        oversampleSpinBox->setObjectName(QString::fromUtf8("oversampleSpinBox"));
        oversampleSpinBox->setMinimum(1);
        oversampleSpinBox->setMaximum(20);

        hboxLayout4->addWidget(oversampleSpinBox);


        vboxLayout1->addLayout(hboxLayout4);


        vboxLayout->addWidget(projectionGroupBox);

        rotationGroupBox = new QGroupBox(ProjectionsDialog);
        rotationGroupBox->setObjectName(QString::fromUtf8("rotationGroupBox"));
        hboxLayout5 = new QHBoxLayout(rotationGroupBox);
        hboxLayout5->setSpacing(6);
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        hboxLayout5->setContentsMargins(9, 9, 9, 9);
        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        XrotLabel = new QLabel(rotationGroupBox);
        XrotLabel->setObjectName(QString::fromUtf8("XrotLabel"));

        vboxLayout2->addWidget(XrotLabel);

        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setSpacing(6);
        hboxLayout6->setObjectName(QString::fromUtf8("hboxLayout6"));
        hboxLayout6->setContentsMargins(0, 0, 0, 0);
        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout6->addItem(spacerItem2);

        XrotSpinBox = new QSpinBox(rotationGroupBox);
        XrotSpinBox->setObjectName(QString::fromUtf8("XrotSpinBox"));
        XrotSpinBox->setMinimum(0);
        XrotSpinBox->setMaximum(360);

        hboxLayout6->addWidget(XrotSpinBox);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout6->addItem(spacerItem3);


        vboxLayout2->addLayout(hboxLayout6);

        XSlider = new QSlider(rotationGroupBox);
        XSlider->setObjectName(QString::fromUtf8("XSlider"));
        XSlider->setMaximum(360);
        XSlider->setOrientation(Qt::Horizontal);
        XSlider->setTickPosition(QSlider::TicksBelow);
        XSlider->setTickInterval(90);

        vboxLayout2->addWidget(XSlider);


        hboxLayout5->addLayout(vboxLayout2);

        line_3 = new QFrame(rotationGroupBox);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::VLine);
        line_3->setFrameShadow(QFrame::Sunken);

        hboxLayout5->addWidget(line_3);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        YrotLabel = new QLabel(rotationGroupBox);
        YrotLabel->setObjectName(QString::fromUtf8("YrotLabel"));

        vboxLayout3->addWidget(YrotLabel);

        hboxLayout7 = new QHBoxLayout();
        hboxLayout7->setSpacing(6);
        hboxLayout7->setObjectName(QString::fromUtf8("hboxLayout7"));
        hboxLayout7->setContentsMargins(0, 0, 0, 0);
        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout7->addItem(spacerItem4);

        YrotSpinBox = new QSpinBox(rotationGroupBox);
        YrotSpinBox->setObjectName(QString::fromUtf8("YrotSpinBox"));
        YrotSpinBox->setMinimum(0);
        YrotSpinBox->setMaximum(360);

        hboxLayout7->addWidget(YrotSpinBox);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout7->addItem(spacerItem5);


        vboxLayout3->addLayout(hboxLayout7);

        YSlider = new QSlider(rotationGroupBox);
        YSlider->setObjectName(QString::fromUtf8("YSlider"));
        YSlider->setMaximum(360);
        YSlider->setOrientation(Qt::Horizontal);
        YSlider->setTickPosition(QSlider::TicksBelow);
        YSlider->setTickInterval(90);

        vboxLayout3->addWidget(YSlider);


        hboxLayout5->addLayout(vboxLayout3);

        line_4 = new QFrame(rotationGroupBox);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::VLine);
        line_4->setFrameShadow(QFrame::Sunken);

        hboxLayout5->addWidget(line_4);

        vboxLayout4 = new QVBoxLayout();
        vboxLayout4->setSpacing(6);
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        vboxLayout4->setContentsMargins(0, 0, 0, 0);
        ZrotLabel = new QLabel(rotationGroupBox);
        ZrotLabel->setObjectName(QString::fromUtf8("ZrotLabel"));

        vboxLayout4->addWidget(ZrotLabel);

        hboxLayout8 = new QHBoxLayout();
        hboxLayout8->setSpacing(6);
        hboxLayout8->setObjectName(QString::fromUtf8("hboxLayout8"));
        hboxLayout8->setContentsMargins(0, 0, 0, 0);
        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout8->addItem(spacerItem6);

        ZrotSpinBox = new QSpinBox(rotationGroupBox);
        ZrotSpinBox->setObjectName(QString::fromUtf8("ZrotSpinBox"));
        ZrotSpinBox->setMinimum(0);
        ZrotSpinBox->setMaximum(360);

        hboxLayout8->addWidget(ZrotSpinBox);

        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout8->addItem(spacerItem7);


        vboxLayout4->addLayout(hboxLayout8);

        ZSlider = new QSlider(rotationGroupBox);
        ZSlider->setObjectName(QString::fromUtf8("ZSlider"));
        ZSlider->setMaximum(360);
        ZSlider->setOrientation(Qt::Horizontal);
        ZSlider->setTickPosition(QSlider::TicksBelow);
        ZSlider->setTickInterval(90);

        vboxLayout4->addWidget(ZSlider);


        hboxLayout5->addLayout(vboxLayout4);


        vboxLayout->addWidget(rotationGroupBox);

        line = new QFrame(ProjectionsDialog);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line);

        hboxLayout9 = new QHBoxLayout();
        hboxLayout9->setSpacing(6);
        hboxLayout9->setObjectName(QString::fromUtf8("hboxLayout9"));
        hboxLayout9->setContentsMargins(0, 0, 0, 0);
        progressBar = new QProgressBar(ProjectionsDialog);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setValue(24);

        hboxLayout9->addWidget(progressBar);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout9->addItem(spacerItem8);

        cancelButton = new QPushButton(ProjectionsDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        hboxLayout9->addWidget(cancelButton);

        okButton = new QPushButton(ProjectionsDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        okButton->setAutoDefault(false);

        hboxLayout9->addWidget(okButton);


        vboxLayout->addLayout(hboxLayout9);


        retranslateUi(ProjectionsDialog);
        QObject::connect(XrotSpinBox, SIGNAL(valueChanged(int)), XSlider, SLOT(setValue(int)));
        QObject::connect(XSlider, SIGNAL(valueChanged(int)), XrotSpinBox, SLOT(setValue(int)));
        QObject::connect(YSlider, SIGNAL(valueChanged(int)), YrotSpinBox, SLOT(setValue(int)));
        QObject::connect(YrotSpinBox, SIGNAL(valueChanged(int)), YSlider, SLOT(setValue(int)));
        QObject::connect(ZrotSpinBox, SIGNAL(valueChanged(int)), ZSlider, SLOT(setValue(int)));
        QObject::connect(ZSlider, SIGNAL(valueChanged(int)), ZrotSpinBox, SLOT(setValue(int)));
        QObject::connect(cancelButton, SIGNAL(clicked()), ProjectionsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(ProjectionsDialog);
    } // setupUi

    void retranslateUi(QDialog *ProjectionsDialog)
    {
        ProjectionsDialog->setWindowTitle(QCoreApplication::translate("ProjectionsDialog", "Projective Transformation", nullptr));
        projectionGroupBox->setTitle(QCoreApplication::translate("ProjectionsDialog", "Projections", nullptr));
        labelSrcProj->setText(QCoreApplication::translate("ProjectionsDialog", "Source Projection:", nullptr));
        sourceProjection->setItemText(0, QCoreApplication::translate("ProjectionsDialog", "Polar", nullptr));
        sourceProjection->setItemText(1, QCoreApplication::translate("ProjectionsDialog", "Angular", nullptr));
        sourceProjection->setItemText(2, QCoreApplication::translate("ProjectionsDialog", "Cylindrical", nullptr));
        sourceProjection->setItemText(3, QCoreApplication::translate("ProjectionsDialog", "Mirror Ball", nullptr));

        labelDestProj->setText(QCoreApplication::translate("ProjectionsDialog", "Destination Projection:", nullptr));
        destProjection->setItemText(0, QCoreApplication::translate("ProjectionsDialog", "Polar", nullptr));
        destProjection->setItemText(1, QCoreApplication::translate("ProjectionsDialog", "Angular", nullptr));
        destProjection->setItemText(2, QCoreApplication::translate("ProjectionsDialog", "Cylindrical", nullptr));
        destProjection->setItemText(3, QCoreApplication::translate("ProjectionsDialog", "Mirror Ball", nullptr));

        labelAngles->setText(QCoreApplication::translate("ProjectionsDialog", "Angles:", nullptr));
        anglesSpinBox->setSuffix(QCoreApplication::translate("ProjectionsDialog", "\302\260", nullptr));
        bilinearCheckBox->setText(QCoreApplication::translate("ProjectionsDialog", "Bilinear Interpolation", nullptr));
        oversampleLabel->setText(QCoreApplication::translate("ProjectionsDialog", "Oversample factor:", nullptr));
        rotationGroupBox->setTitle(QCoreApplication::translate("ProjectionsDialog", "Rotation (degrees)", nullptr));
        XrotLabel->setText(QCoreApplication::translate("ProjectionsDialog", "Yaw:", nullptr));
        XrotSpinBox->setSuffix(QCoreApplication::translate("ProjectionsDialog", "\302\260", nullptr));
        YrotLabel->setText(QCoreApplication::translate("ProjectionsDialog", "Pitch:", nullptr));
        YrotSpinBox->setSuffix(QCoreApplication::translate("ProjectionsDialog", "\302\260", nullptr));
        ZrotLabel->setText(QCoreApplication::translate("ProjectionsDialog", "Roll:", nullptr));
        ZrotSpinBox->setSuffix(QCoreApplication::translate("ProjectionsDialog", "\302\260", nullptr));
        cancelButton->setText(QCoreApplication::translate("ProjectionsDialog", "&Cancel", nullptr));
        okButton->setText(QCoreApplication::translate("ProjectionsDialog", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProjectionsDialog: public Ui_ProjectionsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROJECTIONSDIALOG_H
