/********************************************************************************
** Form generated from reading UI file 'TransplantExifDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSPLANTEXIFDIALOG_H
#define UI_TRANSPLANTEXIFDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_TransplantExifDialog
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout;
    QVBoxLayout *_2;
    QListWidget *leftlist;
    QVBoxLayout *vboxLayout1;
    QToolButton *addleft;
    QToolButton *removeleft;
    QToolButton *moveup_left_button;
    QToolButton *movedown_left_button;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_2;
    QVBoxLayout *_3;
    QListWidget *rightlist;
    QVBoxLayout *vboxLayout2;
    QToolButton *addright;
    QToolButton *removeright;
    QToolButton *moveup_right_button;
    QToolButton *movedown_right_button;
    QSpacerItem *spacerItem1;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout3;
    QListView *Log_Widget;
    QHBoxLayout *hboxLayout1;
    QLabel *filterLabel1;
    QComboBox *filterComboBox;
    QFrame *line;
    QLabel *filterLabel2;
    QLineEdit *filterLineEdit;
    QToolButton *clearTextToolButton;
    QCheckBox *checkBox_dont_overwrite;
    QHBoxLayout *hboxLayout2;
    QProgressBar *progressBar;
    QSpacerItem *spacerItem2;
    QPushButton *cancelbutton;
    QPushButton *TransplantButton;

    void setupUi(QDialog *TransplantExifDialog)
    {
        if (TransplantExifDialog->objectName().isEmpty())
            TransplantExifDialog->setObjectName(QString::fromUtf8("TransplantExifDialog"));
        TransplantExifDialog->resize(708, 486);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/images/luminance-hdr.png"), QSize(), QIcon::Normal, QIcon::Off);
        TransplantExifDialog->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(TransplantExifDialog);
        vboxLayout->setSpacing(5);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(5, 5, 5, 5);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        groupBox_2 = new QGroupBox(TransplantExifDialog);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        gridLayout = new QGridLayout(groupBox_2);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        _2 = new QVBoxLayout();
        _2->setSpacing(6);
        _2->setObjectName(QString::fromUtf8("_2"));
        _2->setContentsMargins(0, 0, 0, 0);
        leftlist = new QListWidget(groupBox_2);
        leftlist->setObjectName(QString::fromUtf8("leftlist"));
        leftlist->setEditTriggers(QAbstractItemView::NoEditTriggers);
        leftlist->setSelectionMode(QAbstractItemView::ContiguousSelection);

        _2->addWidget(leftlist);


        gridLayout->addLayout(_2, 0, 0, 1, 1);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        addleft = new QToolButton(groupBox_2);
        addleft->setObjectName(QString::fromUtf8("addleft"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("list-add");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        addleft->setIcon(icon1);
        addleft->setIconSize(QSize(24, 24));

        vboxLayout1->addWidget(addleft);

        removeleft = new QToolButton(groupBox_2);
        removeleft->setObjectName(QString::fromUtf8("removeleft"));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("list-remove");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        removeleft->setIcon(icon2);
        removeleft->setIconSize(QSize(24, 24));

        vboxLayout1->addWidget(removeleft);

        moveup_left_button = new QToolButton(groupBox_2);
        moveup_left_button->setObjectName(QString::fromUtf8("moveup_left_button"));
        QIcon icon3;
        iconThemeName = QString::fromUtf8("arrow-up");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        moveup_left_button->setIcon(icon3);
        moveup_left_button->setIconSize(QSize(24, 24));

        vboxLayout1->addWidget(moveup_left_button);

        movedown_left_button = new QToolButton(groupBox_2);
        movedown_left_button->setObjectName(QString::fromUtf8("movedown_left_button"));
        QIcon icon4;
        iconThemeName = QString::fromUtf8("arrow-down");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        movedown_left_button->setIcon(icon4);
        movedown_left_button->setIconSize(QSize(24, 24));

        vboxLayout1->addWidget(movedown_left_button);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem);


        gridLayout->addLayout(vboxLayout1, 0, 1, 1, 1);


        hboxLayout->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(TransplantExifDialog);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        gridLayout_2 = new QGridLayout(groupBox_3);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        _3 = new QVBoxLayout();
        _3->setSpacing(6);
        _3->setObjectName(QString::fromUtf8("_3"));
        _3->setContentsMargins(0, 0, 0, 0);
        rightlist = new QListWidget(groupBox_3);
        rightlist->setObjectName(QString::fromUtf8("rightlist"));
        rightlist->setEditTriggers(QAbstractItemView::NoEditTriggers);
        rightlist->setSelectionMode(QAbstractItemView::ContiguousSelection);

        _3->addWidget(rightlist);


        gridLayout_2->addLayout(_3, 0, 1, 1, 1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        addright = new QToolButton(groupBox_3);
        addright->setObjectName(QString::fromUtf8("addright"));
        addright->setIcon(icon1);
        addright->setIconSize(QSize(24, 24));

        vboxLayout2->addWidget(addright);

        removeright = new QToolButton(groupBox_3);
        removeright->setObjectName(QString::fromUtf8("removeright"));
        removeright->setIcon(icon2);
        removeright->setIconSize(QSize(24, 24));

        vboxLayout2->addWidget(removeright);

        moveup_right_button = new QToolButton(groupBox_3);
        moveup_right_button->setObjectName(QString::fromUtf8("moveup_right_button"));
        moveup_right_button->setIcon(icon3);
        moveup_right_button->setIconSize(QSize(24, 24));

        vboxLayout2->addWidget(moveup_right_button);

        movedown_right_button = new QToolButton(groupBox_3);
        movedown_right_button->setObjectName(QString::fromUtf8("movedown_right_button"));
        movedown_right_button->setIcon(icon4);
        movedown_right_button->setIconSize(QSize(24, 24));

        vboxLayout2->addWidget(movedown_right_button);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem1);


        gridLayout_2->addLayout(vboxLayout2, 0, 0, 1, 1);


        hboxLayout->addWidget(groupBox_3);


        vboxLayout->addLayout(hboxLayout);

        groupBox = new QGroupBox(TransplantExifDialog);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        vboxLayout3 = new QVBoxLayout(groupBox);
        vboxLayout3->setSpacing(5);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        vboxLayout3->setContentsMargins(2, 2, 2, 2);
        Log_Widget = new QListView(groupBox);
        Log_Widget->setObjectName(QString::fromUtf8("Log_Widget"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Log_Widget->sizePolicy().hasHeightForWidth());
        Log_Widget->setSizePolicy(sizePolicy);
        Log_Widget->setMaximumSize(QSize(16777215, 110));
        Log_Widget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        Log_Widget->setSelectionMode(QAbstractItemView::ContiguousSelection);
        Log_Widget->setResizeMode(QListView::Adjust);

        vboxLayout3->addWidget(Log_Widget);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        filterLabel1 = new QLabel(groupBox);
        filterLabel1->setObjectName(QString::fromUtf8("filterLabel1"));

        hboxLayout1->addWidget(filterLabel1);

        filterComboBox = new QComboBox(groupBox);
        filterComboBox->addItem(QString());
        filterComboBox->addItem(QString());
        filterComboBox->setObjectName(QString::fromUtf8("filterComboBox"));

        hboxLayout1->addWidget(filterComboBox);

        line = new QFrame(groupBox);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::VLine);
        line->setFrameShadow(QFrame::Sunken);

        hboxLayout1->addWidget(line);

        filterLabel2 = new QLabel(groupBox);
        filterLabel2->setObjectName(QString::fromUtf8("filterLabel2"));

        hboxLayout1->addWidget(filterLabel2);

        filterLineEdit = new QLineEdit(groupBox);
        filterLineEdit->setObjectName(QString::fromUtf8("filterLineEdit"));

        hboxLayout1->addWidget(filterLineEdit);

        clearTextToolButton = new QToolButton(groupBox);
        clearTextToolButton->setObjectName(QString::fromUtf8("clearTextToolButton"));
        clearTextToolButton->setEnabled(false);
        QIcon icon5;
        iconThemeName = QString::fromUtf8("edit-clear");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        clearTextToolButton->setIcon(icon5);

        hboxLayout1->addWidget(clearTextToolButton);


        vboxLayout3->addLayout(hboxLayout1);


        vboxLayout->addWidget(groupBox);

        checkBox_dont_overwrite = new QCheckBox(TransplantExifDialog);
        checkBox_dont_overwrite->setObjectName(QString::fromUtf8("checkBox_dont_overwrite"));
        checkBox_dont_overwrite->setChecked(true);

        vboxLayout->addWidget(checkBox_dont_overwrite);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        progressBar = new QProgressBar(TransplantExifDialog);
        progressBar->setObjectName(QString::fromUtf8("progressBar"));
        progressBar->setValue(0);
        progressBar->setAlignment(Qt::AlignHCenter);
        progressBar->setOrientation(Qt::Horizontal);

        hboxLayout2->addWidget(progressBar);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem2);

        cancelbutton = new QPushButton(TransplantExifDialog);
        cancelbutton->setObjectName(QString::fromUtf8("cancelbutton"));
        cancelbutton->setAutoDefault(false);

        hboxLayout2->addWidget(cancelbutton);

        TransplantButton = new QPushButton(TransplantExifDialog);
        TransplantButton->setObjectName(QString::fromUtf8("TransplantButton"));
        TransplantButton->setEnabled(false);
        TransplantButton->setAutoDefault(false);

        hboxLayout2->addWidget(TransplantButton);


        vboxLayout->addLayout(hboxLayout2);

#if QT_CONFIG(shortcut)
        filterLabel1->setBuddy(filterComboBox);
        filterLabel2->setBuddy(filterLineEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(leftlist, rightlist);
        QWidget::setTabOrder(rightlist, addleft);
        QWidget::setTabOrder(addleft, removeleft);
        QWidget::setTabOrder(removeleft, moveup_left_button);
        QWidget::setTabOrder(moveup_left_button, movedown_left_button);
        QWidget::setTabOrder(movedown_left_button, addright);
        QWidget::setTabOrder(addright, removeright);
        QWidget::setTabOrder(removeright, moveup_right_button);
        QWidget::setTabOrder(moveup_right_button, movedown_right_button);
        QWidget::setTabOrder(movedown_right_button, Log_Widget);
        QWidget::setTabOrder(Log_Widget, filterComboBox);
        QWidget::setTabOrder(filterComboBox, filterLineEdit);
        QWidget::setTabOrder(filterLineEdit, clearTextToolButton);
        QWidget::setTabOrder(clearTextToolButton, checkBox_dont_overwrite);
        QWidget::setTabOrder(checkBox_dont_overwrite, cancelbutton);
        QWidget::setTabOrder(cancelbutton, TransplantButton);

        retranslateUi(TransplantExifDialog);
        QObject::connect(cancelbutton, SIGNAL(clicked()), TransplantExifDialog, SLOT(reject()));
        QObject::connect(clearTextToolButton, SIGNAL(clicked()), filterLineEdit, SLOT(clear()));

        leftlist->setCurrentRow(-1);


        QMetaObject::connectSlotsByName(TransplantExifDialog);
    } // setupUi

    void retranslateUi(QDialog *TransplantExifDialog)
    {
        TransplantExifDialog->setWindowTitle(QCoreApplication::translate("TransplantExifDialog", "Copy Exif data", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("TransplantExifDialog", "From", nullptr));
#if QT_CONFIG(tooltip)
        addleft->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Append files to the list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        removeleft->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Remove selected file(s) from the list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        moveup_left_button->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Move up selected file(s)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        movedown_left_button->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Move down selected file(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_3->setTitle(QCoreApplication::translate("TransplantExifDialog", "To", nullptr));
#if QT_CONFIG(tooltip)
        addright->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Append files to the list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        removeright->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Remove selected file(s) from the list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        moveup_right_button->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Move up selected file(s)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        movedown_right_button->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Move down selected file(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("TransplantExifDialog", "Log", nullptr));
#if QT_CONFIG(tooltip)
        Log_Widget->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Exif operations report", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel1->setText(QCoreApplication::translate("TransplantExifDialog", "Show onl&y:", nullptr));
        filterComboBox->setItemText(0, QCoreApplication::translate("TransplantExifDialog", "All messages", nullptr));
        filterComboBox->setItemText(1, QCoreApplication::translate("TransplantExifDialog", "Errors only", nullptr));

#if QT_CONFIG(tooltip)
        filterComboBox->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Filter messages based on severity", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel2->setText(QCoreApplication::translate("TransplantExifDialog", "Filter &log messages:", nullptr));
#if QT_CONFIG(tooltip)
        clearTextToolButton->setToolTip(QCoreApplication::translate("TransplantExifDialog", "Clear filter text", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBox_dont_overwrite->setText(QCoreApplication::translate("TransplantExifDialog", "Keep existing Exif tags in destination file", nullptr));
        cancelbutton->setText(QCoreApplication::translate("TransplantExifDialog", "&Cancel", nullptr));
        TransplantButton->setText(QCoreApplication::translate("TransplantExifDialog", "&Start", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TransplantExifDialog: public Ui_TransplantExifDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRANSPLANTEXIFDIALOG_H
