/********************************************************************************
** Form generated from reading UI file 'PreferencesDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCESDIALOG_H
#define UI_PREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PreferencesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_16;
    QToolButton *toolButtonTM;
    QSpacerItem *horizontalSpacer_17;
    QSpacerItem *horizontalSpacer_18;
    QToolButton *toolButtonRAW;
    QToolButton *toolButtonCMS;
    QToolButton *toolButtonExtTool;
    QToolButton *toolButtonInterface;
    QToolButton *toolButtonQueue;
    QStackedWidget *stackedPagesWidget;
    QWidget *stackedPagesWidgetPage1_2;
    QGridLayout *gridLayout_12;
    QGroupBox *interfaceGroupBox;
    QFormLayout *formLayout_7;
    QLabel *label_9;
    QComboBox *languageComboBox;
    QLabel *label_4;
    QSpinBox *previewsWidthSpinBox;
    QCheckBox *checkBoxTMOWindowsPreviewPanel;
    QCheckBox *chkPortableMode;
    QHBoxLayout *horizontalLayout_21;
    QComboBox *themeComboBox;
    QCheckBox *chkDarkMode;
    QLabel *label;
    QWidget *page;
    QGridLayout *gridLayout_7;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_8;
    QLabel *label_2;
    QLabel *label_14;
    QSpacerItem *verticalSpacer_3;
    QComboBox *exportFormatCombo;
    QToolButton *exportFormatToolbutton;
    QLineEdit *exportDirectoryEdit;
    QToolButton *exportFileButton;
    QWidget *stackedPagesWidgetPage3_2;
    QGridLayout *gridLayout_14;
    QGroupBox *tonemappingGroupBox;
    QGridLayout *gridLayout;
    QLabel *label_11;
    QLabel *numThreadsLabel;
    QSpinBox *numThreadspinBox;
    QSpacerItem *verticalSpacer;
    QLineEdit *lineEditTempPath;
    QToolButton *chooseCachePathButton;
    QWidget *stackedPagesWidgetPage4_2;
    QGridLayout *gridLayout_6;
    QTabWidget *tabWidget;
    QWidget *tabWidgetPage1;
    QGridLayout *gridLayout_2;
    QFormLayout *formLayout;
    QLabel *qualityLabel;
    QHBoxLayout *horizontalLayout;
    QComboBox *user_qual_comboBox;
    QToolButton *user_qual_toolButton;
    QLabel *medianLabel;
    QHBoxLayout *horizontalLayout_2;
    QSlider *med_passes_horizontalSlider;
    QSpinBox *med_passes_spinBox;
    QToolButton *med_passes_toolButton;
    QCheckBox *do_not_use_fuji_rotate_CB;
    QCheckBox *four_color_rgb_CB;
    QWidget *tabWidgetPage2;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer_4;
    QFormLayout *formLayout_3;
    QLabel *label_3;
    QHBoxLayout *horizontalLayout_3;
    QComboBox *wb_method_comboBox;
    QToolButton *wb_method_toolButton;
    QSpacerItem *horizontalSpacer_4;
    QLabel *TK_label;
    QHBoxLayout *horizontalLayout_5;
    QSlider *TK_horizontalSlider;
    QSpinBox *TK_spinBox;
    QToolButton *TK_toolButton;
    QSpacerItem *horizontalSpacer_5;
    QLabel *green_label;
    QHBoxLayout *horizontalLayout_6;
    QSlider *green_horizontalSlider;
    QDoubleSpinBox *green_doubleSpinBox;
    QToolButton *green_toolButton;
    QSpacerItem *horizontalSpacer;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout_7;
    QComboBox *highlights_comboBox;
    QToolButton *highlights_toolButton;
    QSpacerItem *horizontalSpacer_6;
    QLabel *label_7;
    QHBoxLayout *horizontalLayout_4;
    QSlider *level_horizontalSlider;
    QSpinBox *level_spinBox;
    QToolButton *level_toolButton;
    QSpacerItem *horizontalSpacer_7;
    QCheckBox *false_colors_CB;
    QCheckBox *auto_bright_CB;
    QLabel *label_8;
    QHBoxLayout *horizontalLayout_10;
    QSlider *brightness_horizontalSlider;
    QDoubleSpinBox *brightness_doubleSpinBox;
    QToolButton *brightness_toolButton;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *use_black_CB;
    QHBoxLayout *horizontalLayout_9;
    QSlider *user_black_horizontalSlider;
    QSpinBox *user_black_spinBox;
    QToolButton *user_black_toolButton;
    QSpacerItem *horizontalSpacer_8;
    QCheckBox *use_sat_CB;
    QHBoxLayout *horizontalLayout_8;
    QSlider *user_sat_horizontalSlider;
    QSpinBox *user_sat_spinBox;
    QToolButton *user_sat_toolButton;
    QSpacerItem *horizontalSpacer_3;
    QWidget *tabWidgetPage3;
    QGridLayout *gridLayout_4;
    QSpacerItem *verticalSpacer_2;
    QFormLayout *formLayout_4;
    QCheckBox *use_noise_CB;
    QHBoxLayout *horizontalLayout_11;
    QSlider *threshold_horizontalSlider;
    QSpinBox *threshold_spinBox;
    QToolButton *threshold_toolButton;
    QSpacerItem *horizontalSpacer_9;
    QLabel *threshold_label;
    QCheckBox *use_chroma_CB;
    QHBoxLayout *horizontalLayout_12;
    QSlider *red_horizontalSlider;
    QDoubleSpinBox *red_doubleSpinBox;
    QToolButton *red_toolButton;
    QSpacerItem *horizontalSpacer_10;
    QHBoxLayout *horizontalLayout_13;
    QSlider *blue_horizontalSlider;
    QDoubleSpinBox *blue_doubleSpinBox;
    QToolButton *blue_toolButton;
    QSpacerItem *horizontalSpacer_11;
    QLabel *red_label;
    QLabel *blue_label;
    QWidget *stackedPagesWidgetPage6_2;
    QGridLayout *gridLayout_5;
    QGroupBox *iccGroupBox;
    QGridLayout *gridLayout_9;
    QToolButton *monitor_toolButton;
    QToolButton *printer_toolButton;
    QToolButton *camera_toolButton;
    QLineEdit *camera_lineEdit;
    QLineEdit *printer_lineEdit;
    QLabel *label_15;
    QLabel *label_21;
    QLabel *label_13;
    QLabel *label_16;
    QLabel *label_17;
    QLineEdit *monitor_lineEdit;
    QLabel *label_12;
    QSpacerItem *verticalSpacer_5;
    QWidget *stackedPagesWidgetPage5_2;
    QGridLayout *gridLayout_15;
    QGroupBox *alignGroupBox;
    QFormLayout *formLayout_8;
    QLabel *label_5;
    QLineEdit *aisParamsLineEdit;
    QHBoxLayout *buttonArea_2;
    QSpacerItem *spacer_3;
    QPushButton *cancelButton;
    QPushButton *okButton;

    void setupUi(QWidget *PreferencesDialog)
    {
        if (PreferencesDialog->objectName().isEmpty())
            PreferencesDialog->setObjectName(QString::fromUtf8("PreferencesDialog"));
        PreferencesDialog->resize(1132, 650);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PreferencesDialog->sizePolicy().hasHeightForWidth());
        PreferencesDialog->setSizePolicy(sizePolicy);
        PreferencesDialog->setMaximumSize(QSize(16777215, 650));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        PreferencesDialog->setWindowIcon(icon);
        verticalLayout = new QVBoxLayout(PreferencesDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout_16 = new QGridLayout();
        gridLayout_16->setObjectName(QString::fromUtf8("gridLayout_16"));
        toolButtonTM = new QToolButton(PreferencesDialog);
        toolButtonTM->setObjectName(QString::fromUtf8("toolButtonTM"));
        toolButtonTM->setMinimumSize(QSize(150, 0));
        toolButtonTM->setMaximumSize(QSize(150, 16777215));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/program-icons/tonemap"), QSize(), QIcon::Normal, QIcon::Off);
        toolButtonTM->setIcon(icon1);
        toolButtonTM->setIconSize(QSize(48, 48));
        toolButtonTM->setCheckable(true);
        toolButtonTM->setAutoExclusive(true);
        toolButtonTM->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonTM->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonTM, 0, 3, 1, 1);

        horizontalSpacer_17 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_16->addItem(horizontalSpacer_17, 0, 9, 1, 1);

        horizontalSpacer_18 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_16->addItem(horizontalSpacer_18, 0, 0, 1, 1);

        toolButtonRAW = new QToolButton(PreferencesDialog);
        toolButtonRAW->setObjectName(QString::fromUtf8("toolButtonRAW"));
        toolButtonRAW->setMinimumSize(QSize(150, 0));
        toolButtonRAW->setMaximumSize(QSize(150, 16777215));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/program-icons/rawconverter"), QSize(), QIcon::Normal, QIcon::Off);
        toolButtonRAW->setIcon(icon2);
        toolButtonRAW->setIconSize(QSize(48, 48));
        toolButtonRAW->setCheckable(true);
        toolButtonRAW->setChecked(false);
        toolButtonRAW->setAutoExclusive(true);
        toolButtonRAW->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonRAW->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonRAW, 0, 4, 1, 1);

        toolButtonCMS = new QToolButton(PreferencesDialog);
        toolButtonCMS->setObjectName(QString::fromUtf8("toolButtonCMS"));
        toolButtonCMS->setMinimumSize(QSize(150, 0));
        toolButtonCMS->setMaximumSize(QSize(150, 16777215));
        QIcon icon3;
        QString iconThemeName = QString::fromUtf8("color-management");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        toolButtonCMS->setIcon(icon3);
        toolButtonCMS->setIconSize(QSize(48, 48));
        toolButtonCMS->setCheckable(true);
        toolButtonCMS->setAutoExclusive(true);
        toolButtonCMS->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonCMS->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonCMS, 0, 5, 1, 1);

        toolButtonExtTool = new QToolButton(PreferencesDialog);
        toolButtonExtTool->setObjectName(QString::fromUtf8("toolButtonExtTool"));
        toolButtonExtTool->setMinimumSize(QSize(150, 0));
        toolButtonExtTool->setMaximumSize(QSize(150, 16777215));
        QIcon icon4;
        iconThemeName = QString::fromUtf8("preferences-other");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        toolButtonExtTool->setIcon(icon4);
        toolButtonExtTool->setIconSize(QSize(48, 48));
        toolButtonExtTool->setCheckable(true);
        toolButtonExtTool->setAutoExclusive(true);
        toolButtonExtTool->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonExtTool->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonExtTool, 0, 7, 1, 1);

        toolButtonInterface = new QToolButton(PreferencesDialog);
        toolButtonInterface->setObjectName(QString::fromUtf8("toolButtonInterface"));
        toolButtonInterface->setMinimumSize(QSize(150, 0));
        toolButtonInterface->setMaximumSize(QSize(150, 16777215));
        QIcon icon5;
        iconThemeName = QString::fromUtf8("preferences-system-windows");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        toolButtonInterface->setIcon(icon5);
        toolButtonInterface->setIconSize(QSize(48, 48));
        toolButtonInterface->setCheckable(true);
        toolButtonInterface->setChecked(true);
        toolButtonInterface->setAutoExclusive(true);
        toolButtonInterface->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonInterface->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonInterface, 0, 1, 1, 1);

        toolButtonQueue = new QToolButton(PreferencesDialog);
        toolButtonQueue->setObjectName(QString::fromUtf8("toolButtonQueue"));
        toolButtonQueue->setMinimumSize(QSize(150, 0));
        toolButtonQueue->setMaximumSize(QSize(150, 16777215));
        QIcon icon6;
        iconThemeName = QString::fromUtf8("folder-pictures");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        toolButtonQueue->setIcon(icon6);
        toolButtonQueue->setIconSize(QSize(48, 48));
        toolButtonQueue->setCheckable(true);
        toolButtonQueue->setChecked(false);
        toolButtonQueue->setAutoExclusive(true);
        toolButtonQueue->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        toolButtonQueue->setAutoRaise(true);

        gridLayout_16->addWidget(toolButtonQueue, 0, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_16);

        stackedPagesWidget = new QStackedWidget(PreferencesDialog);
        stackedPagesWidget->setObjectName(QString::fromUtf8("stackedPagesWidget"));
        stackedPagesWidget->setMaximumSize(QSize(16777215, 16777215));
        stackedPagesWidgetPage1_2 = new QWidget();
        stackedPagesWidgetPage1_2->setObjectName(QString::fromUtf8("stackedPagesWidgetPage1_2"));
        gridLayout_12 = new QGridLayout(stackedPagesWidgetPage1_2);
        gridLayout_12->setObjectName(QString::fromUtf8("gridLayout_12"));
        gridLayout_12->setContentsMargins(0, 0, 0, 0);
        interfaceGroupBox = new QGroupBox(stackedPagesWidgetPage1_2);
        interfaceGroupBox->setObjectName(QString::fromUtf8("interfaceGroupBox"));
        formLayout_7 = new QFormLayout(interfaceGroupBox);
        formLayout_7->setObjectName(QString::fromUtf8("formLayout_7"));
        formLayout_7->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        label_9 = new QLabel(interfaceGroupBox);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout_7->setWidget(0, QFormLayout::LabelRole, label_9);

        languageComboBox = new QComboBox(interfaceGroupBox);
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/program-icons/CZEC0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon7, QString::fromUtf8("\304\215esk\303\275"));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/program-icons/DK"), QSize(), QIcon::Normal, QIcon::Off);
        languageComboBox->addItem(icon8, QString::fromUtf8("Dansk"));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/program-icons/GERM0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon9, QString::fromUtf8("Deutsch"));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/program-icons/UNKG0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon10, QString::fromUtf8("English"));
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/program-icons/SPAN0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon11, QString::fromUtf8("Espa\303\261ol"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/program-icons/FRAN0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon12, QString::fromUtf8("Fran\303\247ais"));
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/program-icons/INDN0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon13, QString::fromUtf8("Indonesia"));
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/program-icons/ITAL0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon14, QString::fromUtf8("Italiano"));
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/program-icons/HUNG0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon15, QString::fromUtf8("Magyar"));
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/program-icons/POLA0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon16, QString::fromUtf8("Polski"));
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/program-icons/BRA0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon17, QString::fromUtf8("Portugu\303\252s (Brasil)"));
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/program-icons/RUSS0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon18, QString::fromUtf8("\320\240\321\203\321\201\321\201\320\272\320\270\320\271"));
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/program-icons/RMNA0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon19, QString::fromUtf8("Rom\303\242n\304\203"));
        QIcon icon20;
        icon20.addFile(QString::fromUtf8(":/program-icons/FINL0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon20, QString::fromUtf8("Suomi"));
        QIcon icon21;
        icon21.addFile(QString::fromUtf8(":/program-icons/TURK0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon21, QString::fromUtf8("T\303\274rk"));
        QIcon icon22;
        icon22.addFile(QString::fromUtf8(":/program-icons/CHIN0001"), QSize(), QIcon::Normal, QIcon::On);
        languageComboBox->addItem(icon22, QString::fromUtf8("\344\270\255\346\226\207"));
        languageComboBox->setObjectName(QString::fromUtf8("languageComboBox"));
        languageComboBox->setMinimumSize(QSize(250, 0));
        languageComboBox->setMaximumSize(QSize(250, 16777215));

        formLayout_7->setWidget(0, QFormLayout::FieldRole, languageComboBox);

        label_4 = new QLabel(interfaceGroupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout_7->setWidget(4, QFormLayout::LabelRole, label_4);

        previewsWidthSpinBox = new QSpinBox(interfaceGroupBox);
        previewsWidthSpinBox->setObjectName(QString::fromUtf8("previewsWidthSpinBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(previewsWidthSpinBox->sizePolicy().hasHeightForWidth());
        previewsWidthSpinBox->setSizePolicy(sizePolicy1);
        previewsWidthSpinBox->setMaximumSize(QSize(250, 16777215));
        previewsWidthSpinBox->setMinimum(128);
        previewsWidthSpinBox->setMaximum(2048);
        previewsWidthSpinBox->setValue(400);

        formLayout_7->setWidget(4, QFormLayout::FieldRole, previewsWidthSpinBox);

        checkBoxTMOWindowsPreviewPanel = new QCheckBox(interfaceGroupBox);
        checkBoxTMOWindowsPreviewPanel->setObjectName(QString::fromUtf8("checkBoxTMOWindowsPreviewPanel"));
        checkBoxTMOWindowsPreviewPanel->setChecked(true);

        formLayout_7->setWidget(5, QFormLayout::FieldRole, checkBoxTMOWindowsPreviewPanel);

        chkPortableMode = new QCheckBox(interfaceGroupBox);
        chkPortableMode->setObjectName(QString::fromUtf8("chkPortableMode"));

        formLayout_7->setWidget(6, QFormLayout::FieldRole, chkPortableMode);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName(QString::fromUtf8("horizontalLayout_21"));
        themeComboBox = new QComboBox(interfaceGroupBox);
        themeComboBox->setObjectName(QString::fromUtf8("themeComboBox"));

        horizontalLayout_21->addWidget(themeComboBox);

        chkDarkMode = new QCheckBox(interfaceGroupBox);
        chkDarkMode->setObjectName(QString::fromUtf8("chkDarkMode"));

        horizontalLayout_21->addWidget(chkDarkMode);


        formLayout_7->setLayout(1, QFormLayout::FieldRole, horizontalLayout_21);

        label = new QLabel(interfaceGroupBox);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout_7->setWidget(1, QFormLayout::LabelRole, label);


        gridLayout_12->addWidget(interfaceGroupBox, 0, 0, 1, 1);

        stackedPagesWidget->addWidget(stackedPagesWidgetPage1_2);
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        gridLayout_7 = new QGridLayout(page);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        gridLayout_7->setContentsMargins(0, 0, 0, 0);
        groupBox = new QGroupBox(page);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        groupBox->setTitle(QString::fromUtf8(""));
        gridLayout_8 = new QGridLayout(groupBox);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout_8->addWidget(label_2, 1, 0, 1, 1);

        label_14 = new QLabel(groupBox);
        label_14->setObjectName(QString::fromUtf8("label_14"));
        label_14->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_8->addWidget(label_14, 0, 0, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_8->addItem(verticalSpacer_3, 2, 0, 1, 1);

        exportFormatCombo = new QComboBox(groupBox);
        exportFormatCombo->setObjectName(QString::fromUtf8("exportFormatCombo"));

        gridLayout_8->addWidget(exportFormatCombo, 1, 1, 1, 1);

        exportFormatToolbutton = new QToolButton(groupBox);
        exportFormatToolbutton->setObjectName(QString::fromUtf8("exportFormatToolbutton"));

        gridLayout_8->addWidget(exportFormatToolbutton, 1, 2, 1, 1);

        exportDirectoryEdit = new QLineEdit(groupBox);
        exportDirectoryEdit->setObjectName(QString::fromUtf8("exportDirectoryEdit"));
        exportDirectoryEdit->setReadOnly(true);

        gridLayout_8->addWidget(exportDirectoryEdit, 0, 1, 1, 3);

        exportFileButton = new QToolButton(groupBox);
        exportFileButton->setObjectName(QString::fromUtf8("exportFileButton"));
        QIcon icon23;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon23 = QIcon::fromTheme(iconThemeName);
        } else {
            icon23.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        exportFileButton->setIcon(icon23);
        exportFileButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout_8->addWidget(exportFileButton, 0, 4, 1, 1);

        gridLayout_8->setColumnStretch(3, 1);

        gridLayout_7->addWidget(groupBox, 0, 0, 1, 1);

        stackedPagesWidget->addWidget(page);
        stackedPagesWidgetPage3_2 = new QWidget();
        stackedPagesWidgetPage3_2->setObjectName(QString::fromUtf8("stackedPagesWidgetPage3_2"));
        gridLayout_14 = new QGridLayout(stackedPagesWidgetPage3_2);
        gridLayout_14->setObjectName(QString::fromUtf8("gridLayout_14"));
        gridLayout_14->setContentsMargins(0, 0, 0, 0);
        tonemappingGroupBox = new QGroupBox(stackedPagesWidgetPage3_2);
        tonemappingGroupBox->setObjectName(QString::fromUtf8("tonemappingGroupBox"));
        gridLayout = new QGridLayout(tonemappingGroupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_11 = new QLabel(tonemappingGroupBox);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(label_11, 0, 0, 1, 1);

        numThreadsLabel = new QLabel(tonemappingGroupBox);
        numThreadsLabel->setObjectName(QString::fromUtf8("numThreadsLabel"));
        numThreadsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        numThreadsLabel->setWordWrap(true);

        gridLayout->addWidget(numThreadsLabel, 1, 0, 1, 1);

        numThreadspinBox = new QSpinBox(tonemappingGroupBox);
        numThreadspinBox->setObjectName(QString::fromUtf8("numThreadspinBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(numThreadspinBox->sizePolicy().hasHeightForWidth());
        numThreadspinBox->setSizePolicy(sizePolicy2);
        numThreadspinBox->setMinimum(1);
        numThreadspinBox->setMaximum(10);

        gridLayout->addWidget(numThreadspinBox, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer, 2, 0, 1, 1);

        lineEditTempPath = new QLineEdit(tonemappingGroupBox);
        lineEditTempPath->setObjectName(QString::fromUtf8("lineEditTempPath"));
        lineEditTempPath->setReadOnly(true);

        gridLayout->addWidget(lineEditTempPath, 0, 1, 1, 2);

        chooseCachePathButton = new QToolButton(tonemappingGroupBox);
        chooseCachePathButton->setObjectName(QString::fromUtf8("chooseCachePathButton"));
        chooseCachePathButton->setIcon(icon23);
        chooseCachePathButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout->addWidget(chooseCachePathButton, 0, 3, 1, 1);

        gridLayout->setColumnStretch(2, 1);

        gridLayout_14->addWidget(tonemappingGroupBox, 0, 0, 1, 1);

        stackedPagesWidget->addWidget(stackedPagesWidgetPage3_2);
        stackedPagesWidgetPage4_2 = new QWidget();
        stackedPagesWidgetPage4_2->setObjectName(QString::fromUtf8("stackedPagesWidgetPage4_2"));
        gridLayout_6 = new QGridLayout(stackedPagesWidgetPage4_2);
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        gridLayout_6->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(stackedPagesWidgetPage4_2);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabWidgetPage1 = new QWidget();
        tabWidgetPage1->setObjectName(QString::fromUtf8("tabWidgetPage1"));
        gridLayout_2 = new QGridLayout(tabWidgetPage1);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        qualityLabel = new QLabel(tabWidgetPage1);
        qualityLabel->setObjectName(QString::fromUtf8("qualityLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, qualityLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        user_qual_comboBox = new QComboBox(tabWidgetPage1);
        user_qual_comboBox->addItem(QString());
        user_qual_comboBox->addItem(QString());
        user_qual_comboBox->addItem(QString());
        user_qual_comboBox->addItem(QString());
        user_qual_comboBox->setObjectName(QString::fromUtf8("user_qual_comboBox"));
        user_qual_comboBox->setMinimumSize(QSize(250, 0));
        user_qual_comboBox->setMaximumSize(QSize(250, 16777215));

        horizontalLayout->addWidget(user_qual_comboBox);

        user_qual_toolButton = new QToolButton(tabWidgetPage1);
        user_qual_toolButton->setObjectName(QString::fromUtf8("user_qual_toolButton"));
        user_qual_toolButton->setEnabled(false);
        QIcon icon24;
        iconThemeName = QString::fromUtf8("edit-undo");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon24 = QIcon::fromTheme(iconThemeName);
        } else {
            icon24.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        user_qual_toolButton->setIcon(icon24);

        horizontalLayout->addWidget(user_qual_toolButton);


        formLayout->setLayout(3, QFormLayout::FieldRole, horizontalLayout);

        medianLabel = new QLabel(tabWidgetPage1);
        medianLabel->setObjectName(QString::fromUtf8("medianLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, medianLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        med_passes_horizontalSlider = new QSlider(tabWidgetPage1);
        med_passes_horizontalSlider->setObjectName(QString::fromUtf8("med_passes_horizontalSlider"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(med_passes_horizontalSlider->sizePolicy().hasHeightForWidth());
        med_passes_horizontalSlider->setSizePolicy(sizePolicy3);
        med_passes_horizontalSlider->setMinimumSize(QSize(250, 0));
        med_passes_horizontalSlider->setMaximumSize(QSize(250, 16777215));
        med_passes_horizontalSlider->setMaximum(10);
        med_passes_horizontalSlider->setPageStep(1);
        med_passes_horizontalSlider->setOrientation(Qt::Horizontal);
        med_passes_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        med_passes_horizontalSlider->setTickInterval(5);

        horizontalLayout_2->addWidget(med_passes_horizontalSlider);

        med_passes_spinBox = new QSpinBox(tabWidgetPage1);
        med_passes_spinBox->setObjectName(QString::fromUtf8("med_passes_spinBox"));
        med_passes_spinBox->setMinimumSize(QSize(70, 0));
        med_passes_spinBox->setMaximumSize(QSize(70, 16777215));
        med_passes_spinBox->setMaximum(10);

        horizontalLayout_2->addWidget(med_passes_spinBox);

        med_passes_toolButton = new QToolButton(tabWidgetPage1);
        med_passes_toolButton->setObjectName(QString::fromUtf8("med_passes_toolButton"));
        med_passes_toolButton->setEnabled(false);
        med_passes_toolButton->setIcon(icon24);

        horizontalLayout_2->addWidget(med_passes_toolButton);


        formLayout->setLayout(4, QFormLayout::FieldRole, horizontalLayout_2);

        do_not_use_fuji_rotate_CB = new QCheckBox(tabWidgetPage1);
        do_not_use_fuji_rotate_CB->setObjectName(QString::fromUtf8("do_not_use_fuji_rotate_CB"));

        formLayout->setWidget(2, QFormLayout::FieldRole, do_not_use_fuji_rotate_CB);

        four_color_rgb_CB = new QCheckBox(tabWidgetPage1);
        four_color_rgb_CB->setObjectName(QString::fromUtf8("four_color_rgb_CB"));

        formLayout->setWidget(1, QFormLayout::FieldRole, four_color_rgb_CB);


        gridLayout_2->addLayout(formLayout, 0, 0, 1, 1);

        tabWidget->addTab(tabWidgetPage1, QString());
        tabWidgetPage2 = new QWidget();
        tabWidgetPage2->setObjectName(QString::fromUtf8("tabWidgetPage2"));
        gridLayout_3 = new QGridLayout(tabWidgetPage2);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_3->addItem(verticalSpacer_4, 3, 0, 1, 1);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        label_3 = new QLabel(tabWidgetPage2);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout_3->setWidget(0, QFormLayout::LabelRole, label_3);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        wb_method_comboBox = new QComboBox(tabWidgetPage2);
        wb_method_comboBox->addItem(QString());
        wb_method_comboBox->addItem(QString());
        wb_method_comboBox->addItem(QString());
        wb_method_comboBox->addItem(QString());
        wb_method_comboBox->setObjectName(QString::fromUtf8("wb_method_comboBox"));
        QSizePolicy sizePolicy4(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(wb_method_comboBox->sizePolicy().hasHeightForWidth());
        wb_method_comboBox->setSizePolicy(sizePolicy4);
        wb_method_comboBox->setMinimumSize(QSize(250, 0));

        horizontalLayout_3->addWidget(wb_method_comboBox);

        wb_method_toolButton = new QToolButton(tabWidgetPage2);
        wb_method_toolButton->setObjectName(QString::fromUtf8("wb_method_toolButton"));
        wb_method_toolButton->setEnabled(false);
        wb_method_toolButton->setIcon(icon24);

        horizontalLayout_3->addWidget(wb_method_toolButton);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_4);


        formLayout_3->setLayout(0, QFormLayout::FieldRole, horizontalLayout_3);

        TK_label = new QLabel(tabWidgetPage2);
        TK_label->setObjectName(QString::fromUtf8("TK_label"));
        TK_label->setEnabled(false);
        TK_label->setWordWrap(true);

        formLayout_3->setWidget(1, QFormLayout::LabelRole, TK_label);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        TK_horizontalSlider = new QSlider(tabWidgetPage2);
        TK_horizontalSlider->setObjectName(QString::fromUtf8("TK_horizontalSlider"));
        TK_horizontalSlider->setEnabled(false);
        sizePolicy4.setHeightForWidth(TK_horizontalSlider->sizePolicy().hasHeightForWidth());
        TK_horizontalSlider->setSizePolicy(sizePolicy4);
        TK_horizontalSlider->setMinimumSize(QSize(250, 0));
        TK_horizontalSlider->setMinimum(2000);
        TK_horizontalSlider->setMaximum(12000);
        TK_horizontalSlider->setSingleStep(100);
        TK_horizontalSlider->setPageStep(1000);
        TK_horizontalSlider->setValue(6500);
        TK_horizontalSlider->setOrientation(Qt::Horizontal);
        TK_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        TK_horizontalSlider->setTickInterval(5000);

        horizontalLayout_5->addWidget(TK_horizontalSlider);

        TK_spinBox = new QSpinBox(tabWidgetPage2);
        TK_spinBox->setObjectName(QString::fromUtf8("TK_spinBox"));
        TK_spinBox->setEnabled(false);
        sizePolicy3.setHeightForWidth(TK_spinBox->sizePolicy().hasHeightForWidth());
        TK_spinBox->setSizePolicy(sizePolicy3);
        TK_spinBox->setMinimumSize(QSize(70, 0));
        TK_spinBox->setMaximumSize(QSize(70, 16777215));
        TK_spinBox->setMinimum(2000);
        TK_spinBox->setMaximum(12000);
        TK_spinBox->setSingleStep(100);
        TK_spinBox->setValue(6500);

        horizontalLayout_5->addWidget(TK_spinBox);

        TK_toolButton = new QToolButton(tabWidgetPage2);
        TK_toolButton->setObjectName(QString::fromUtf8("TK_toolButton"));
        TK_toolButton->setEnabled(false);
        TK_toolButton->setIcon(icon24);

        horizontalLayout_5->addWidget(TK_toolButton);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        formLayout_3->setLayout(1, QFormLayout::FieldRole, horizontalLayout_5);

        green_label = new QLabel(tabWidgetPage2);
        green_label->setObjectName(QString::fromUtf8("green_label"));
        green_label->setEnabled(false);

        formLayout_3->setWidget(2, QFormLayout::LabelRole, green_label);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        green_horizontalSlider = new QSlider(tabWidgetPage2);
        green_horizontalSlider->setObjectName(QString::fromUtf8("green_horizontalSlider"));
        green_horizontalSlider->setEnabled(false);
        sizePolicy4.setHeightForWidth(green_horizontalSlider->sizePolicy().hasHeightForWidth());
        green_horizontalSlider->setSizePolicy(sizePolicy4);
        green_horizontalSlider->setMinimumSize(QSize(250, 0));
        green_horizontalSlider->setMinimum(20);
        green_horizontalSlider->setMaximum(2500);
        green_horizontalSlider->setSingleStep(10);
        green_horizontalSlider->setPageStep(100);
        green_horizontalSlider->setValue(1000);
        green_horizontalSlider->setOrientation(Qt::Horizontal);
        green_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        green_horizontalSlider->setTickInterval(1240);

        horizontalLayout_6->addWidget(green_horizontalSlider);

        green_doubleSpinBox = new QDoubleSpinBox(tabWidgetPage2);
        green_doubleSpinBox->setObjectName(QString::fromUtf8("green_doubleSpinBox"));
        green_doubleSpinBox->setEnabled(false);
        green_doubleSpinBox->setMinimumSize(QSize(70, 0));
        green_doubleSpinBox->setMaximumSize(QSize(70, 16777215));
        green_doubleSpinBox->setMinimum(0.200000000000000);
        green_doubleSpinBox->setMaximum(2.500000000000000);
        green_doubleSpinBox->setSingleStep(0.100000000000000);
        green_doubleSpinBox->setValue(1.000000000000000);

        horizontalLayout_6->addWidget(green_doubleSpinBox);

        green_toolButton = new QToolButton(tabWidgetPage2);
        green_toolButton->setObjectName(QString::fromUtf8("green_toolButton"));
        green_toolButton->setEnabled(false);
        green_toolButton->setIcon(icon24);

        horizontalLayout_6->addWidget(green_toolButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer);


        formLayout_3->setLayout(2, QFormLayout::FieldRole, horizontalLayout_6);

        label_6 = new QLabel(tabWidgetPage2);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout_3->setWidget(3, QFormLayout::LabelRole, label_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        highlights_comboBox = new QComboBox(tabWidgetPage2);
        highlights_comboBox->addItem(QString());
        highlights_comboBox->addItem(QString());
        highlights_comboBox->addItem(QString());
        highlights_comboBox->addItem(QString());
        highlights_comboBox->setObjectName(QString::fromUtf8("highlights_comboBox"));
        sizePolicy4.setHeightForWidth(highlights_comboBox->sizePolicy().hasHeightForWidth());
        highlights_comboBox->setSizePolicy(sizePolicy4);
        highlights_comboBox->setMinimumSize(QSize(250, 0));

        horizontalLayout_7->addWidget(highlights_comboBox);

        highlights_toolButton = new QToolButton(tabWidgetPage2);
        highlights_toolButton->setObjectName(QString::fromUtf8("highlights_toolButton"));
        highlights_toolButton->setEnabled(false);
        highlights_toolButton->setIcon(icon24);

        horizontalLayout_7->addWidget(highlights_toolButton);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_6);


        formLayout_3->setLayout(3, QFormLayout::FieldRole, horizontalLayout_7);

        label_7 = new QLabel(tabWidgetPage2);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout_3->setWidget(4, QFormLayout::LabelRole, label_7);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        level_horizontalSlider = new QSlider(tabWidgetPage2);
        level_horizontalSlider->setObjectName(QString::fromUtf8("level_horizontalSlider"));
        sizePolicy4.setHeightForWidth(level_horizontalSlider->sizePolicy().hasHeightForWidth());
        level_horizontalSlider->setSizePolicy(sizePolicy4);
        level_horizontalSlider->setMinimumSize(QSize(250, 0));
        level_horizontalSlider->setMaximum(6);
        level_horizontalSlider->setPageStep(2);
        level_horizontalSlider->setOrientation(Qt::Horizontal);
        level_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        level_horizontalSlider->setTickInterval(3);

        horizontalLayout_4->addWidget(level_horizontalSlider);

        level_spinBox = new QSpinBox(tabWidgetPage2);
        level_spinBox->setObjectName(QString::fromUtf8("level_spinBox"));
        level_spinBox->setMinimumSize(QSize(70, 0));
        level_spinBox->setMaximumSize(QSize(70, 16777215));
        level_spinBox->setMaximum(6);

        horizontalLayout_4->addWidget(level_spinBox);

        level_toolButton = new QToolButton(tabWidgetPage2);
        level_toolButton->setObjectName(QString::fromUtf8("level_toolButton"));
        level_toolButton->setEnabled(false);
        level_toolButton->setIcon(icon24);

        horizontalLayout_4->addWidget(level_toolButton);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_7);


        formLayout_3->setLayout(4, QFormLayout::FieldRole, horizontalLayout_4);

        false_colors_CB = new QCheckBox(tabWidgetPage2);
        false_colors_CB->setObjectName(QString::fromUtf8("false_colors_CB"));
        false_colors_CB->setEnabled(false);

        formLayout_3->setWidget(5, QFormLayout::FieldRole, false_colors_CB);

        auto_bright_CB = new QCheckBox(tabWidgetPage2);
        auto_bright_CB->setObjectName(QString::fromUtf8("auto_bright_CB"));
        auto_bright_CB->setChecked(true);

        formLayout_3->setWidget(6, QFormLayout::FieldRole, auto_bright_CB);

        label_8 = new QLabel(tabWidgetPage2);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout_3->setWidget(7, QFormLayout::LabelRole, label_8);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        brightness_horizontalSlider = new QSlider(tabWidgetPage2);
        brightness_horizontalSlider->setObjectName(QString::fromUtf8("brightness_horizontalSlider"));
        brightness_horizontalSlider->setEnabled(false);
        sizePolicy4.setHeightForWidth(brightness_horizontalSlider->sizePolicy().hasHeightForWidth());
        brightness_horizontalSlider->setSizePolicy(sizePolicy4);
        brightness_horizontalSlider->setMinimumSize(QSize(250, 0));
        brightness_horizontalSlider->setMaximumSize(QSize(16777215, 16777215));
        brightness_horizontalSlider->setMaximum(100);
        brightness_horizontalSlider->setPageStep(10);
        brightness_horizontalSlider->setValue(10);
        brightness_horizontalSlider->setOrientation(Qt::Horizontal);
        brightness_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        brightness_horizontalSlider->setTickInterval(50);

        horizontalLayout_10->addWidget(brightness_horizontalSlider);

        brightness_doubleSpinBox = new QDoubleSpinBox(tabWidgetPage2);
        brightness_doubleSpinBox->setObjectName(QString::fromUtf8("brightness_doubleSpinBox"));
        brightness_doubleSpinBox->setEnabled(false);
        brightness_doubleSpinBox->setMinimumSize(QSize(70, 0));
        brightness_doubleSpinBox->setMaximumSize(QSize(70, 16777215));
        brightness_doubleSpinBox->setMaximum(10.000000000000000);
        brightness_doubleSpinBox->setSingleStep(0.100000000000000);
        brightness_doubleSpinBox->setValue(1.000000000000000);

        horizontalLayout_10->addWidget(brightness_doubleSpinBox);

        brightness_toolButton = new QToolButton(tabWidgetPage2);
        brightness_toolButton->setObjectName(QString::fromUtf8("brightness_toolButton"));
        brightness_toolButton->setEnabled(false);
        brightness_toolButton->setIcon(icon24);

        horizontalLayout_10->addWidget(brightness_toolButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_2);


        formLayout_3->setLayout(7, QFormLayout::FieldRole, horizontalLayout_10);

        use_black_CB = new QCheckBox(tabWidgetPage2);
        use_black_CB->setObjectName(QString::fromUtf8("use_black_CB"));
        use_black_CB->setLayoutDirection(Qt::RightToLeft);

        formLayout_3->setWidget(8, QFormLayout::LabelRole, use_black_CB);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        user_black_horizontalSlider = new QSlider(tabWidgetPage2);
        user_black_horizontalSlider->setObjectName(QString::fromUtf8("user_black_horizontalSlider"));
        user_black_horizontalSlider->setEnabled(false);
        sizePolicy4.setHeightForWidth(user_black_horizontalSlider->sizePolicy().hasHeightForWidth());
        user_black_horizontalSlider->setSizePolicy(sizePolicy4);
        user_black_horizontalSlider->setMinimumSize(QSize(0, 0));
        user_black_horizontalSlider->setMaximumSize(QSize(16777215, 16777215));
        user_black_horizontalSlider->setMaximum(1000);
        user_black_horizontalSlider->setPageStep(100);
        user_black_horizontalSlider->setOrientation(Qt::Horizontal);
        user_black_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        user_black_horizontalSlider->setTickInterval(500);

        horizontalLayout_9->addWidget(user_black_horizontalSlider);

        user_black_spinBox = new QSpinBox(tabWidgetPage2);
        user_black_spinBox->setObjectName(QString::fromUtf8("user_black_spinBox"));
        user_black_spinBox->setEnabled(false);
        user_black_spinBox->setMinimumSize(QSize(70, 0));
        user_black_spinBox->setMaximum(1000);

        horizontalLayout_9->addWidget(user_black_spinBox);

        user_black_toolButton = new QToolButton(tabWidgetPage2);
        user_black_toolButton->setObjectName(QString::fromUtf8("user_black_toolButton"));
        user_black_toolButton->setEnabled(false);
        user_black_toolButton->setIcon(icon24);

        horizontalLayout_9->addWidget(user_black_toolButton);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_8);


        formLayout_3->setLayout(8, QFormLayout::FieldRole, horizontalLayout_9);

        use_sat_CB = new QCheckBox(tabWidgetPage2);
        use_sat_CB->setObjectName(QString::fromUtf8("use_sat_CB"));
        use_sat_CB->setLayoutDirection(Qt::RightToLeft);

        formLayout_3->setWidget(9, QFormLayout::LabelRole, use_sat_CB);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        user_sat_horizontalSlider = new QSlider(tabWidgetPage2);
        user_sat_horizontalSlider->setObjectName(QString::fromUtf8("user_sat_horizontalSlider"));
        user_sat_horizontalSlider->setEnabled(false);
        sizePolicy4.setHeightForWidth(user_sat_horizontalSlider->sizePolicy().hasHeightForWidth());
        user_sat_horizontalSlider->setSizePolicy(sizePolicy4);
        user_sat_horizontalSlider->setMinimumSize(QSize(0, 0));
        user_sat_horizontalSlider->setMaximum(20000);
        user_sat_horizontalSlider->setSingleStep(100);
        user_sat_horizontalSlider->setPageStep(1000);
        user_sat_horizontalSlider->setValue(20000);
        user_sat_horizontalSlider->setOrientation(Qt::Horizontal);
        user_sat_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        user_sat_horizontalSlider->setTickInterval(10000);

        horizontalLayout_8->addWidget(user_sat_horizontalSlider);

        user_sat_spinBox = new QSpinBox(tabWidgetPage2);
        user_sat_spinBox->setObjectName(QString::fromUtf8("user_sat_spinBox"));
        user_sat_spinBox->setEnabled(false);
        user_sat_spinBox->setMinimumSize(QSize(70, 0));
        user_sat_spinBox->setMaximum(20000);
        user_sat_spinBox->setSingleStep(100);
        user_sat_spinBox->setValue(20000);

        horizontalLayout_8->addWidget(user_sat_spinBox);

        user_sat_toolButton = new QToolButton(tabWidgetPage2);
        user_sat_toolButton->setObjectName(QString::fromUtf8("user_sat_toolButton"));
        user_sat_toolButton->setEnabled(false);
        user_sat_toolButton->setIcon(icon24);

        horizontalLayout_8->addWidget(user_sat_toolButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_3);


        formLayout_3->setLayout(9, QFormLayout::FieldRole, horizontalLayout_8);


        gridLayout_3->addLayout(formLayout_3, 0, 0, 1, 1);

        tabWidget->addTab(tabWidgetPage2, QString());
        tabWidgetPage3 = new QWidget();
        tabWidgetPage3->setObjectName(QString::fromUtf8("tabWidgetPage3"));
        gridLayout_4 = new QGridLayout(tabWidgetPage3);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        verticalSpacer_2 = new QSpacerItem(20, 187, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer_2, 1, 0, 1, 1);

        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        use_noise_CB = new QCheckBox(tabWidgetPage3);
        use_noise_CB->setObjectName(QString::fromUtf8("use_noise_CB"));
        use_noise_CB->setChecked(true);

        formLayout_4->setWidget(0, QFormLayout::FieldRole, use_noise_CB);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        threshold_horizontalSlider = new QSlider(tabWidgetPage3);
        threshold_horizontalSlider->setObjectName(QString::fromUtf8("threshold_horizontalSlider"));
        threshold_horizontalSlider->setEnabled(true);
        threshold_horizontalSlider->setMinimumSize(QSize(250, 0));
        threshold_horizontalSlider->setMaximumSize(QSize(250, 16777215));
        threshold_horizontalSlider->setMinimum(10);
        threshold_horizontalSlider->setMaximum(1000);
        threshold_horizontalSlider->setSingleStep(10);
        threshold_horizontalSlider->setPageStep(100);
        threshold_horizontalSlider->setValue(100);
        threshold_horizontalSlider->setOrientation(Qt::Horizontal);
        threshold_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        threshold_horizontalSlider->setTickInterval(440);

        horizontalLayout_11->addWidget(threshold_horizontalSlider);

        threshold_spinBox = new QSpinBox(tabWidgetPage3);
        threshold_spinBox->setObjectName(QString::fromUtf8("threshold_spinBox"));
        threshold_spinBox->setEnabled(true);
        threshold_spinBox->setMinimumSize(QSize(70, 0));
        threshold_spinBox->setMaximumSize(QSize(70, 16777215));
        threshold_spinBox->setMinimum(10);
        threshold_spinBox->setMaximum(1000);
        threshold_spinBox->setSingleStep(10);
        threshold_spinBox->setValue(100);

        horizontalLayout_11->addWidget(threshold_spinBox);

        threshold_toolButton = new QToolButton(tabWidgetPage3);
        threshold_toolButton->setObjectName(QString::fromUtf8("threshold_toolButton"));
        threshold_toolButton->setEnabled(false);
        threshold_toolButton->setIcon(icon24);

        horizontalLayout_11->addWidget(threshold_toolButton);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_9);


        formLayout_4->setLayout(1, QFormLayout::FieldRole, horizontalLayout_11);

        threshold_label = new QLabel(tabWidgetPage3);
        threshold_label->setObjectName(QString::fromUtf8("threshold_label"));
        threshold_label->setEnabled(true);

        formLayout_4->setWidget(1, QFormLayout::LabelRole, threshold_label);

        use_chroma_CB = new QCheckBox(tabWidgetPage3);
        use_chroma_CB->setObjectName(QString::fromUtf8("use_chroma_CB"));

        formLayout_4->setWidget(3, QFormLayout::FieldRole, use_chroma_CB);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        red_horizontalSlider = new QSlider(tabWidgetPage3);
        red_horizontalSlider->setObjectName(QString::fromUtf8("red_horizontalSlider"));
        red_horizontalSlider->setEnabled(false);
        red_horizontalSlider->setMinimumSize(QSize(250, 0));
        red_horizontalSlider->setMaximumSize(QSize(250, 16777215));
        red_horizontalSlider->setMinimum(1);
        red_horizontalSlider->setMaximum(10000);
        red_horizontalSlider->setSingleStep(100);
        red_horizontalSlider->setPageStep(500);
        red_horizontalSlider->setValue(5000);
        red_horizontalSlider->setOrientation(Qt::Horizontal);
        red_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        red_horizontalSlider->setTickInterval(1000000);

        horizontalLayout_12->addWidget(red_horizontalSlider);

        red_doubleSpinBox = new QDoubleSpinBox(tabWidgetPage3);
        red_doubleSpinBox->setObjectName(QString::fromUtf8("red_doubleSpinBox"));
        red_doubleSpinBox->setEnabled(false);
        red_doubleSpinBox->setMinimumSize(QSize(70, 0));
        red_doubleSpinBox->setMaximumSize(QSize(70, 16777215));
        red_doubleSpinBox->setDecimals(3);
        red_doubleSpinBox->setMinimum(0.001000000000000);
        red_doubleSpinBox->setMaximum(2.000000000000000);
        red_doubleSpinBox->setSingleStep(0.100000000000000);
        red_doubleSpinBox->setValue(1.000000000000000);

        horizontalLayout_12->addWidget(red_doubleSpinBox);

        red_toolButton = new QToolButton(tabWidgetPage3);
        red_toolButton->setObjectName(QString::fromUtf8("red_toolButton"));
        red_toolButton->setEnabled(false);
        red_toolButton->setIcon(icon24);

        horizontalLayout_12->addWidget(red_toolButton);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_10);


        formLayout_4->setLayout(4, QFormLayout::FieldRole, horizontalLayout_12);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName(QString::fromUtf8("horizontalLayout_13"));
        blue_horizontalSlider = new QSlider(tabWidgetPage3);
        blue_horizontalSlider->setObjectName(QString::fromUtf8("blue_horizontalSlider"));
        blue_horizontalSlider->setEnabled(false);
        blue_horizontalSlider->setMinimumSize(QSize(250, 0));
        blue_horizontalSlider->setMaximumSize(QSize(250, 16777215));
        blue_horizontalSlider->setMinimum(1);
        blue_horizontalSlider->setMaximum(10000);
        blue_horizontalSlider->setSingleStep(100);
        blue_horizontalSlider->setPageStep(500);
        blue_horizontalSlider->setValue(5000);
        blue_horizontalSlider->setOrientation(Qt::Horizontal);
        blue_horizontalSlider->setTickPosition(QSlider::TicksBelow);
        blue_horizontalSlider->setTickInterval(1000000);

        horizontalLayout_13->addWidget(blue_horizontalSlider);

        blue_doubleSpinBox = new QDoubleSpinBox(tabWidgetPage3);
        blue_doubleSpinBox->setObjectName(QString::fromUtf8("blue_doubleSpinBox"));
        blue_doubleSpinBox->setEnabled(false);
        blue_doubleSpinBox->setMinimumSize(QSize(70, 0));
        blue_doubleSpinBox->setMaximumSize(QSize(70, 16777215));
        blue_doubleSpinBox->setDecimals(3);
        blue_doubleSpinBox->setMinimum(0.001000000000000);
        blue_doubleSpinBox->setMaximum(2.000000000000000);
        blue_doubleSpinBox->setSingleStep(0.100000000000000);
        blue_doubleSpinBox->setValue(1.000000000000000);

        horizontalLayout_13->addWidget(blue_doubleSpinBox);

        blue_toolButton = new QToolButton(tabWidgetPage3);
        blue_toolButton->setObjectName(QString::fromUtf8("blue_toolButton"));
        blue_toolButton->setEnabled(false);
        blue_toolButton->setIcon(icon24);

        horizontalLayout_13->addWidget(blue_toolButton);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_11);


        formLayout_4->setLayout(5, QFormLayout::FieldRole, horizontalLayout_13);

        red_label = new QLabel(tabWidgetPage3);
        red_label->setObjectName(QString::fromUtf8("red_label"));
        red_label->setEnabled(false);

        formLayout_4->setWidget(4, QFormLayout::LabelRole, red_label);

        blue_label = new QLabel(tabWidgetPage3);
        blue_label->setObjectName(QString::fromUtf8("blue_label"));
        blue_label->setEnabled(false);

        formLayout_4->setWidget(5, QFormLayout::LabelRole, blue_label);


        gridLayout_4->addLayout(formLayout_4, 0, 0, 1, 1);

        tabWidget->addTab(tabWidgetPage3, QString());

        gridLayout_6->addWidget(tabWidget, 1, 0, 1, 1);

        stackedPagesWidget->addWidget(stackedPagesWidgetPage4_2);
        stackedPagesWidgetPage6_2 = new QWidget();
        stackedPagesWidgetPage6_2->setObjectName(QString::fromUtf8("stackedPagesWidgetPage6_2"));
        gridLayout_5 = new QGridLayout(stackedPagesWidgetPage6_2);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        gridLayout_5->setContentsMargins(0, 0, 0, 0);
        iccGroupBox = new QGroupBox(stackedPagesWidgetPage6_2);
        iccGroupBox->setObjectName(QString::fromUtf8("iccGroupBox"));
        gridLayout_9 = new QGridLayout(iccGroupBox);
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        monitor_toolButton = new QToolButton(iccGroupBox);
        monitor_toolButton->setObjectName(QString::fromUtf8("monitor_toolButton"));
        monitor_toolButton->setIcon(icon23);
        monitor_toolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout_9->addWidget(monitor_toolButton, 0, 3, 1, 1);

        printer_toolButton = new QToolButton(iccGroupBox);
        printer_toolButton->setObjectName(QString::fromUtf8("printer_toolButton"));
        printer_toolButton->setIcon(icon23);
        printer_toolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout_9->addWidget(printer_toolButton, 4, 3, 1, 1);

        camera_toolButton = new QToolButton(iccGroupBox);
        camera_toolButton->setObjectName(QString::fromUtf8("camera_toolButton"));
        camera_toolButton->setEnabled(true);
        camera_toolButton->setIcon(icon23);
        camera_toolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        gridLayout_9->addWidget(camera_toolButton, 3, 3, 1, 1);

        camera_lineEdit = new QLineEdit(iccGroupBox);
        camera_lineEdit->setObjectName(QString::fromUtf8("camera_lineEdit"));
        camera_lineEdit->setEnabled(true);
        camera_lineEdit->setMaximumSize(QSize(16777215, 16777215));

        gridLayout_9->addWidget(camera_lineEdit, 3, 2, 1, 1);

        printer_lineEdit = new QLineEdit(iccGroupBox);
        printer_lineEdit->setObjectName(QString::fromUtf8("printer_lineEdit"));

        gridLayout_9->addWidget(printer_lineEdit, 4, 2, 1, 1);

        label_15 = new QLabel(iccGroupBox);
        label_15->setObjectName(QString::fromUtf8("label_15"));
        label_15->setPixmap(QPixmap(QString::fromUtf8(":/program-icons/printer")));

        gridLayout_9->addWidget(label_15, 4, 0, 1, 1);

        label_21 = new QLabel(iccGroupBox);
        label_21->setObjectName(QString::fromUtf8("label_21"));
        label_21->setEnabled(true);

        gridLayout_9->addWidget(label_21, 3, 1, 1, 1);

        label_13 = new QLabel(iccGroupBox);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        gridLayout_9->addWidget(label_13, 0, 1, 1, 1);

        label_16 = new QLabel(iccGroupBox);
        label_16->setObjectName(QString::fromUtf8("label_16"));

        gridLayout_9->addWidget(label_16, 4, 1, 1, 1);

        label_17 = new QLabel(iccGroupBox);
        label_17->setObjectName(QString::fromUtf8("label_17"));
        label_17->setPixmap(QPixmap(QString::fromUtf8(":/program-icons/camera-photo")));

        gridLayout_9->addWidget(label_17, 3, 0, 1, 1);

        monitor_lineEdit = new QLineEdit(iccGroupBox);
        monitor_lineEdit->setObjectName(QString::fromUtf8("monitor_lineEdit"));

        gridLayout_9->addWidget(monitor_lineEdit, 0, 2, 1, 1);

        label_12 = new QLabel(iccGroupBox);
        label_12->setObjectName(QString::fromUtf8("label_12"));
        label_12->setPixmap(QPixmap(QString::fromUtf8(":/program-icons/video-television")));
        label_12->setScaledContents(false);

        gridLayout_9->addWidget(label_12, 0, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_9->addItem(verticalSpacer_5, 5, 2, 1, 1);


        gridLayout_5->addWidget(iccGroupBox, 0, 0, 1, 1);

        stackedPagesWidget->addWidget(stackedPagesWidgetPage6_2);
        stackedPagesWidgetPage5_2 = new QWidget();
        stackedPagesWidgetPage5_2->setObjectName(QString::fromUtf8("stackedPagesWidgetPage5_2"));
        gridLayout_15 = new QGridLayout(stackedPagesWidgetPage5_2);
        gridLayout_15->setObjectName(QString::fromUtf8("gridLayout_15"));
        gridLayout_15->setContentsMargins(0, 0, 0, 0);
        alignGroupBox = new QGroupBox(stackedPagesWidgetPage5_2);
        alignGroupBox->setObjectName(QString::fromUtf8("alignGroupBox"));
        formLayout_8 = new QFormLayout(alignGroupBox);
        formLayout_8->setObjectName(QString::fromUtf8("formLayout_8"));
        label_5 = new QLabel(alignGroupBox);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_5->setWordWrap(true);

        formLayout_8->setWidget(0, QFormLayout::LabelRole, label_5);

        aisParamsLineEdit = new QLineEdit(alignGroupBox);
        aisParamsLineEdit->setObjectName(QString::fromUtf8("aisParamsLineEdit"));
        sizePolicy4.setHeightForWidth(aisParamsLineEdit->sizePolicy().hasHeightForWidth());
        aisParamsLineEdit->setSizePolicy(sizePolicy4);
        aisParamsLineEdit->setMinimumSize(QSize(250, 0));

        formLayout_8->setWidget(0, QFormLayout::FieldRole, aisParamsLineEdit);


        gridLayout_15->addWidget(alignGroupBox, 0, 0, 1, 1);

        stackedPagesWidget->addWidget(stackedPagesWidgetPage5_2);

        verticalLayout->addWidget(stackedPagesWidget);

        buttonArea_2 = new QHBoxLayout();
        buttonArea_2->setObjectName(QString::fromUtf8("buttonArea_2"));
        spacer_3 = new QSpacerItem(131, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);

        buttonArea_2->addItem(spacer_3);

        cancelButton = new QPushButton(PreferencesDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        buttonArea_2->addWidget(cancelButton);

        okButton = new QPushButton(PreferencesDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        buttonArea_2->addWidget(okButton);


        verticalLayout->addLayout(buttonArea_2);

        QWidget::setTabOrder(toolButtonInterface, toolButtonQueue);
        QWidget::setTabOrder(toolButtonQueue, toolButtonTM);
        QWidget::setTabOrder(toolButtonTM, toolButtonRAW);
        QWidget::setTabOrder(toolButtonRAW, toolButtonCMS);
        QWidget::setTabOrder(toolButtonCMS, toolButtonExtTool);
        QWidget::setTabOrder(toolButtonExtTool, languageComboBox);
        QWidget::setTabOrder(languageComboBox, themeComboBox);
        QWidget::setTabOrder(themeComboBox, chkDarkMode);
        QWidget::setTabOrder(chkDarkMode, previewsWidthSpinBox);
        QWidget::setTabOrder(previewsWidthSpinBox, checkBoxTMOWindowsPreviewPanel);
        QWidget::setTabOrder(checkBoxTMOWindowsPreviewPanel, chkPortableMode);
        QWidget::setTabOrder(chkPortableMode, exportDirectoryEdit);
        QWidget::setTabOrder(exportDirectoryEdit, exportFileButton);
        QWidget::setTabOrder(exportFileButton, exportFormatCombo);
        QWidget::setTabOrder(exportFormatCombo, exportFormatToolbutton);
        QWidget::setTabOrder(exportFormatToolbutton, lineEditTempPath);
        QWidget::setTabOrder(lineEditTempPath, chooseCachePathButton);
        QWidget::setTabOrder(chooseCachePathButton, numThreadspinBox);
        QWidget::setTabOrder(numThreadspinBox, tabWidget);
        QWidget::setTabOrder(tabWidget, four_color_rgb_CB);
        QWidget::setTabOrder(four_color_rgb_CB, do_not_use_fuji_rotate_CB);
        QWidget::setTabOrder(do_not_use_fuji_rotate_CB, user_qual_comboBox);
        QWidget::setTabOrder(user_qual_comboBox, user_qual_toolButton);
        QWidget::setTabOrder(user_qual_toolButton, med_passes_horizontalSlider);
        QWidget::setTabOrder(med_passes_horizontalSlider, med_passes_spinBox);
        QWidget::setTabOrder(med_passes_spinBox, med_passes_toolButton);
        QWidget::setTabOrder(med_passes_toolButton, wb_method_comboBox);
        QWidget::setTabOrder(wb_method_comboBox, wb_method_toolButton);
        QWidget::setTabOrder(wb_method_toolButton, TK_horizontalSlider);
        QWidget::setTabOrder(TK_horizontalSlider, TK_spinBox);
        QWidget::setTabOrder(TK_spinBox, TK_toolButton);
        QWidget::setTabOrder(TK_toolButton, green_horizontalSlider);
        QWidget::setTabOrder(green_horizontalSlider, green_doubleSpinBox);
        QWidget::setTabOrder(green_doubleSpinBox, green_toolButton);
        QWidget::setTabOrder(green_toolButton, highlights_comboBox);
        QWidget::setTabOrder(highlights_comboBox, highlights_toolButton);
        QWidget::setTabOrder(highlights_toolButton, level_horizontalSlider);
        QWidget::setTabOrder(level_horizontalSlider, level_spinBox);
        QWidget::setTabOrder(level_spinBox, level_toolButton);
        QWidget::setTabOrder(level_toolButton, false_colors_CB);
        QWidget::setTabOrder(false_colors_CB, auto_bright_CB);
        QWidget::setTabOrder(auto_bright_CB, brightness_horizontalSlider);
        QWidget::setTabOrder(brightness_horizontalSlider, brightness_doubleSpinBox);
        QWidget::setTabOrder(brightness_doubleSpinBox, brightness_toolButton);
        QWidget::setTabOrder(brightness_toolButton, use_black_CB);
        QWidget::setTabOrder(use_black_CB, user_black_horizontalSlider);
        QWidget::setTabOrder(user_black_horizontalSlider, user_black_spinBox);
        QWidget::setTabOrder(user_black_spinBox, user_black_toolButton);
        QWidget::setTabOrder(user_black_toolButton, user_sat_horizontalSlider);
        QWidget::setTabOrder(user_sat_horizontalSlider, user_sat_spinBox);
        QWidget::setTabOrder(user_sat_spinBox, user_sat_toolButton);
        QWidget::setTabOrder(user_sat_toolButton, monitor_lineEdit);
        QWidget::setTabOrder(monitor_lineEdit, monitor_toolButton);
        QWidget::setTabOrder(monitor_toolButton, camera_lineEdit);
        QWidget::setTabOrder(camera_lineEdit, camera_toolButton);
        QWidget::setTabOrder(camera_toolButton, printer_lineEdit);
        QWidget::setTabOrder(printer_lineEdit, printer_toolButton);
        QWidget::setTabOrder(printer_toolButton, aisParamsLineEdit);
        QWidget::setTabOrder(aisParamsLineEdit, red_horizontalSlider);
        QWidget::setTabOrder(red_horizontalSlider, blue_doubleSpinBox);
        QWidget::setTabOrder(blue_doubleSpinBox, threshold_horizontalSlider);
        QWidget::setTabOrder(threshold_horizontalSlider, blue_toolButton);
        QWidget::setTabOrder(blue_toolButton, use_sat_CB);
        QWidget::setTabOrder(use_sat_CB, use_noise_CB);
        QWidget::setTabOrder(use_noise_CB, use_chroma_CB);
        QWidget::setTabOrder(use_chroma_CB, red_toolButton);
        QWidget::setTabOrder(red_toolButton, red_doubleSpinBox);
        QWidget::setTabOrder(red_doubleSpinBox, cancelButton);
        QWidget::setTabOrder(cancelButton, threshold_spinBox);
        QWidget::setTabOrder(threshold_spinBox, okButton);
        QWidget::setTabOrder(okButton, threshold_toolButton);
        QWidget::setTabOrder(threshold_toolButton, blue_horizontalSlider);

        retranslateUi(PreferencesDialog);
        QObject::connect(threshold_horizontalSlider, SIGNAL(valueChanged(int)), threshold_spinBox, SLOT(setValue(int)));
        QObject::connect(threshold_spinBox, SIGNAL(valueChanged(int)), threshold_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(user_black_spinBox, SIGNAL(valueChanged(int)), user_black_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(user_black_horizontalSlider, SIGNAL(valueChanged(int)), user_black_spinBox, SLOT(setValue(int)));
        QObject::connect(user_sat_spinBox, SIGNAL(valueChanged(int)), user_sat_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(level_spinBox, SIGNAL(valueChanged(int)), level_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(user_sat_horizontalSlider, SIGNAL(valueChanged(int)), user_sat_spinBox, SLOT(setValue(int)));
        QObject::connect(level_horizontalSlider, SIGNAL(valueChanged(int)), level_spinBox, SLOT(setValue(int)));
        QObject::connect(TK_spinBox, SIGNAL(valueChanged(int)), TK_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(med_passes_horizontalSlider, SIGNAL(valueChanged(int)), med_passes_spinBox, SLOT(setValue(int)));
        QObject::connect(med_passes_spinBox, SIGNAL(valueChanged(int)), med_passes_horizontalSlider, SLOT(setValue(int)));
        QObject::connect(TK_horizontalSlider, SIGNAL(valueChanged(int)), TK_spinBox, SLOT(setValue(int)));

        stackedPagesWidget->setCurrentIndex(0);
        tabWidget->setCurrentIndex(1);
        wb_method_comboBox->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(PreferencesDialog);
    } // setupUi

    void retranslateUi(QWidget *PreferencesDialog)
    {
        PreferencesDialog->setWindowTitle(QCoreApplication::translate("PreferencesDialog", "Preferences", nullptr));
        toolButtonTM->setText(QCoreApplication::translate("PreferencesDialog", "Tone Mapping", nullptr));
        toolButtonRAW->setText(QCoreApplication::translate("PreferencesDialog", "RAW Conversion", nullptr));
        toolButtonCMS->setText(QCoreApplication::translate("PreferencesDialog", "Color Management", nullptr));
        toolButtonExtTool->setText(QCoreApplication::translate("PreferencesDialog", "External Tools", nullptr));
        toolButtonInterface->setText(QCoreApplication::translate("PreferencesDialog", "Interface", nullptr));
        toolButtonQueue->setText(QCoreApplication::translate("PreferencesDialog", "Fast export", nullptr));
        label_9->setText(QCoreApplication::translate("PreferencesDialog", "Language", nullptr));

        label_4->setText(QCoreApplication::translate("PreferencesDialog", "Default Previews Width", nullptr));
        previewsWidthSpinBox->setSuffix(QCoreApplication::translate("PreferencesDialog", "px", nullptr));
        checkBoxTMOWindowsPreviewPanel->setText(QCoreApplication::translate("PreferencesDialog", "Always show Preview Panel", nullptr));
#if QT_CONFIG(tooltip)
        chkPortableMode->setToolTip(QCoreApplication::translate("PreferencesDialog", "Saves the settings along the program files, to be portable", nullptr));
#endif // QT_CONFIG(tooltip)
        chkPortableMode->setText(QCoreApplication::translate("PreferencesDialog", "Portable mode", nullptr));
        chkDarkMode->setText(QCoreApplication::translate("PreferencesDialog", "Dark mode (beta)", nullptr));
        label->setText(QCoreApplication::translate("PreferencesDialog", "Theme", nullptr));
        label_2->setText(QCoreApplication::translate("PreferencesDialog", "Format", nullptr));
        label_14->setText(QCoreApplication::translate("PreferencesDialog", "Export directory", nullptr));
        exportFormatToolbutton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        exportFileButton->setText(QCoreApplication::translate("PreferencesDialog", "&Browse...", nullptr));
        label_11->setText(QCoreApplication::translate("PreferencesDialog", "Temporary Working Folder", nullptr));
#if QT_CONFIG(tooltip)
        numThreadsLabel->setToolTip(QCoreApplication::translate("PreferencesDialog", "Amount of threads to use on multicore/SMP machines", nullptr));
#endif // QT_CONFIG(tooltip)
        numThreadsLabel->setText(QCoreApplication::translate("PreferencesDialog", "Batch Tonemapping Number of Threads", nullptr));
#if QT_CONFIG(tooltip)
        numThreadspinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "Amount of threads to use on multicore/SMP machines", nullptr));
#endif // QT_CONFIG(tooltip)
        chooseCachePathButton->setText(QCoreApplication::translate("PreferencesDialog", "&Browse...", nullptr));
        qualityLabel->setText(QCoreApplication::translate("PreferencesDialog", "Quality:", nullptr));
        user_qual_comboBox->setItemText(0, QCoreApplication::translate("PreferencesDialog", "Bilinear", nullptr));
        user_qual_comboBox->setItemText(1, QCoreApplication::translate("PreferencesDialog", "VNG", nullptr));
        user_qual_comboBox->setItemText(2, QCoreApplication::translate("PreferencesDialog", "PPG", nullptr));
        user_qual_comboBox->setItemText(3, QCoreApplication::translate("PreferencesDialog", "AHD", nullptr));

#if QT_CONFIG(tooltip)
        user_qual_comboBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Quality (interpolation)</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Select here the demosaicing RAW images decoding interpolation method. A demosaicing algorithm is a digital image process used to interpolate a complete image from the partial raw data received from the color-filtered image sensor internal to many digital cameras in form of a matrix of colored pixels. Also known as CFA interpolation or color reconstruction, another commo"
                        "n spelling is demosaicing. There are 4 methods to demosaicing RAW images:</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Bilinear</span>: use high-speed but low-quality bilinear interpolation (default - for slow computer). In this method, the red value of a non-red pixel is computed as the average of the adjacent red pixels, and similar for blue and green.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">VNG</span>: use Variable Number of Gradients interpolation. This method computes gradients near the pixel of interest and uses the lower gradients (representing smoother and more similar parts of the image) to make an estimate.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600"
                        ";\">PPG</span>: use Patterned Pixel Grouping interpolation. Pixel Grouping uses assumptions about natural scenery in making estimates. It has fewer color artifacts on natural images than the Variable Number of Gradients method.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">AHD</span>: use Adaptive Homogeneity-Directed interpolation. This method selects the direction of interpolation so as to maximize a homogeneity metric, thus typically minimizing color artifacts.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        user_qual_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        medianLabel->setText(QCoreApplication::translate("PreferencesDialog", "Median:", nullptr));
#if QT_CONFIG(tooltip)
        med_passes_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Median Filter</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Set here the passes used by median filter applied after interpolation to Red-Green and Blue-Green channels.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        med_passes_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
#if QT_CONFIG(tooltip)
        do_not_use_fuji_rotate_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Do not stretch or rotate pixels</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">For Fuji Super CCD cameras, show the image tilted 45 degrees. For cameras with non-square pixels, do not stretch the image to its correct aspect ratio. In any case,                                              this option guarantees that each output pixel corresponds to one RAW pixel.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        do_not_use_fuji_rotate_CB->setText(QCoreApplication::translate("PreferencesDialog", "Do not stretch or rotate pixels", nullptr));
#if QT_CONFIG(tooltip)
        four_color_rgb_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Interpolate RGB as four colors</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The default is to assume that all green pixels are the same. If even-row green pixels are more sensitive to ultraviolet light than odd-row this difference causes a mesh                             pattern in the output; using this option solves this problem with minimal loss of detail.</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin"
                        "-right:0px; -qt-block-indent:0; text-indent:0px;\">To resume, this option blurs the image a little, but it eliminates false 2x2 mesh patterns with VNG quality method or mazes with AHD quality method.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        four_color_rgb_CB->setText(QCoreApplication::translate("PreferencesDialog", "Interpolate RGB using 4 colours", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage1), QCoreApplication::translate("PreferencesDialog", "General", nullptr));
        label_3->setText(QCoreApplication::translate("PreferencesDialog", "White Balance", nullptr));
        wb_method_comboBox->setItemText(0, QCoreApplication::translate("PreferencesDialog", "Predefined D65", nullptr));
        wb_method_comboBox->setItemText(1, QCoreApplication::translate("PreferencesDialog", "Camera", nullptr));
        wb_method_comboBox->setItemText(2, QCoreApplication::translate("PreferencesDialog", "Auto", nullptr));
        wb_method_comboBox->setItemText(3, QCoreApplication::translate("PreferencesDialog", "Manual", nullptr));

#if QT_CONFIG(tooltip)
        wb_method_comboBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">White Balance Method</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Configure the raw white balance :</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Default D65</span>: Use a standard daylight D65 white balance (dcraw defaults)</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block"
                        "-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Camera</span>: Use the white balance specified by the camera. If not available, reverts to default neutral white balance</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Automatic</span>: Calculates an automatic white balance averaging the entire image</p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Manual</span>: Set a custom temperature and green level values</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        wb_method_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        TK_label->setText(QCoreApplication::translate("PreferencesDialog", "Temperature (Kelvin)", nullptr));
#if QT_CONFIG(tooltip)
        TK_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Temperature</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Set here the color temperature in Kelvin.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        TK_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        green_label->setText(QCoreApplication::translate("PreferencesDialog", "Green", nullptr));
#if QT_CONFIG(tooltip)
        green_doubleSpinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "Set here the green component to set magenta color cast removal level", nullptr));
#endif // QT_CONFIG(tooltip)
        green_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        label_6->setText(QCoreApplication::translate("PreferencesDialog", "Highlights", nullptr));
        highlights_comboBox->setItemText(0, QCoreApplication::translate("PreferencesDialog", "Solid White", nullptr));
        highlights_comboBox->setItemText(1, QCoreApplication::translate("PreferencesDialog", "Do not transform", nullptr));
        highlights_comboBox->setItemText(2, QCoreApplication::translate("PreferencesDialog", "Blend", nullptr));
        highlights_comboBox->setItemText(3, QCoreApplication::translate("PreferencesDialog", "Reconstruct", nullptr));

#if QT_CONFIG(tooltip)
        highlights_comboBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Highlights</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt;\">Select here the highlight clipping method:</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Solid White</span><span style=\""
                        " font-family:'Sans Serif'; font-size:10pt;\">: clip all highlights to solid white</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Do not transform</span><span style=\" font-family:'Sans Serif'; font-size:10pt;\">: leave highlights unclipped in various shades of pink</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Blend</span><span style=\" font-family:'Sans Serif'; font-size:10pt;\">: Blend clipped and unclipped values together for a gradual fade to white</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Reconstruct</span><span s"
                        "tyle=\" font-family:'Sans Serif'; font-size:10pt;\">: reconstruct highlights using a level value</span></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        highlights_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        label_7->setText(QCoreApplication::translate("PreferencesDialog", "Level", nullptr));
#if QT_CONFIG(tooltip)
        level_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Level</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Specify the reconstruct highlight level. Low values favor whites and high values favor colors.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        level_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        false_colors_CB->setText(QCoreApplication::translate("PreferencesDialog", "Correct false colors in highlights", nullptr));
        auto_bright_CB->setText(QCoreApplication::translate("PreferencesDialog", "Auto Brightness", nullptr));
        label_8->setText(QCoreApplication::translate("PreferencesDialog", "Brightness", nullptr));
#if QT_CONFIG(tooltip)
        brightness_doubleSpinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Brighness</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Specify the brightness level of output image. The default value is 1.0</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        brightness_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
#if QT_CONFIG(tooltip)
        use_black_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Black point</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Use a specific black point value to decode RAW pictures. If you set this option to off, the Black Point value will be automatically computed.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        use_black_CB->setText(QCoreApplication::translate("PreferencesDialog", "Black", nullptr));
#if QT_CONFIG(tooltip)
        user_black_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Black point value</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Specify specific black point value of the output image.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        user_black_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
#if QT_CONFIG(tooltip)
        use_sat_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Saturation</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt;\">When highlights appear pink, you need to lower the saturation level. The default saturation is usually correct.</span></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        use_sat_CB->setText(QCoreApplication::translate("PreferencesDialog", "Saturation", nullptr));
#if QT_CONFIG(tooltip)
        user_sat_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:600;\">Saturation value</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif'; font-size:10pt;\">Specify the specific saturation level. When highlights appear pink, you need to lower the saturation level.</span></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        user_sat_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage2), QCoreApplication::translate("PreferencesDialog", "Color correction", nullptr));
#if QT_CONFIG(tooltip)
        use_noise_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Enable Noise Reduction</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Use wavelets to erase noise while preserving real detail.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        use_noise_CB->setText(QCoreApplication::translate("PreferencesDialog", "Enable noise reduction", nullptr));
#if QT_CONFIG(tooltip)
        threshold_spinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Threshold</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Set here the noise reduction threshold value to use.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        threshold_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        threshold_label->setText(QCoreApplication::translate("PreferencesDialog", "Threshold:", nullptr));
#if QT_CONFIG(tooltip)
        use_chroma_CB->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Enable Chromatic Aberration correction</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Enlarge the raw red and blue layers by the given factors, typically 0.999 to 1.001, to correct chromatic aberration.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        use_chroma_CB->setText(QCoreApplication::translate("PreferencesDialog", "Enable Chromatic Aberration correction", nullptr));
#if QT_CONFIG(tooltip)
        red_doubleSpinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Red multiplier</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Set here the magnification factor of the red layer</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        red_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
#if QT_CONFIG(tooltip)
        blue_doubleSpinBox->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Blue multiplier</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Set here the magnification factor of the blue layer</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        blue_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "...", nullptr));
        red_label->setText(QCoreApplication::translate("PreferencesDialog", "Red Component", nullptr));
        blue_label->setText(QCoreApplication::translate("PreferencesDialog", "Blue Component", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage3), QCoreApplication::translate("PreferencesDialog", "NR and CA Correction", nullptr));
        monitor_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "Browse...", nullptr));
        printer_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "Browse...", nullptr));
        camera_toolButton->setText(QCoreApplication::translate("PreferencesDialog", "Browse...", nullptr));
        label_15->setText(QString());
        label_21->setText(QCoreApplication::translate("PreferencesDialog", "Camera profile", nullptr));
        label_13->setText(QCoreApplication::translate("PreferencesDialog", "Monitor Profile", nullptr));
        label_16->setText(QCoreApplication::translate("PreferencesDialog", "Printer profile", nullptr));
        label_17->setText(QString());
        label_12->setText(QString());
        label_5->setText(QCoreApplication::translate("PreferencesDialog", "align_image_stack command line arguments", nullptr));
#if QT_CONFIG(tooltip)
        aisParamsLineEdit->setToolTip(QCoreApplication::translate("PreferencesDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'DejaVu Sans'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:'Sans Serif';\">Read </span><span style=\" font-family:'Sans Serif'; font-style:italic;\">Help &gt; Contents &gt; Setting up Luminance &gt; External tools</span><span style=\" font-family:'Sans Serif';\"> to find out more about align_image_stack command line arguments</span></p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        cancelButton->setText(QCoreApplication::translate("PreferencesDialog", "&Cancel", nullptr));
        okButton->setText(QCoreApplication::translate("PreferencesDialog", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PreferencesDialog: public Ui_PreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCESDIALOG_H
