/****************************************************************************
** Meta object code from reading C++ file 'PreferencesDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/Preferences/PreferencesDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PreferencesDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PreferencesDialog_t {
    QByteArrayData data[49];
    char stringdata0[1433];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesDialog_t qt_meta_stringdata_PreferencesDialog = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PreferencesDialog"
QT_MOC_LITERAL(1, 18, 19), // "on_okButton_clicked"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 23), // "on_cancelButton_clicked"
QT_MOC_LITERAL(4, 63, 32), // "on_chooseCachePathButton_clicked"
QT_MOC_LITERAL(5, 96, 14), // "enterWhatsThis"
QT_MOC_LITERAL(6, 111, 41), // "on_user_qual_comboBox_current..."
QT_MOC_LITERAL(7, 153, 34), // "on_med_passes_spinBox_valueCh..."
QT_MOC_LITERAL(8, 188, 41), // "on_wb_method_comboBox_current..."
QT_MOC_LITERAL(9, 230, 26), // "on_TK_spinBox_valueChanged"
QT_MOC_LITERAL(10, 257, 42), // "on_highlights_comboBox_curren..."
QT_MOC_LITERAL(11, 300, 29), // "on_level_spinBox_valueChanged"
QT_MOC_LITERAL(12, 330, 34), // "on_user_black_spinBox_valueCh..."
QT_MOC_LITERAL(13, 365, 32), // "on_user_sat_spinBox_valueChanged"
QT_MOC_LITERAL(14, 398, 33), // "on_threshold_spinBox_valueCha..."
QT_MOC_LITERAL(15, 432, 28), // "on_use_black_CB_stateChanged"
QT_MOC_LITERAL(16, 461, 26), // "on_use_sat_CB_stateChanged"
QT_MOC_LITERAL(17, 488, 28), // "on_use_noise_CB_stateChanged"
QT_MOC_LITERAL(18, 517, 29), // "on_use_chroma_CB_stateChanged"
QT_MOC_LITERAL(19, 547, 43), // "on_brightness_horizontalSlide..."
QT_MOC_LITERAL(20, 591, 40), // "on_brightness_doubleSpinBox_v..."
QT_MOC_LITERAL(21, 632, 36), // "on_red_horizontalSlider_value..."
QT_MOC_LITERAL(22, 669, 33), // "on_red_doubleSpinBox_valueCha..."
QT_MOC_LITERAL(23, 703, 37), // "on_blue_horizontalSlider_valu..."
QT_MOC_LITERAL(24, 741, 34), // "on_blue_doubleSpinBox_valueCh..."
QT_MOC_LITERAL(25, 776, 38), // "on_green_horizontalSlider_val..."
QT_MOC_LITERAL(26, 815, 35), // "on_green_doubleSpinBox_valueC..."
QT_MOC_LITERAL(27, 851, 31), // "on_user_qual_toolButton_clicked"
QT_MOC_LITERAL(28, 883, 32), // "on_med_passes_toolButton_clicked"
QT_MOC_LITERAL(29, 916, 31), // "on_wb_method_toolButton_clicked"
QT_MOC_LITERAL(30, 948, 24), // "on_TK_toolButton_clicked"
QT_MOC_LITERAL(31, 973, 32), // "on_highlights_toolButton_clicked"
QT_MOC_LITERAL(32, 1006, 27), // "on_level_toolButton_clicked"
QT_MOC_LITERAL(33, 1034, 32), // "on_brightness_toolButton_clicked"
QT_MOC_LITERAL(34, 1067, 32), // "on_user_black_toolButton_clicked"
QT_MOC_LITERAL(35, 1100, 30), // "on_user_sat_toolButton_clicked"
QT_MOC_LITERAL(36, 1131, 31), // "on_threshold_toolButton_clicked"
QT_MOC_LITERAL(37, 1163, 25), // "on_red_toolButton_clicked"
QT_MOC_LITERAL(38, 1189, 26), // "on_blue_toolButton_clicked"
QT_MOC_LITERAL(39, 1216, 27), // "on_green_toolButton_clicked"
QT_MOC_LITERAL(40, 1244, 27), // "on_exportFileButton_clicked"
QT_MOC_LITERAL(41, 1272, 28), // "on_camera_toolButton_clicked"
QT_MOC_LITERAL(42, 1301, 29), // "on_monitor_toolButton_clicked"
QT_MOC_LITERAL(43, 1331, 29), // "on_printer_toolButton_clicked"
QT_MOC_LITERAL(44, 1361, 18), // "toolButton_clicked"
QT_MOC_LITERAL(45, 1380, 16), // "openColorProfile"
QT_MOC_LITERAL(46, 1397, 10), // "QLineEdit*"
QT_MOC_LITERAL(47, 1408, 8), // "lineEdit"
QT_MOC_LITERAL(48, 1417, 15) // "on_themeChanged"

    },
    "PreferencesDialog\0on_okButton_clicked\0"
    "\0on_cancelButton_clicked\0"
    "on_chooseCachePathButton_clicked\0"
    "enterWhatsThis\0on_user_qual_comboBox_currentIndexChanged\0"
    "on_med_passes_spinBox_valueChanged\0"
    "on_wb_method_comboBox_currentIndexChanged\0"
    "on_TK_spinBox_valueChanged\0"
    "on_highlights_comboBox_currentIndexChanged\0"
    "on_level_spinBox_valueChanged\0"
    "on_user_black_spinBox_valueChanged\0"
    "on_user_sat_spinBox_valueChanged\0"
    "on_threshold_spinBox_valueChanged\0"
    "on_use_black_CB_stateChanged\0"
    "on_use_sat_CB_stateChanged\0"
    "on_use_noise_CB_stateChanged\0"
    "on_use_chroma_CB_stateChanged\0"
    "on_brightness_horizontalSlider_valueChanged\0"
    "on_brightness_doubleSpinBox_valueChanged\0"
    "on_red_horizontalSlider_valueChanged\0"
    "on_red_doubleSpinBox_valueChanged\0"
    "on_blue_horizontalSlider_valueChanged\0"
    "on_blue_doubleSpinBox_valueChanged\0"
    "on_green_horizontalSlider_valueChanged\0"
    "on_green_doubleSpinBox_valueChanged\0"
    "on_user_qual_toolButton_clicked\0"
    "on_med_passes_toolButton_clicked\0"
    "on_wb_method_toolButton_clicked\0"
    "on_TK_toolButton_clicked\0"
    "on_highlights_toolButton_clicked\0"
    "on_level_toolButton_clicked\0"
    "on_brightness_toolButton_clicked\0"
    "on_user_black_toolButton_clicked\0"
    "on_user_sat_toolButton_clicked\0"
    "on_threshold_toolButton_clicked\0"
    "on_red_toolButton_clicked\0"
    "on_blue_toolButton_clicked\0"
    "on_green_toolButton_clicked\0"
    "on_exportFileButton_clicked\0"
    "on_camera_toolButton_clicked\0"
    "on_monitor_toolButton_clicked\0"
    "on_printer_toolButton_clicked\0"
    "toolButton_clicked\0openColorProfile\0"
    "QLineEdit*\0lineEdit\0on_themeChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      45,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  239,    2, 0x08 /* Private */,
       3,    0,  240,    2, 0x08 /* Private */,
       4,    0,  241,    2, 0x08 /* Private */,
       5,    0,  242,    2, 0x08 /* Private */,
       6,    1,  243,    2, 0x08 /* Private */,
       7,    1,  246,    2, 0x08 /* Private */,
       8,    1,  249,    2, 0x08 /* Private */,
       9,    1,  252,    2, 0x08 /* Private */,
      10,    1,  255,    2, 0x08 /* Private */,
      11,    1,  258,    2, 0x08 /* Private */,
      12,    1,  261,    2, 0x08 /* Private */,
      13,    1,  264,    2, 0x08 /* Private */,
      14,    1,  267,    2, 0x08 /* Private */,
      15,    1,  270,    2, 0x08 /* Private */,
      16,    1,  273,    2, 0x08 /* Private */,
      17,    1,  276,    2, 0x08 /* Private */,
      18,    1,  279,    2, 0x08 /* Private */,
      19,    1,  282,    2, 0x08 /* Private */,
      20,    1,  285,    2, 0x08 /* Private */,
      21,    1,  288,    2, 0x08 /* Private */,
      22,    1,  291,    2, 0x08 /* Private */,
      23,    1,  294,    2, 0x08 /* Private */,
      24,    1,  297,    2, 0x08 /* Private */,
      25,    1,  300,    2, 0x08 /* Private */,
      26,    1,  303,    2, 0x08 /* Private */,
      27,    0,  306,    2, 0x08 /* Private */,
      28,    0,  307,    2, 0x08 /* Private */,
      29,    0,  308,    2, 0x08 /* Private */,
      30,    0,  309,    2, 0x08 /* Private */,
      31,    0,  310,    2, 0x08 /* Private */,
      32,    0,  311,    2, 0x08 /* Private */,
      33,    0,  312,    2, 0x08 /* Private */,
      34,    0,  313,    2, 0x08 /* Private */,
      35,    0,  314,    2, 0x08 /* Private */,
      36,    0,  315,    2, 0x08 /* Private */,
      37,    0,  316,    2, 0x08 /* Private */,
      38,    0,  317,    2, 0x08 /* Private */,
      39,    0,  318,    2, 0x08 /* Private */,
      40,    0,  319,    2, 0x08 /* Private */,
      41,    0,  320,    2, 0x08 /* Private */,
      42,    0,  321,    2, 0x08 /* Private */,
      43,    0,  322,    2, 0x08 /* Private */,
      44,    1,  323,    2, 0x08 /* Private */,
      45,    1,  326,    2, 0x08 /* Private */,
      48,    0,  329,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 46,   47,
    QMetaType::Void,

       0        // eod
};

void PreferencesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PreferencesDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->on_okButton_clicked(); break;
        case 1: _t->on_cancelButton_clicked(); break;
        case 2: _t->on_chooseCachePathButton_clicked(); break;
        case 3: _t->enterWhatsThis(); break;
        case 4: _t->on_user_qual_comboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->on_med_passes_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->on_wb_method_comboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->on_TK_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->on_highlights_comboBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->on_level_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->on_user_black_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->on_user_sat_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->on_threshold_spinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->on_use_black_CB_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->on_use_sat_CB_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->on_use_noise_CB_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_use_chroma_CB_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->on_brightness_horizontalSlider_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->on_brightness_doubleSpinBox_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 19: _t->on_red_horizontalSlider_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->on_red_doubleSpinBox_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 21: _t->on_blue_horizontalSlider_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->on_blue_doubleSpinBox_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 23: _t->on_green_horizontalSlider_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->on_green_doubleSpinBox_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 25: _t->on_user_qual_toolButton_clicked(); break;
        case 26: _t->on_med_passes_toolButton_clicked(); break;
        case 27: _t->on_wb_method_toolButton_clicked(); break;
        case 28: _t->on_TK_toolButton_clicked(); break;
        case 29: _t->on_highlights_toolButton_clicked(); break;
        case 30: _t->on_level_toolButton_clicked(); break;
        case 31: _t->on_brightness_toolButton_clicked(); break;
        case 32: _t->on_user_black_toolButton_clicked(); break;
        case 33: _t->on_user_sat_toolButton_clicked(); break;
        case 34: _t->on_threshold_toolButton_clicked(); break;
        case 35: _t->on_red_toolButton_clicked(); break;
        case 36: _t->on_blue_toolButton_clicked(); break;
        case 37: _t->on_green_toolButton_clicked(); break;
        case 38: _t->on_exportFileButton_clicked(); break;
        case 39: _t->on_camera_toolButton_clicked(); break;
        case 40: _t->on_monitor_toolButton_clicked(); break;
        case 41: _t->on_printer_toolButton_clicked(); break;
        case 42: _t->toolButton_clicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 43: _t->openColorProfile((*reinterpret_cast< QLineEdit*(*)>(_a[1]))); break;
        case 44: _t->on_themeChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PreferencesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesDialog.data,
    qt_meta_data_PreferencesDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PreferencesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 45;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
