/********************************************************************************
** Form generated from reading UI file 'MainWindow.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *fileNewAction;
    QAction *fileOpenAction;
    QAction *fileSaveAsAction;
    QAction *fileExitAction;
    QAction *rotateccw;
    QAction *rotatecw;
    QAction *Low_dynamic_range;
    QAction *Fit_to_dynamic_range;
    QAction *Shrink_dynamic_range;
    QAction *Extend_dynamic_range;
    QAction *Decrease_exposure;
    QAction *Increase_exposure;
    QAction *documentationAction;
    QAction *normalSizeAct;
    QAction *zoomInAct;
    QAction *zoomOutAct;
    QAction *fitToWindowAct;
    QAction *OptionsAction;
    QAction *actionResizeHDR;
    QAction *Transplant_Exif_Data_action;
    QAction *actionAbout_Qt;
    QAction *actionText_Under_Icons;
    QAction *actionIcons_Only;
    QAction *actionText_Only;
    QAction *actionText_Alongside_Icons;
    QAction *actionBatch_Tone_Mapping;
    QAction *actionWhat_s_This;
    QAction *actionAbout_Luminance;
    QAction *actionSave_Hdr_Preview;
    QAction *action_Projective_Transformation;
    QAction *cropToSelectionAction;
    QAction *removeSelectionAction;
    QAction *actionDonate;
    QAction *actionLock;
    QAction *actionMinimize;
    QAction *actionMaximize;
    QAction *actionBring_All_to_Front;
    QAction *actionShowPreviewPanel;
    QAction *actionFix_Histogram;
    QAction *fileSaveAllAction;
    QAction *actionRemove_Tab;
    QAction *actionFill_to_Window;
    QAction *actionBatch_HDR;
    QAction *actionSoft_Proofing;
    QAction *actionGamut_Check;
    QAction *actionShow_on_the_right;
    QAction *actionShow_on_the_bottom;
    QAction *actionRealtimePreviews;
    QAction *actionUpdateAvailable;
    QAction *actionFits_Importer;
    QAction *actionWhite_Balance;
    QAction *actionExportToHTML;
    QAction *actionShow_Full_Screen;
    QAction *actionShow_Image_Full_Screen;
    QAction *actionSupported_Cameras;
    QAction *actionSelect_Interpolation_Method;
    QAction *actionOnline_Documentation;
    QAction *actionDevelopers_Website;
    QAction *actionLuminance_HDR_Website;
    QWidget *centralwidget;
    QMenuBar *menubar;
    QMenu *menuSettings;
    QMenu *menuHelp;
    QMenu *menuLuminance_HDR_Online;
    QMenu *menuWindows;
    QMenu *menuView;
    QMenu *menuHDR_Histogram;
    QMenu *menuToolbars;
    QMenu *menuPreview_Panel_2;
    QMenu *menuFile;
    QMenu *menuEdit;
    QStatusBar *statusbar;
    QToolBar *toolBar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName(QString::fromUtf8("MainWindow"));
        MainWindow->resize(1651, 756);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        MainWindow->setWindowIcon(icon);
        MainWindow->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        MainWindow->setUnifiedTitleAndToolBarOnMac(true);
        fileNewAction = new QAction(MainWindow);
        fileNewAction->setObjectName(QString::fromUtf8("fileNewAction"));
        QIcon icon1;
        QString iconThemeName = QString::fromUtf8("insert-image");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon1 = QIcon::fromTheme(iconThemeName);
        } else {
            icon1.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fileNewAction->setIcon(icon1);
        fileOpenAction = new QAction(MainWindow);
        fileOpenAction->setObjectName(QString::fromUtf8("fileOpenAction"));
        QIcon icon2;
        iconThemeName = QString::fromUtf8("document-open");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon2 = QIcon::fromTheme(iconThemeName);
        } else {
            icon2.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fileOpenAction->setIcon(icon2);
        fileSaveAsAction = new QAction(MainWindow);
        fileSaveAsAction->setObjectName(QString::fromUtf8("fileSaveAsAction"));
        fileSaveAsAction->setEnabled(false);
        QIcon icon3;
        iconThemeName = QString::fromUtf8("document-save");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon3 = QIcon::fromTheme(iconThemeName);
        } else {
            icon3.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fileSaveAsAction->setIcon(icon3);
        fileExitAction = new QAction(MainWindow);
        fileExitAction->setObjectName(QString::fromUtf8("fileExitAction"));
        QIcon icon4;
        iconThemeName = QString::fromUtf8("application-exit");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon4 = QIcon::fromTheme(iconThemeName);
        } else {
            icon4.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fileExitAction->setIcon(icon4);
        rotateccw = new QAction(MainWindow);
        rotateccw->setObjectName(QString::fromUtf8("rotateccw"));
        rotateccw->setEnabled(false);
        QIcon icon5;
        iconThemeName = QString::fromUtf8("object-rotate-left");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon5 = QIcon::fromTheme(iconThemeName);
        } else {
            icon5.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        rotateccw->setIcon(icon5);
        rotatecw = new QAction(MainWindow);
        rotatecw->setObjectName(QString::fromUtf8("rotatecw"));
        rotatecw->setEnabled(false);
        QIcon icon6;
        iconThemeName = QString::fromUtf8("object-rotate-right");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon6 = QIcon::fromTheme(iconThemeName);
        } else {
            icon6.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        rotatecw->setIcon(icon6);
        Low_dynamic_range = new QAction(MainWindow);
        Low_dynamic_range->setObjectName(QString::fromUtf8("Low_dynamic_range"));
        Low_dynamic_range->setEnabled(false);
        Fit_to_dynamic_range = new QAction(MainWindow);
        Fit_to_dynamic_range->setObjectName(QString::fromUtf8("Fit_to_dynamic_range"));
        Fit_to_dynamic_range->setEnabled(false);
        Shrink_dynamic_range = new QAction(MainWindow);
        Shrink_dynamic_range->setObjectName(QString::fromUtf8("Shrink_dynamic_range"));
        Shrink_dynamic_range->setEnabled(false);
        Extend_dynamic_range = new QAction(MainWindow);
        Extend_dynamic_range->setObjectName(QString::fromUtf8("Extend_dynamic_range"));
        Extend_dynamic_range->setEnabled(false);
        Decrease_exposure = new QAction(MainWindow);
        Decrease_exposure->setObjectName(QString::fromUtf8("Decrease_exposure"));
        Decrease_exposure->setEnabled(false);
        Increase_exposure = new QAction(MainWindow);
        Increase_exposure->setObjectName(QString::fromUtf8("Increase_exposure"));
        Increase_exposure->setEnabled(false);
        documentationAction = new QAction(MainWindow);
        documentationAction->setObjectName(QString::fromUtf8("documentationAction"));
        documentationAction->setEnabled(false);
        QIcon icon7;
        iconThemeName = QString::fromUtf8("help-contents");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon7 = QIcon::fromTheme(iconThemeName);
        } else {
            icon7.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        documentationAction->setIcon(icon7);
        normalSizeAct = new QAction(MainWindow);
        normalSizeAct->setObjectName(QString::fromUtf8("normalSizeAct"));
        normalSizeAct->setCheckable(true);
        normalSizeAct->setEnabled(false);
        QIcon icon8;
        iconThemeName = QString::fromUtf8("zoom-original");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon8 = QIcon::fromTheme(iconThemeName);
        } else {
            icon8.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        normalSizeAct->setIcon(icon8);
        zoomInAct = new QAction(MainWindow);
        zoomInAct->setObjectName(QString::fromUtf8("zoomInAct"));
        zoomInAct->setEnabled(false);
        QIcon icon9;
        iconThemeName = QString::fromUtf8("zoom-in");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon9 = QIcon::fromTheme(iconThemeName);
        } else {
            icon9.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        zoomInAct->setIcon(icon9);
        zoomOutAct = new QAction(MainWindow);
        zoomOutAct->setObjectName(QString::fromUtf8("zoomOutAct"));
        zoomOutAct->setEnabled(false);
        QIcon icon10;
        iconThemeName = QString::fromUtf8("zoom-out");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon10 = QIcon::fromTheme(iconThemeName);
        } else {
            icon10.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        zoomOutAct->setIcon(icon10);
        fitToWindowAct = new QAction(MainWindow);
        fitToWindowAct->setObjectName(QString::fromUtf8("fitToWindowAct"));
        fitToWindowAct->setCheckable(true);
        fitToWindowAct->setChecked(false);
        fitToWindowAct->setEnabled(false);
        QIcon icon11;
        iconThemeName = QString::fromUtf8("zoom");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon11 = QIcon::fromTheme(iconThemeName);
        } else {
            icon11.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fitToWindowAct->setIcon(icon11);
        OptionsAction = new QAction(MainWindow);
        OptionsAction->setObjectName(QString::fromUtf8("OptionsAction"));
        QIcon icon12;
        iconThemeName = QString::fromUtf8("preferences-system");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon12 = QIcon::fromTheme(iconThemeName);
        } else {
            icon12.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        OptionsAction->setIcon(icon12);
        actionResizeHDR = new QAction(MainWindow);
        actionResizeHDR->setObjectName(QString::fromUtf8("actionResizeHDR"));
        actionResizeHDR->setEnabled(false);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/program-icons/scale"), QSize(), QIcon::Normal, QIcon::Off);
        actionResizeHDR->setIcon(icon13);
        Transplant_Exif_Data_action = new QAction(MainWindow);
        Transplant_Exif_Data_action->setObjectName(QString::fromUtf8("Transplant_Exif_Data_action"));
        QIcon icon14;
        iconThemeName = QString::fromUtf8("selection-move-to-layer-below");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon14 = QIcon::fromTheme(iconThemeName);
        } else {
            icon14.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        Transplant_Exif_Data_action->setIcon(icon14);
        actionAbout_Qt = new QAction(MainWindow);
        actionAbout_Qt->setObjectName(QString::fromUtf8("actionAbout_Qt"));
        actionText_Under_Icons = new QAction(MainWindow);
        actionText_Under_Icons->setObjectName(QString::fromUtf8("actionText_Under_Icons"));
        actionText_Under_Icons->setCheckable(true);
        actionIcons_Only = new QAction(MainWindow);
        actionIcons_Only->setObjectName(QString::fromUtf8("actionIcons_Only"));
        actionIcons_Only->setCheckable(true);
        actionText_Only = new QAction(MainWindow);
        actionText_Only->setObjectName(QString::fromUtf8("actionText_Only"));
        actionText_Only->setCheckable(true);
        actionText_Alongside_Icons = new QAction(MainWindow);
        actionText_Alongside_Icons->setObjectName(QString::fromUtf8("actionText_Alongside_Icons"));
        actionText_Alongside_Icons->setCheckable(true);
        actionBatch_Tone_Mapping = new QAction(MainWindow);
        actionBatch_Tone_Mapping->setObjectName(QString::fromUtf8("actionBatch_Tone_Mapping"));
        QIcon icon15;
        iconThemeName = QString::fromUtf8("system-run");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon15 = QIcon::fromTheme(iconThemeName);
        } else {
            icon15.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionBatch_Tone_Mapping->setIcon(icon15);
        actionWhat_s_This = new QAction(MainWindow);
        actionWhat_s_This->setObjectName(QString::fromUtf8("actionWhat_s_This"));
        QIcon icon16;
        iconThemeName = QString::fromUtf8("help-whatsthis");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon16 = QIcon::fromTheme(iconThemeName);
        } else {
            icon16.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionWhat_s_This->setIcon(icon16);
        actionAbout_Luminance = new QAction(MainWindow);
        actionAbout_Luminance->setObjectName(QString::fromUtf8("actionAbout_Luminance"));
        QIcon icon17;
        iconThemeName = QString::fromUtf8("help-about");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon17 = QIcon::fromTheme(iconThemeName);
        } else {
            icon17.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionAbout_Luminance->setIcon(icon17);
        actionSave_Hdr_Preview = new QAction(MainWindow);
        actionSave_Hdr_Preview->setObjectName(QString::fromUtf8("actionSave_Hdr_Preview"));
        actionSave_Hdr_Preview->setEnabled(false);
        QIcon icon18;
        iconThemeName = QString::fromUtf8("document-save-as-template");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon18 = QIcon::fromTheme(iconThemeName);
        } else {
            icon18.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionSave_Hdr_Preview->setIcon(icon18);
        action_Projective_Transformation = new QAction(MainWindow);
        action_Projective_Transformation->setObjectName(QString::fromUtf8("action_Projective_Transformation"));
        action_Projective_Transformation->setEnabled(false);
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/program-icons/perspective"), QSize(), QIcon::Normal, QIcon::Off);
        action_Projective_Transformation->setIcon(icon19);
        cropToSelectionAction = new QAction(MainWindow);
        cropToSelectionAction->setObjectName(QString::fromUtf8("cropToSelectionAction"));
        cropToSelectionAction->setEnabled(false);
        QIcon icon20;
        iconThemeName = QString::fromUtf8("transform-crop");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon20 = QIcon::fromTheme(iconThemeName);
        } else {
            icon20.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        cropToSelectionAction->setIcon(icon20);
        removeSelectionAction = new QAction(MainWindow);
        removeSelectionAction->setObjectName(QString::fromUtf8("removeSelectionAction"));
        removeSelectionAction->setEnabled(false);
        QIcon icon21;
        iconThemeName = QString::fromUtf8("edit-select-none");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon21 = QIcon::fromTheme(iconThemeName);
        } else {
            icon21.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        removeSelectionAction->setIcon(icon21);
        actionDonate = new QAction(MainWindow);
        actionDonate->setObjectName(QString::fromUtf8("actionDonate"));
        QIcon icon22;
        iconThemeName = QString::fromUtf8("help-donate");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon22 = QIcon::fromTheme(iconThemeName);
        } else {
            icon22.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionDonate->setIcon(icon22);
        actionLock = new QAction(MainWindow);
        actionLock->setObjectName(QString::fromUtf8("actionLock"));
        actionLock->setCheckable(true);
        actionLock->setEnabled(false);
        QIcon icon23;
        iconThemeName = QString::fromUtf8("system-lock-screen");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon23 = QIcon::fromTheme(iconThemeName);
        } else {
            icon23.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionLock->setIcon(icon23);
        actionMinimize = new QAction(MainWindow);
        actionMinimize->setObjectName(QString::fromUtf8("actionMinimize"));
        QIcon icon24;
        iconThemeName = QString::fromUtf8("window-minimize-symbolic");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon24 = QIcon::fromTheme(iconThemeName);
        } else {
            icon24.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionMinimize->setIcon(icon24);
        actionMaximize = new QAction(MainWindow);
        actionMaximize->setObjectName(QString::fromUtf8("actionMaximize"));
        QIcon icon25;
        iconThemeName = QString::fromUtf8("window-maximize-symbolic");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon25 = QIcon::fromTheme(iconThemeName);
        } else {
            icon25.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionMaximize->setIcon(icon25);
        actionBring_All_to_Front = new QAction(MainWindow);
        actionBring_All_to_Front->setObjectName(QString::fromUtf8("actionBring_All_to_Front"));
        actionShowPreviewPanel = new QAction(MainWindow);
        actionShowPreviewPanel->setObjectName(QString::fromUtf8("actionShowPreviewPanel"));
        actionShowPreviewPanel->setCheckable(true);
        actionShowPreviewPanel->setChecked(false);
        actionShowPreviewPanel->setEnabled(true);
        actionFix_Histogram = new QAction(MainWindow);
        actionFix_Histogram->setObjectName(QString::fromUtf8("actionFix_Histogram"));
        actionFix_Histogram->setCheckable(true);
        actionFix_Histogram->setEnabled(false);
        QIcon icon26;
        icon26.addFile(QString::fromUtf8(":/program-icons/adjust-levels"), QSize(), QIcon::Normal, QIcon::Off);
        actionFix_Histogram->setIcon(icon26);
        fileSaveAllAction = new QAction(MainWindow);
        fileSaveAllAction->setObjectName(QString::fromUtf8("fileSaveAllAction"));
        fileSaveAllAction->setEnabled(false);
        QIcon icon27;
        iconThemeName = QString::fromUtf8("document-save-all");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon27 = QIcon::fromTheme(iconThemeName);
        } else {
            icon27.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        fileSaveAllAction->setIcon(icon27);
        actionRemove_Tab = new QAction(MainWindow);
        actionRemove_Tab->setObjectName(QString::fromUtf8("actionRemove_Tab"));
        actionRemove_Tab->setEnabled(false);
        QIcon icon28;
        iconThemeName = QString::fromUtf8("tab-close");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon28 = QIcon::fromTheme(iconThemeName);
        } else {
            icon28.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionRemove_Tab->setIcon(icon28);
        actionFill_to_Window = new QAction(MainWindow);
        actionFill_to_Window->setObjectName(QString::fromUtf8("actionFill_to_Window"));
        actionFill_to_Window->setCheckable(true);
        actionFill_to_Window->setEnabled(false);
        QIcon icon29;
        iconThemeName = QString::fromUtf8("zoom-fit-best");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon29 = QIcon::fromTheme(iconThemeName);
        } else {
            icon29.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionFill_to_Window->setIcon(icon29);
        actionBatch_HDR = new QAction(MainWindow);
        actionBatch_HDR->setObjectName(QString::fromUtf8("actionBatch_HDR"));
        actionBatch_HDR->setIcon(icon15);
        actionSoft_Proofing = new QAction(MainWindow);
        actionSoft_Proofing->setObjectName(QString::fromUtf8("actionSoft_Proofing"));
        actionSoft_Proofing->setCheckable(true);
        actionSoft_Proofing->setEnabled(false);
        QIcon icon30;
        icon30.addFile(QString::fromUtf8(":/program-icons/soft-proofing"), QSize(), QIcon::Normal, QIcon::Off);
        actionSoft_Proofing->setIcon(icon30);
        actionGamut_Check = new QAction(MainWindow);
        actionGamut_Check->setObjectName(QString::fromUtf8("actionGamut_Check"));
        actionGamut_Check->setCheckable(true);
        actionGamut_Check->setEnabled(false);
        QIcon icon31;
        icon31.addFile(QString::fromUtf8(":/program-icons/gamut"), QSize(), QIcon::Normal, QIcon::Off);
        actionGamut_Check->setIcon(icon31);
        actionShow_on_the_right = new QAction(MainWindow);
        actionShow_on_the_right->setObjectName(QString::fromUtf8("actionShow_on_the_right"));
        actionShow_on_the_right->setCheckable(true);
        actionShow_on_the_right->setChecked(true);
        actionShow_on_the_bottom = new QAction(MainWindow);
        actionShow_on_the_bottom->setObjectName(QString::fromUtf8("actionShow_on_the_bottom"));
        actionShow_on_the_bottom->setCheckable(true);
        actionRealtimePreviews = new QAction(MainWindow);
        actionRealtimePreviews->setObjectName(QString::fromUtf8("actionRealtimePreviews"));
        actionRealtimePreviews->setCheckable(true);
        actionRealtimePreviews->setChecked(true);
        actionUpdateAvailable = new QAction(MainWindow);
        actionUpdateAvailable->setObjectName(QString::fromUtf8("actionUpdateAvailable"));
        QIcon icon32;
        iconThemeName = QString::fromUtf8("system-software-update");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon32 = QIcon::fromTheme(iconThemeName);
        } else {
            icon32.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionUpdateAvailable->setIcon(icon32);
        actionUpdateAvailable->setVisible(false);
        actionFits_Importer = new QAction(MainWindow);
        actionFits_Importer->setObjectName(QString::fromUtf8("actionFits_Importer"));
        QIcon icon33;
        icon33.addFile(QString::fromUtf8(":/program-icons/applications-astronomy"), QSize(), QIcon::Normal, QIcon::Off);
        actionFits_Importer->setIcon(icon33);
        actionWhite_Balance = new QAction(MainWindow);
        actionWhite_Balance->setObjectName(QString::fromUtf8("actionWhite_Balance"));
        actionWhite_Balance->setEnabled(false);
        QIcon icon34;
        iconThemeName = QString::fromUtf8("whitebalance");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon34 = QIcon::fromTheme(iconThemeName);
        } else {
            icon34.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionWhite_Balance->setIcon(icon34);
        actionExportToHTML = new QAction(MainWindow);
        actionExportToHTML->setObjectName(QString::fromUtf8("actionExportToHTML"));
        actionExportToHTML->setEnabled(false);
        QIcon icon35;
        iconThemeName = QString::fromUtf8("globe");
        if (QIcon::hasThemeIcon(iconThemeName)) {
            icon35 = QIcon::fromTheme(iconThemeName);
        } else {
            icon35.addFile(QString::fromUtf8("."), QSize(), QIcon::Normal, QIcon::Off);
        }
        actionExportToHTML->setIcon(icon35);
        actionShow_Full_Screen = new QAction(MainWindow);
        actionShow_Full_Screen->setObjectName(QString::fromUtf8("actionShow_Full_Screen"));
        actionShow_Full_Screen->setCheckable(true);
        actionShow_Image_Full_Screen = new QAction(MainWindow);
        actionShow_Image_Full_Screen->setObjectName(QString::fromUtf8("actionShow_Image_Full_Screen"));
        actionShow_Image_Full_Screen->setCheckable(false);
        actionShow_Image_Full_Screen->setEnabled(false);
        actionSupported_Cameras = new QAction(MainWindow);
        actionSupported_Cameras->setObjectName(QString::fromUtf8("actionSupported_Cameras"));
        QIcon icon36;
        icon36.addFile(QString::fromUtf8(":/program-icons/raw-icon"), QSize(), QIcon::Normal, QIcon::Off);
        actionSupported_Cameras->setIcon(icon36);
        actionSelect_Interpolation_Method = new QAction(MainWindow);
        actionSelect_Interpolation_Method->setObjectName(QString::fromUtf8("actionSelect_Interpolation_Method"));
        actionSelect_Interpolation_Method->setCheckable(true);
        actionSelect_Interpolation_Method->setChecked(false);
        actionOnline_Documentation = new QAction(MainWindow);
        actionOnline_Documentation->setObjectName(QString::fromUtf8("actionOnline_Documentation"));
        actionDevelopers_Website = new QAction(MainWindow);
        actionDevelopers_Website->setObjectName(QString::fromUtf8("actionDevelopers_Website"));
        actionLuminance_HDR_Website = new QAction(MainWindow);
        actionLuminance_HDR_Website->setObjectName(QString::fromUtf8("actionLuminance_HDR_Website"));
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName(QString::fromUtf8("menubar"));
        menubar->setGeometry(QRect(0, 0, 1651, 27));
        menuSettings = new QMenu(menubar);
        menuSettings->setObjectName(QString::fromUtf8("menuSettings"));
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName(QString::fromUtf8("menuHelp"));
        menuLuminance_HDR_Online = new QMenu(menuHelp);
        menuLuminance_HDR_Online->setObjectName(QString::fromUtf8("menuLuminance_HDR_Online"));
        menuWindows = new QMenu(menubar);
        menuWindows->setObjectName(QString::fromUtf8("menuWindows"));
        menuView = new QMenu(menubar);
        menuView->setObjectName(QString::fromUtf8("menuView"));
        menuHDR_Histogram = new QMenu(menuView);
        menuHDR_Histogram->setObjectName(QString::fromUtf8("menuHDR_Histogram"));
        menuHDR_Histogram->setEnabled(false);
        menuToolbars = new QMenu(menuView);
        menuToolbars->setObjectName(QString::fromUtf8("menuToolbars"));
        menuPreview_Panel_2 = new QMenu(menuView);
        menuPreview_Panel_2->setObjectName(QString::fromUtf8("menuPreview_Panel_2"));
        menuFile = new QMenu(menubar);
        menuFile->setObjectName(QString::fromUtf8("menuFile"));
        menuEdit = new QMenu(menubar);
        menuEdit->setObjectName(QString::fromUtf8("menuEdit"));
        MainWindow->setMenuBar(menubar);
        statusbar = new QStatusBar(MainWindow);
        statusbar->setObjectName(QString::fromUtf8("statusbar"));
        MainWindow->setStatusBar(statusbar);
        toolBar = new QToolBar(MainWindow);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        toolBar->setEnabled(true);
        toolBar->setOrientation(Qt::Horizontal);
        toolBar->setIconSize(QSize(22, 22));
        toolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        MainWindow->addToolBar(Qt::TopToolBarArea, toolBar);

        menubar->addAction(menuFile->menuAction());
        menubar->addAction(menuEdit->menuAction());
        menubar->addAction(menuView->menuAction());
        menubar->addAction(menuSettings->menuAction());
        menubar->addAction(menuWindows->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menuSettings->addAction(actionShowPreviewPanel);
        menuSettings->addAction(actionRealtimePreviews);
        menuSettings->addAction(actionShow_Image_Full_Screen);
        menuSettings->addAction(actionSelect_Interpolation_Method);
        menuSettings->addSeparator();
        menuSettings->addAction(Transplant_Exif_Data_action);
        menuSettings->addAction(actionBatch_HDR);
        menuSettings->addAction(actionBatch_Tone_Mapping);
        menuSettings->addAction(actionFits_Importer);
        menuSettings->addAction(actionExportToHTML);
        menuSettings->addAction(OptionsAction);
        menuHelp->addAction(documentationAction);
        menuHelp->addAction(actionWhat_s_This);
        menuHelp->addSeparator();
        menuHelp->addAction(actionUpdateAvailable);
        menuHelp->addAction(actionSupported_Cameras);
        menuHelp->addAction(actionDonate);
        menuHelp->addAction(actionAbout_Luminance);
        menuHelp->addAction(actionAbout_Qt);
        menuHelp->addAction(menuLuminance_HDR_Online->menuAction());
        menuLuminance_HDR_Online->addAction(actionOnline_Documentation);
        menuLuminance_HDR_Online->addAction(actionDevelopers_Website);
        menuLuminance_HDR_Online->addAction(actionLuminance_HDR_Website);
        menuWindows->addAction(actionMinimize);
        menuWindows->addAction(actionMaximize);
        menuWindows->addAction(actionShow_Full_Screen);
        menuWindows->addSeparator();
        menuWindows->addAction(actionBring_All_to_Front);
        menuWindows->addSeparator();
        menuWindows->addAction(actionRemove_Tab);
        menuView->addAction(fitToWindowAct);
        menuView->addAction(actionFill_to_Window);
        menuView->addAction(normalSizeAct);
        menuView->addAction(zoomInAct);
        menuView->addAction(zoomOutAct);
        menuView->addAction(actionLock);
        menuView->addAction(menuHDR_Histogram->menuAction());
        menuView->addSeparator();
        menuView->addAction(menuToolbars->menuAction());
        menuView->addAction(menuPreview_Panel_2->menuAction());
        menuHDR_Histogram->addAction(Fit_to_dynamic_range);
        menuHDR_Histogram->addAction(Low_dynamic_range);
        menuHDR_Histogram->addAction(Shrink_dynamic_range);
        menuHDR_Histogram->addAction(Extend_dynamic_range);
        menuHDR_Histogram->addAction(Decrease_exposure);
        menuHDR_Histogram->addAction(Increase_exposure);
        menuToolbars->addAction(actionIcons_Only);
        menuToolbars->addAction(actionText_Under_Icons);
        menuToolbars->addAction(actionText_Alongside_Icons);
        menuToolbars->addAction(actionText_Only);
        menuToolbars->addSeparator();
        menuPreview_Panel_2->addAction(actionShow_on_the_right);
        menuPreview_Panel_2->addAction(actionShow_on_the_bottom);
        menuFile->addAction(fileNewAction);
        menuFile->addAction(fileOpenAction);
        menuFile->addAction(fileSaveAsAction);
        menuFile->addAction(actionSave_Hdr_Preview);
        menuFile->addSeparator();
        menuFile->addAction(fileExitAction);
        menuEdit->addAction(rotateccw);
        menuEdit->addAction(rotatecw);
        menuEdit->addSeparator();
        menuEdit->addAction(cropToSelectionAction);
        menuEdit->addAction(removeSelectionAction);
        menuEdit->addSeparator();
        menuEdit->addAction(actionResizeHDR);
        menuEdit->addAction(action_Projective_Transformation);
        menuEdit->addSeparator();
        menuEdit->addAction(actionFix_Histogram);
        toolBar->addAction(fileNewAction);
        toolBar->addAction(fileOpenAction);
        toolBar->addAction(fileSaveAsAction);
        toolBar->addAction(fileSaveAllAction);
        toolBar->addSeparator();
        toolBar->addAction(fitToWindowAct);
        toolBar->addAction(actionFill_to_Window);
        toolBar->addAction(normalSizeAct);
        toolBar->addSeparator();
        toolBar->addAction(actionFix_Histogram);
        toolBar->addAction(actionWhite_Balance);
        toolBar->addSeparator();
        toolBar->addAction(actionSoft_Proofing);
        toolBar->addAction(actionGamut_Check);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Luminance HDR", nullptr));
        fileNewAction->setText(QCoreApplication::translate("MainWindow", "&New HDR image...", nullptr));
        fileNewAction->setIconText(QCoreApplication::translate("MainWindow", "New HDR image", nullptr));
#if QT_CONFIG(tooltip)
        fileNewAction->setToolTip(QCoreApplication::translate("MainWindow", "Create a new HDR image from a set of LDR or Raw images", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileNewAction->setStatusTip(QCoreApplication::translate("MainWindow", "Create a new HDR image from a set of LDR or Raw images", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fileNewAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Create a new HDR image from a set of JPEG, Raw, or 8/16 bpc TIFF files", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fileNewAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        fileOpenAction->setText(QCoreApplication::translate("MainWindow", "&Open HDR image...", nullptr));
        fileOpenAction->setIconText(QCoreApplication::translate("MainWindow", "Open HDR image", nullptr));
#if QT_CONFIG(tooltip)
        fileOpenAction->setToolTip(QCoreApplication::translate("MainWindow", "Load an existing HDR image file (OpenEXR, Radiance RGBE, PFS stream, RAW or TIFF)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileOpenAction->setStatusTip(QCoreApplication::translate("MainWindow", "Load an existing HDR image file (OpenEXR, Radiance RGBE, PFS stream, RAW or TIFF)", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fileOpenAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Load an existing HDR image file (OpenEXR, Radiance RGBE, PFS stream, RAW or TIFF)", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fileOpenAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        fileSaveAsAction->setText(QCoreApplication::translate("MainWindow", "&Save as...", nullptr));
        fileSaveAsAction->setIconText(QCoreApplication::translate("MainWindow", "Save as...", nullptr));
#if QT_CONFIG(tooltip)
        fileSaveAsAction->setToolTip(QCoreApplication::translate("MainWindow", "Save image (HDR or LDR)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileSaveAsAction->setStatusTip(QCoreApplication::translate("MainWindow", "Save image (HDR or LDR)", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fileSaveAsAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Save image (HDR or LDR)", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fileSaveAsAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        fileExitAction->setText(QCoreApplication::translate("MainWindow", "&Quit", nullptr));
#if QT_CONFIG(tooltip)
        fileExitAction->setToolTip(QCoreApplication::translate("MainWindow", "Quit Luminance HDR", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileExitAction->setStatusTip(QCoreApplication::translate("MainWindow", "Quit Luminance HDR", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fileExitAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Exit", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fileExitAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        rotateccw->setText(QCoreApplication::translate("MainWindow", "Rotate c&ounter-clockwise", nullptr));
        rotateccw->setIconText(QCoreApplication::translate("MainWindow", "Rotate counter-clockwise", nullptr));
#if QT_CONFIG(tooltip)
        rotateccw->setToolTip(QCoreApplication::translate("MainWindow", "Rotate counter-clockwise the HDR image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        rotateccw->setStatusTip(QCoreApplication::translate("MainWindow", "Rotate counter-clockwise the HDR image", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        rotateccw->setWhatsThis(QCoreApplication::translate("MainWindow", "Rotate counter-clockwise the HDR image", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        rotateccw->setShortcut(QCoreApplication::translate("MainWindow", "<", nullptr));
#endif // QT_CONFIG(shortcut)
        rotatecw->setText(QCoreApplication::translate("MainWindow", "Rotat&e clockwise", nullptr));
        rotatecw->setIconText(QCoreApplication::translate("MainWindow", "Rotate clockwise", nullptr));
#if QT_CONFIG(tooltip)
        rotatecw->setToolTip(QCoreApplication::translate("MainWindow", "Rotate clockwise the HDR image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        rotatecw->setStatusTip(QCoreApplication::translate("MainWindow", "Rotate clockwise the HDR image", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        rotatecw->setWhatsThis(QCoreApplication::translate("MainWindow", "Rotate clockwise the HDR image", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        rotatecw->setShortcut(QCoreApplication::translate("MainWindow", ">", nullptr));
#endif // QT_CONFIG(shortcut)
        Low_dynamic_range->setText(QCoreApplication::translate("MainWindow", "&Low Dynamic Range", nullptr));
#if QT_CONFIG(shortcut)
        Low_dynamic_range->setShortcut(QCoreApplication::translate("MainWindow", "L", nullptr));
#endif // QT_CONFIG(shortcut)
        Fit_to_dynamic_range->setText(QCoreApplication::translate("MainWindow", "&Fit to Dynamic Range", nullptr));
#if QT_CONFIG(tooltip)
        Fit_to_dynamic_range->setToolTip(QCoreApplication::translate("MainWindow", "Fit to dynamic range", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        Fit_to_dynamic_range->setShortcut(QCoreApplication::translate("MainWindow", "\\", nullptr));
#endif // QT_CONFIG(shortcut)
        Shrink_dynamic_range->setText(QCoreApplication::translate("MainWindow", "&Shrink Dynamic Range", nullptr));
#if QT_CONFIG(shortcut)
        Shrink_dynamic_range->setShortcut(QCoreApplication::translate("MainWindow", "[", nullptr));
#endif // QT_CONFIG(shortcut)
        Extend_dynamic_range->setText(QCoreApplication::translate("MainWindow", "&Extend Dynamic Range", nullptr));
#if QT_CONFIG(shortcut)
        Extend_dynamic_range->setShortcut(QCoreApplication::translate("MainWindow", "]", nullptr));
#endif // QT_CONFIG(shortcut)
        Decrease_exposure->setText(QCoreApplication::translate("MainWindow", "&Decrease Exposure", nullptr));
#if QT_CONFIG(shortcut)
        Decrease_exposure->setShortcut(QCoreApplication::translate("MainWindow", "0", nullptr));
#endif // QT_CONFIG(shortcut)
        Increase_exposure->setText(QCoreApplication::translate("MainWindow", "&Increase Exposure", nullptr));
#if QT_CONFIG(shortcut)
        Increase_exposure->setShortcut(QCoreApplication::translate("MainWindow", "9", nullptr));
#endif // QT_CONFIG(shortcut)
        documentationAction->setText(QCoreApplication::translate("MainWindow", "&Contents...", nullptr));
        documentationAction->setIconText(QCoreApplication::translate("MainWindow", "About...", nullptr));
#if QT_CONFIG(tooltip)
        documentationAction->setToolTip(QCoreApplication::translate("MainWindow", "Open user manual for Luminance HDR", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        documentationAction->setStatusTip(QCoreApplication::translate("MainWindow", "Open user manual for Luminance HDR", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        documentationAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Open user manual for Luminance HDR", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        documentationAction->setShortcut(QCoreApplication::translate("MainWindow", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
        normalSizeAct->setText(QCoreApplication::translate("MainWindow", "&Normal Size", nullptr));
        normalSizeAct->setIconText(QCoreApplication::translate("MainWindow", "Normal size", nullptr));
#if QT_CONFIG(tooltip)
        normalSizeAct->setToolTip(QCoreApplication::translate("MainWindow", "View HDR image at its normal size", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        normalSizeAct->setStatusTip(QCoreApplication::translate("MainWindow", "View HDR image at its normal size", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        normalSizeAct->setWhatsThis(QCoreApplication::translate("MainWindow", "View HDR image at its normal size", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        normalSizeAct->setShortcut(QCoreApplication::translate("MainWindow", "O", nullptr));
#endif // QT_CONFIG(shortcut)
        zoomInAct->setText(QCoreApplication::translate("MainWindow", "Zoom &In", nullptr));
#if QT_CONFIG(tooltip)
        zoomInAct->setToolTip(QCoreApplication::translate("MainWindow", "Zoom in", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        zoomInAct->setStatusTip(QCoreApplication::translate("MainWindow", "Zoom in", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        zoomInAct->setWhatsThis(QCoreApplication::translate("MainWindow", "Zoom in", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        zoomInAct->setShortcut(QCoreApplication::translate("MainWindow", "+", nullptr));
#endif // QT_CONFIG(shortcut)
        zoomOutAct->setText(QCoreApplication::translate("MainWindow", "Zoom &Out", nullptr));
#if QT_CONFIG(tooltip)
        zoomOutAct->setToolTip(QCoreApplication::translate("MainWindow", "Zoom out", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        zoomOutAct->setStatusTip(QCoreApplication::translate("MainWindow", "Zoom out", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        zoomOutAct->setWhatsThis(QCoreApplication::translate("MainWindow", "Zoom out", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        zoomOutAct->setShortcut(QCoreApplication::translate("MainWindow", "-", nullptr));
#endif // QT_CONFIG(shortcut)
        fitToWindowAct->setText(QCoreApplication::translate("MainWindow", "&Fit in Window", nullptr));
        fitToWindowAct->setIconText(QCoreApplication::translate("MainWindow", "Fit in Window", nullptr));
#if QT_CONFIG(tooltip)
        fitToWindowAct->setToolTip(QCoreApplication::translate("MainWindow", "Fit in Window", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fitToWindowAct->setStatusTip(QCoreApplication::translate("MainWindow", "Make the HDR image fit its window", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fitToWindowAct->setWhatsThis(QCoreApplication::translate("MainWindow", "Make the HDR image fit its window", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        fitToWindowAct->setShortcut(QCoreApplication::translate("MainWindow", "W", nullptr));
#endif // QT_CONFIG(shortcut)
        OptionsAction->setText(QCoreApplication::translate("MainWindow", "&Preferences...", nullptr));
        OptionsAction->setIconText(QCoreApplication::translate("MainWindow", "Luminance Options", nullptr));
#if QT_CONFIG(tooltip)
        OptionsAction->setToolTip(QCoreApplication::translate("MainWindow", "Set various options of Luminance", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        OptionsAction->setStatusTip(QCoreApplication::translate("MainWindow", "Set various options of Luminance", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        OptionsAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Set various options of Luminance", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        OptionsAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        actionResizeHDR->setText(QCoreApplication::translate("MainWindow", "&Resize...", nullptr));
        actionResizeHDR->setIconText(QCoreApplication::translate("MainWindow", "Resize the HDR image", nullptr));
#if QT_CONFIG(tooltip)
        actionResizeHDR->setToolTip(QCoreApplication::translate("MainWindow", "Change size of the HDR image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionResizeHDR->setStatusTip(QCoreApplication::translate("MainWindow", "Change size of the HDR image", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionResizeHDR->setWhatsThis(QCoreApplication::translate("MainWindow", "Change size of the HDR image", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionResizeHDR->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
        Transplant_Exif_Data_action->setText(QCoreApplication::translate("MainWindow", "Copy &Exif Data...", nullptr));
        Transplant_Exif_Data_action->setIconText(QCoreApplication::translate("MainWindow", "Copy Exif Data", nullptr));
#if QT_CONFIG(tooltip)
        Transplant_Exif_Data_action->setToolTip(QCoreApplication::translate("MainWindow", "Copy Exif data between two sets of files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        Transplant_Exif_Data_action->setStatusTip(QCoreApplication::translate("MainWindow", "Copy Exif data between two sets of files", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        Transplant_Exif_Data_action->setWhatsThis(QCoreApplication::translate("MainWindow", "Copy Exif data between two sets of files", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        Transplant_Exif_Data_action->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+E", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAbout_Qt->setText(QCoreApplication::translate("MainWindow", "&About Qt", nullptr));
#if QT_CONFIG(tooltip)
        actionAbout_Qt->setToolTip(QCoreApplication::translate("MainWindow", "Show information about Qt", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionAbout_Qt->setStatusTip(QCoreApplication::translate("MainWindow", "Show information about Qt library that is used by Luminance", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionAbout_Qt->setWhatsThis(QCoreApplication::translate("MainWindow", "Show information about Qt library that is used by Luminance", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionText_Under_Icons->setText(QCoreApplication::translate("MainWindow", "Text &under Icons", nullptr));
        actionIcons_Only->setText(QCoreApplication::translate("MainWindow", "&Icons Only", nullptr));
        actionText_Only->setText(QCoreApplication::translate("MainWindow", "&Text Only", nullptr));
        actionText_Alongside_Icons->setText(QCoreApplication::translate("MainWindow", "Text &Alongside Icons", nullptr));
        actionBatch_Tone_Mapping->setText(QCoreApplication::translate("MainWindow", "&Batch Tone Mapping...", nullptr));
#if QT_CONFIG(tooltip)
        actionBatch_Tone_Mapping->setToolTip(QCoreApplication::translate("MainWindow", "Convert multiple HDR images to LDR", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionBatch_Tone_Mapping->setStatusTip(QCoreApplication::translate("MainWindow", "Convert multiple HDR images to LDR using existing settings files", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionBatch_Tone_Mapping->setWhatsThis(QCoreApplication::translate("MainWindow", "Convert multiple HDR images to LDR using existing settings files", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionBatch_Tone_Mapping->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+B", nullptr));
#endif // QT_CONFIG(shortcut)
        actionWhat_s_This->setText(QCoreApplication::translate("MainWindow", "&What's This?", nullptr));
#if QT_CONFIG(tooltip)
        actionWhat_s_This->setToolTip(QCoreApplication::translate("MainWindow", "Get verbose information about user interface elements", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionWhat_s_This->setStatusTip(QCoreApplication::translate("MainWindow", "Get verbose information about user interface elements", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionWhat_s_This->setWhatsThis(QString());
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionWhat_s_This->setShortcut(QCoreApplication::translate("MainWindow", "Shift+F1", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAbout_Luminance->setText(QCoreApplication::translate("MainWindow", "About &Luminance HDR", nullptr));
#if QT_CONFIG(tooltip)
        actionAbout_Luminance->setToolTip(QCoreApplication::translate("MainWindow", "Show information about Luminance HDR", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionAbout_Luminance->setStatusTip(QCoreApplication::translate("MainWindow", "Show information about Luminance HDR, its authors and contributors", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionAbout_Luminance->setWhatsThis(QCoreApplication::translate("MainWindow", "Show information about Luminance HDR, its authors and contributors", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionAbout_Luminance->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+I", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSave_Hdr_Preview->setText(QCoreApplication::translate("MainWindow", "Save HDR image &preview...", nullptr));
#if QT_CONFIG(tooltip)
        actionSave_Hdr_Preview->setToolTip(QCoreApplication::translate("MainWindow", "Save the current HDR Preview to an LDR file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionSave_Hdr_Preview->setStatusTip(QCoreApplication::translate("MainWindow", "Save the current HDR Preview to an LDR file", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionSave_Hdr_Preview->setWhatsThis(QCoreApplication::translate("MainWindow", "Save the current HDR Preview to an LDR file", nullptr));
#endif // QT_CONFIG(whatsthis)
        action_Projective_Transformation->setText(QCoreApplication::translate("MainWindow", "&Projective Transformation...", nullptr));
#if QT_CONFIG(tooltip)
        action_Projective_Transformation->setToolTip(QCoreApplication::translate("MainWindow", "Apply projective transformation to the current HDR image", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        action_Projective_Transformation->setStatusTip(QCoreApplication::translate("MainWindow", "Apply projective transformation to the current HDR image", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        action_Projective_Transformation->setWhatsThis(QCoreApplication::translate("MainWindow", "Apply projective transformation to the current HDR image", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        action_Projective_Transformation->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+J", nullptr));
#endif // QT_CONFIG(shortcut)
        cropToSelectionAction->setText(QCoreApplication::translate("MainWindow", "&Crop to Selection", nullptr));
        cropToSelectionAction->setIconText(QCoreApplication::translate("MainWindow", "Crop to Selection", nullptr));
#if QT_CONFIG(tooltip)
        cropToSelectionAction->setToolTip(QCoreApplication::translate("MainWindow", "Crop the image to selection rectangle", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        cropToSelectionAction->setStatusTip(QCoreApplication::translate("MainWindow", "Crop the image to selection rectangle", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        cropToSelectionAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Click and drag inside image area to create a selection, then use this button to crop to a new HDR image", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        cropToSelectionAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+X", nullptr));
#endif // QT_CONFIG(shortcut)
        removeSelectionAction->setText(QCoreApplication::translate("MainWindow", "Remove &Selection", nullptr));
#if QT_CONFIG(tooltip)
        removeSelectionAction->setToolTip(QCoreApplication::translate("MainWindow", "Remove selection rectangle", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        removeSelectionAction->setStatusTip(QCoreApplication::translate("MainWindow", "Remove selection rectangle from image area", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        removeSelectionAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Remove selection rectangle from image area", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        removeSelectionAction->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Z", nullptr));
#endif // QT_CONFIG(shortcut)
        actionDonate->setText(QCoreApplication::translate("MainWindow", "Make a &Donation", nullptr));
#if QT_CONFIG(shortcut)
        actionDonate->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+D", nullptr));
#endif // QT_CONFIG(shortcut)
        actionLock->setText(QCoreApplication::translate("MainWindow", "&Lock Viewers", nullptr));
#if QT_CONFIG(tooltip)
        actionLock->setToolTip(QCoreApplication::translate("MainWindow", "Synchronize viewers mode (fit, fill or 1:1).", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        actionLock->setWhatsThis(QCoreApplication::translate("MainWindow", "Lock the images for sync view", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionLock->setShortcut(QCoreApplication::translate("MainWindow", "S", nullptr));
#endif // QT_CONFIG(shortcut)
        actionMinimize->setText(QCoreApplication::translate("MainWindow", "&Minimize", nullptr));
        actionMaximize->setText(QCoreApplication::translate("MainWindow", "Ma&ximize", nullptr));
        actionBring_All_to_Front->setText(QCoreApplication::translate("MainWindow", "&Bring All to Front", nullptr));
        actionShowPreviewPanel->setText(QCoreApplication::translate("MainWindow", "&Show Preview Panel", nullptr));
#if QT_CONFIG(tooltip)
        actionShowPreviewPanel->setToolTip(QCoreApplication::translate("MainWindow", "Show/Hide Preview Panel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionShowPreviewPanel->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+V", nullptr));
#endif // QT_CONFIG(shortcut)
        actionFix_Histogram->setText(QCoreApplication::translate("MainWindow", "Adjust &Levels", nullptr));
        actionFix_Histogram->setIconText(QCoreApplication::translate("MainWindow", "Adjust Levels", nullptr));
#if QT_CONFIG(tooltip)
        actionFix_Histogram->setToolTip(QCoreApplication::translate("MainWindow", "Adjust Levels", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        actionFix_Histogram->setStatusTip(QCoreApplication::translate("MainWindow", "Tune the histogram for the current LDR", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        actionFix_Histogram->setWhatsThis(QCoreApplication::translate("MainWindow", "Tune the histogram for the current LDR", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(shortcut)
        actionFix_Histogram->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+L", nullptr));
#endif // QT_CONFIG(shortcut)
        fileSaveAllAction->setText(QCoreApplication::translate("MainWindow", "Save All", nullptr));
#if QT_CONFIG(tooltip)
        fileSaveAllAction->setToolTip(QCoreApplication::translate("MainWindow", "Save All LDR files", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        fileSaveAllAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Save All LDR files", nullptr));
#endif // QT_CONFIG(whatsthis)
        actionRemove_Tab->setText(QCoreApplication::translate("MainWindow", "&Remove Tab", nullptr));
#if QT_CONFIG(shortcut)
        actionRemove_Tab->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+W", nullptr));
#endif // QT_CONFIG(shortcut)
        actionFill_to_Window->setText(QCoreApplication::translate("MainWindow", "Fill &Window", nullptr));
#if QT_CONFIG(tooltip)
        actionFill_to_Window->setToolTip(QCoreApplication::translate("MainWindow", "Fill window with the current viewer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionFill_to_Window->setShortcut(QCoreApplication::translate("MainWindow", "F", nullptr));
#endif // QT_CONFIG(shortcut)
        actionBatch_HDR->setText(QCoreApplication::translate("MainWindow", "Batch &HDR...", nullptr));
#if QT_CONFIG(shortcut)
        actionBatch_HDR->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+H", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSoft_Proofing->setText(QCoreApplication::translate("MainWindow", "Soft Proofing", nullptr));
        actionGamut_Check->setText(QCoreApplication::translate("MainWindow", "Gamut Check", nullptr));
        actionShow_on_the_right->setText(QCoreApplication::translate("MainWindow", "&Show on the right", nullptr));
        actionShow_on_the_bottom->setText(QCoreApplication::translate("MainWindow", "Show &on the bottom", nullptr));
        actionRealtimePreviews->setText(QCoreApplication::translate("MainWindow", "&Realtime Previews", nullptr));
        actionUpdateAvailable->setText(QCoreApplication::translate("MainWindow", "&Update", nullptr));
        actionFits_Importer->setText(QCoreApplication::translate("MainWindow", "&FITS Importer...", nullptr));
#if QT_CONFIG(tooltip)
        actionFits_Importer->setToolTip(QCoreApplication::translate("MainWindow", "Import four FITS files representing luminosity, red, green and blue channels", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionFits_Importer->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+F", nullptr));
#endif // QT_CONFIG(shortcut)
        actionWhite_Balance->setText(QCoreApplication::translate("MainWindow", "White Balance", nullptr));
#if QT_CONFIG(tooltip)
        actionWhite_Balance->setToolTip(QCoreApplication::translate("MainWindow", "Fix color cast, can be slow", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportToHTML->setText(QCoreApplication::translate("MainWindow", "E&xport to HTML", nullptr));
#if QT_CONFIG(tooltip)
        actionExportToHTML->setToolTip(QCoreApplication::translate("MainWindow", "Export to web page with embedded HDR viewer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionExportToHTML->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+L", nullptr));
#endif // QT_CONFIG(shortcut)
        actionShow_Full_Screen->setText(QCoreApplication::translate("MainWindow", "&Show Full Screen", nullptr));
#if QT_CONFIG(shortcut)
        actionShow_Full_Screen->setShortcut(QCoreApplication::translate("MainWindow", "F11", nullptr));
#endif // QT_CONFIG(shortcut)
        actionShow_Image_Full_Screen->setText(QCoreApplication::translate("MainWindow", "Show &Image Full Screen", nullptr));
#if QT_CONFIG(shortcut)
        actionShow_Image_Full_Screen->setShortcut(QCoreApplication::translate("MainWindow", "F10", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSupported_Cameras->setText(QCoreApplication::translate("MainWindow", "&Supported RAW Cameras", nullptr));
#if QT_CONFIG(tooltip)
        actionSupported_Cameras->setToolTip(QCoreApplication::translate("MainWindow", "Lists all cameras supported by the program", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSupported_Cameras->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+M", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSelect_Interpolation_Method->setText(QCoreApplication::translate("MainWindow", "&Lanczos/Bilinear interpolation", nullptr));
#if QT_CONFIG(tooltip)
        actionSelect_Interpolation_Method->setToolTip(QCoreApplication::translate("MainWindow", "<html><head/><body><p>Switch between Lanczos and Bilinear interpolation</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSelect_Interpolation_Method->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+K", nullptr));
#endif // QT_CONFIG(shortcut)
        actionOnline_Documentation->setText(QCoreApplication::translate("MainWindow", "&Online Documentation", nullptr));
#if QT_CONFIG(tooltip)
        actionOnline_Documentation->setToolTip(QCoreApplication::translate("MainWindow", "Open Online Documentation Web Site with the System Browser", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDevelopers_Website->setText(QCoreApplication::translate("MainWindow", "&Developers Web Site", nullptr));
        actionDevelopers_Website->setIconText(QCoreApplication::translate("MainWindow", "Developers Web Site", nullptr));
#if QT_CONFIG(tooltip)
        actionDevelopers_Website->setToolTip(QCoreApplication::translate("MainWindow", "Open Developers Web Site with the System Browser", nullptr));
#endif // QT_CONFIG(tooltip)
        actionLuminance_HDR_Website->setText(QCoreApplication::translate("MainWindow", "&Luminance HDR Web Site", nullptr));
        actionLuminance_HDR_Website->setIconText(QCoreApplication::translate("MainWindow", "Luminance HDR Web Site", nullptr));
#if QT_CONFIG(tooltip)
        actionLuminance_HDR_Website->setToolTip(QCoreApplication::translate("MainWindow", "Open Main Web Site with the System Browser", nullptr));
#endif // QT_CONFIG(tooltip)
        menuSettings->setTitle(QCoreApplication::translate("MainWindow", "&Tools", nullptr));
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        menuLuminance_HDR_Online->setTitle(QCoreApplication::translate("MainWindow", "Luminance &HDR Online", nullptr));
        menuWindows->setTitle(QCoreApplication::translate("MainWindow", "&Windows", nullptr));
        menuView->setTitle(QCoreApplication::translate("MainWindow", "&View", nullptr));
        menuHDR_Histogram->setTitle(QCoreApplication::translate("MainWindow", "HDR &Histogram", nullptr));
        menuToolbars->setTitle(QCoreApplication::translate("MainWindow", "&Toolbars", nullptr));
        menuPreview_Panel_2->setTitle(QCoreApplication::translate("MainWindow", "&Preview Panel", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "&File", nullptr));
        menuEdit->setTitle(QCoreApplication::translate("MainWindow", "&Edit", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("MainWindow", "Main Toolbar", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
