/********************************************************************************
** Form generated from reading UI file 'about.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUT_H
#define UI_ABOUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AboutLuminance
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QLabel *label_version;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QWidget *tab_2;
    QVBoxLayout *vboxLayout1;
    QTextBrowser *authorsBox;
    QWidget *tab_4;
    QVBoxLayout *vboxLayout2;
    QTextBrowser *thanksToBox;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_2;
    QTextBrowser *ChangelogBox;
    QWidget *tab_3;
    QVBoxLayout *vboxLayout3;
    QTextBrowser *GPLbox;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *pushButton;

    void setupUi(QDialog *AboutLuminance)
    {
        if (AboutLuminance->objectName().isEmpty())
            AboutLuminance->setObjectName(QString::fromUtf8("AboutLuminance"));
        AboutLuminance->resize(535, 375);
        gridLayout = new QGridLayout(AboutLuminance);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_2 = new QLabel(AboutLuminance);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy);
        label_2->setMinimumSize(QSize(32, 32));
        label_2->setMaximumSize(QSize(32, 32));
        label_2->setPixmap(QPixmap(QString::fromUtf8(":/program-icons/luminance-hdr")));
        label_2->setScaledContents(true);

        horizontalLayout->addWidget(label_2);

        label_version = new QLabel(AboutLuminance);
        label_version->setObjectName(QString::fromUtf8("label_version"));
        QFont font;
        font.setFamily(QString::fromUtf8("Sans Serif"));
        font.setPointSize(13);
        label_version->setFont(font);
        label_version->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout->addWidget(label_version);


        verticalLayout->addLayout(horizontalLayout);

        tabWidget = new QTabWidget(AboutLuminance);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        vboxLayout = new QVBoxLayout(tab);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        label = new QLabel(tab);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font1;
        font1.setPointSize(11);
        label->setFont(font1);
        label->setTextFormat(Qt::AutoText);
        label->setOpenExternalLinks(true);

        vboxLayout->addWidget(label);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        vboxLayout1 = new QVBoxLayout(tab_2);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        authorsBox = new QTextBrowser(tab_2);
        authorsBox->setObjectName(QString::fromUtf8("authorsBox"));
        authorsBox->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        authorsBox->setOpenExternalLinks(true);

        vboxLayout1->addWidget(authorsBox);

        tabWidget->addTab(tab_2, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        vboxLayout2 = new QVBoxLayout(tab_4);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        thanksToBox = new QTextBrowser(tab_4);
        thanksToBox->setObjectName(QString::fromUtf8("thanksToBox"));
        thanksToBox->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

        vboxLayout2->addWidget(thanksToBox);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName(QString::fromUtf8("tab_5"));
        verticalLayout_2 = new QVBoxLayout(tab_5);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        ChangelogBox = new QTextBrowser(tab_5);
        ChangelogBox->setObjectName(QString::fromUtf8("ChangelogBox"));

        verticalLayout_2->addWidget(ChangelogBox);

        tabWidget->addTab(tab_5, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        vboxLayout3 = new QVBoxLayout(tab_3);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        GPLbox = new QTextBrowser(tab_3);
        GPLbox->setObjectName(QString::fromUtf8("GPLbox"));
        QFont font2;
        font2.setPointSize(10);
        GPLbox->setFont(font2);
        GPLbox->setAutoFillBackground(true);
        GPLbox->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        GPLbox->setAutoFormatting(QTextEdit::AutoAll);
        GPLbox->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Sans Serif';\"><br /></p></body></html>"));
        GPLbox->setTabStopWidth(4);

        vboxLayout3->addWidget(GPLbox);

        tabWidget->addTab(tab_3, QString());

        verticalLayout->addWidget(tabWidget);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        pushButton = new QPushButton(AboutLuminance);
        pushButton->setObjectName(QString::fromUtf8("pushButton"));

        hboxLayout->addWidget(pushButton);


        verticalLayout->addLayout(hboxLayout);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(AboutLuminance);
        QObject::connect(pushButton, SIGNAL(clicked()), AboutLuminance, SLOT(accept()));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AboutLuminance);
    } // setupUi

    void retranslateUi(QDialog *AboutLuminance)
    {
        AboutLuminance->setWindowTitle(QCoreApplication::translate("AboutLuminance", "About Luminance HDR", nullptr));
        label_2->setText(QString());
        label_version->setText(QCoreApplication::translate("AboutLuminance", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; color:#0078dc;\">Luminance HDR</span></p></body></html>", nullptr));
        label->setText(QCoreApplication::translate("AboutLuminance", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:24pt; font-weight:600; color:#55aaff;\">Luminance HDR</span></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Create and tone-map HDR images.</p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -"
                        "qt-block-indent:0; text-indent:0px;\">(c) 2006-2009 Giuseppe Rota</p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(c) 2010-2018 Franco Comida, Davide Anastasia</p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(c) 2019-2021 Franco Comida</p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p align=\"center\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://github.com/LuminanceHDR/LuminanceHDR\"><span style=\" text-decoration: un"
                        "derline; color:#32a4e6;\">https://github.com/LuminanceHDR/LuminanceHDR</span></a></p></body></html>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("AboutLuminance", "&About", nullptr));
        authorsBox->setHtml(QCoreApplication::translate("AboutLuminance", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:14pt; font-weight:600;\">DEVELOPERS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Main developer of QtPfsGui</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0;"
                        " text-indent:0px;\"><span style=\" font-size:10pt;\">    Giuseppe Rota, </span><a href=\"mailto:grota@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">grota@users.sourceforge.net</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Main Developers of Luminance HDR</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Franco Comida, </span><a href=\"mailto:fcomida@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">fcomida@users.sourceforge.net</span></a></p>\n"
"<p style=\" margin-top:0px"
                        "; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Davide Anastasia, </span><a href=\"mailto:davideanastasia@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">davideanastasia@users.sourceforge.net</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Improvements, bugfixing</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Daniel Kaneider, </span><a href=\"mailto:danielkaneider@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: under"
                        "line; color:#32a4e6;\">danielkaneider@users.sourceforge.net</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Ingo Weyrich, </span><a href=\"mailto:heckflosse67@gmx.de\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">heckflosse67@gmx.de</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">pfs{tools|calibration|tmo} codebase</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Grzegorz Krawczyk, </span><a href=\"mailto:krawczyk@mpi-"
                        "sb.mpg.de\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">krawczyk@mpi-sb.mpg.de</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Rafal Mantiuk, </span><a href=\"mailto:mantiuk@mpi-sb.mpg.de\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">mantiuk@mpi-sb.mpg.de</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">G.Ward's Mean Threshold Bitmap (MTB) algorithm</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:1"
                        "0pt;\">    Nicholas Phillips, </span><a href=\"mailto:ngphillips@gmail.com\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">ngphillips@gmail.com</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:14pt; font-weight:600;\">CONTRIBUTORS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Arne Hagenah</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-i"
                        "ndent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Ignacy Gawedzki</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Jean-Christophe Frisch, </span><a href=\"mailto:natureh.510@gmail.com\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">natureh.510@gmail.com</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Maciej Dworak, </span><a href=\"https://github.com/Beep6581\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">https://github.com/Beep6581</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px"
                        "; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:14pt; font-weight:600;\">PACKAGE MAINTAINERS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Linux:</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">    Fedora</span><span style=\" font-size:10pt;\">: Franco Comida</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">    FreeBSD: </span><span style=\" font-size:10pt;\">Joao Rocha Braga Filho, Bertrand Pe"
                        "tit and Steffen Beyer</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">     </span><span style=\" font-size:10pt; font-weight:600;\">Gentoo</span><span style=\" font-size:10pt;\">: Rene Zbinden</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">    OpenSUSE</span><span style=\" font-size:10pt;\">: Peter Linnell, Rauch Christian</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">macOS</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-le"
                        "ft:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Robin Mills, </span><a href=\"mailto:robin@clanmills.com\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">robin@clanmills.com</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Richard Barber, </span><a href=\"https://github.com/Benitoite\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">https://github.com/Benitoite</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Windows</span></p>\n"
"<p style=\" margin-top:0px; marg"
                        "in-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Daniel Kaneider, </span><a href=\"mailto:danielkaneider@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">danielkaneider@users.sourceforge.net</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:14pt; font-weight:600;\">TRANSLATORS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\""
                        " font-size:10pt;\">    </span><a href=\"https://www.transifex.com/luminance-hdr/teams/\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">Luminance HDR translation team</span></a><span style=\" font-size:10pt;\"> at Transifex</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Chinese</span><span style=\" font-size:10pt;\">: Lei Zhao</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Czech</span><span style=\" font-size:10pt;\">: Jan Gemela</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" f"
                        "ont-size:10pt; font-weight:600;\">French</span><span style=\" font-size:10pt;\">: Cyril Lavier</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">German</span><span style=\" font-size:10pt;\">: Christian Raue</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Norwegian</span><span style=\" font-size:10pt;\">: \303\205smund \303\230deg\303\245rd, </span><a href=\"mailto:mandus@users.sourceforge.net\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">mandus@users.sourceforge.net</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font"
                        "-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Polish</span><span style=\" font-size:10pt;\">: Maciej Dziardziel</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Romanian</span><span style=\" font-size:10pt;\">: Andrei R\304\203c\304\203\305\237an</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Russian</span><span style=\" font-size:10pt;\">: Alexandre Prokoudine, </span><a href=\"mailto:alexandre.prokoudine@gmail.com\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">alexandre.prokoudine@gmail.com</span></a></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -q"
                        "t-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Spanish</span><span style=\" font-size:10pt;\">: Alberto Garcia</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    </span><span style=\" font-size:10pt; font-weight:600;\">Turkish</span><span style=\" font-size:10pt;\">: Mustafa G\303\274nay</span></p></body></html>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("AboutLuminance", "A&uthors", nullptr));
        thanksToBox->setHtml(QCoreApplication::translate("AboutLuminance", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">The community on </span><a href=\"https://discuss.pixls.us/c/software/luminancehdr\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">Pixls.us</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;"
                        " font-weight:600;\">The community on </span><a href=\"https://www.flickr.com/groups/qtpfsgui/\"><span style=\" font-size:10pt; text-decoration: underline; color:#32a4e6;\">Flickr</span></a></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Hugin's align_image_stack and more</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Pablo D'Angelo</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; "
                        "margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">DCRaw</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Dave Coffin</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:10pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">And also</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Erik Ouchterlony</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt"
                        ";\">    Antoine Latter</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Frank Boehme</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Sloan Poe</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt;\">    Elizabeth Oldham</span></p></body></html>", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("AboutLuminance", "&Thanks To", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("AboutLuminance", "Changelog", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("AboutLuminance", "&License Agreement", nullptr));
        pushButton->setText(QCoreApplication::translate("AboutLuminance", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutLuminance: public Ui_AboutLuminance {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUT_H
