/********************************************************************************
** Form generated from reading UI file 'GammaAndLevels.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GAMMAANDLEVELS_H
#define UI_GAMMAANDLEVELS_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LevelsDialog
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QLabel *label_6;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout1;
    QLabel *label;
    QSpacerItem *spacerItem2;
    QVBoxLayout *vboxLayout1;
    QFrame *inputStuffFrame;
    QHBoxLayout *hboxLayout2;
    QLabel *label_4;
    QSpinBox *black_in_spinbox;
    QSpacerItem *spacerItem3;
    QLabel *label_3;
    QDoubleSpinBox *gamma_spinbox;
    QSpacerItem *spacerItem4;
    QLabel *label_5;
    QSpinBox *white_in_spinbox;
    QHBoxLayout *hboxLayout3;
    QLabel *label_2;
    QSpacerItem *spacerItem5;
    QVBoxLayout *vboxLayout2;
    QWidget *out_levels;
    QHBoxLayout *hboxLayout4;
    QSpinBox *black_out_spinbox;
    QSpacerItem *spacerItem6;
    QSpinBox *white_out_spinbox;
    QFrame *line;
    QHBoxLayout *hboxLayout5;
    QPushButton *ResetButton;
    QSpacerItem *spacerItem7;
    QPushButton *cancelButton;
    QPushButton *okButton;

    void setupUi(QDialog *LevelsDialog)
    {
        if (LevelsDialog->objectName().isEmpty())
            LevelsDialog->setObjectName(QString::fromUtf8("LevelsDialog"));
        LevelsDialog->setWindowModality(Qt::ApplicationModal);
        LevelsDialog->resize(523, 379);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/view-object-histogram-linear"), QSize(), QIcon::Normal, QIcon::Off);
        LevelsDialog->setWindowIcon(icon);
        LevelsDialog->setModal(true);
        vboxLayout = new QVBoxLayout(LevelsDialog);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        label_6 = new QLabel(LevelsDialog);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        hboxLayout->addWidget(label_6);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(LevelsDialog);
        label->setObjectName(QString::fromUtf8("label"));

        hboxLayout1->addWidget(label);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);


        vboxLayout->addLayout(hboxLayout1);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        inputStuffFrame = new QFrame(LevelsDialog);
        inputStuffFrame->setObjectName(QString::fromUtf8("inputStuffFrame"));
        inputStuffFrame->setMinimumSize(QSize(401, 80));
        inputStuffFrame->setFrameShape(QFrame::StyledPanel);
        inputStuffFrame->setFrameShadow(QFrame::Sunken);

        vboxLayout1->addWidget(inputStuffFrame);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        label_4 = new QLabel(LevelsDialog);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        hboxLayout2->addWidget(label_4);

        black_in_spinbox = new QSpinBox(LevelsDialog);
        black_in_spinbox->setObjectName(QString::fromUtf8("black_in_spinbox"));
        black_in_spinbox->setMaximum(255);

        hboxLayout2->addWidget(black_in_spinbox);

        spacerItem3 = new QSpacerItem(16, 25, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem3);

        label_3 = new QLabel(LevelsDialog);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        hboxLayout2->addWidget(label_3);

        gamma_spinbox = new QDoubleSpinBox(LevelsDialog);
        gamma_spinbox->setObjectName(QString::fromUtf8("gamma_spinbox"));
        gamma_spinbox->setMinimum(0.100000000000000);
        gamma_spinbox->setMaximum(10.000000000000000);
        gamma_spinbox->setSingleStep(0.010000000000000);
        gamma_spinbox->setValue(1.000000000000000);

        hboxLayout2->addWidget(gamma_spinbox);

        spacerItem4 = new QSpacerItem(16, 25, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem4);

        label_5 = new QLabel(LevelsDialog);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        hboxLayout2->addWidget(label_5);

        white_in_spinbox = new QSpinBox(LevelsDialog);
        white_in_spinbox->setObjectName(QString::fromUtf8("white_in_spinbox"));
        white_in_spinbox->setMaximum(255);
        white_in_spinbox->setValue(255);

        hboxLayout2->addWidget(white_in_spinbox);


        vboxLayout1->addLayout(hboxLayout2);


        vboxLayout->addLayout(vboxLayout1);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        label_2 = new QLabel(LevelsDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        hboxLayout3->addWidget(label_2);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem5);


        vboxLayout->addLayout(hboxLayout3);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        out_levels = new QWidget(LevelsDialog);
        out_levels->setObjectName(QString::fromUtf8("out_levels"));
        out_levels->setMinimumSize(QSize(401, 21));

        vboxLayout2->addWidget(out_levels);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        black_out_spinbox = new QSpinBox(LevelsDialog);
        black_out_spinbox->setObjectName(QString::fromUtf8("black_out_spinbox"));
        black_out_spinbox->setMaximum(255);

        hboxLayout4->addWidget(black_out_spinbox);

        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout4->addItem(spacerItem6);

        white_out_spinbox = new QSpinBox(LevelsDialog);
        white_out_spinbox->setObjectName(QString::fromUtf8("white_out_spinbox"));
        white_out_spinbox->setMaximum(255);
        white_out_spinbox->setValue(255);

        hboxLayout4->addWidget(white_out_spinbox);


        vboxLayout2->addLayout(hboxLayout4);


        vboxLayout->addLayout(vboxLayout2);

        line = new QFrame(LevelsDialog);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setSpacing(6);
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        ResetButton = new QPushButton(LevelsDialog);
        ResetButton->setObjectName(QString::fromUtf8("ResetButton"));

        hboxLayout5->addWidget(ResetButton);

        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem7);

        cancelButton = new QPushButton(LevelsDialog);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout5->addWidget(cancelButton);

        okButton = new QPushButton(LevelsDialog);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout5->addWidget(okButton);


        vboxLayout->addLayout(hboxLayout5);


        retranslateUi(LevelsDialog);
        QObject::connect(cancelButton, SIGNAL(clicked()), LevelsDialog, SLOT(reject()));
        QObject::connect(okButton, SIGNAL(clicked()), LevelsDialog, SLOT(accept()));

        QMetaObject::connectSlotsByName(LevelsDialog);
    } // setupUi

    void retranslateUi(QDialog *LevelsDialog)
    {
        LevelsDialog->setWindowTitle(QCoreApplication::translate("LevelsDialog", "Levels and Gamma", nullptr));
        label_6->setText(QCoreApplication::translate("LevelsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Adjust Color Levels</span></p></body></html>", nullptr));
        label->setText(QCoreApplication::translate("LevelsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Input Levels</span></p></body></html>", nullptr));
        label_4->setText(QCoreApplication::translate("LevelsDialog", "Clip black", nullptr));
        label_3->setText(QCoreApplication::translate("LevelsDialog", "Gamma", nullptr));
        label_5->setText(QCoreApplication::translate("LevelsDialog", "Clip White", nullptr));
        label_2->setText(QCoreApplication::translate("LevelsDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Output Levels</span></p></body></html>", nullptr));
        ResetButton->setText(QCoreApplication::translate("LevelsDialog", "&Reset", nullptr));
        cancelButton->setText(QCoreApplication::translate("LevelsDialog", "&Cancel", nullptr));
        okButton->setText(QCoreApplication::translate("LevelsDialog", "&OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LevelsDialog: public Ui_LevelsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GAMMAANDLEVELS_H
