/********************************************************************************
** Form generated from reading UI file 'DonationDialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DONATIONDIALOG_H
#define UI_DONATIONDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DonationDialog
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QPushButton *yesButton;
    QPushButton *askMeLaterButton;
    QPushButton *noButton;

    void setupUi(QWidget *DonationDialog)
    {
        if (DonationDialog->objectName().isEmpty())
            DonationDialog->setObjectName(QString::fromUtf8("DonationDialog"));
        DonationDialog->resize(576, 188);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DonationDialog->sizePolicy().hasHeightForWidth());
        DonationDialog->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/program-icons/luminance-hdr"), QSize(), QIcon::Normal, QIcon::Off);
        DonationDialog->setWindowIcon(icon);
        gridLayout = new QGridLayout(DonationDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(DonationDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        yesButton = new QPushButton(DonationDialog);
        yesButton->setObjectName(QString::fromUtf8("yesButton"));

        horizontalLayout->addWidget(yesButton);

        askMeLaterButton = new QPushButton(DonationDialog);
        askMeLaterButton->setObjectName(QString::fromUtf8("askMeLaterButton"));
        askMeLaterButton->setAutoDefault(true);

        horizontalLayout->addWidget(askMeLaterButton);

        noButton = new QPushButton(DonationDialog);
        noButton->setObjectName(QString::fromUtf8("noButton"));

        horizontalLayout->addWidget(noButton);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);

        QWidget::setTabOrder(yesButton, askMeLaterButton);
        QWidget::setTabOrder(askMeLaterButton, noButton);

        retranslateUi(DonationDialog);

        askMeLaterButton->setDefault(false);


        QMetaObject::connectSlotsByName(DonationDialog);
    } // setupUi

    void retranslateUi(QWidget *DonationDialog)
    {
        DonationDialog->setWindowTitle(QCoreApplication::translate("DonationDialog", "Luminance HDR - Make a Donation", nullptr));
        label->setText(QCoreApplication::translate("DonationDialog", "<html><head/><body><p><span style=\" font-weight:600;\">Luminance HDR</span> is open-source software and its development required hundreds of hours of work.</p><p>If you like it,  you are using it in your HDR workflow and you would like to see it gradually improved, please support its authors by making a donation.</p><p>Would you like to make a donation for Luminance HDR now?</p></body></html>", nullptr));
        yesButton->setText(QCoreApplication::translate("DonationDialog", "Yes, I'd love to!", nullptr));
        askMeLaterButton->setText(QCoreApplication::translate("DonationDialog", "Ask me again later", nullptr));
        noButton->setText(QCoreApplication::translate("DonationDialog", "No, Stop bothering me!", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DonationDialog: public Ui_DonationDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DONATIONDIALOG_H
