/****************************************************************************
** Meta object code from reading C++ file 'TransplantExifDialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../luminance-hdr-2.6.1.1/src/TransplantExif/TransplantExifDialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TransplantExifDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TransplantExifDialog_t {
    QByteArrayData data[14];
    char stringdata0[200];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TransplantExifDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TransplantExifDialog_t qt_meta_stringdata_TransplantExifDialog = {
    {
QT_MOC_LITERAL(0, 0, 20), // "TransplantExifDialog"
QT_MOC_LITERAL(1, 21, 20), // "transplant_requested"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 14), // "help_requested"
QT_MOC_LITERAL(4, 58, 11), // "moveup_left"
QT_MOC_LITERAL(5, 70, 12), // "moveup_right"
QT_MOC_LITERAL(6, 83, 13), // "movedown_left"
QT_MOC_LITERAL(7, 97, 14), // "movedown_right"
QT_MOC_LITERAL(8, 112, 11), // "remove_left"
QT_MOC_LITERAL(9, 124, 12), // "remove_right"
QT_MOC_LITERAL(10, 137, 11), // "append_left"
QT_MOC_LITERAL(11, 149, 12), // "append_right"
QT_MOC_LITERAL(12, 162, 13), // "filterChanged"
QT_MOC_LITERAL(13, 176, 23) // "filterComboBoxActivated"

    },
    "TransplantExifDialog\0transplant_requested\0"
    "\0help_requested\0moveup_left\0moveup_right\0"
    "movedown_left\0movedown_right\0remove_left\0"
    "remove_right\0append_left\0append_right\0"
    "filterChanged\0filterComboBoxActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TransplantExifDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x08 /* Private */,
       3,    0,   75,    2, 0x08 /* Private */,
       4,    0,   76,    2, 0x08 /* Private */,
       5,    0,   77,    2, 0x08 /* Private */,
       6,    0,   78,    2, 0x08 /* Private */,
       7,    0,   79,    2, 0x08 /* Private */,
       8,    0,   80,    2, 0x08 /* Private */,
       9,    0,   81,    2, 0x08 /* Private */,
      10,    0,   82,    2, 0x08 /* Private */,
      11,    0,   83,    2, 0x08 /* Private */,
      12,    1,   84,    2, 0x08 /* Private */,
      13,    1,   87,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void TransplantExifDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TransplantExifDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->transplant_requested(); break;
        case 1: _t->help_requested(); break;
        case 2: _t->moveup_left(); break;
        case 3: _t->moveup_right(); break;
        case 4: _t->movedown_left(); break;
        case 5: _t->movedown_right(); break;
        case 6: _t->remove_left(); break;
        case 7: _t->remove_right(); break;
        case 8: _t->append_left(); break;
        case 9: _t->append_right(); break;
        case 10: _t->filterChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->filterComboBoxActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TransplantExifDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_TransplantExifDialog.data,
    qt_meta_data_TransplantExifDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TransplantExifDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TransplantExifDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TransplantExifDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int TransplantExifDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
