/********************************************************************************
** Form generated from reading UI file 'ThresholdWidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_THRESHOLDWIDGET_H
#define UI_THRESHOLDWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>

QT_BEGIN_NAMESPACE

class Ui_ThresholdWidget
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSlider *thresholdHorizontalSlider;
    QDoubleSpinBox *thresholdDoubleSpinBox;

    void setupUi(QFrame *ThresholdWidget)
    {
        if (ThresholdWidget->objectName().isEmpty())
            ThresholdWidget->setObjectName(QString::fromUtf8("ThresholdWidget"));
        ThresholdWidget->setWindowModality(Qt::ApplicationModal);
        ThresholdWidget->resize(400, 49);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ThresholdWidget->sizePolicy().hasHeightForWidth());
        ThresholdWidget->setSizePolicy(sizePolicy);
        ThresholdWidget->setFrameShape(QFrame::StyledPanel);
        ThresholdWidget->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(ThresholdWidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(ThresholdWidget);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        thresholdHorizontalSlider = new QSlider(ThresholdWidget);
        thresholdHorizontalSlider->setObjectName(QString::fromUtf8("thresholdHorizontalSlider"));
        thresholdHorizontalSlider->setMinimum(800);
        thresholdHorizontalSlider->setMaximum(999);
        thresholdHorizontalSlider->setValue(985);
        thresholdHorizontalSlider->setOrientation(Qt::Horizontal);
        thresholdHorizontalSlider->setTickPosition(QSlider::TicksBelow);
        thresholdHorizontalSlider->setTickInterval(10);

        horizontalLayout->addWidget(thresholdHorizontalSlider);

        thresholdDoubleSpinBox = new QDoubleSpinBox(ThresholdWidget);
        thresholdDoubleSpinBox->setObjectName(QString::fromUtf8("thresholdDoubleSpinBox"));
        thresholdDoubleSpinBox->setDecimals(3);
        thresholdDoubleSpinBox->setMinimum(0.800000000000000);
        thresholdDoubleSpinBox->setMaximum(0.999000000000000);
        thresholdDoubleSpinBox->setSingleStep(0.001000000000000);
        thresholdDoubleSpinBox->setValue(0.985000000000000);

        horizontalLayout->addWidget(thresholdDoubleSpinBox);


        gridLayout->addLayout(horizontalLayout, 0, 0, 1, 1);


        retranslateUi(ThresholdWidget);

        QMetaObject::connectSlotsByName(ThresholdWidget);
    } // setupUi

    void retranslateUi(QFrame *ThresholdWidget)
    {
        ThresholdWidget->setWindowTitle(QCoreApplication::translate("ThresholdWidget", "Frame", nullptr));
        label->setText(QCoreApplication::translate("ThresholdWidget", "Threshold", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ThresholdWidget: public Ui_ThresholdWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_THRESHOLDWIDGET_H
